
ord_names = {
    1:'accept',
    2:'bind',
    3:'closesocket',
    4:'connect',
    5:'getpeername',
    6:'getsockname',
    7:'getsockopt',
    8:'htonl',
    9:'htons',
    10:'ioctlsocket',
    11:'inet_addr',
    12:'inet_ntoa',
    13:'listen',
    14:'ntohl',
    15:'ntohs',
    16:'recv',
    17:'recvfrom',
    18:'select',
    19:'send',
    20:'sendto',
    21:'setsockopt',
    22:'shutdown',
    23:'socket',
    24:'GetAddrInfoW',
    25:'GetNameInfoW',
    26:'WSApSetPostRoutine',
    27:'FreeAddrInfoW',
    28:'WPUCompleteOverlappedRequest',
    29:'WSAAccept',
    30:'WSAAddressToStringA',
    31:'WSAAddressToStringW',
    32:'WSACloseEvent',
    33:'WSAConnect',
    34:'WSACreateEvent',
    35:'WSADuplicateSocketA',
    36:'WSADuplicateSocketW',
    37:'WSAEnumNameSpaceProvidersA',
    38:'WSAEnumNameSpaceProvidersW',
    39:'WSAEnumNetworkEvents',
    40:'WSAEnumProtocolsA',
    41:'WSAEnumProtocolsW',
    42:'WSAEventSelect',
    43:'WSAGetOverlappedResult',
    44:'WSAGetQOSByName',
    45:'WSAGetServiceClassInfoA',
    46:'WSAGetServiceClassInfoW',
    47:'WSAGetServiceClassNameByClassIdA',
    48:'WSAGetServiceClassNameByClassIdW',
    49:'WSAHtonl',
    50:'WSAHtons',
    51:'gethostbyaddr',
    52:'gethostbyname',
    53:'getprotobyname',
    54:'getprotobynumber',
    55:'getservbyname',
    56:'getservbyport',
    57:'gethostname',
    58:'WSAInstallServiceClassA',
    59:'WSAInstallServiceClassW',
    60:'WSAIoctl',
    61:'WSAJoinLeaf',
    62:'WSALookupServiceBeginA',
    63:'WSALookupServiceBeginW',
    64:'WSALookupServiceEnd',
    65:'WSALookupServiceNextA',
    66:'WSALookupServiceNextW',
    67:'WSANSPIoctl',
    68:'WSANtohl',
    69:'WSANtohs',
    70:'WSAProviderConfigChange',
    71:'WSARecv',
    72:'WSARecvDisconnect',
    73:'WSARecvFrom',
    74:'WSARemoveServiceClass',
    75:'WSAResetEvent',
    76:'WSASend',
    77:'WSASendDisconnect',
    78:'WSASendTo',
    79:'WSASetEvent',
    80:'WSASetServiceA',
    81:'WSASetServiceW',
    82:'WSASocketA',
    83:'WSASocketW',
    84:'WSAStringToAddressA',
    85:'WSAStringToAddressW',
    86:'WSAWaitForMultipleEvents',
    87:'WSCDeinstallProvider',
    88:'WSCEnableNSProvider',
    89:'WSCEnumProtocols',
    90:'WSCGetProviderPath',
    91:'WSCInstallNameSpace',
    92:'WSCInstallProvider',
    93:'WSCUnInstallNameSpace',
    94:'WSCUpdateProvider',
    95:'WSCWriteNameSpaceOrder',
    96:'WSCWriteProviderOrder',
    97:'freeaddrinfo',
    98:'getaddrinfo',
    99:'getnameinfo',
    101:'WSAAsyncSelect',
    102:'WSAAsyncGetHostByAddr',
    103:'WSAAsyncGetHostByName',
    104:'WSAAsyncGetProtoByNumber',
    105:'WSAAsyncGetProtoByName',
    106:'WSAAsyncGetServByPort',
    107:'WSAAsyncGetServByName',
    108:'WSACancelAsyncRequest',
    109:'WSASetBlockingHook',
    110:'WSAUnhookBlockingHook',
    111:'WSAGetLastError',
    112:'WSASetLastError',
    113:'WSACancelBlockingCall',
    114:'WSAIsBlocking',
    115:'WSAStartup',
    116:'WSACleanup',
    151:'__WSAFDIsSet',
    500:'WEP',
}
