'''
This directory should contain tidbits and trinkets which help the maintainers upkeep the code.

This repository of information/tools began with commented-out-notes from the symbolic switchcase module, to keep track of things like symbolic states and various kinds of switchcases we've encountered and had to analyze.

vivisect.tests.testelf relies upon a number of python modules which store unittest data.  Those modules are generated by scripts which are ugly and not maintained as part of the official Vivisect code... so we store them here so they're never lost (eg.  atlas gets hit by a bus)
'''
