VMCS_NAMES = {
    0x0400: 'Guest_ES_Selector',
    0x0402: 'Guest_CS_Selector',
    0x0404: 'Guest_SS_Selector',
    0x0406: 'Guest_DS_Selector',
    0x0408: 'Guest_FS_Selector',
    0x040a: 'Guest_GS_Selector',
    0x040c: 'Guest_LDTR_Selector',
    0x040e: 'Guest_TR_Selector',

    0x0800: 'Host_ES_Selector',
    0x0802: 'Host_CS_Selector',
    0x0804: 'Host_SS_Selector',
    0x0806: 'Host_DS_Selector',
    0x0808: 'Host_FS_Selector',
    0x080a: 'Host_GS_Selector',
    0x080c: 'Host_TR_Selector',

    0x2000: 'Address_of_IO_bitmap_A_full',
    0x2001: 'Address_of_IO_bitmap_A_HIGH',
    0x2002: 'Address_of_IO_bitmap_B_full',
    0x2003: 'Address_of_IO_bitmap_B_HIGH',
    0x2004: 'Address_of_MSR_bitmaps_full',
    0x2005: 'Address_of_MSR_bitmaps_HIGH',
    0x2006: 'VM-exit_MSR_store_address_full',
    0x2007: 'VM-exit_MSR_store_address_HIGH',
    0x2008: 'VM-exit_MSR_load_address_full',
    0x2009: 'VM-exit_MSR_load_address_HIGH',
    0x200a: 'VM-entry_MSR_load_address_full',
    0x200b: 'VM-entry_MSR_load_address_HIGH',
    0x200c: 'Executive-VMCS_pointer_full',
    0x200d: 'Executive-VMCS_pointer_HIGH',

    0x2010: 'TSC_offset_full',
    0x2011: 'TSC_offset_HIGH',
    0x2012: 'Virtual_APIC_address_full',
    0x2013: 'Virtual_APIC_address_HIGH',
    0x2014: 'APIC-access_address_full',
    0x2015: 'APIC-access_address_HIGH',

    0x2800: 'VMCS_link_pointer_full',
    0x2801: 'VMCS_link_pointer_HIGH',
    0x2802: 'Guest_IA32_DEBUGCTL_full',
    0x2803: 'Guest_IA32_DEBUGCTL_HIGH',

    0x4000: 'Pin -based_VM-execution_controls',
    0x4002: 'Primary_process-based_VM-execution_controls',
    0x4004: 'Exception_bitmap',
    0x4006: 'Page_fault_error-code_mask',
    0x4008: 'Page-fault_error-code_match',
    0x400a: 'CR3-target_count',
    0x400c: 'VM-exit_controls',
    0x400e: 'VM-exit_MSR-store_count',
    0x4010: 'VM-exit_MSR-load_count',
    0x4012: 'VM-entry_controls',
    0x4014: 'VM-entry_MSR-load_count',
    0x4016: 'VM-entry_interruption-information_field',
    0x4018: 'VM-entry_exception_error_code',
    0x401a: 'VM-entry_instruction_length',
    0x401c: 'TPR_threshold',
    0x401e: 'Secondary_processor-based_VM-execution_controls',
    0x4020: 'PLE_Gap',
    0x4022: 'PLE_Window',

    0x4400: 'VM-instruction_error',
    0x4402: 'Exit_reason',
    0x4404: 'VM-exit_interruption_information',
    0x4406: 'VM-exit_interruption_error_code',
    0x4408: 'IDT-vectoring_information_field',
    0x440a: 'IDT-vectoring_error_code',
    0x440c: 'VM-exit_instruction_length',
    0x440e: 'VM-exit_instruction_information',

    0x4800: 'Guest_ES_limit',
    0x4802: 'Guest_CS_limit',
    0x4804: 'Guest_SS_limit',
    0x4806: 'Guest_DS_limit',
    0x4808: 'Guest_FS_limit',
    0x480a: 'Guest_GS_limit',
    0x480c: 'Guest_LDTR_limit',
    0x480e: 'Guest_TR_limit',
    0x4810: 'Guest_GDTR_limit',
    0x4812: 'Guest_IDTR_limit',
    0x4814: 'Guest_ES_access_rights',
    0x4816: 'Guest_CS_access_rights',
    0x4818: 'Guest_SS_access_rights',
    0x481a: 'Guest_DS_access_rights',
    0x481c: 'Guest_FS_access_rights',
    0x481e: 'Guest_GS_access_rights',
    0x4820: 'Guest_LDTR_access_rights',
    0x4822: 'Guest_TR_access_rights',
    0x4824: 'Guest_Interruptability_state',
    0x4826: 'Guest_activity_state',
    0x4828: 'Guest_SMBASE',
    0x482a: 'Guest_IA32_SYSENTER_CS',

    0x4c00: 'Host_IA32_SYSENTER_CS',

    0x6000: 'CR0_guest_host_mask',
    0x6002: 'CR4_guest_host_mask',
    0x6004: 'CR0_read_shadow',
    0x6006: 'CR4_read_shadow',
    0x6008: 'CR3_target_value_0',
    0x600a: 'CR3_target_value_1',
    0x600c: 'CR3_target_value_2',
    0x600e: 'CR3_target_value_3',

    0x6400: 'Exit_qualification',
    0x6402: 'IO_RCX',
    0x6404: 'IO_RSI',
    0x6406: 'IO_RDI',
    0x6408: 'IO_RIP',
    0x640a: 'Guest_linear_address',

    0x6800: 'Guest_CR0',
    0x6802: 'Guest_CR3',
    0x6804: 'Guest_CR4',
    0x6806: 'Guest_ES_base',
    0x6808: 'Guest_CS_base',
    0x680a: 'Guest_SS_base',
    0x680c: 'Guest_DS_base',
    0x680e: 'Guest_FS_base',
    0x6810: 'Guest_GS_base',
    0x6812: 'Guest_LDTR_base',
    0x6814: 'Guest_TR_base',
    0x6816: 'Guest_GDTR_base',
    0x6818: 'Guest_IDTR_base',
    0x681a: 'Guest_DR7',
    0x681c: 'Guest_RSP',
    0x681e: 'Guest_RIP',
    0x6820: 'Guest_RFLAGS',
    0x6822: 'Guest_pending_debug_exceptions',
    0x6824: 'Guest_IA32_SYSENTER_ESP',
    0x6826: 'Guest_IA32_SYSENTER_EIP',

    0x6800: 'Host_CR0',
    0x6802: 'Host_CR3',
    0x6804: 'Host_CR4',
    0x6806: 'Host_FS_base',
    0x6808: 'Host_GS_base',
    0x680a: 'Host_TR_base',
    0x680c: 'Host_GDTR_base',
    0x680e: 'Host_IDTR_base',
    0x6810: 'Host_IA32_SYSENTER_ESP',
    0x6812: 'Host_IA32_SYSENTER_EIP',
    0x6814: 'Host_RSP',
    0x6816: 'Host_RIP',
}
