ord_names = {
    100:'ThunRTMain',
    101:'VBDllUnRegisterServer',
    102:'VBDllCanUnloadNow',
    103:'VBDllRegisterServer',
    104:'VBDllGetClassObject',
    105:'UserDllMain',
    106:'DllRegisterServer',
    107:'DllUnregisterServer',
    108:'__vbaAryLock',
    109:'__vbaBoolErrVar',
    110:'__vbaRedimVar2',
    111:'__vbaStrErrVarCopy',
    112:'__vbaVarLateMemCallLd',
    113:'__vbaVarLateMemCallLdRf',
    114:'__vbaVarLateMemCallSt',
    115:'__vbaVarLateMemSt',
    116:'__vbaVarLateMemStAd',
    117:'__vbaAryVarVarg',
    118:'__vbaFpCDblR4',
    119:'__vbaFpCDblR8',
    120:'__vbaFpCSngR4',
    121:'__vbaFpCSngR8',
    122:'__vbaFpCmpCy',
    123:'__vbaFpCy',
    124:'__vbaFpI2',
    125:'__vbaFpI4',
    126:'__vbaFpR4',
    127:'__vbaFpR8',
    128:'__vbaFpUI1',
    129:'__vbaFreeObj',
    130:'__vbaFreeStr',
    131:'__vbaFreeVar',
    132:'__vbaFreeVarg',
    133:'__vbaI2Abs',
    134:'__vbaI2I4',
    135:'__vbaI2Sgn',
    136:'__vbaI4Abs',
    137:'__vbaI4Sgn',
    138:'__vbaStrCopy',
    139:'__vbaStrMove',
    140:'__vbaUI1I2',
    141:'__vbaUI1I4',
    142:'__vbaUI1Sgn',
    143:'__vbaVarCopy',
    144:'__vbaVarDup',
    145:'__vbaVarMove',
    146:'__vbaVarVargNofree',
    147:'__vbaVarZero',
    148:'__vbaVargParmRef',
    149:'__vbaVargVar',
    150:'__vbaVargVarCopy',
    151:'__vbaVargVarMove',
    152:'__vbaVargVarRef',
    153:'DLLGetDocumentation',
    154:'DllCanUnloadNow',
    155:'DllGetClassObject',
    156:'_CIatan',
    157:'_CIcos',
    158:'_CIexp',
    159:'_CIlog',
    160:'_CIsin',
    161:'_CIsqrt',
    162:'_CItan',
    163:'__vbaAptOffset',
    164:'__vbaAryConstruct2',
    165:'__vbaAryConstruct',
    166:'__vbaAryCopy',
    167:'__vbaAryDestruct',
    168:'__vbaAryMove',
    169:'__vbaAryRebase1Var',
    170:'__vbaAryRecCopy',
    171:'__vbaAryRecMove',
    172:'__vbaAryUnlock',
    173:'__vbaAryVar',
    174:'__vbaBoolStr',
    175:'__vbaBoolVar',
    176:'__vbaBoolVarNull',
    177:'__vbaCVarAryUdt',
    178:'__vbaCastObj',
    179:'__vbaCastObjVar',
    180:'__vbaCheckType',
    181:'__vbaCheckTypeVar',
    182:'__vbaChkstk',
    183:'__vbaCopyBytes',
    184:'__vbaCopyBytesZero',
    185:'__vbaCyAbs',
    186:'ProcCallEngine',
    187:'DllFunctionCall',
    188:'__vbaCyAdd',
    189:'__vbaCyErrVar',
    190:'CopyRecord',
    191:'__vbaCyFix',
    192:'__vbaCyForInit',
    193:'__vbaCyForNext',
    194:'__vbaCyI2',
    195:'TipGetAddressOfPredeclaredInstance',
    196:'__vbaCyI4',
    197:'__vbaCyInt',
    198:'__vbaCyMul',
    199:'MethCallEngine',
    200:'__vbaCyMulI2',
    201:'__vbaCySgn',
    202:'__vbaCyStr',
    203:'__vbaCySub',
    204:'__vbaCyUI1',
    205:'__vbaCyVar',
    206:'__vbaDateR4',
    207:'__vbaDateR8',
    208:'__vbaDateStr',
    209:'__vbaDateVar',
    210:'__vbaDerefAry',
    211:'__vbaDerefAry1',
    212:'__vbaEnd',
    213:'__vbaErase',
    214:'__vbaEraseKeepData',
    215:'__vbaEraseNoPop',
    216:'__vbaError',
    217:'__vbaErrorOverflow',
    218:'__vbaExceptHandler',
    219:'__vbaExitEachAry',
    220:'TipSetOption',
    221:'__vbaExitEachColl',
    222:'__vbaExitEachVar',
    223:'__vbaExitProc',
    224:'__vbaFPException',
    225:'__vbaFPFix',
    226:'__vbaFPInt',
    227:'TipUnloadProject',
    228:'__vbaFailedFriend',
    229:'__vbaFileClose',
    230:'TipCreateInstanceProject2',
    231:'EbResetProject',
    232:'EbGetHandleOfExecutingProject',
    233:'__vbaFileCloseAll',
    234:'__vbaFileLock',
    235:'__vbaFileOpen',
    236:'__vbaFileSeek',
    237:'__vbaFixstrConstruct',
    238:'__vbaForEachAry',
    239:'__vbaForEachCollAd',
    240:'__vbaForEachCollObj',
    241:'__vbaForEachCollVar',
    242:'__vbaForEachVar',
    243:'__vbaFreeObjList',
    244:'__vbaFreeStrList',
    245:'__vbaFreeVarList',
    246:'__vbaGenerateBoundsError',
    247:'__vbaGet3',
    248:'__vbaGet4',
    249:'__vbaGetFxStr3',
    250:'__vbaGetFxStr4',
    251:'__vbaGetOwner3',
    252:'__vbaGetOwner4',
    253:'__vbaGosub',
    254:'__vbaGosubFree',
    255:'__vbaGosubReturn',
    256:'__vbaHresultCheck',
    257:'__vbaHresultCheckNonvirt',
    258:'__vbaHresultCheckObj',
    259:'__vbaI2Cy',
    260:'__vbaI2ErrVar',
    261:'__vbaI2ForNextCheck',
    262:'__vbaI2Str',
    263:'__vbaI2Var',
    264:'__vbaI4Cy',
    265:'EbResetProjectNormal',
    266:'TipUnloadInstance',
    267:'__vbaI4ErrVar',
    268:'EbLibraryLoad',
    269:'EbLibraryUnload',
    270:'__vbaI4ForNextCheck',
    271:'EbLoadRunTime',
    272:'__vbaI4Str',
    273:'__vbaI4Var',
    274:'EbCreateContext',
    275:'EbDestroyContext',
    276:'EbSetContextWorkerThread',
    277:'__vbaInStr',
    278:'__vbaInStrB',
    279:'__vbaInStrVar',
    280:'__vbaInStrVarB',
    281:'__vbaInputFile',
    282:'__vbaLateIdCall',
    283:'__vbaLateIdCallLd',
    284:'EbGetErrorInfo',
    285:'__vbaLateIdCallSt',
    286:'__vbaLateIdNamedCall',
    287:'__vbaLateIdNamedCallLd',
    288:'__vbaLateIdNamedCallSt',
    289:'__vbaLateIdNamedStAd',
    290:'__vbaLateIdSt',
    291:'__vbaLateIdStAd',
    292:'__vbaLateMemCall',
    293:'__vbaLateMemCallLd',
    294:'__vbaLateMemCallSt',
    295:'__vbaLateMemNamedCall',
    296:'__vbaLateMemNamedCallLd',
    297:'__vbaLateMemNamedCallSt',
    298:'EbIsProjectOnStack',
    299:'TipCreateInstanceEx',
    300:'GetMem2',
    301:'GetMem4',
    302:'GetMem8',
    303:'GetMemStr',
    304:'GetMemVar',
    305:'GetMemObj',
    306:'PutMem2',
    307:'PutMem4',
    308:'PutMem8',
    309:'PutMemStr',
    310:'PutMemVar',
    311:'PutMemObj',
    312:'SetMemVar',
    313:'SetMemObj',
    314:'GetMemNewObj',
    315:'PutMemNewObj',
    316:'SetMemNewObj',
    317:'GetMem1',
    318:'PutMem1',
    319:'GetMemEvent',
    320:'PutMemEvent',
    321:'SetMemEvent',
    322:'__vbaLateMemNamedStAd',
    323:'__vbaLateMemSt',
    324:'__vbaLateMemStAd',
    325:'__vbaLbound',
    326:'__vbaLdZeroAry',
    327:'__vbaLenBstr',
    328:'__vbaLenBstrB',
    329:'__vbaLenVar',
    330:'__vbaLenVarB',
    331:'__vbaLineInputStr',
    332:'__vbaLineInputVar',
    333:'__vbaLsetFixstr',
    334:'__vbaLsetFixstrFree',
    335:'__vbaMidStmtBstr',
    336:'__vbaMidStmtBstrB',
    337:'__vbaMidStmtVar',
    338:'__vbaMidStmtVarB',
    339:'__vbaNameFile',
    340:'__vbaNew2',
    341:'__vbaNew',
    342:'__vbaNextEachAry',
    343:'__vbaNextEachCollAd',
    344:'__vbaNextEachCollObj',
    345:'__vbaNextEachCollVar',
    346:'__vbaNextEachVar',
    347:'__vbaObjAddref',
    348:'__vbaObjIs',
    349:'__vbaObjSet',
    350:'__vbaObjSetAddref',
    351:'__vbaObjVar',
    352:'__vbaOnError',
    353:'__vbaOnGoCheck',
    354:'__vbaPowerR8',
    355:'__vbaPrintFile',
    356:'__vbaPrintObj',
    357:'__vbaPut3',
    358:'__vbaPut4',
    359:'__vbaPutFxStr3',
    360:'__vbaPutFxStr4',
    361:'__vbaPutOwner3',
    362:'__vbaPutOwner4',
    363:'__vbaR4Cy',
    364:'__vbaR4ErrVar',
    365:'__vbaR4ForNextCheck',
    366:'__vbaR4Sgn',
    367:'__vbaR4Str',
    368:'__vbaR4Var',
    369:'__vbaR8Cy',
    370:'__vbaR8ErrVar',
    371:'__vbaR8FixI2',
    372:'__vbaR8FixI4',
    373:'__vbaR8ForNextCheck',
    374:'__vbaR8IntI2',
    375:'__vbaR8IntI4',
    376:'__vbaR8Sgn',
    377:'__vbaR8Str',
    378:'__vbaR8Var',
    379:'__vbaRaiseEvent',
    380:'__vbaRecAnsiToUni',
    381:'__vbaRecAssign',
    382:'__vbaRecDestruct',
    383:'__vbaRecDestructAnsi',
    384:'__vbaRecUniToAnsi',
    385:'__vbaRedim',
    386:'__vbaRedimPreserve',
    387:'__vbaRedimPreserveVar',
    388:'__vbaRedimPreserveVar2',
    389:'__vbaRedimVar',
    390:'__vbaRefVarAry',
    391:'__vbaResume',
    392:'__vbaRsetFixstr',
    393:'__vbaRsetFixstrFree',
    394:'__vbaSetSystemError',
    395:'__vbaStopExe',
    396:'__vbaStr2Vec',
    397:'__vbaStrAryToAnsi',
    398:'__vbaStrAryToUnicode',
    399:'__vbaStrBool',
    400:'EVENT_SINK_QueryInterface',
    401:'EVENT_SINK_AddRef',
    402:'EVENT_SINK_Release',
    403:'EVENT_SINK_GetIDsOfNames',
    404:'EVENT_SINK_Invoke',
    405:'__vbaStrCat',
    406:'__vbaStrCmp',
    407:'__vbaStrComp',
    408:'__vbaStrCompVar',
    409:'__vbaStrCy',
    410:'BASIC_CLASS_QueryInterface',
    411:'BASIC_CLASS_AddRef',
    412:'BASIC_CLASS_Release',
    413:'BASIC_CLASS_GetIDsOfNames',
    414:'BASIC_CLASS_Invoke',
    415:'__vbaStrDate',
    416:'__vbaStrFixstr',
    417:'__vbaStrI2',
    418:'__vbaStrI4',
    419:'__vbaStrLike',
    420:'BASIC_DISPINTERFACE_GetTICount',
    421:'BASIC_DISPINTERFACE_GetTypeInfo',
    422:'__vbaStrR4',
    423:'__vbaStrR8',
    424:'__vbaStrTextCmp',
    425:'__vbaStrTextLike',
    426:'__vbaStrToAnsi',
    427:'__vbaStrToUnicode',
    428:'__vbaStrUI1',
    429:'__vbaStrVarCopy',
    430:'Zombie_QueryInterface',
    431:'Zombie_AddRef',
    432:'Zombie_Release',
    433:'Zombie_GetTypeInfoCount',
    434:'Zombie_GetTypeInfo',
    435:'Zombie_GetIDsOfNames',
    436:'Zombie_Invoke',
    437:'__vbaStrVarMove',
    438:'__vbaStrVarVal',
    439:'__vbaUI1Cy',
    440:'EVENT_SINK2_AddRef',
    441:'EVENT_SINK2_Release',
    442:'__vbaUI1ErrVar',
    443:'__vbaUI1Str',
    444:'__vbaUI1Var',
    445:'__vbaUbound',
    446:'__vbaUdtVar',
    447:'__vbaUnkVar',
    448:'__vbaVar2Vec',
    449:'__vbaVarAbs',
    450:'__vbaVarAdd',
    451:'__vbaVarAnd',
    452:'__vbaVarCat',
    453:'__vbaVarCmpEq',
    454:'__vbaVarCmpGe',
    455:'__vbaVarCmpGt',
    456:'__vbaVarCmpLe',
    457:'__vbaVarCmpLt',
    458:'__vbaVarCmpNe',
    459:'__vbaVarDateVar',
    460:'__vbaVarDiv',
    461:'__vbaVarEqv',
    462:'__vbaVarErrI4',
    463:'__vbaVarFix',
    464:'__vbaVarForInit',
    465:'__vbaVarForNext',
    466:'__vbaVarIdiv',
    467:'__vbaVarImp',
    468:'__vbaVarIndexLoad',
    469:'__vbaVarIndexLoadRef',
    470:'__vbaVarIndexLoadRefLock',
    471:'__vbaVarIndexStore',
    472:'__vbaVarIndexStoreObj',
    473:'__vbaVarInt',
    474:'__vbaVarLike',
    475:'__vbaVarLikeVar',
    476:'__vbaVarMod',
    477:'__vbaVarMul',
    478:'__vbaVarNeg',
    479:'__vbaVarNot',
    480:'__vbaVarOr',
    481:'__vbaVarPow',
    482:'__vbaVarSetObj',
    483:'__vbaVarSetObjAddref',
    484:'__vbaVarSetUnk',
    485:'__vbaVarSetUnkAddref',
    486:'__vbaVarSetVar',
    487:'__vbaVarSetVarAddref',
    488:'__vbaVarSub',
    489:'__vbaVarTextCmpEq',
    490:'__vbaVarTextCmpGe',
    491:'__vbaVarTextCmpGt',
    492:'__vbaVarTextCmpLe',
    493:'__vbaVarTextCmpLt',
    494:'__vbaVarTextCmpNe',
    495:'__vbaVarTextLike',
    496:'__vbaVarTextLikeVar',
    497:'__vbaVarTextTstEq',
    498:'__vbaVarTextTstGe',
    499:'__vbaVarTextTstGt',
    500:'__vbaVarTextTstLe',
    501:'__vbaVarTextTstLt',
    502:'__vbaVarTextTstNe',
    503:'__vbaVarTstEq',
    504:'__vbaVarTstGe',
    505:'__vbaVarTstGt',
    506:'__vbaVarTstLe',
    507:'__vbaVarTstLt',
    508:'__vbaVarTstNe',
    509:'__vbaVarXor',
    510:'__vbaVargObj',
    511:'__vbaVargObjAddref',
    512:'rtcLeftBstr',
    513:'rtcLeftVar',
    514:'rtcRightBstr',
    515:'rtcRightVar',
    516:'rtcAnsiValueBstr',
    517:'rtcLowerCaseBstr',
    518:'rtcLowerCaseVar',
    519:'rtcTrimBstr',
    520:'rtcTrimVar',
    521:'rtcLeftTrimBstr',
    522:'rtcLeftTrimVar',
    523:'rtcRightTrimBstr',
    524:'rtcRightTrimVar',
    525:'rtcSpaceBstr',
    526:'rtcSpaceVar',
    527:'rtcUpperCaseBstr',
    528:'rtcUpperCaseVar',
    529:'rtcKillFiles',
    530:'rtcChangeDir',
    531:'rtcMakeDir',
    532:'rtcRemoveDir',
    533:'rtcChangeDrive',
    534:'rtcBeep',
    535:'rtcGetTimer',
    536:'rtcStrFromVar',
    537:'rtcBstrFromAnsi',
    538:'rtcPackDate',
    539:'rtcPackTime',
    540:'rtcGetDateValue',
    541:'rtcGetTimeValue',
    542:'rtcGetDayOfMonth',
    543:'rtcGetHourOfDay',
    544:'rtcGetMinuteOfHour',
    545:'rtcGetMonthOfYear',
    546:'rtcGetPresentDate',
    547:'rtcGetSecondOfMinute',
    548:'rtcSetDateVar',
    549:'rtcSetDateBstr',
    550:'rtcSetTimeVar',
    551:'rtcSetTimeBstr',
    552:'rtcGetDayOfWeek',
    553:'rtcGetYear',
    554:'rtcFileReset',
    555:'rtcFileAttributes',
    556:'rtcIsArray',
    557:'rtcIsDate',
    558:'rtcIsEmpty',
    559:'rtcIsError',
    560:'rtcIsNull',
    561:'rtcIsNumeric',
    562:'rtcIsObject',
    563:'rtcVarType',
    564:'rtDecFromVar',
    565:'rtcFileWidth',
    566:'rtcInputCount',
    567:'rtcInputCountVar',
    568:'rtcFileSeek',
    569:'rtcFileLocation',
    570:'rtcFileLength',
    571:'rtcEndOfFile',
    572:'rtcHexBstrFromVar',
    573:'rtcHexVarFromVar',
    574:'rtcOctBstrFromVar',
    575:'rtcOctVarFromVar',
    576:'rtcFileCopy',
    577:'rtcFileDateTime',
    578:'rtcFileLen',
    579:'rtcGetFileAttr',
    580:'rtcSetFileAttr',
    581:'rtcR8ValFromBstr',
    582:'rtcSin',
    583:'rtcCos',
    584:'rtcTan',
    585:'rtcAtn',
    586:'rtcExp',
    587:'rtcLog',
    588:'rtcRgb',
    589:'rtcQBColor',
    590:'rtcMacId',
    591:'rtcTypeName',
    592:'rtcIsMissing',
    593:'rtcRandomNext',
    594:'rtcRandomize',
    595:'rtcMsgBox',
    596:'rtcInputBox',
    597:'rtcAppActivate',
    598:'rtcDoEvents',
    599:'rtcSendKeys',
    600:'rtcShell',
    601:'rtcArray',
    602:'__vbaVargUnk',
    603:'__vbaVargUnkAddref',
    604:'__vbaVerifyVarObj',
    605:'rtcGetErl',
    606:'rtcStringBstr',
    607:'rtcStringVar',
    608:'rtcVarBstrFromAnsi',
    609:'rtcGetDateBstr',
    610:'rtcGetDateVar',
    611:'rtcGetTimeBstr',
    612:'rtcGetTimeVar',
    613:'rtcVarStrFromVar',
    614:'rtcSqr',
    615:'rtcIMEStatus',
    616:'rtcLeftCharBstr',
    617:'rtcLeftCharVar',
    618:'rtcRightCharBstr',
    619:'rtcRightCharVar',
    620:'rtcInputCharCount',
    621:'rtcInputCharCountVar',
    622:'rtcStrConvVar',
    623:'__vbaWriteFile',
    624:'rtcGetHostLCID',
    625:'rtcCreateObject',
    626:'rtcGetObject',
    627:'rtcAppleScript',
    628:'rtcMidBstr',
    629:'rtcMidVar',
    630:'rtcInStr',
    631:'rtcMidCharBstr',
    632:'rtcMidCharVar',
    633:'rtcInStrChar',
    634:'rtBstrFromErrVar',
    635:'rtBoolFromErrVar',
    636:'rtCyFromErrVar',
    637:'rtI2FromErrVar',
    638:'rtI4FromErrVar',
    639:'rtR4FromErrVar',
    640:'rtR8FromErrVar',
    641:'rtcDateFromVar',
    642:'rtcVarFromVar',
    643:'rtcCVErrFromVar',
    644:'VarPtr',
    645:'rtcDir',
    646:'rtcCurrentDirBstr',
    647:'rtcCurrentDir',
    648:'rtcFreeFile',
    649:'rtcCompareBstr',
    650:'rtcBstrFromFormatVar',
    651:'rtcBstrFromError',
    652:'rtcVarFromError',
    653:'rtcLenCharVar',
    654:'rtcLenVar',
    655:'rtcFixVar',
    656:'rtcAbsVar',
    657:'rtcIntVar',
    658:'rtcSgnVar',
    659:'_adj_fdiv_m16i',
    660:'rtcVarFromFormatVar',
    661:'rtcDateAdd',
    662:'rtcDateDiff',
    663:'rtcDatePart',
    664:'rtcPartition',
    665:'rtcChoose',
    666:'rtcEnvironVar',
    667:'rtcEnvironBstr',
    668:'rtcSwitch',
    669:'rtcCommandBstr',
    670:'rtcCommandVar',
    671:'rtcSLN',
    672:'rtcSYD',
    673:'rtcDDB',
    674:'rtcIPMT',
    675:'rtcPPMT',
    676:'rtcPMT',
    677:'rtcPV',
    678:'rtcFV',
    679:'rtcNPer',
    680:'rtcRate',
    681:'rtcImmediateIf',
    682:'rtcIRR',
    683:'rtcMIRR',
    684:'rtcNPV',
    685:'rtcErrObj',
    686:'rtUI1FromErrVar',
    687:'rtcVarDateFromVar',
    688:'_adj_fdiv_m32',
    689:'rtcGetSetting',
    690:'rtcSaveSetting',
    691:'rtcDeleteSetting',
    692:'rtcGetAllSettings',
    693:'rtcByteValueBstr',
    694:'rtcBstrFromByte',
    695:'rtcVarBstrFromByte',
    696:'rtcCharValueBstr',
    697:'rtcBstrFromChar',
    698:'rtcVarBstrFromChar',
    699:'rtcSetCurrentCalendar',
    700:'rtcGetCurrentCalendar',
    701:'_adj_fdiv_m32i',
    702:'rtcFormatNumber',
    703:'rtcFormatCurrency',
    704:'rtcFormatPercent',
    705:'rtcFormatDateTime',
    706:'rtcWeekdayName',
    707:'rtcMonthName',
    708:'rtcFilter',
    709:'rtcInStrRev',
    710:'rtcJoin',
    711:'rtcSplit',
    712:'rtcReplace',
    713:'rtcStrReverse',
    714:'rtcRound',
    715:'rtcCallByName',
    716:'rtcCreateObject2',
    717:'rtcStrConvVar2',
    718:'_adj_fdiv_m64',
    719:'_adj_fdiv_r',
    720:'_adj_fdivr_m16i',
    721:'_adj_fdivr_m32',
    722:'_adj_fdivr_m32i',
    723:'_adj_fdivr_m64',
    724:'_adj_fpatan',
    725:'_adj_fprem',
    726:'_adj_fprem1',
    727:'_adj_fptan',
    728:'_allmul',
    999:'TipInvokeMethod2',
    1016:'TipInvokeMethod',
    1024:'IID_IVbaHost',
    1025:'EbGetObjConnectionCounts',
    2000:'CreateIExprSrvObj',
    2010:'EbGetVBAObject',
}
