ord_names = {
    1:'OleUIAddVerbMenuA',
    2:'OleUICanConvertOrActivateAs',
    3:'OleUIInsertObjectA',
    4:'OleUIPasteSpecialA',
    5:'OleUIEditLinksA',
    6:'OleUIChangeIconA',
    7:'OleUIConvertA',
    8:'OleUIBusyA',
    9:'OleUIUpdateLinksA',
    10:'OleUIPromptUserA',
    11:'OleUIObjectPropertiesA',
    12:'OleUIChangeSourceA',
    13:'OleUIAddVerbMenuW',
    14:'OleUIBusyW',
    15:'OleUIChangeIconW',
    16:'OleUIChangeSourceW',
    17:'OleUIConvertW',
    18:'OleUIEditLinksW',
    19:'OleUIInsertObjectW',
    20:'OleUIObjectPropertiesW',
    21:'OleUIPasteSpecialW',
    22:'OleUIPromptUserW',
    23:'OleUIUpdateLinksW',
}
