
OPTYPE_Reg      =   1
OPTYPE_RegMem   =   2
OPTYPE_const    =   3
OPTYPE_imm8     =   4
OPTYPE_imm16    =   5
OPTYPE_RegAlt   =   6
OPTYPE_Ind      =   7
OPTYPE_Cond     =   8
OPTYPE_immmem16 =   9
OPTYPE_immmem8  =   10
OPTYPE_RegMemDisp = 11


COND_NZ         =   1
COND_Z          =   2
COND_NC         =   3
COND_PO         =   4
COND_PE         =   5
COND_P          =   6
COND_M          =   7
