ord_names = {
    2:'MenuHelp',
    3:'ShowHideMenuCtl',
    4:'GetEffectiveClientRect',
    5:'DrawStatusTextA',
    6:'CreateStatusWindowA',
    7:'CreateToolbar',
    8:'CreateMappedBitmap',
    12:'CreatePropertySheetPage',
    13:'MakeDragList',
    14:'LBItemFromPt',
    15:'DrawInsert',
    16:'CreateUpDownControl',
    17:'InitCommonControls',
    18:'CreatePropertySheetPageA',
    19:'CreatePropertySheetPageW',
    20:'CreateStatusWindow',
    21:'CreateStatusWindowW',
    22:'CreateToolbarEx',
    23:'DestroyPropertySheetPage',
    24:'DllGetVersion',
    25:'DllInstall',
    26:'DrawStatusText',
    27:'DrawStatusTextW',
    28:'FlatSB_EnableScrollBar',
    29:'FlatSB_GetScrollInfo',
    30:'FlatSB_GetScrollPos',
    31:'FlatSB_GetScrollProp',
    32:'FlatSB_GetScrollRange',
    33:'FlatSB_SetScrollInfo',
    34:'FlatSB_SetScrollPos',
    35:'FlatSB_SetScrollProp',
    36:'FlatSB_SetScrollRange',
    37:'FlatSB_ShowScrollBar',
    38:'GetMUILanguage',
    39:'ImageList_Add',
    40:'ImageList_AddIcon',
    41:'ImageList_AddMasked',
    42:'ImageList_BeginDrag',
    43:'ImageList_Copy',
    44:'ImageList_Create',
    45:'ImageList_Destroy',
    46:'ImageList_DragEnter',
    47:'ImageList_DragLeave',
    48:'ImageList_DragMove',
    49:'ImageList_DragShowNolock',
    50:'ImageList_Draw',
    51:'ImageList_DrawEx',
    52:'ImageList_DrawIndirect',
    53:'ImageList_Duplicate',
    54:'ImageList_EndDrag',
    55:'ImageList_GetBkColor',
    56:'ImageList_GetDragImage',
    57:'ImageList_GetFlags',
    58:'ImageList_GetIcon',
    59:'ImageList_GetIconSize',
    60:'ImageList_GetImageCount',
    61:'ImageList_GetImageInfo',
    62:'ImageList_GetImageRect',
    63:'ImageList_LoadImage',
    64:'ImageList_LoadImageA',
    65:'ImageList_LoadImageW',
    66:'ImageList_Merge',
    67:'ImageList_Read',
    68:'ImageList_Remove',
    69:'ImageList_Replace',
    70:'ImageList_ReplaceIcon',
    75:'ImageList_SetBkColor',
    76:'ImageList_SetDragCursorImage',
    77:'ImageList_SetFilter',
    78:'ImageList_SetFlags',
    79:'ImageList_SetIconSize',
    80:'ImageList_SetImageCount',
    81:'ImageList_SetOverlayImage',
    82:'ImageList_Write',
    83:'InitCommonControlsEx',
    84:'InitMUILanguage',
    85:'InitializeFlatSB',
    86:'PropertySheet',
    87:'PropertySheetA',
    88:'PropertySheetW',
    89:'UninitializeFlatSB',
    90:'_TrackMouseEvent',
    152:'FreeMRUList',
    236:'Str_SetPtrW',
    320:'DSA_Create',
    321:'DSA_Destroy',
    323:'DSA_GetItemPtr',
    324:'DSA_InsertItem',
    327:'DSA_DeleteAllItems',
    328:'DPA_Create',
    329:'DPA_Destroy',
    332:'DPA_GetPtr',
    334:'DPA_InsertPtr',
    335:'DPA_SetPtr',
    336:'DPA_DeletePtr',
    337:'DPA_DeleteAllPtrs',
    338:'DPA_Sort',
    339:'DPA_Search',
    385:'DPA_EnumCallback',
    386:'DPA_DestroyCallback',
    388:'DSA_DestroyCallback',
    400:'CreateMRUListW',
    401:'AddMRUStringW',
    403:'EnumMRUListW',
    410:'SetWindowSubclass',
    412:'RemoveWindowSubclass',
    413:'DefSubclassProc',
}
