# APIs for Windows 32-bit MSVC runtime library (msvcrt).
# Format:  rettype, retname, callconv, exactname, arglist(type, name)
#          arglist type is one of ['int', 'void *']
#          arglist name is one of [None, 'funcptr', 'obj', 'ptr']

api_defs = {
    'msvcrt.main_entry':( 'int', None, 'stdcall', 'msvcrt.main_entry', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt.$i10_output':( 'int', None, 'cdecl', 'msvcrt.$I10_OUTPUT', (('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('void *', 'ptr')) ),
    'msvcrt.??0__non_rtti_object@@qae@abv0@@z':( 'int', None, 'stdcall', 'msvcrt.??0__non_rtti_object@@QAE@ABV0@@Z', ( ('int', None),) ),
    'msvcrt.??0__non_rtti_object@@qae@pbd@z':( 'int', None, 'stdcall', 'msvcrt.??0__non_rtti_object@@QAE@PBD@Z', ( ('int', None),) ),
    'msvcrt.??0bad_cast@@aae@pbqbd@z':( 'int', None, 'stdcall', 'msvcrt.??0bad_cast@@AAE@PBQBD@Z', ( ('int', None),) ),
    'msvcrt.??0bad_cast@@qae@abqbd@z':( 'int', None, 'stdcall', 'msvcrt.??0bad_cast@@QAE@ABQBD@Z', ( ('int', None),) ),
    'msvcrt.??0bad_cast@@qae@abv0@@z':( 'int', None, 'stdcall', 'msvcrt.??0bad_cast@@QAE@ABV0@@Z', ( ('int', None),) ),
    'msvcrt.??0bad_cast@@qae@pbd@z':( 'int', None, 'stdcall', 'msvcrt.??0bad_cast@@QAE@PBD@Z', ( ('int', None),) ),
    'msvcrt.??0bad_typeid@@qae@abv0@@z':( 'int', None, 'stdcall', 'msvcrt.??0bad_typeid@@QAE@ABV0@@Z', ( ('int', None),) ),
    'msvcrt.??0bad_typeid@@qae@pbd@z':( 'int', None, 'stdcall', 'msvcrt.??0bad_typeid@@QAE@PBD@Z', ( ('int', None),) ),
    'msvcrt.??0exception@@qae@abqbd@z':( 'int', None, 'stdcall', 'msvcrt.??0exception@@QAE@ABQBD@Z', ( ('int', None),) ),
    'msvcrt.??0exception@@qae@abv0@@z':( 'int', None, 'stdcall', 'msvcrt.??0exception@@QAE@ABV0@@Z', ( ('int', None),) ),
    'msvcrt.??0exception@@qae@xz':( 'int', None, 'stdcall', 'msvcrt.??0exception@@QAE@XZ', () ),
    'msvcrt.??1__non_rtti_object@@uae@xz':( 'int', None, 'stdcall', 'msvcrt.??1__non_rtti_object@@UAE@XZ', () ),
    'msvcrt.??1bad_cast@@uae@xz':( 'int', None, 'stdcall', 'msvcrt.??1bad_cast@@UAE@XZ', () ),
    'msvcrt.??1bad_typeid@@uae@xz':( 'int', None, 'stdcall', 'msvcrt.??1bad_typeid@@UAE@XZ', () ),
    'msvcrt.??1exception@@uae@xz':( 'int', None, 'stdcall', 'msvcrt.??1exception@@UAE@XZ', () ),
    'msvcrt.??1type_info@@uae@xz':( 'int', None, 'stdcall', 'msvcrt.??1type_info@@UAE@XZ', () ),
    'msvcrt.??2@yapaxi@z':( 'int', None, 'cdecl', 'msvcrt.??2@YAPAXI@Z', (('int', None),) ),
    'msvcrt.??3@yaxpax@z':( 'void', None, 'cdecl', 'msvcrt.??3@YAXPAX@Z', (('void *', 'ptr'),) ),
    'msvcrt.??4__non_rtti_object@@qaeaav0@abv0@@z':( 'int', None, 'stdcall', 'msvcrt.??4__non_rtti_object@@QAEAAV0@ABV0@@Z', ( ('int', None),) ),
    'msvcrt.??4bad_cast@@qaeaav0@abv0@@z':( 'int', None, 'stdcall', 'msvcrt.??4bad_cast@@QAEAAV0@ABV0@@Z', ( ('int', None),) ),
    'msvcrt.??4bad_typeid@@qaeaav0@abv0@@z':( 'int', None, 'stdcall', 'msvcrt.??4bad_typeid@@QAEAAV0@ABV0@@Z', ( ('int', None),) ),
    'msvcrt.??4exception@@qaeaav0@abv0@@z':( 'int', None, 'stdcall', 'msvcrt.??4exception@@QAEAAV0@ABV0@@Z', ( ('int', None),) ),
    'msvcrt.??8type_info@@qbehabv0@@z':( 'int', None, 'stdcall', 'msvcrt.??8type_info@@QBEHABV0@@Z', ( ('int', None),) ),
    'msvcrt.??9type_info@@qbehabv0@@z':( 'int', None, 'stdcall', 'msvcrt.??9type_info@@QBEHABV0@@Z', ( ('int', None),) ),
    'msvcrt.??_7exception@@6b@':( 'int', None, 'cdecl', 'msvcrt.??_7exception@@6B@', () ),
    'msvcrt.??_e__non_rtti_object@@uaepaxi@z':( 'int', None, 'stdcall', 'msvcrt.??_E__non_rtti_object@@UAEPAXI@Z', ( ('int', None),) ),
    'msvcrt.??_ebad_cast@@uaepaxi@z':( 'int', None, 'stdcall', 'msvcrt.??_Ebad_cast@@UAEPAXI@Z', ( ('int', None),) ),
    'msvcrt.??_ebad_typeid@@uaepaxi@z':( 'int', None, 'stdcall', 'msvcrt.??_Ebad_typeid@@UAEPAXI@Z', ( ('int', None),) ),
    'msvcrt.??_eexception@@uaepaxi@z':( 'int', None, 'stdcall', 'msvcrt.??_Eexception@@UAEPAXI@Z', ( ('int', None),) ),
    'msvcrt.??_fbad_cast@@qaexxz':( 'int', None, 'stdcall', 'msvcrt.??_Fbad_cast@@QAEXXZ', () ),
    'msvcrt.??_fbad_typeid@@qaexxz':( 'int', None, 'stdcall', 'msvcrt.??_Fbad_typeid@@QAEXXZ', () ),
    'msvcrt.??_g__non_rtti_object@@uaepaxi@z':( 'int', None, 'stdcall', 'msvcrt.??_G__non_rtti_object@@UAEPAXI@Z', ( ('int', None),) ),
    'msvcrt.??_gbad_cast@@uaepaxi@z':( 'int', None, 'stdcall', 'msvcrt.??_Gbad_cast@@UAEPAXI@Z', ( ('int', None),) ),
    'msvcrt.??_gbad_typeid@@uaepaxi@z':( 'int', None, 'stdcall', 'msvcrt.??_Gbad_typeid@@UAEPAXI@Z', ( ('int', None),) ),
    'msvcrt.??_gexception@@uaepaxi@z':( 'int', None, 'stdcall', 'msvcrt.??_Gexception@@UAEPAXI@Z', ( ('int', None),) ),
    'msvcrt.??_u@yapaxi@z':( 'int', None, 'cdecl', 'msvcrt.??_U@YAPAXI@Z', (('int', None),) ),
    'msvcrt.??_v@yaxpax@z':( 'void', None, 'cdecl', 'msvcrt.??_V@YAXPAX@Z', (('void *', 'ptr'),) ),
    'msvcrt.?_query_new_handler@@yap6ahi@zxz':( 'int', None, 'cdecl', 'msvcrt.?_query_new_handler@@YAP6AHI@ZXZ', () ),
    'msvcrt.?_query_new_mode@@yahxz':( 'int', None, 'cdecl', 'msvcrt.?_query_new_mode@@YAHXZ', () ),
    'msvcrt.?_set_new_handler@@yap6ahi@zp6ahi@z@z':( 'int', None, 'cdecl', 'msvcrt.?_set_new_handler@@YAP6AHI@ZP6AHI@Z@Z', (('int', None),) ),
    'msvcrt.?_set_new_mode@@yahh@z':( 'int', None, 'cdecl', 'msvcrt.?_set_new_mode@@YAHH@Z', (('int', None),) ),
    'msvcrt.?_set_se_translator@@yap6axipau_exception_pointers@@@zp6axi0@z@z':( 'int', None, 'cdecl', 'msvcrt.?_set_se_translator@@YAP6AXIPAU_EXCEPTION_POINTERS@@@ZP6AXI0@Z@Z', (('int', None),) ),
    'msvcrt.?before@type_info@@qbehabv1@@z':( 'int', None, 'stdcall', 'msvcrt.?before@type_info@@QBEHABV1@@Z', ( ('int', None),) ),
    'msvcrt.?name@type_info@@qbepbdxz':( 'int', None, 'stdcall', 'msvcrt.?name@type_info@@QBEPBDXZ', () ),
    'msvcrt.?raw_name@type_info@@qbepbdxz':( 'int', None, 'cdecl', 'msvcrt.?raw_name@type_info@@QBEPBDXZ', () ),
    'msvcrt.?set_new_handler@@yap6axxzp6axxz@z':( 'int', None, 'cdecl', 'msvcrt.?set_new_handler@@YAP6AXXZP6AXXZ@Z', () ),
    'msvcrt.?set_terminate@@yap6axxzp6axxz@z':( 'int', None, 'cdecl', 'msvcrt.?set_terminate@@YAP6AXXZP6AXXZ@Z', (('int', None),) ),
    'msvcrt.?set_unexpected@@yap6axxzp6axxz@z':( 'int', None, 'cdecl', 'msvcrt.?set_unexpected@@YAP6AXXZP6AXXZ@Z', (('int', None),) ),
    'msvcrt.?terminate@@yaxxz':( 'int', None, 'cdecl', 'msvcrt.?terminate@@YAXXZ', () ),
    'msvcrt.?unexpected@@yaxxz':( 'int', None, 'cdecl', 'msvcrt.?unexpected@@YAXXZ', () ),
    'msvcrt.?what@exception@@ubepbdxz':( 'int', None, 'stdcall', 'msvcrt.?what@exception@@UBEPBDXZ', () ),
    'msvcrt._ciacos':( 'int', None, 'cdecl', 'msvcrt._CIacos', () ),
    'msvcrt._ciasin':( 'int', None, 'cdecl', 'msvcrt._CIasin', () ),
    'msvcrt._ciatan':( 'int', None, 'cdecl', 'msvcrt._CIatan', () ),
    'msvcrt._ciatan2':( 'int', None, 'cdecl', 'msvcrt._CIatan2', () ),
    'msvcrt._cicos':( 'int', None, 'cdecl', 'msvcrt._CIcos', () ),
    'msvcrt._cicosh':( 'int', None, 'cdecl', 'msvcrt._CIcosh', () ),
    'msvcrt._ciexp':( 'int', None, 'cdecl', 'msvcrt._CIexp', () ),
    'msvcrt._cifmod':( 'int', None, 'cdecl', 'msvcrt._CIfmod', () ),
    'msvcrt._cilog':( 'int', None, 'cdecl', 'msvcrt._CIlog', () ),
    'msvcrt._cilog10':( 'int', None, 'cdecl', 'msvcrt._CIlog10', () ),
    'msvcrt._cipow':( 'int', None, 'cdecl', 'msvcrt._CIpow', () ),
    'msvcrt._cisin':( 'int', None, 'cdecl', 'msvcrt._CIsin', () ),
    'msvcrt._cisinh':( 'int', None, 'cdecl', 'msvcrt._CIsinh', () ),
    'msvcrt._cisqrt':( 'int', None, 'cdecl', 'msvcrt._CIsqrt', () ),
    'msvcrt._citan':( 'int', None, 'cdecl', 'msvcrt._CItan', () ),
    'msvcrt._citanh':( 'int', None, 'cdecl', 'msvcrt._CItanh', () ),
    'msvcrt._cxxthrowexception':( 'void', None, 'stdcall', 'msvcrt._CxxThrowException', (('void *', 'ptr'), ('int', None)) ),
    'msvcrt._eh_prolog':( 'int', None, 'cdecl', 'msvcrt._EH_prolog', () ),
    'msvcrt._getdays':( 'int', None, 'cdecl', 'msvcrt._Getdays', () ),
    'msvcrt._getmonths':( 'int', None, 'cdecl', 'msvcrt._Getmonths', () ),
    'msvcrt._gettnames':( 'int', None, 'cdecl', 'msvcrt._Gettnames', () ),
    'msvcrt._huge':( 'int', None, 'cdecl', 'msvcrt._HUGE', () ),
    'msvcrt._strftime':( 'int', None, 'cdecl', 'msvcrt._Strftime', (('int', None), ('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt._xcptfilter':( 'int', None, 'cdecl', 'msvcrt._XcptFilter', (('int', None), ('int', None)) ),
    'msvcrt.__cxxcallunwinddtor':( 'int', None, 'cdecl', 'msvcrt.__CxxCallUnwindDtor', (('void *', 'funcptr'), ('int', None)) ),
    'msvcrt.__cxxdetectrethrow':( 'int', None, 'cdecl', 'msvcrt.__CxxDetectRethrow', (('int', None),) ),
    'msvcrt.__cxxexceptionfilter':( 'int', None, 'cdecl', 'msvcrt.__CxxExceptionFilter', (('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt.__cxxframehandler':( 'int', None, 'bfastcall', 'msvcrt.__CxxFrameHandler', (('int', None), ('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt.__cxxlongjmpunwind':( 'int', None, 'stdcall', 'msvcrt.__CxxLongjmpUnwind', (('int', None),) ),
    'msvcrt.__cxxqueryexceptionsize':( 'int', None, 'cdecl', 'msvcrt.__CxxQueryExceptionSize', () ),
    'msvcrt.__cxxregisterexceptionobject':( 'int', None, 'cdecl', 'msvcrt.__CxxRegisterExceptionObject', (('int', None), ('void *', 'ptr')) ),
    'msvcrt.__cxxunregisterexceptionobject':( 'int', None, 'cdecl', 'msvcrt.__CxxUnregisterExceptionObject', (('int', None), ('int', None)) ),
    'msvcrt.__destructexceptionobject':( 'int', None, 'cdecl', 'msvcrt.__DestructExceptionObject', (('int', None),) ),
    'msvcrt.__rtcasttovoid':( 'int', None, 'cdecl', 'msvcrt.__RTCastToVoid', () ),
    'msvcrt.__rtdynamiccast':( 'int', None, 'cdecl', 'msvcrt.__RTDynamicCast', ( ('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt.__rttypeid':( 'int', None, 'cdecl', 'msvcrt.__RTtypeid', (('int', None),) ),
    'msvcrt.__stringtold':( 'int', None, 'cdecl', 'msvcrt.__STRINGTOLD', (('int', None), ('void *', 'ptr'), ('int', None), ('int', None)) ),
    'msvcrt.___lc_codepage_func':( 'int', None, 'cdecl', 'msvcrt.___lc_codepage_func', () ),
    'msvcrt.___lc_handle_func':( 'int', None, 'cdecl', 'msvcrt.___lc_handle_func', () ),
    'msvcrt.___mb_cur_max_func':( 'int', None, 'cdecl', 'msvcrt.___mb_cur_max_func', () ),
    'msvcrt.___setlc_active_func':( 'int', None, 'cdecl', 'msvcrt.___setlc_active_func', () ),
    'msvcrt.___unguarded_readlc_active_add_func':( 'int', None, 'cdecl', 'msvcrt.___unguarded_readlc_active_add_func', () ),
    'msvcrt.__argc':( 'int', None, 'cdecl', 'msvcrt.__argc', () ),
    'msvcrt.__argv':( 'int', None, 'cdecl', 'msvcrt.__argv', () ),
    'msvcrt.__badioinfo':( 'int', None, 'cdecl', 'msvcrt.__badioinfo', () ),
    'msvcrt.__crtcomparestringa':( 'int', None, 'cdecl', 'msvcrt.__crtCompareStringA', (('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt.__crtcomparestringw':( 'int', None, 'cdecl', 'msvcrt.__crtCompareStringW', (('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt.__crtgetlocaleinfow':( 'int', None, 'cdecl', 'msvcrt.__crtGetLocaleInfoW', (('int', None), ('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt.__crtgetstringtypew':( 'int', None, 'cdecl', 'msvcrt.__crtGetStringTypeW', (('int', None), ('int', None), ('int', None), ('void *', 'ptr'), ('int', None), ('int', None)) ),
    'msvcrt.__crtlcmapstringa':( 'int', None, 'cdecl', 'msvcrt.__crtLCMapStringA', (('int', None), ('int', None), ('void *', 'ptr'), ('int', None), ('void *', 'ptr'), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt.__crtlcmapstringw':( 'int', None, 'cdecl', 'msvcrt.__crtLCMapStringW', (('int', None), ('int', None), ('int', None), ('int', None), ('void *', 'ptr'), ('int', None), ('int', None)) ),
    'msvcrt.__dllonexit':( 'int', None, 'cdecl', 'msvcrt.__dllonexit', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt.__doserrno':( 'int', None, 'cdecl', 'msvcrt.__doserrno', () ),
    'msvcrt.__fpecode':( 'int', None, 'cdecl', 'msvcrt.__fpecode', () ),
    'msvcrt.__getmainargs':( 'int', None, 'cdecl', 'msvcrt.__getmainargs', (('int', None), ('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt.__initenv':( 'int', None, 'cdecl', 'msvcrt.__initenv', () ),
    'msvcrt.__iob_func':( 'int', None, 'cdecl', 'msvcrt.__iob_func', () ),
    'msvcrt.__isascii':( 'int', None, 'cdecl', 'msvcrt.__isascii', (('int', None),) ),
    'msvcrt.__iscsym':( 'int', None, 'cdecl', 'msvcrt.__iscsym', (('int', None),) ),
    'msvcrt.__iscsymf':( 'int', None, 'cdecl', 'msvcrt.__iscsymf', (('int', None),) ),
    'msvcrt.__lc_codepage':( 'int', None, 'cdecl', 'msvcrt.__lc_codepage', () ),
    'msvcrt.__lc_collate_cp':( 'int', None, 'cdecl', 'msvcrt.__lc_collate_cp', () ),
    'msvcrt.__lc_handle':( 'int', None, 'cdecl', 'msvcrt.__lc_handle', () ),
    'msvcrt.__lconv_init':( 'int', None, 'cdecl', 'msvcrt.__lconv_init', () ),
    'msvcrt.__mb_cur_max':( 'int', None, 'bfastcall', 'msvcrt.__mb_cur_max', (('int', None),) ),
    'msvcrt.__p___argc':( 'int', None, 'cdecl', 'msvcrt.__p___argc', () ),
    'msvcrt.__p___argv':( 'int', None, 'cdecl', 'msvcrt.__p___argv', () ),
    'msvcrt.__p___initenv':( 'int', None, 'cdecl', 'msvcrt.__p___initenv', () ),
    'msvcrt.__p___mb_cur_max':( 'int', None, 'cdecl', 'msvcrt.__p___mb_cur_max', () ),
    'msvcrt.__p___wargv':( 'int', None, 'cdecl', 'msvcrt.__p___wargv', () ),
    'msvcrt.__p___winitenv':( 'int', None, 'cdecl', 'msvcrt.__p___winitenv', () ),
    'msvcrt.__p__acmdln':( 'int', None, 'cdecl', 'msvcrt.__p__acmdln', () ),
    'msvcrt.__p__amblksiz':( 'int', None, 'cdecl', 'msvcrt.__p__amblksiz', () ),
    'msvcrt.__p__commode':( 'int', None, 'cdecl', 'msvcrt.__p__commode', () ),
    'msvcrt.__p__daylight':( 'int', None, 'cdecl', 'msvcrt.__p__daylight', () ),
    'msvcrt.__p__dstbias':( 'int', None, 'cdecl', 'msvcrt.__p__dstbias', () ),
    'msvcrt.__p__environ':( 'int', None, 'cdecl', 'msvcrt.__p__environ', () ),
    'msvcrt.__p__fileinfo':( 'int', None, 'cdecl', 'msvcrt.__p__fileinfo', () ),
    'msvcrt.__p__fmode':( 'int', None, 'cdecl', 'msvcrt.__p__fmode', () ),
    'msvcrt.__p__iob':( 'int', None, 'cdecl', 'msvcrt.__p__iob', () ),
    'msvcrt.__p__mbcasemap':( 'int', None, 'cdecl', 'msvcrt.__p__mbcasemap', () ),
    'msvcrt.__p__mbctype':( 'int', None, 'cdecl', 'msvcrt.__p__mbctype', () ),
    'msvcrt.__p__osver':( 'int', None, 'cdecl', 'msvcrt.__p__osver', () ),
    'msvcrt.__p__pctype':( 'int', None, 'cdecl', 'msvcrt.__p__pctype', () ),
    'msvcrt.__p__pgmptr':( 'int', None, 'cdecl', 'msvcrt.__p__pgmptr', () ),
    'msvcrt.__p__pwctype':( 'int', None, 'cdecl', 'msvcrt.__p__pwctype', () ),
    'msvcrt.__p__timezone':( 'int', None, 'cdecl', 'msvcrt.__p__timezone', () ),
    'msvcrt.__p__tzname':( 'int', None, 'cdecl', 'msvcrt.__p__tzname', () ),
    'msvcrt.__p__wcmdln':( 'int', None, 'cdecl', 'msvcrt.__p__wcmdln', () ),
    'msvcrt.__p__wenviron':( 'int', None, 'cdecl', 'msvcrt.__p__wenviron', () ),
    'msvcrt.__p__winmajor':( 'int', None, 'cdecl', 'msvcrt.__p__winmajor', () ),
    'msvcrt.__p__winminor':( 'int', None, 'cdecl', 'msvcrt.__p__winminor', () ),
    'msvcrt.__p__winver':( 'int', None, 'cdecl', 'msvcrt.__p__winver', () ),
    'msvcrt.__p__wpgmptr':( 'int', None, 'cdecl', 'msvcrt.__p__wpgmptr', () ),
    'msvcrt.__pctype_func':( 'int', None, 'cdecl', 'msvcrt.__pctype_func', () ),
    'msvcrt.__pioinfo':( 'int', None, 'cdecl', 'msvcrt.__pioinfo', () ),
    'msvcrt.__pxcptinfoptrs':( 'int', None, 'cdecl', 'msvcrt.__pxcptinfoptrs', () ),
    'msvcrt.__set_app_type':( 'int', None, 'cdecl', 'msvcrt.__set_app_type', (('int', None),) ),
    'msvcrt.__setlc_active':( 'int', None, 'cdecl', 'msvcrt.__setlc_active', () ),
    'msvcrt.__setusermatherr':( 'int', None, 'cdecl', 'msvcrt.__setusermatherr', (('int', None),) ),
    'msvcrt.__threadhandle':( 'int', None, 'stdcall', 'msvcrt.__threadhandle', () ),
    'msvcrt.__threadid':( 'int', None, 'stdcall', 'msvcrt.__threadid', () ),
    'msvcrt.__toascii':( 'int', None, 'cdecl', 'msvcrt.__toascii', (('int', None),) ),
    'msvcrt.__undname':( 'int', None, 'cdecl', 'msvcrt.__unDName', (('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt.__undnameex':( 'int', None, 'cdecl', 'msvcrt.__unDNameEx', (('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt.__uncaught_exception':( 'int', None, 'cdecl', 'msvcrt.__uncaught_exception', () ),
    'msvcrt.__unguarded_readlc_active':( 'int', None, 'cdecl', 'msvcrt.__unguarded_readlc_active', () ),
    'msvcrt.__wargv':( 'int', None, 'cdecl', 'msvcrt.__wargv', () ),
    'msvcrt.__wcserror':( 'int', None, 'cdecl', 'msvcrt.__wcserror', (('int', None),) ),
    'msvcrt.__wgetmainargs':( 'int', None, 'msfastcall', 'msvcrt.__wgetmainargs', (('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt.__winitenv':( 'int', None, 'cdecl', 'msvcrt.__winitenv', () ),
    'msvcrt._abnormal_termination':( 'int', None, 'cdecl', 'msvcrt._abnormal_termination', () ),
    'msvcrt._access':( 'int', None, 'cdecl', 'msvcrt._access', (('void *', 'ptr'), ('int', None)) ),
    'msvcrt._acmdln':( 'int', None, 'cdecl', 'msvcrt._acmdln', () ),
    'msvcrt._adj_fdiv_m16i':( 'int', None, 'cdecl', 'msvcrt._adj_fdiv_m16i', () ),
    'msvcrt._adj_fdiv_m32':( 'int', None, 'cdecl', 'msvcrt._adj_fdiv_m32', (('int', None),) ),
    'msvcrt._adj_fdiv_m32i':( 'int', None, 'cdecl', 'msvcrt._adj_fdiv_m32i', () ),
    'msvcrt._adj_fdiv_m64':( 'int', None, 'cdecl', 'msvcrt._adj_fdiv_m64', (('int', None), ('int', None)) ),
    'msvcrt._adj_fdiv_r':( 'int', None, 'cdecl', 'msvcrt._adj_fdiv_r', () ),
    'msvcrt._adj_fdivr_m16i':( 'int', None, 'cdecl', 'msvcrt._adj_fdivr_m16i', () ),
    'msvcrt._adj_fdivr_m32':( 'int', None, 'cdecl', 'msvcrt._adj_fdivr_m32', (('int', None),) ),
    'msvcrt._adj_fdivr_m32i':( 'int', None, 'cdecl', 'msvcrt._adj_fdivr_m32i', () ),
    'msvcrt._adj_fdivr_m64':( 'int', None, 'cdecl', 'msvcrt._adj_fdivr_m64', (('int', None), ('int', None)) ),
    'msvcrt._adj_fpatan':( 'int', None, 'cdecl', 'msvcrt._adj_fpatan', () ),
    'msvcrt._adj_fprem':( 'int', None, 'cdecl', 'msvcrt._adj_fprem', () ),
    'msvcrt._adj_fprem1':( 'int', None, 'cdecl', 'msvcrt._adj_fprem1', () ),
    'msvcrt._adj_fptan':( 'int', None, 'cdecl', 'msvcrt._adj_fptan', () ),
    'msvcrt._adjust_fdiv':( 'int', None, 'cdecl', 'msvcrt._adjust_fdiv', () ),
    'msvcrt._aexit_rtn':( 'int', None, 'cdecl', 'msvcrt._aexit_rtn', () ),
    'msvcrt._aligned_free':( 'int', None, 'cdecl', 'msvcrt._aligned_free', (('int', None),) ),
    'msvcrt._aligned_malloc':( 'int', None, 'cdecl', 'msvcrt._aligned_malloc', (('int', None), ('int', None)) ),
    'msvcrt._aligned_offset_malloc':( 'int', None, 'cdecl', 'msvcrt._aligned_offset_malloc', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._aligned_offset_realloc':( 'int', None, 'cdecl', 'msvcrt._aligned_offset_realloc', (('void *', 'ptr'), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt._aligned_realloc':( 'int', None, 'cdecl', 'msvcrt._aligned_realloc', (('void *', 'ptr'), ('int', None), ('int', None)) ),
    'msvcrt._amsg_exit':( 'int', None, 'cdecl', 'msvcrt._amsg_exit', (('int', None),) ),
    'msvcrt._assert':( 'int', None, 'cdecl', 'msvcrt._assert', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._atodbl':( 'int', None, 'cdecl', 'msvcrt._atodbl', (('void *', 'ptr'), ('int', None)) ),
    'msvcrt._atoi64':( 'int', None, 'cdecl', 'msvcrt._atoi64', (('int', None),) ),
    'msvcrt._atoldbl':( 'int', None, 'cdecl', 'msvcrt._atoldbl', (('int', None), ('int', None)) ),
    'msvcrt._beep':( 'int', None, 'cdecl', 'msvcrt._beep', (('int', None), ('int', None)) ),
    'msvcrt._beginthread':( 'int', None, 'cdecl', 'msvcrt._beginthread', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._beginthreadex':( 'int', None, 'cdecl', 'msvcrt._beginthreadex', (('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt._c_exit':( 'int', None, 'cdecl', 'msvcrt._c_exit', () ),
    'msvcrt._cabs':( 'int', None, 'cdecl', 'msvcrt._cabs', () ),
    'msvcrt._callnewh':( 'int', None, 'cdecl', 'msvcrt._callnewh', (('int', None),) ),
    'msvcrt._cexit':( 'int', None, 'cdecl', 'msvcrt._cexit', () ),
    'msvcrt._cgets':( 'int', None, 'stdcall', 'msvcrt._cgets', ( ('int', None),) ),
    'msvcrt._cgetws':( 'int', None, 'cdecl', 'msvcrt._cgetws', (('int', None),) ),
    'msvcrt._chdir':( 'int', None, 'cdecl', 'msvcrt._chdir', (('int', None),) ),
    'msvcrt._chdrive':( 'int', None, 'cdecl', 'msvcrt._chdrive', (('int', None),) ),
    'msvcrt._chgsign':( 'int', None, 'cdecl', 'msvcrt._chgsign', (('int', None), ('int', None)) ),
    'msvcrt._chkesp':( 'int', None, 'cdecl', 'msvcrt._chkesp', () ),
    'msvcrt._chmod':( 'int', None, 'cdecl', 'msvcrt._chmod', (('int', None), ('int', None)) ),
    'msvcrt._chsize':( 'int', None, 'cdecl', 'msvcrt._chsize', (('int', None), ('int', None)) ),
    'msvcrt._clearfp':( 'int', None, 'cdecl', 'msvcrt._clearfp', () ),
    'msvcrt._close':( 'int', None, 'cdecl', 'msvcrt._close', (('int', None),) ),
    'msvcrt._commit':( 'int', None, 'cdecl', 'msvcrt._commit', (('int', None),) ),
    'msvcrt._commode':( 'int', None, 'cdecl', 'msvcrt._commode', () ),
    'msvcrt._control87':( 'int', None, 'cdecl', 'msvcrt._control87', () ),
    'msvcrt._controlfp':( 'int', None, 'cdecl', 'msvcrt._controlfp', (('int', None), ('int', None)) ),
    'msvcrt._copysign':( 'int', None, 'cdecl', 'msvcrt._copysign', (('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt._cprintf':( 'int', None, 'cdecl', 'msvcrt._cprintf', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._cputs':( 'int', None, 'cdecl', 'msvcrt._cputs', (('int', None),) ),
    'msvcrt._cputws':( 'int', None, 'cdecl', 'msvcrt._cputws', (('void *', 'ptr'),) ),
    'msvcrt._creat':( 'int', None, 'cdecl', 'msvcrt._creat', (('StringA', None), ('int', None)) ),
    'msvcrt._cscanf':( 'int', None, 'cdecl', 'msvcrt._cscanf', () ),
    'msvcrt._ctime64':( 'int', None, 'cdecl', 'msvcrt._ctime64', (('int', None),) ),
    'msvcrt._ctype':( 'int', None, 'cdecl', 'msvcrt._ctype', () ),
    'msvcrt._cwait':( 'int', None, 'cdecl', 'msvcrt._cwait', (('void *', 'ptr'), ('HANDLE', None)) ),
    'msvcrt._cwprintf':( 'int', None, 'cdecl', 'msvcrt._cwprintf', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._cwscanf':( 'int', None, 'cdecl', 'msvcrt._cwscanf', () ),
    'msvcrt._daylight':( 'int', None, 'bfastcall', 'msvcrt._daylight', (('int', None),) ),
    'msvcrt._dstbias':( 'int', None, 'cdecl', 'msvcrt._dstbias', () ),
    'msvcrt._dup':( 'int', None, 'cdecl', 'msvcrt._dup', (('int', None),) ),
    'msvcrt._dup2':( 'int', None, 'cdecl', 'msvcrt._dup2', (('int', None), ('int', None)) ),
    'msvcrt._ecvt':( 'int', None, 'cdecl', 'msvcrt._ecvt', (('int', None), ('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt._endthread':( 'int', None, 'cdecl', 'msvcrt._endthread', () ),
    'msvcrt._endthreadex':( 'int', None, 'cdecl', 'msvcrt._endthreadex', (('int', None),) ),
    'msvcrt._environ':( 'int', None, 'cdecl', 'msvcrt._environ', () ),
    'msvcrt._eof':( 'int', None, 'cdecl', 'msvcrt._eof', (('int', None),) ),
    'msvcrt._errno':( 'int', None, 'cdecl', 'msvcrt._errno', () ),
    'msvcrt._except_handler2':( 'int', None, 'cdecl', 'msvcrt._except_handler2', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._except_handler3':( 'int', None, 'cdecl', 'msvcrt._except_handler3', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._execl':( 'int', None, 'cdecl', 'msvcrt._execl', (('void *', 'ptr'),) ),
    'msvcrt._execle':( 'int', None, 'cdecl', 'msvcrt._execle', (('void *', 'ptr'), ('int', None), ('int', None)) ),
    'msvcrt._execlp':( 'int', None, 'cdecl', 'msvcrt._execlp', (('void *', 'ptr'),) ),
    'msvcrt._execlpe':( 'int', None, 'cdecl', 'msvcrt._execlpe', (('void *', 'ptr'), ('int', None), ('int', None)) ),
    'msvcrt._execv':( 'int', None, 'cdecl', 'msvcrt._execv', (('void *', 'ptr'), ('int', None)) ),
    'msvcrt._execve':( 'int', None, 'cdecl', 'msvcrt._execve', (('void *', 'ptr'), ('int', None), ('int', None)) ),
    'msvcrt._execvp':( 'int', None, 'cdecl', 'msvcrt._execvp', (('void *', 'ptr'), ('int', None)) ),
    'msvcrt._execvpe':( 'int', None, 'cdecl', 'msvcrt._execvpe', (('void *', 'ptr'), ('int', None), ('int', None)) ),
    'msvcrt._exit':( 'int', None, 'cdecl', 'msvcrt._exit', (('int', None),) ),
    'msvcrt._expand':( 'int', None, 'cdecl', 'msvcrt._expand', (('int', None), ('int', None)) ),
    'msvcrt._fcloseall':( 'int', None, 'cdecl', 'msvcrt._fcloseall', () ),
    'msvcrt._fcvt':( 'int', None, 'cdecl', 'msvcrt._fcvt', (('int', None), ('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt._fdopen':( 'int', None, 'cdecl', 'msvcrt._fdopen', (('int', None), ('int', None)) ),
    'msvcrt._fgetchar':( 'int', None, 'cdecl', 'msvcrt._fgetchar', () ),
    'msvcrt._fgetwchar':( 'int', None, 'cdecl', 'msvcrt._fgetwchar', () ),
    'msvcrt._filbuf':( 'int', None, 'cdecl', 'msvcrt._filbuf', (('int', None),) ),
    'msvcrt._fileinfo':( 'int', None, 'cdecl', 'msvcrt._fileinfo', () ),
    'msvcrt._filelength':( 'int', None, 'cdecl', 'msvcrt._filelength', (('int', None),) ),
    'msvcrt._filelengthi64':( 'int', None, 'cdecl', 'msvcrt._filelengthi64', (('int', None),) ),
    'msvcrt._fileno':( 'int', None, 'cdecl', 'msvcrt._fileno', (('int', None),) ),
    'msvcrt._findclose':( 'int', None, 'cdecl', 'msvcrt._findclose', (('void *', 'ptr'),) ),
    'msvcrt._findfirst':( 'int', None, 'cdecl', 'msvcrt._findfirst', (('int', None), ('int', None)) ),
    'msvcrt._findfirst64':( 'int', None, 'cdecl', 'msvcrt._findfirst64', (('int', None), ('int', None)) ),
    'msvcrt._findfirsti64':( 'int', None, 'cdecl', 'msvcrt._findfirsti64', (('int', None), ('int', None)) ),
    'msvcrt._findnext':( 'int', None, 'cdecl', 'msvcrt._findnext', (('int', None), ('int', None)) ),
    'msvcrt._findnext64':( 'int', None, 'cdecl', 'msvcrt._findnext64', (('int', None), ('int', None)) ),
    'msvcrt._findnexti64':( 'int', None, 'cdecl', 'msvcrt._findnexti64', (('int', None), ('int', None)) ),
    'msvcrt._finite':( 'int', None, 'cdecl', 'msvcrt._finite', (('int', None), ('int', None)) ),
    'msvcrt._flsbuf':( 'int', None, 'cdecl', 'msvcrt._flsbuf', (('int', None), ('void *', 'ptr')) ),
    'msvcrt._flushall':( 'int', None, 'cdecl', 'msvcrt._flushall', () ),
    'msvcrt._fmode':( 'int', None, 'cdecl', 'msvcrt._fmode', () ),
    'msvcrt._fpclass':( 'int', None, 'cdecl', 'msvcrt._fpclass', (('int', None), ('int', None)) ),
    'msvcrt._fpieee_flt':( 'int', None, 'cdecl', 'msvcrt._fpieee_flt', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._fpreset':( 'int', None, 'cdecl', 'msvcrt._fpreset', () ),
    'msvcrt._fputchar':( 'int', None, 'cdecl', 'msvcrt._fputchar', (('int', None),) ),
    'msvcrt._fputwchar':( 'int', None, 'cdecl', 'msvcrt._fputwchar', (('int', None),) ),
    'msvcrt._fsopen':( 'int', None, 'cdecl', 'msvcrt._fsopen', (('StringA', None), ('int', None), ('int', None)) ),
    'msvcrt._fstat':( 'int', None, 'cdecl', 'msvcrt._fstat', (('int', None), ('int', None)) ),
    'msvcrt._fstat64':( 'int', None, 'cdecl', 'msvcrt._fstat64', (('int', None), ('int', None)) ),
    'msvcrt._fstati64':( 'int', None, 'cdecl', 'msvcrt._fstati64', (('int', None), ('int', None)) ),
    'msvcrt._ftime':( 'int', None, 'cdecl', 'msvcrt._ftime', (('int', None),) ),
    'msvcrt._ftime64':( 'int', None, 'cdecl', 'msvcrt._ftime64', (('int', None),) ),
    'msvcrt._ftol':( 'int', None, 'cdecl', 'msvcrt._ftol', () ),
    'msvcrt._fullpath':( 'int', None, 'cdecl', 'msvcrt._fullpath', (('void *', 'ptr'), ('int', None), ('int', None)) ),
    'msvcrt._futime':( 'int', None, 'cdecl', 'msvcrt._futime', (('int', None), ('int', None)) ),
    'msvcrt._futime64':( 'int', None, 'cdecl', 'msvcrt._futime64', (('int', None), ('int', None)) ),
    'msvcrt._gcvt':( 'int', None, 'cdecl', 'msvcrt._gcvt', (('int', None), ('int', None), ('int', None), ('void *', 'ptr')) ),
    'msvcrt._get_heap_handle':( 'int', None, 'cdecl', 'msvcrt._get_heap_handle', () ),
    'msvcrt._get_osfhandle':( 'int', None, 'cdecl', 'msvcrt._get_osfhandle', (('int', None),) ),
    'msvcrt._get_sbh_threshold':( 'int', None, 'cdecl', 'msvcrt._get_sbh_threshold', () ),
    'msvcrt._getch':( 'int', None, 'cdecl', 'msvcrt._getch', () ),
    'msvcrt._getche':( 'int', None, 'cdecl', 'msvcrt._getche', () ),
    'msvcrt._getcwd':( 'int', None, 'cdecl', 'msvcrt._getcwd', (('int', None), ('int', None)) ),
    'msvcrt._getdcwd':( 'int', None, 'cdecl', 'msvcrt._getdcwd', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._getdiskfree':( 'int', None, 'cdecl', 'msvcrt._getdiskfree', (('int', None), ('int', None)) ),
    'msvcrt._getdllprocaddr':( 'int', None, 'cdecl', 'msvcrt._getdllprocaddr', (('HMODULE', None), ('StringA', None), ('int', None)) ),
    'msvcrt._getdrive':( 'int', None, 'cdecl', 'msvcrt._getdrive', () ),
    'msvcrt._getdrives':( 'int', None, 'stdcall', 'msvcrt._getdrives', () ),
    'msvcrt._getmaxstdio':( 'int', None, 'cdecl', 'msvcrt._getmaxstdio', () ),
    'msvcrt._getmbcp':( 'int', None, 'cdecl', 'msvcrt._getmbcp', () ),
    'msvcrt._getpid':( 'int', None, 'stdcall', 'msvcrt._getpid', () ),
    'msvcrt._getsystime':( 'int', None, 'cdecl', 'msvcrt._getsystime', (('int', None),) ),
    'msvcrt._getw':( 'int', None, 'cdecl', 'msvcrt._getw', (('int', None),) ),
    'msvcrt._getwch':( 'int', None, 'cdecl', 'msvcrt._getwch', () ),
    'msvcrt._getwche':( 'int', None, 'cdecl', 'msvcrt._getwche', () ),
    'msvcrt._getws':( 'int', None, 'cdecl', 'msvcrt._getws', (('int', None),) ),
    'msvcrt._global_unwind2':( 'int', None, 'cdecl', 'msvcrt._global_unwind2', (('int', None),) ),
    'msvcrt._gmtime64':( 'int', None, 'cdecl', 'msvcrt._gmtime64', (('void *', 'ptr'),) ),
    'msvcrt._heapadd':( 'int', None, 'cdecl', 'msvcrt._heapadd', () ),
    'msvcrt._heapchk':( 'int', None, 'cdecl', 'msvcrt._heapchk', () ),
    'msvcrt._heapmin':( 'int', None, 'cdecl', 'msvcrt._heapmin', () ),
    'msvcrt._heapset':( 'int', None, 'cdecl', 'msvcrt._heapset', () ),
    'msvcrt._heapused':( 'int', None, 'cdecl', 'msvcrt._heapused', () ),
    'msvcrt._heapwalk':( 'int', None, 'cdecl', 'msvcrt._heapwalk', (('int', None),) ),
    'msvcrt._hypot':( 'int', None, 'cdecl', 'msvcrt._hypot', () ),
    'msvcrt._i64toa':( 'int', None, 'msfastcall', 'msvcrt._i64toa', (('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt._i64tow':( 'int', None, 'msfastcall', 'msvcrt._i64tow', (('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt._initterm':( 'int', None, 'cdecl', 'msvcrt._initterm', (('int', None), ('int', None)) ),
    'msvcrt._inp':( 'int', None, 'cdecl', 'msvcrt._inp', (('int', None),) ),
    'msvcrt._inpd':( 'int', None, 'cdecl', 'msvcrt._inpd', (('int', None),) ),
    'msvcrt._inpw':( 'int', None, 'cdecl', 'msvcrt._inpw', (('int', None),) ),
    'msvcrt._iob':( 'int', None, 'cdecl', 'msvcrt._iob', () ),
    'msvcrt._isatty':( 'int', None, 'cdecl', 'msvcrt._isatty', (('int', None),) ),
    'msvcrt._isctype':( 'int', None, 'cdecl', 'msvcrt._isctype', (('int', None), ('int', None)) ),
    'msvcrt._ismbbalnum':( 'int', None, 'cdecl', 'msvcrt._ismbbalnum', (('int', None),) ),
    'msvcrt._ismbbalpha':( 'int', None, 'cdecl', 'msvcrt._ismbbalpha', (('int', None),) ),
    'msvcrt._ismbbgraph':( 'int', None, 'cdecl', 'msvcrt._ismbbgraph', (('int', None),) ),
    'msvcrt._ismbbkalnum':( 'int', None, 'cdecl', 'msvcrt._ismbbkalnum', (('int', None),) ),
    'msvcrt._ismbbkana':( 'int', None, 'cdecl', 'msvcrt._ismbbkana', (('int', None),) ),
    'msvcrt._ismbbkprint':( 'int', None, 'cdecl', 'msvcrt._ismbbkprint', (('int', None),) ),
    'msvcrt._ismbbkpunct':( 'int', None, 'cdecl', 'msvcrt._ismbbkpunct', (('int', None),) ),
    'msvcrt._ismbblead':( 'int', None, 'cdecl', 'msvcrt._ismbblead', (('int', None),) ),
    'msvcrt._ismbbprint':( 'int', None, 'cdecl', 'msvcrt._ismbbprint', (('int', None),) ),
    'msvcrt._ismbbpunct':( 'int', None, 'cdecl', 'msvcrt._ismbbpunct', (('int', None),) ),
    'msvcrt._ismbbtrail':( 'int', None, 'cdecl', 'msvcrt._ismbbtrail', (('int', None),) ),
    'msvcrt._ismbcalnum':( 'int', None, 'cdecl', 'msvcrt._ismbcalnum', (('int', None),) ),
    'msvcrt._ismbcalpha':( 'int', None, 'cdecl', 'msvcrt._ismbcalpha', (('int', None),) ),
    'msvcrt._ismbcdigit':( 'int', None, 'cdecl', 'msvcrt._ismbcdigit', (('int', None),) ),
    'msvcrt._ismbcgraph':( 'int', None, 'cdecl', 'msvcrt._ismbcgraph', (('int', None),) ),
    'msvcrt._ismbchira':( 'int', None, 'cdecl', 'msvcrt._ismbchira', (('int', None),) ),
    'msvcrt._ismbckata':( 'int', None, 'cdecl', 'msvcrt._ismbckata', (('int', None),) ),
    'msvcrt._ismbcl0':( 'int', None, 'cdecl', 'msvcrt._ismbcl0', (('int', None),) ),
    'msvcrt._ismbcl1':( 'int', None, 'cdecl', 'msvcrt._ismbcl1', (('int', None),) ),
    'msvcrt._ismbcl2':( 'int', None, 'cdecl', 'msvcrt._ismbcl2', (('int', None),) ),
    'msvcrt._ismbclegal':( 'int', None, 'cdecl', 'msvcrt._ismbclegal', (('int', None),) ),
    'msvcrt._ismbclower':( 'int', None, 'cdecl', 'msvcrt._ismbclower', (('int', None),) ),
    'msvcrt._ismbcprint':( 'int', None, 'cdecl', 'msvcrt._ismbcprint', (('int', None),) ),
    'msvcrt._ismbcpunct':( 'int', None, 'cdecl', 'msvcrt._ismbcpunct', (('int', None),) ),
    'msvcrt._ismbcspace':( 'int', None, 'cdecl', 'msvcrt._ismbcspace', (('int', None),) ),
    'msvcrt._ismbcsymbol':( 'int', None, 'cdecl', 'msvcrt._ismbcsymbol', (('int', None),) ),
    'msvcrt._ismbcupper':( 'int', None, 'cdecl', 'msvcrt._ismbcupper', (('int', None),) ),
    'msvcrt._ismbslead':( 'int', None, 'cdecl', 'msvcrt._ismbslead', (('int', None), ('int', None)) ),
    'msvcrt._ismbstrail':( 'int', None, 'cdecl', 'msvcrt._ismbstrail', (('int', None), ('int', None)) ),
    'msvcrt._isnan':( 'int', None, 'cdecl', 'msvcrt._isnan', (('int', None), ('int', None)) ),
    'msvcrt._itoa':( 'int', None, 'cdecl', 'msvcrt._itoa', (('int', None), ('void *', 'obj'), ('int', None)) ),
    'msvcrt._itow':( 'int', None, 'cdecl', 'msvcrt._itow', (('int', None), ('void *', 'obj'), ('int', None)) ),
    'msvcrt._j0':( 'int', None, 'cdecl', 'msvcrt._j0', () ),
    'msvcrt._j1':( 'int', None, 'cdecl', 'msvcrt._j1', () ),
    'msvcrt._jn':( 'int', None, 'cdecl', 'msvcrt._jn', (('int', None),) ),
    'msvcrt._kbhit':( 'int', None, 'cdecl', 'msvcrt._kbhit', () ),
    'msvcrt._lfind':( 'int', None, 'cdecl', 'msvcrt._lfind', (('int', None), ('int', None), ('int', None), ('int', None), ('void *', 'funcptr')) ),
    'msvcrt._loaddll':( 'int', None, 'cdecl', 'msvcrt._loaddll', (('FileNameA', None),) ),
    'msvcrt._local_unwind2':( 'int', None, 'cdecl', 'msvcrt._local_unwind2', (('void *', 'ptr'), ('int', None)) ),
    'msvcrt._localtime64':( 'int', None, 'cdecl', 'msvcrt._localtime64', (('void *', 'ptr'),) ),
    'msvcrt._lock':( 'int', None, 'cdecl', 'msvcrt._lock', (('int', None),) ),
    'msvcrt._locking':( 'int', None, 'cdecl', 'msvcrt._locking', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._logb':( 'int', None, 'cdecl', 'msvcrt._logb', (('int', None), ('int', None)) ),
    'msvcrt._longjmpex':( 'int', None, 'cdecl', 'msvcrt._longjmpex', (('int', None), ('int', None)) ),
    'msvcrt._lrotl':( 'int', None, 'cdecl', 'msvcrt._lrotl', (('int', None), ('int', None)) ),
    'msvcrt._lrotr':( 'int', None, 'cdecl', 'msvcrt._lrotr', (('int', None), ('int', None)) ),
    'msvcrt._lsearch':( 'int', None, 'cdecl', 'msvcrt._lsearch', (('int', None), ('int', None), ('int', None), ('int', None), ('void *', 'funcptr')) ),
    'msvcrt._lseek':( 'int', None, 'cdecl', 'msvcrt._lseek', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._lseeki64':( 'int', None, 'cdecl', 'msvcrt._lseeki64', (('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt._ltoa':( 'int', None, 'cdecl', 'msvcrt._ltoa', (('int', None), ('void *', 'obj'), ('int', None)) ),
    'msvcrt._ltow':( 'int', None, 'cdecl', 'msvcrt._ltow', (('int', None), ('void *', 'obj'), ('int', None)) ),
    'msvcrt._makepath':( 'int', None, 'cdecl', 'msvcrt._makepath', (('int', None), ('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt._mbbtombc':( 'int', None, 'cdecl', 'msvcrt._mbbtombc', (('int', None),) ),
    'msvcrt._mbbtype':( 'int', None, 'cdecl', 'msvcrt._mbbtype', (('int', None), ('int', None)) ),
    'msvcrt._mbcasemap':( 'int', None, 'cdecl', 'msvcrt._mbcasemap', () ),
    'msvcrt._mbccpy':( 'int', None, 'cdecl', 'msvcrt._mbccpy', (('int', None), ('int', None)) ),
    'msvcrt._mbcjistojms':( 'int', None, 'cdecl', 'msvcrt._mbcjistojms', (('int', None),) ),
    'msvcrt._mbcjmstojis':( 'int', None, 'cdecl', 'msvcrt._mbcjmstojis', (('int', None),) ),
    'msvcrt._mbclen':( 'int', None, 'cdecl', 'msvcrt._mbclen', (('int', None),) ),
    'msvcrt._mbctohira':( 'int', None, 'cdecl', 'msvcrt._mbctohira', (('int', None),) ),
    'msvcrt._mbctokata':( 'int', None, 'cdecl', 'msvcrt._mbctokata', (('int', None),) ),
    'msvcrt._mbctolower':( 'int', None, 'cdecl', 'msvcrt._mbctolower', (('int', None),) ),
    'msvcrt._mbctombb':( 'int', None, 'cdecl', 'msvcrt._mbctombb', (('int', None),) ),
    'msvcrt._mbctoupper':( 'int', None, 'cdecl', 'msvcrt._mbctoupper', (('int', None),) ),
    'msvcrt._mbctype':( 'int', None, 'cdecl', 'msvcrt._mbctype', () ),
    'msvcrt._mbsbtype':( 'int', None, 'cdecl', 'msvcrt._mbsbtype', (('int', None), ('int', None)) ),
    'msvcrt._mbscat':( 'int', None, 'cdecl', 'msvcrt._mbscat', (('int', None), ('int', None)) ),
    'msvcrt._mbschr':( 'int', None, 'cdecl', 'msvcrt._mbschr', (('int', None), ('int', None)) ),
    'msvcrt._mbscmp':( 'int', None, 'cdecl', 'msvcrt._mbscmp', (('int', None), ('int', None)) ),
    'msvcrt._mbscoll':( 'int', None, 'cdecl', 'msvcrt._mbscoll', (('int', None), ('int', None)) ),
    'msvcrt._mbscpy':( 'int', None, 'cdecl', 'msvcrt._mbscpy', (('int', None), ('void *', 'ptr')) ),
    'msvcrt._mbscspn':( 'int', None, 'cdecl', 'msvcrt._mbscspn', (('int', None), ('int', None)) ),
    'msvcrt._mbsdec':( 'int', None, 'cdecl', 'msvcrt._mbsdec', (('int', None), ('int', None)) ),
    'msvcrt._mbsdup':( 'int', None, 'cdecl', 'msvcrt._mbsdup', (('int', None),) ),
    'msvcrt._mbsicmp':( 'int', None, 'cdecl', 'msvcrt._mbsicmp', (('void *', 'ptr'), ('void *', 'ptr')) ),
    'msvcrt._mbsicoll':( 'int', None, 'cdecl', 'msvcrt._mbsicoll', (('int', None), ('int', None)) ),
    'msvcrt._mbsinc':( 'int', None, 'cdecl', 'msvcrt._mbsinc', (('int', None),) ),
    'msvcrt._mbslen':( 'int', None, 'cdecl', 'msvcrt._mbslen', (('int', None),) ),
    'msvcrt._mbslwr':( 'int', None, 'cdecl', 'msvcrt._mbslwr', (('void *', 'ptr'),) ),
    'msvcrt._mbsnbcat':( 'int', None, 'cdecl', 'msvcrt._mbsnbcat', (('void *', 'ptr'), ('int', None), ('int', None)) ),
    'msvcrt._mbsnbcmp':( 'int', None, 'cdecl', 'msvcrt._mbsnbcmp', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._mbsnbcnt':( 'int', None, 'cdecl', 'msvcrt._mbsnbcnt', (('int', None), ('int', None)) ),
    'msvcrt._mbsnbcoll':( 'int', None, 'cdecl', 'msvcrt._mbsnbcoll', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._mbsnbcpy':( 'int', None, 'cdecl', 'msvcrt._mbsnbcpy', (('void *', 'ptr'), ('void *', 'ptr'), ('int', None)) ),
    'msvcrt._mbsnbicmp':( 'int', None, 'cdecl', 'msvcrt._mbsnbicmp', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._mbsnbicoll':( 'int', None, 'cdecl', 'msvcrt._mbsnbicoll', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._mbsnbset':( 'int', None, 'cdecl', 'msvcrt._mbsnbset', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._mbsncat':( 'int', None, 'cdecl', 'msvcrt._mbsncat', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._mbsnccnt':( 'int', None, 'cdecl', 'msvcrt._mbsnccnt', (('int', None), ('int', None)) ),
    'msvcrt._mbsncmp':( 'int', None, 'cdecl', 'msvcrt._mbsncmp', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._mbsncoll':( 'int', None, 'cdecl', 'msvcrt._mbsncoll', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._mbsncpy':( 'int', None, 'cdecl', 'msvcrt._mbsncpy', (('void *', 'ptr'), ('void *', 'ptr'), ('int', None)) ),
    'msvcrt._mbsnextc':( 'int', None, 'cdecl', 'msvcrt._mbsnextc', (('int', None),) ),
    'msvcrt._mbsnicmp':( 'int', None, 'cdecl', 'msvcrt._mbsnicmp', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._mbsnicoll':( 'int', None, 'cdecl', 'msvcrt._mbsnicoll', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._mbsninc':( 'int', None, 'cdecl', 'msvcrt._mbsninc', (('int', None), ('int', None)) ),
    'msvcrt._mbsnset':( 'int', None, 'cdecl', 'msvcrt._mbsnset', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._mbspbrk':( 'int', None, 'cdecl', 'msvcrt._mbspbrk', (('int', None), ('int', None)) ),
    'msvcrt._mbsrchr':( 'int', None, 'cdecl', 'msvcrt._mbsrchr', (('int', None), ('int', None)) ),
    'msvcrt._mbsrev':( 'int', None, 'cdecl', 'msvcrt._mbsrev', (('int', None),) ),
    'msvcrt._mbsset':( 'int', None, 'cdecl', 'msvcrt._mbsset', (('int', None), ('int', None)) ),
    'msvcrt._mbsspn':( 'int', None, 'cdecl', 'msvcrt._mbsspn', (('int', None), ('int', None)) ),
    'msvcrt._mbsspnp':( 'int', None, 'cdecl', 'msvcrt._mbsspnp', (('int', None), ('int', None)) ),
    'msvcrt._mbsstr':( 'int', None, 'cdecl', 'msvcrt._mbsstr', (('int', None), ('int', None)) ),
    'msvcrt._mbstok':( 'int', None, 'cdecl', 'msvcrt._mbstok', (('int', None), ('int', None)) ),
    'msvcrt._mbstrlen':( 'int', None, 'cdecl', 'msvcrt._mbstrlen', (('int', None),) ),
    'msvcrt._mbsupr':( 'int', None, 'cdecl', 'msvcrt._mbsupr', (('void *', 'ptr'),) ),
    'msvcrt._memccpy':( 'int', None, 'cdecl', 'msvcrt._memccpy', (('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt._memicmp':( 'int', None, 'cdecl', 'msvcrt._memicmp', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._mkdir':( 'int', None, 'cdecl', 'msvcrt._mkdir', (('StringA', None),) ),
    'msvcrt._mktemp':( 'int', None, 'cdecl', 'msvcrt._mktemp', (('void *', 'ptr'),) ),
    'msvcrt._mktime64':( 'int', None, 'cdecl', 'msvcrt._mktime64', (('int', None),) ),
    'msvcrt._msize':( 'int', None, 'cdecl', 'msvcrt._msize', (('int', None),) ),
    'msvcrt._nextafter':( 'int', None, 'cdecl', 'msvcrt._nextafter', (('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt._onexit':( 'int', None, 'cdecl', 'msvcrt._onexit', (('int', None),) ),
    'msvcrt._open':( 'int', None, 'cdecl', 'msvcrt._open', (('StringA', None), ('int', None), ('int', None)) ),
    'msvcrt._open_osfhandle':( 'int', None, 'cdecl', 'msvcrt._open_osfhandle', (('int', None), ('int', None)) ),
    'msvcrt._osplatform':( 'int', None, 'cdecl', 'msvcrt._osplatform', () ),
    'msvcrt._osver':( 'int', None, 'cdecl', 'msvcrt._osver', () ),
    'msvcrt._outp':( 'int', None, 'cdecl', 'msvcrt._outp', (('int', None), ('int', None)) ),
    'msvcrt._outpd':( 'int', None, 'cdecl', 'msvcrt._outpd', (('int', None), ('int', None)) ),
    'msvcrt._outpw':( 'int', None, 'cdecl', 'msvcrt._outpw', (('int', None), ('int', None)) ),
    'msvcrt._pclose':( 'int', None, 'cdecl', 'msvcrt._pclose', (('int', None),) ),
    'msvcrt._pctype':( 'int', None, 'cdecl', 'msvcrt._pctype', () ),
    'msvcrt._pgmptr':( 'int', None, 'cdecl', 'msvcrt._pgmptr', () ),
    'msvcrt._pipe':( 'int', None, 'cdecl', 'msvcrt._pipe', (('void *', 'ptr'), ('int', None), ('int', None)) ),
    'msvcrt._popen':( 'int', None, 'cdecl', 'msvcrt._popen', (('int', None), ('int', None)) ),
    'msvcrt._purecall':( 'int', None, 'cdecl', 'msvcrt._purecall', () ),
    'msvcrt._putch':( 'int', None, 'cdecl', 'msvcrt._putch', (('int', None),) ),
    'msvcrt._putenv':( 'int', None, 'cdecl', 'msvcrt._putenv', (('int', None),) ),
    'msvcrt._putw':( 'int', None, 'cdecl', 'msvcrt._putw', (('int', None), ('void *', 'ptr')) ),
    'msvcrt._putwch':( 'int', None, 'cdecl', 'msvcrt._putwch', (('int', None),) ),
    'msvcrt._putws':( 'int', None, 'cdecl', 'msvcrt._putws', (('int', None),) ),
    'msvcrt._pwctype':( 'int', None, 'cdecl', 'msvcrt._pwctype', () ),
    'msvcrt._read':( 'int', None, 'cdecl', 'msvcrt._read', (('int', None), ('StringA', None), ('DWORD', None)) ),
    'msvcrt._resetstkoflw':( 'int', None, 'cdecl', 'msvcrt._resetstkoflw', () ),
    'msvcrt._rmdir':( 'int', None, 'cdecl', 'msvcrt._rmdir', (('int', None),) ),
    'msvcrt._rmtmp':( 'int', None, 'cdecl', 'msvcrt._rmtmp', () ),
    'msvcrt._rotl':( 'int', None, 'cdecl', 'msvcrt._rotl', (('int', None), ('int', None)) ),
    'msvcrt._rotr':( 'int', None, 'cdecl', 'msvcrt._rotr', (('int', None), ('int', None)) ),
    'msvcrt._safe_fdiv':( 'int', None, 'cdecl', 'msvcrt._safe_fdiv', () ),
    'msvcrt._safe_fdivr':( 'int', None, 'cdecl', 'msvcrt._safe_fdivr', () ),
    'msvcrt._safe_fprem':( 'int', None, 'cdecl', 'msvcrt._safe_fprem', () ),
    'msvcrt._safe_fprem1':( 'int', None, 'cdecl', 'msvcrt._safe_fprem1', () ),
    'msvcrt._scalb':( 'int', None, 'cdecl', 'msvcrt._scalb', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._scprintf':( 'int', None, 'cdecl', 'msvcrt._scprintf', (('int', None),) ),
    'msvcrt._scwprintf':( 'int', None, 'cdecl', 'msvcrt._scwprintf', (('int', None),) ),
    'msvcrt._searchenv':( 'int', None, 'cdecl', 'msvcrt._searchenv', (('int', None), ('int', None), ('void *', 'ptr')) ),
    'msvcrt._seh_longjmp_unwind':( 'int', None, 'stdcall', 'msvcrt._seh_longjmp_unwind', (('int', None),) ),
    'msvcrt._set_sse2_enable':( 'int', None, 'cdecl', 'msvcrt._set_SSE2_enable', (('int', None),) ),
    'msvcrt._set_error_mode':( 'int', None, 'cdecl', 'msvcrt._set_error_mode', (('int', None),) ),
    'msvcrt._set_sbh_threshold':( 'int', None, 'cdecl', 'msvcrt._set_sbh_threshold', (('int', None),) ),
    'msvcrt._seterrormode':( 'int', None, 'cdecl', 'msvcrt._seterrormode', (('int', None),) ),
    'msvcrt._setjmp':( 'int', None, 'cdecl', 'msvcrt._setjmp', (('int', None),) ),
    'msvcrt._setjmp3':( 'int', None, 'cdecl', 'msvcrt._setjmp3', (('int', None), ('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt._setmaxstdio':( 'int', None, 'cdecl', 'msvcrt._setmaxstdio', (('int', None),) ),
    'msvcrt._setmbcp':( 'int', None, 'cdecl', 'msvcrt._setmbcp', (('int', None),) ),
    'msvcrt._setmode':( 'int', None, 'cdecl', 'msvcrt._setmode', (('int', None), ('int', None)) ),
    'msvcrt._setsystime':( 'int', None, 'cdecl', 'msvcrt._setsystime', (('int', None), ('int', None)) ),
    'msvcrt._sleep':( 'int', None, 'cdecl', 'msvcrt._sleep', (('int', None),) ),
    'msvcrt._snprintf':( 'int', None, 'cdecl', 'msvcrt._snprintf', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._snscanf':( 'int', None, 'cdecl', 'msvcrt._snscanf', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._snwprintf':( 'int', None, 'cdecl', 'msvcrt._snwprintf', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._snwscanf':( 'int', None, 'cdecl', 'msvcrt._snwscanf', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._sopen':( 'int', None, 'cdecl', 'msvcrt._sopen', (('StringA', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt._spawnl':( 'int', None, 'cdecl', 'msvcrt._spawnl', (('int', None), ('void *', 'ptr')) ),
    'msvcrt._spawnle':( 'int', None, 'cdecl', 'msvcrt._spawnle', (('int', None), ('void *', 'ptr'), ('int', None), ('int', None)) ),
    'msvcrt._spawnlp':( 'int', None, 'cdecl', 'msvcrt._spawnlp', (('int', None), ('void *', 'ptr')) ),
    'msvcrt._spawnlpe':( 'int', None, 'cdecl', 'msvcrt._spawnlpe', (('int', None), ('void *', 'ptr'), ('int', None), ('int', None)) ),
    'msvcrt._spawnv':( 'int', None, 'cdecl', 'msvcrt._spawnv', (('int', None), ('void *', 'ptr'), ('void *', 'ptr')) ),
    'msvcrt._spawnve':( 'int', None, 'cdecl', 'msvcrt._spawnve', (('int', None), ('void *', 'ptr'), ('void *', 'ptr'), ('int', None)) ),
    'msvcrt._spawnvp':( 'int', None, 'cdecl', 'msvcrt._spawnvp', (('int', None), ('void *', 'ptr'), ('int', None)) ),
    'msvcrt._spawnvpe':( 'int', None, 'cdecl', 'msvcrt._spawnvpe', (('int', None), ('void *', 'ptr'), ('void *', 'ptr'), ('int', None)) ),
    'msvcrt._splitpath':( 'int', None, 'cdecl', 'msvcrt._splitpath', (('void *', 'ptr'), ('void *', 'ptr'), ('void *', 'ptr'), ('void *', 'ptr'), ('void *', 'ptr')) ),
    'msvcrt._stat':( 'int', None, 'cdecl', 'msvcrt._stat', (('int', None), ('int', None)) ),
    'msvcrt._stat64':( 'int', None, 'cdecl', 'msvcrt._stat64', (('int', None), ('int', None)) ),
    'msvcrt._stati64':( 'int', None, 'cdecl', 'msvcrt._stati64', (('int', None), ('int', None)) ),
    'msvcrt._statusfp':( 'int', None, 'cdecl', 'msvcrt._statusfp', () ),
    'msvcrt._strcmpi':( 'int', None, 'cdecl', 'msvcrt._strcmpi', (('int', None), ('void *', 'ptr')) ),
    'msvcrt._strdate':( 'int', None, 'cdecl', 'msvcrt._strdate', (('int', None),) ),
    'msvcrt._strdup':( 'int', None, 'cdecl', 'msvcrt._strdup', (('int', None),) ),
    'msvcrt._strerror':( 'int', None, 'cdecl', 'msvcrt._strerror', (('int', None),) ),
    'msvcrt._stricmp':( 'int', None, 'cdecl', 'msvcrt._stricmp', (('int', None), ('void *', 'ptr')) ),
    'msvcrt._stricoll':( 'int', None, 'cdecl', 'msvcrt._stricoll', (('int', None), ('int', None)) ),
    'msvcrt._strlwr':( 'int', None, 'cdecl', 'msvcrt._strlwr', (('void *', 'ptr'),) ),
    'msvcrt._strncoll':( 'int', None, 'cdecl', 'msvcrt._strncoll', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._strnicmp':( 'int', None, 'cdecl', 'msvcrt._strnicmp', (('int', None), ('void *', 'ptr'), ('int', None)) ),
    'msvcrt._strnicoll':( 'int', None, 'cdecl', 'msvcrt._strnicoll', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._strnset':( 'int', None, 'cdecl', 'msvcrt._strnset', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._strrev':( 'int', None, 'cdecl', 'msvcrt._strrev', (('int', None),) ),
    'msvcrt._strset':( 'int', None, 'cdecl', 'msvcrt._strset', (('int', None), ('int', None)) ),
    'msvcrt._strtime':( 'int', None, 'cdecl', 'msvcrt._strtime', (('int', None),) ),
    'msvcrt._strtoi64':( 'int', None, 'cdecl', 'msvcrt._strtoi64', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._strtoui64':( 'int', None, 'cdecl', 'msvcrt._strtoui64', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._strupr':( 'int', None, 'cdecl', 'msvcrt._strupr', (('void *', 'ptr'),) ),
    'msvcrt._swab':( 'int', None, 'cdecl', 'msvcrt._swab', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._sys_errlist':( 'int', None, 'cdecl', 'msvcrt._sys_errlist', () ),
    'msvcrt._sys_nerr':( 'int', None, 'bfastcall', 'msvcrt._sys_nerr', (('int', None),) ),
    'msvcrt._tell':( 'int', None, 'cdecl', 'msvcrt._tell', (('int', None),) ),
    'msvcrt._telli64':( 'int', None, 'cdecl', 'msvcrt._telli64', (('int', None),) ),
    'msvcrt._tempnam':( 'int', None, 'cdecl', 'msvcrt._tempnam', (('void *', 'ptr'), ('int', None)) ),
    'msvcrt._time64':( 'int', None, 'cdecl', 'msvcrt._time64', (('void *', 'ptr'),) ),
    'msvcrt._timezone':( 'int', None, 'bfastcall', 'msvcrt._timezone', (('int', None),) ),
    'msvcrt._tolower':( 'int', None, 'cdecl', 'msvcrt._tolower', (('int', None),) ),
    'msvcrt._toupper':( 'int', None, 'cdecl', 'msvcrt._toupper', (('int', None),) ),
    'msvcrt._tzname':( 'int', None, 'cdecl', 'msvcrt._tzname', () ),
    'msvcrt._tzset':( 'int', None, 'cdecl', 'msvcrt._tzset', () ),
    'msvcrt._ui64toa':( 'int', None, 'msfastcall', 'msvcrt._ui64toa', (('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt._ui64tow':( 'int', None, 'msfastcall', 'msvcrt._ui64tow', (('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt._ultoa':( 'int', None, 'cdecl', 'msvcrt._ultoa', (('int', None), ('void *', 'obj'), ('int', None)) ),
    'msvcrt._ultow':( 'int', None, 'cdecl', 'msvcrt._ultow', (('int', None), ('void *', 'obj'), ('int', None)) ),
    'msvcrt._umask':( 'int', None, 'cdecl', 'msvcrt._umask', (('int', None),) ),
    'msvcrt._ungetch':( 'int', None, 'cdecl', 'msvcrt._ungetch', (('int', None),) ),
    'msvcrt._ungetwch':( 'int', None, 'cdecl', 'msvcrt._ungetwch', (('int', None),) ),
    'msvcrt._unlink':( 'int', None, 'cdecl', 'msvcrt._unlink', (('int', None),) ),
    'msvcrt._unloaddll':( 'int', None, 'cdecl', 'msvcrt._unloaddll', (('int', None),) ),
    'msvcrt._unlock':( 'int', None, 'cdecl', 'msvcrt._unlock', (('int', None),) ),
    'msvcrt._utime':( 'int', None, 'cdecl', 'msvcrt._utime', (('StringA', None), ('int', None)) ),
    'msvcrt._utime64':( 'int', None, 'cdecl', 'msvcrt._utime64', (('StringA', None), ('int', None)) ),
    'msvcrt._vscprintf':( 'int', None, 'cdecl', 'msvcrt._vscprintf', (('int', None), ('int', None)) ),
    'msvcrt._vscwprintf':( 'int', None, 'cdecl', 'msvcrt._vscwprintf', (('int', None), ('int', None)) ),
    'msvcrt._vsnprintf':( 'int', None, 'cdecl', 'msvcrt._vsnprintf', (('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt._vsnwprintf':( 'int', None, 'cdecl', 'msvcrt._vsnwprintf', (('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt._waccess':( 'int', None, 'cdecl', 'msvcrt._waccess', (('void *', 'ptr'), ('int', None)) ),
    'msvcrt._wasctime':( 'int', None, 'cdecl', 'msvcrt._wasctime', (('int', None),) ),
    'msvcrt._wchdir':( 'int', None, 'cdecl', 'msvcrt._wchdir', (('int', None),) ),
    'msvcrt._wchmod':( 'int', None, 'cdecl', 'msvcrt._wchmod', (('int', None), ('int', None)) ),
    'msvcrt._wcmdln':( 'int', None, 'cdecl', 'msvcrt._wcmdln', () ),
    'msvcrt._wcreat':( 'int', None, 'cdecl', 'msvcrt._wcreat', (('StringW', None), ('int', None)) ),
    'msvcrt._wcsdup':( 'int', None, 'cdecl', 'msvcrt._wcsdup', (('int', None),) ),
    'msvcrt._wcserror':( 'int', None, 'cdecl', 'msvcrt._wcserror', (('int', None),) ),
    'msvcrt._wcsicmp':( 'int', None, 'cdecl', 'msvcrt._wcsicmp', (('int', None), ('int', None)) ),
    'msvcrt._wcsicoll':( 'int', None, 'cdecl', 'msvcrt._wcsicoll', (('int', None), ('int', None)) ),
    'msvcrt._wcslwr':( 'int', None, 'cdecl', 'msvcrt._wcslwr', (('int', None),) ),
    'msvcrt._wcsncoll':( 'int', None, 'cdecl', 'msvcrt._wcsncoll', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._wcsnicmp':( 'int', None, 'cdecl', 'msvcrt._wcsnicmp', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._wcsnicoll':( 'int', None, 'cdecl', 'msvcrt._wcsnicoll', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._wcsnset':( 'int', None, 'cdecl', 'msvcrt._wcsnset', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._wcsrev':( 'int', None, 'cdecl', 'msvcrt._wcsrev', (('int', None),) ),
    'msvcrt._wcsset':( 'int', None, 'cdecl', 'msvcrt._wcsset', (('int', None), ('int', None)) ),
    'msvcrt._wcstoi64':( 'int', None, 'cdecl', 'msvcrt._wcstoi64', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._wcstoui64':( 'int', None, 'cdecl', 'msvcrt._wcstoui64', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._wcsupr':( 'int', None, 'cdecl', 'msvcrt._wcsupr', (('int', None),) ),
    'msvcrt._wctime':( 'int', None, 'cdecl', 'msvcrt._wctime', (('void *', 'ptr'),) ),
    'msvcrt._wctime64':( 'int', None, 'cdecl', 'msvcrt._wctime64', (('int', None),) ),
    'msvcrt._wenviron':( 'int', None, 'cdecl', 'msvcrt._wenviron', () ),
    'msvcrt._wexecl':( 'int', None, 'cdecl', 'msvcrt._wexecl', () ),
    'msvcrt._wexecle':( 'int', None, 'cdecl', 'msvcrt._wexecle', ( ('int', None), ('int', None)) ),
    'msvcrt._wexeclp':( 'int', None, 'cdecl', 'msvcrt._wexeclp', () ),
    'msvcrt._wexeclpe':( 'int', None, 'cdecl', 'msvcrt._wexeclpe', ( ('int', None), ('int', None)) ),
    'msvcrt._wexecv':( 'int', None, 'cdecl', 'msvcrt._wexecv', ( ('int', None),) ),
    'msvcrt._wexecve':( 'int', None, 'cdecl', 'msvcrt._wexecve', ( ('int', None), ('int', None)) ),
    'msvcrt._wexecvp':( 'int', None, 'cdecl', 'msvcrt._wexecvp', ( ('int', None),) ),
    'msvcrt._wexecvpe':( 'int', None, 'cdecl', 'msvcrt._wexecvpe', ( ('int', None), ('int', None)) ),
    'msvcrt._wfdopen':( 'int', None, 'cdecl', 'msvcrt._wfdopen', (('int', None), ('int', None)) ),
    'msvcrt._wfindfirst':( 'int', None, 'cdecl', 'msvcrt._wfindfirst', (('int', None), ('int', None)) ),
    'msvcrt._wfindfirst64':( 'int', None, 'cdecl', 'msvcrt._wfindfirst64', (('int', None), ('int', None)) ),
    'msvcrt._wfindfirsti64':( 'int', None, 'cdecl', 'msvcrt._wfindfirsti64', (('int', None), ('int', None)) ),
    'msvcrt._wfindnext':( 'int', None, 'cdecl', 'msvcrt._wfindnext', (('int', None), ('int', None)) ),
    'msvcrt._wfindnext64':( 'int', None, 'cdecl', 'msvcrt._wfindnext64', (('int', None), ('int', None)) ),
    'msvcrt._wfindnexti64':( 'int', None, 'cdecl', 'msvcrt._wfindnexti64', (('int', None), ('int', None)) ),
    'msvcrt._wfopen':( 'int', None, 'cdecl', 'msvcrt._wfopen', (('StringW', None), ('int', None)) ),
    'msvcrt._wfreopen':( 'int', None, 'cdecl', 'msvcrt._wfreopen', (('StringW', None), ('int', None), ('int', None)) ),
    'msvcrt._wfsopen':( 'int', None, 'cdecl', 'msvcrt._wfsopen', (('StringW', None), ('int', None), ('int', None)) ),
    'msvcrt._wfullpath':( 'int', None, 'cdecl', 'msvcrt._wfullpath', (('void *', 'ptr'), ('int', None), ('int', None)) ),
    'msvcrt._wgetcwd':( 'int', None, 'cdecl', 'msvcrt._wgetcwd', (('int', None), ('int', None)) ),
    'msvcrt._wgetdcwd':( 'int', None, 'cdecl', 'msvcrt._wgetdcwd', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt._wgetenv':( 'int', None, 'cdecl', 'msvcrt._wgetenv', (('int', None),) ),
    'msvcrt._winmajor':( 'int', None, 'cdecl', 'msvcrt._winmajor', () ),
    'msvcrt._winminor':( 'int', None, 'cdecl', 'msvcrt._winminor', () ),
    'msvcrt._winver':( 'int', None, 'cdecl', 'msvcrt._winver', () ),
    'msvcrt._wmakepath':( 'int', None, 'cdecl', 'msvcrt._wmakepath', (('int', None), ('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt._wmkdir':( 'int', None, 'cdecl', 'msvcrt._wmkdir', (('StringW', None),) ),
    'msvcrt._wmktemp':( 'int', None, 'cdecl', 'msvcrt._wmktemp', (('void *', 'ptr'),) ),
    'msvcrt._wopen':( 'int', None, 'cdecl', 'msvcrt._wopen', (('StringW', None), ('int', None), ('int', None)) ),
    'msvcrt._wperror':( 'int', None, 'cdecl', 'msvcrt._wperror', (('void *', 'ptr'),) ),
    'msvcrt._wpgmptr':( 'int', None, 'cdecl', 'msvcrt._wpgmptr', () ),
    'msvcrt._wpopen':( 'int', None, 'cdecl', 'msvcrt._wpopen', (('int', None), ('int', None)) ),
    'msvcrt._wputenv':( 'int', None, 'cdecl', 'msvcrt._wputenv', (('int', None),) ),
    'msvcrt._wremove':( 'int', None, 'cdecl', 'msvcrt._wremove', (('int', None),) ),
    'msvcrt._wrename':( 'int', None, 'cdecl', 'msvcrt._wrename', (('int', None), ('int', None)) ),
    'msvcrt._write':( 'int', None, 'cdecl', 'msvcrt._write', (('int', None), ('void *', 'ptr'), ('DWORD', None)) ),
    'msvcrt._wrmdir':( 'int', None, 'cdecl', 'msvcrt._wrmdir', (('int', None),) ),
    'msvcrt._wsearchenv':( 'int', None, 'cdecl', 'msvcrt._wsearchenv', (('void *', 'ptr'), ('int', None), ('void *', 'ptr')) ),
    'msvcrt._wsetlocale':( 'int', None, 'cdecl', 'msvcrt._wsetlocale', ( ('void *', 'ptr'),) ),
    'msvcrt._wsopen':( 'int', None, 'cdecl', 'msvcrt._wsopen', (('StringW', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt._wspawnl':( 'int', None, 'cdecl', 'msvcrt._wspawnl', (('int', None), ('void *', 'obj')) ),
    'msvcrt._wspawnle':( 'int', None, 'cdecl', 'msvcrt._wspawnle', (('int', None), ('void *', 'obj'), ('int', None), ('int', None)) ),
    'msvcrt._wspawnlp':( 'int', None, 'cdecl', 'msvcrt._wspawnlp', (('int', None), ('void *', 'obj')) ),
    'msvcrt._wspawnlpe':( 'int', None, 'cdecl', 'msvcrt._wspawnlpe', (('int', None), ('void *', 'obj'), ('int', None), ('int', None)) ),
    'msvcrt._wspawnv':( 'int', None, 'cdecl', 'msvcrt._wspawnv', (('int', None), ('void *', 'obj'), ('void *', 'ptr')) ),
    'msvcrt._wspawnve':( 'int', None, 'cdecl', 'msvcrt._wspawnve', (('int', None), ('void *', 'obj'), ('void *', 'ptr'), ('int', None)) ),
    'msvcrt._wspawnvp':( 'int', None, 'cdecl', 'msvcrt._wspawnvp', (('int', None), ('void *', 'obj'), ('int', None)) ),
    'msvcrt._wspawnvpe':( 'int', None, 'cdecl', 'msvcrt._wspawnvpe', (('int', None), ('void *', 'obj'), ('void *', 'ptr'), ('int', None)) ),
    'msvcrt._wsplitpath':( 'int', None, 'cdecl', 'msvcrt._wsplitpath', (('void *', 'ptr'), ('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt._wstat':( 'int', None, 'cdecl', 'msvcrt._wstat', (('int', None), ('int', None)) ),
    'msvcrt._wstat64':( 'int', None, 'cdecl', 'msvcrt._wstat64', (('int', None), ('int', None)) ),
    'msvcrt._wstati64':( 'int', None, 'cdecl', 'msvcrt._wstati64', (('int', None), ('int', None)) ),
    'msvcrt._wstrdate':( 'int', None, 'cdecl', 'msvcrt._wstrdate', (('int', None),) ),
    'msvcrt._wstrtime':( 'int', None, 'cdecl', 'msvcrt._wstrtime', (('int', None),) ),
    'msvcrt._wsystem':( 'int', None, 'cdecl', 'msvcrt._wsystem', (('int', None),) ),
    'msvcrt._wtempnam':( 'int', None, 'cdecl', 'msvcrt._wtempnam', (('void *', 'ptr'), ('void *', 'ptr')) ),
    'msvcrt._wtmpnam':( 'int', None, 'cdecl', 'msvcrt._wtmpnam', (('int', None),) ),
    'msvcrt._wtof':( 'int', None, 'cdecl', 'msvcrt._wtof', (('void *', 'ptr'),) ),
    'msvcrt._wtoi':( 'int', None, 'cdecl', 'msvcrt._wtoi', (('int', None),) ),
    'msvcrt._wtoi64':( 'int', None, 'cdecl', 'msvcrt._wtoi64', (('int', None),) ),
    'msvcrt._wtol':( 'int', None, 'cdecl', 'msvcrt._wtol', (('int', None),) ),
    'msvcrt._wunlink':( 'int', None, 'cdecl', 'msvcrt._wunlink', (('int', None),) ),
    'msvcrt._wutime':( 'int', None, 'cdecl', 'msvcrt._wutime', (('StringW', None), ('int', None)) ),
    'msvcrt._wutime64':( 'int', None, 'cdecl', 'msvcrt._wutime64', (('StringW', None), ('int', None)) ),
    'msvcrt._y0':( 'int', None, 'cdecl', 'msvcrt._y0', () ),
    'msvcrt._y1':( 'int', None, 'cdecl', 'msvcrt._y1', () ),
    'msvcrt._yn':( 'int', None, 'cdecl', 'msvcrt._yn', () ),
    'msvcrt.abort':( 'int', None, 'cdecl', 'msvcrt.abort', () ),
    'msvcrt.abs':( 'int', None, 'cdecl', 'msvcrt.abs', (('int', None),) ),
    'msvcrt.acos':( 'int', None, 'cdecl', 'msvcrt.acos', () ),
    'msvcrt.asctime':( 'int', None, 'cdecl', 'msvcrt.asctime', (('int', None),) ),
    'msvcrt.asin':( 'int', None, 'cdecl', 'msvcrt.asin', () ),
    'msvcrt.atan':( 'int', None, 'cdecl', 'msvcrt.atan', () ),
    'msvcrt.atan2':( 'int', None, 'cdecl', 'msvcrt.atan2', (('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt.atexit':( 'int', None, 'cdecl', 'msvcrt.atexit', (('int', None),) ),
    'msvcrt.atof':( 'int', None, 'cdecl', 'msvcrt.atof', (('void *', 'ptr'),) ),
    'msvcrt.atoi':( 'int', None, 'cdecl', 'msvcrt.atoi', (('int', None),) ),
    'msvcrt.atol':( 'int', None, 'cdecl', 'msvcrt.atol', (('void *', 'ptr'),) ),
    'msvcrt.bsearch':( 'int', None, 'cdecl', 'msvcrt.bsearch', (('int', None), ('int', None), ('int', None), ('int', None), ('void *', 'funcptr')) ),
    'msvcrt.calloc':( 'int', None, 'cdecl', 'msvcrt.calloc', (('int', None), ('int', None)) ),
    'msvcrt.ceil':( 'int', None, 'cdecl', 'msvcrt.ceil', () ),
    'msvcrt.clearerr':( 'int', None, 'cdecl', 'msvcrt.clearerr', (('int', None),) ),
    'msvcrt.clock':( 'int', None, 'msfastcall', 'msvcrt.clock', (('int', None), ('int', None)) ),
    'msvcrt.cos':( 'int', None, 'cdecl', 'msvcrt.cos', () ),
    'msvcrt.cosh':( 'int', None, 'cdecl', 'msvcrt.cosh', (('int', None), ('int', None)) ),
    'msvcrt.ctime':( 'int', None, 'cdecl', 'msvcrt.ctime', (('int', None),) ),
    'msvcrt.difftime':( 'int', None, 'cdecl', 'msvcrt.difftime', (('int', None), ('int', None)) ),
    'msvcrt.div':( 'int', None, 'cdecl', 'msvcrt.div', (('int', None), ('int', None)) ),
    'msvcrt.exit':( 'int', None, 'cdecl', 'msvcrt.exit', (('int', None),) ),
    'msvcrt.exp':( 'int', None, 'cdecl', 'msvcrt.exp', (('int', None), ('int', None)) ),
    'msvcrt.fabs':( 'int', None, 'cdecl', 'msvcrt.fabs', (('int', None), ('int', None)) ),
    'msvcrt.fclose':( 'int', None, 'cdecl', 'msvcrt.fclose', (('int', None),) ),
    'msvcrt.feof':( 'int', None, 'cdecl', 'msvcrt.feof', (('int', None),) ),
    'msvcrt.ferror':( 'int', None, 'cdecl', 'msvcrt.ferror', (('int', None),) ),
    'msvcrt.fflush':( 'int', None, 'cdecl', 'msvcrt.fflush', (('int', None),) ),
    'msvcrt.fgetc':( 'int', None, 'cdecl', 'msvcrt.fgetc', (('int', None),) ),
    'msvcrt.fgetpos':( 'int', None, 'cdecl', 'msvcrt.fgetpos', (('int', None), ('int', None)) ),
    'msvcrt.fgets':( 'int', None, 'cdecl', 'msvcrt.fgets', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt.fgetwc':( 'int', None, 'cdecl', 'msvcrt.fgetwc', (('int', None),) ),
    'msvcrt.fgetws':( 'int', None, 'cdecl', 'msvcrt.fgetws', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt.floor':( 'int', None, 'cdecl', 'msvcrt.floor', () ),
    'msvcrt.fmod':( 'int', None, 'cdecl', 'msvcrt.fmod', (('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt.fopen':( 'int', None, 'cdecl', 'msvcrt.fopen', (('StringA', None), ('int', None)) ),
    'msvcrt.fprintf':( 'int', None, 'cdecl', 'msvcrt.fprintf', (('void *', 'ptr'), ('int', None)) ),
    'msvcrt.fputc':( 'int', None, 'cdecl', 'msvcrt.fputc', (('int', None), ('int', None)) ),
    'msvcrt.fputs':( 'int', None, 'cdecl', 'msvcrt.fputs', (('void *', 'ptr'), ('int', None)) ),
    'msvcrt.fputwc':( 'int', None, 'cdecl', 'msvcrt.fputwc', (('int', None), ('int', None)) ),
    'msvcrt.fputws':( 'int', None, 'cdecl', 'msvcrt.fputws', (('int', None), ('int', None)) ),
    'msvcrt.fread':( 'int', None, 'cdecl', 'msvcrt.fread', (('StringA', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt.free':( 'void', None, 'cdecl', 'msvcrt.free', (('void *', 'ptr'),) ),
    'msvcrt.freopen':( 'int', None, 'cdecl', 'msvcrt.freopen', (('StringA', None), ('int', None), ('int', None)) ),
    'msvcrt.frexp':( 'int', None, 'cdecl', 'msvcrt.frexp', () ),
    'msvcrt.fscanf':( 'int', None, 'cdecl', 'msvcrt.fscanf', (('void *', 'ptr'), ('int', None)) ),
    'msvcrt.fseek':( 'int', None, 'cdecl', 'msvcrt.fseek', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt.fsetpos':( 'int', None, 'cdecl', 'msvcrt.fsetpos', (('int', None), ('int', None)) ),
    'msvcrt.ftell':( 'int', None, 'cdecl', 'msvcrt.ftell', (('int', None),) ),
    'msvcrt.fwprintf':( 'int', None, 'cdecl', 'msvcrt.fwprintf', (('void *', 'ptr'), ('int', None)) ),
    'msvcrt.fwrite':( 'int', None, 'cdecl', 'msvcrt.fwrite', (('void *', 'ptr'), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt.fwscanf':( 'int', None, 'cdecl', 'msvcrt.fwscanf', (('void *', 'ptr'), ('int', None)) ),
    'msvcrt.getc':( 'int', None, 'cdecl', 'msvcrt.getc', (('int', None),) ),
    'msvcrt.getchar':( 'int', None, 'cdecl', 'msvcrt.getchar', () ),
    'msvcrt.getenv':( 'int', None, 'cdecl', 'msvcrt.getenv', (('int', None),) ),
    'msvcrt.gets':( 'int', None, 'cdecl', 'msvcrt.gets', (('int', None),) ),
    'msvcrt.getwc':( 'int', None, 'cdecl', 'msvcrt.getwc', (('int', None),) ),
    'msvcrt.getwchar':( 'int', None, 'cdecl', 'msvcrt.getwchar', () ),
    'msvcrt.gmtime':( 'int', None, 'cdecl', 'msvcrt.gmtime', (('void *', 'ptr'),) ),
    'msvcrt.is_wctype':( 'int', None, 'cdecl', 'msvcrt.is_wctype', (('int', None), ('int', None)) ),
    'msvcrt.isalnum':( 'int', None, 'cdecl', 'msvcrt.isalnum', (('int', None),) ),
    'msvcrt.isalpha':( 'int', None, 'cdecl', 'msvcrt.isalpha', (('int', None),) ),
    'msvcrt.iscntrl':( 'int', None, 'cdecl', 'msvcrt.iscntrl', (('int', None),) ),
    'msvcrt.isdigit':( 'int', None, 'cdecl', 'msvcrt.isdigit', (('int', None),) ),
    'msvcrt.isgraph':( 'int', None, 'cdecl', 'msvcrt.isgraph', (('int', None),) ),
    'msvcrt.isleadbyte':( 'int', None, 'cdecl', 'msvcrt.isleadbyte', (('int', None),) ),
    'msvcrt.islower':( 'int', None, 'cdecl', 'msvcrt.islower', (('int', None),) ),
    'msvcrt.isprint':( 'int', None, 'cdecl', 'msvcrt.isprint', (('int', None),) ),
    'msvcrt.ispunct':( 'int', None, 'cdecl', 'msvcrt.ispunct', (('int', None),) ),
    'msvcrt.isspace':( 'int', None, 'cdecl', 'msvcrt.isspace', (('int', None),) ),
    'msvcrt.isupper':( 'int', None, 'cdecl', 'msvcrt.isupper', (('int', None),) ),
    'msvcrt.iswalnum':( 'int', None, 'cdecl', 'msvcrt.iswalnum', (('int', None),) ),
    'msvcrt.iswalpha':( 'int', None, 'cdecl', 'msvcrt.iswalpha', (('int', None),) ),
    'msvcrt.iswascii':( 'int', None, 'cdecl', 'msvcrt.iswascii', (('int', None),) ),
    'msvcrt.iswcntrl':( 'int', None, 'cdecl', 'msvcrt.iswcntrl', (('int', None),) ),
    'msvcrt.iswctype':( 'int', None, 'cdecl', 'msvcrt.iswctype', (('int', None), ('int', None)) ),
    'msvcrt.iswdigit':( 'int', None, 'cdecl', 'msvcrt.iswdigit', (('int', None),) ),
    'msvcrt.iswgraph':( 'int', None, 'cdecl', 'msvcrt.iswgraph', (('int', None),) ),
    'msvcrt.iswlower':( 'int', None, 'cdecl', 'msvcrt.iswlower', (('int', None),) ),
    'msvcrt.iswprint':( 'int', None, 'cdecl', 'msvcrt.iswprint', (('int', None),) ),
    'msvcrt.iswpunct':( 'int', None, 'cdecl', 'msvcrt.iswpunct', (('int', None),) ),
    'msvcrt.iswspace':( 'int', None, 'cdecl', 'msvcrt.iswspace', (('int', None),) ),
    'msvcrt.iswupper':( 'int', None, 'cdecl', 'msvcrt.iswupper', (('int', None),) ),
    'msvcrt.iswxdigit':( 'int', None, 'cdecl', 'msvcrt.iswxdigit', (('int', None),) ),
    'msvcrt.isxdigit':( 'int', None, 'cdecl', 'msvcrt.isxdigit', (('int', None),) ),
    'msvcrt.labs':( 'int', None, 'cdecl', 'msvcrt.labs', (('int', None),) ),
    'msvcrt.ldexp':( 'int', None, 'cdecl', 'msvcrt.ldexp', (('int', None), ('int', None)) ),
    'msvcrt.ldiv':( 'int', None, 'cdecl', 'msvcrt.ldiv', (('int', None), ('int', None)) ),
    'msvcrt.localeconv':( 'int', None, 'cdecl', 'msvcrt.localeconv', () ),
    'msvcrt.localtime':( 'int', None, 'cdecl', 'msvcrt.localtime', (('void *', 'ptr'),) ),
    'msvcrt.log':( 'int', None, 'cdecl', 'msvcrt.log', () ),
    'msvcrt.log10':( 'int', None, 'cdecl', 'msvcrt.log10', () ),
    'msvcrt.longjmp':( 'int', None, 'cdecl', 'msvcrt.longjmp', (('int', None), ('int', None)) ),
    'msvcrt.malloc':( 'int', None, 'cdecl', 'msvcrt.malloc', (('DWORD', None),) ),
    'msvcrt.mblen':( 'int', None, 'cdecl', 'msvcrt.mblen', (('int', None), ('int', None)) ),
    'msvcrt.mbstowcs':( 'int', None, 'cdecl', 'msvcrt.mbstowcs', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt.mbtowc':( 'int', None, 'cdecl', 'msvcrt.mbtowc', (('void *', 'ptr'), ('void *', 'ptr'), ('int', None)) ),
    'msvcrt.memchr':( 'int', None, 'cdecl', 'msvcrt.memchr', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt.memcmp':( 'int', None, 'cdecl', 'msvcrt.memcmp', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt.memcpy':( 'int', None, 'cdecl', 'msvcrt.memcpy', (('void *', 'ptr'), ('void *', 'ptr'), ('int', None)) ),
    'msvcrt.memmove':( 'int', None, 'cdecl', 'msvcrt.memmove', (('int', None), ('void *', 'ptr'), ('int', None)) ),
    'msvcrt.memset':( 'int', None, 'cdecl', 'msvcrt.memset', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt.mktime':( 'int', None, 'cdecl', 'msvcrt.mktime', (('int', None),) ),
    'msvcrt.modf':( 'int', None, 'cdecl', 'msvcrt.modf', (('int', None), ('int', None)) ),
    'msvcrt.perror':( 'int', None, 'cdecl', 'msvcrt.perror', (('void *', 'ptr'),) ),
    'msvcrt.pow':( 'int', None, 'cdecl', 'msvcrt.pow', () ),
    'msvcrt.printf':( 'int', None, 'cdecl', 'msvcrt.printf', (('int', None),) ),
    'msvcrt.putc':( 'int', None, 'cdecl', 'msvcrt.putc', (('int', None), ('int', None)) ),
    'msvcrt.putchar':( 'int', None, 'cdecl', 'msvcrt.putchar', (('int', None),) ),
    'msvcrt.puts':( 'int', None, 'cdecl', 'msvcrt.puts', (('void *', 'ptr'),) ),
    'msvcrt.putwc':( 'int', None, 'cdecl', 'msvcrt.putwc', (('int', None), ('int', None)) ),
    'msvcrt.putwchar':( 'int', None, 'cdecl', 'msvcrt.putwchar', (('int', None),) ),
    'msvcrt.qsort':( 'int', None, 'stdcall', 'msvcrt.qsort', ( ('void *', 'funcptr'), ('int', None), ('int', None), ('void *', 'funcptr'), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt.quick_exit':( 'int', None, 'cdecl', 'msvcrt.quick_exit', (('int', None),) ),
    'msvcrt.raise':( 'int', None, 'cdecl', 'msvcrt.raise', (('int', None),) ),
    'msvcrt.rand':( 'int', None, 'cdecl', 'msvcrt.rand', () ),
    'msvcrt.realloc':( 'int', None, 'cdecl', 'msvcrt.realloc', (('void *', 'ptr'), ('DWORD', None)) ),
    'msvcrt.remove':( 'int', None, 'cdecl', 'msvcrt.remove', (('int', None),) ),
    'msvcrt.rename':( 'int', None, 'cdecl', 'msvcrt.rename', (('int', None), ('int', None)) ),
    'msvcrt.rewind':( 'int', None, 'cdecl', 'msvcrt.rewind', (('int', None),) ),
    'msvcrt.scanf':( 'int', None, 'cdecl', 'msvcrt.scanf', (('int', None),) ),
    'msvcrt.setbuf':( 'int', None, 'cdecl', 'msvcrt.setbuf', (('int', None), ('int', None)) ),
    'msvcrt.setlocale':( 'int', None, 'cdecl', 'msvcrt.setlocale', ( ('void *', 'ptr'),) ),
    'msvcrt.setvbuf':( 'int', None, 'cdecl', 'msvcrt.setvbuf', (('int', None), ('int', None), ('int', None), ('DWORD', None)) ),
    'msvcrt.signal':( 'int', None, 'cdecl', 'msvcrt.signal', (('int', None), ('int', None)) ),
    'msvcrt.sin':( 'int', None, 'cdecl', 'msvcrt.sin', () ),
    'msvcrt.sinh':( 'int', None, 'cdecl', 'msvcrt.sinh', (('int', None), ('int', None)) ),
    'msvcrt.sprintf':( 'int', None, 'cdecl', 'msvcrt.sprintf', (('int', None), ('int', None)) ),
    'msvcrt.sqrt':( 'int', None, 'cdecl', 'msvcrt.sqrt', () ),
    'msvcrt.srand':( 'int', None, 'cdecl', 'msvcrt.srand', (('int', None),) ),
    'msvcrt.sscanf':( 'int', None, 'cdecl', 'msvcrt.sscanf', (('int', None), ('int', None)) ),
    'msvcrt.strcat':( 'int', None, 'cdecl', 'msvcrt.strcat', (('int', None), ('int', None)) ),
    'msvcrt.strchr':( 'int', None, 'cdecl', 'msvcrt.strchr', (('int', None), ('int', None)) ),
    'msvcrt.strcmp':( 'int', None, 'cdecl', 'msvcrt.strcmp', (('int', None), ('int', None)) ),
    'msvcrt.strcoll':( 'int', None, 'cdecl', 'msvcrt.strcoll', (('int', None), ('int', None)) ),
    'msvcrt.strcpy':( 'int', None, 'cdecl', 'msvcrt.strcpy', (('int', None), ('void *', 'ptr')) ),
    'msvcrt.strcspn':( 'int', None, 'cdecl', 'msvcrt.strcspn', (('int', None), ('int', None)) ),
    'msvcrt.strerror':( 'int', None, 'cdecl', 'msvcrt.strerror', (('int', None),) ),
    'msvcrt.strftime':( 'int', None, 'cdecl', 'msvcrt.strftime', (('int', None), ('int', None), ('int', None), ('int', None)) ),
    'msvcrt.strlen':( 'int', None, 'cdecl', 'msvcrt.strlen', (('int', None),) ),
    'msvcrt.strncat':( 'int', None, 'cdecl', 'msvcrt.strncat', (('void *', 'ptr'), ('int', None), ('int', None)) ),
    'msvcrt.strncmp':( 'int', None, 'cdecl', 'msvcrt.strncmp', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt.strncpy':( 'int', None, 'cdecl', 'msvcrt.strncpy', (('void *', 'ptr'), ('void *', 'ptr'), ('int', None)) ),
    'msvcrt.strpbrk':( 'int', None, 'cdecl', 'msvcrt.strpbrk', (('int', None), ('int', None)) ),
    'msvcrt.strrchr':( 'int', None, 'cdecl', 'msvcrt.strrchr', (('int', None), ('int', None)) ),
    'msvcrt.strspn':( 'int', None, 'cdecl', 'msvcrt.strspn', (('int', None), ('int', None)) ),
    'msvcrt.strstr':( 'int', None, 'cdecl', 'msvcrt.strstr', (('int', None), ('int', None)) ),
    'msvcrt.strtod':( 'int', None, 'cdecl', 'msvcrt.strtod', (('void *', 'ptr'), ('int', None)) ),
    'msvcrt.strtok':( 'int', None, 'cdecl', 'msvcrt.strtok', (('int', None), ('int', None)) ),
    'msvcrt.strtol':( 'int', None, 'cdecl', 'msvcrt.strtol', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt.strtoul':( 'int', None, 'cdecl', 'msvcrt.strtoul', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt.strxfrm':( 'int', None, 'cdecl', 'msvcrt.strxfrm', (('void *', 'ptr'), ('void *', 'ptr'), ('int', None)) ),
    'msvcrt.swprintf':( 'int', None, 'cdecl', 'msvcrt.swprintf', (('int', None), ('int', None)) ),
    'msvcrt.swscanf':( 'int', None, 'cdecl', 'msvcrt.swscanf', (('int', None), ('int', None)) ),
    'msvcrt.system':( 'int', None, 'cdecl', 'msvcrt.system', (('int', None),) ),
    'msvcrt.tan':( 'int', None, 'cdecl', 'msvcrt.tan', () ),
    'msvcrt.tanh':( 'int', None, 'cdecl', 'msvcrt.tanh', (('int', None), ('int', None)) ),
    'msvcrt.time':( 'int', None, 'cdecl', 'msvcrt.time', (('void *', 'ptr'),) ),
    'msvcrt.tmpfile':( 'int', None, 'cdecl', 'msvcrt.tmpfile', () ),
    'msvcrt.tmpnam':( 'int', None, 'cdecl', 'msvcrt.tmpnam', (('int', None),) ),
    'msvcrt.tolower':( 'int', None, 'cdecl', 'msvcrt.tolower', (('int', None),) ),
    'msvcrt.toupper':( 'int', None, 'cdecl', 'msvcrt.toupper', (('int', None),) ),
    'msvcrt.towlower':( 'int', None, 'cdecl', 'msvcrt.towlower', (('int', None),) ),
    'msvcrt.towupper':( 'int', None, 'cdecl', 'msvcrt.towupper', (('int', None),) ),
    'msvcrt.ungetc':( 'int', None, 'cdecl', 'msvcrt.ungetc', (('int', None), ('int', None)) ),
    'msvcrt.ungetwc':( 'int', None, 'cdecl', 'msvcrt.ungetwc', (('int', None), ('int', None)) ),
    'msvcrt.vfprintf':( 'int', None, 'cdecl', 'msvcrt.vfprintf', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt.vfwprintf':( 'int', None, 'cdecl', 'msvcrt.vfwprintf', (('void *', 'ptr'), ('int', None), ('int', None)) ),
    'msvcrt.vprintf':( 'int', None, 'cdecl', 'msvcrt.vprintf', (('int', None), ('int', None)) ),
    'msvcrt.vsprintf':( 'int', None, 'cdecl', 'msvcrt.vsprintf', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt.vswprintf':( 'int', None, 'cdecl', 'msvcrt.vswprintf', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt.vwprintf':( 'int', None, 'cdecl', 'msvcrt.vwprintf', (('int', None), ('int', None)) ),
    'msvcrt.wcscat':( 'int', None, 'cdecl', 'msvcrt.wcscat', (('int', None), ('int', None)) ),
    'msvcrt.wcschr':( 'int', None, 'cdecl', 'msvcrt.wcschr', (('int', None), ('int', None)) ),
    'msvcrt.wcscmp':( 'int', None, 'cdecl', 'msvcrt.wcscmp', (('int', None), ('int', None)) ),
    'msvcrt.wcscoll':( 'int', None, 'cdecl', 'msvcrt.wcscoll', (('int', None), ('int', None)) ),
    'msvcrt.wcscpy':( 'int', None, 'cdecl', 'msvcrt.wcscpy', (('int', None), ('void *', 'ptr')) ),
    'msvcrt.wcscspn':( 'int', None, 'cdecl', 'msvcrt.wcscspn', (('int', None), ('int', None)) ),
    'msvcrt.wcsftime':( 'int', None, 'cdecl', 'msvcrt.wcsftime', (('int', None), ('int', None), ('void *', 'ptr'), ('int', None)) ),
    'msvcrt.wcslen':( 'int', None, 'cdecl', 'msvcrt.wcslen', (('void *', 'ptr'),) ),
    'msvcrt.wcsncat':( 'int', None, 'cdecl', 'msvcrt.wcsncat', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt.wcsncmp':( 'int', None, 'cdecl', 'msvcrt.wcsncmp', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt.wcsncpy':( 'int', None, 'cdecl', 'msvcrt.wcsncpy', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt.wcspbrk':( 'int', None, 'cdecl', 'msvcrt.wcspbrk', (('int', None), ('int', None)) ),
    'msvcrt.wcsrchr':( 'int', None, 'cdecl', 'msvcrt.wcsrchr', (('int', None), ('int', None)) ),
    'msvcrt.wcsspn':( 'int', None, 'cdecl', 'msvcrt.wcsspn', (('int', None), ('int', None)) ),
    'msvcrt.wcsstr':( 'int', None, 'cdecl', 'msvcrt.wcsstr', (('int', None), ('int', None)) ),
    'msvcrt.wcstod':( 'int', None, 'cdecl', 'msvcrt.wcstod', (('int', None), ('int', None)) ),
    'msvcrt.wcstok':( 'int', None, 'cdecl', 'msvcrt.wcstok', (('int', None), ('int', None)) ),
    'msvcrt.wcstol':( 'int', None, 'cdecl', 'msvcrt.wcstol', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt.wcstombs':( 'int', None, 'cdecl', 'msvcrt.wcstombs', (('void *', 'ptr'), ('void *', 'ptr'), ('int', None)) ),
    'msvcrt.wcstoul':( 'int', None, 'cdecl', 'msvcrt.wcstoul', (('int', None), ('int', None), ('int', None)) ),
    'msvcrt.wcsxfrm':( 'int', None, 'cdecl', 'msvcrt.wcsxfrm', (('void *', 'ptr'), ('void *', 'ptr'), ('int', None)) ),
    'msvcrt.wctomb':( 'int', None, 'cdecl', 'msvcrt.wctomb', (('void *', 'ptr'), ('int', None)) ),
    'msvcrt.wprintf':( 'int', None, 'cdecl', 'msvcrt.wprintf', (('int', None),) ),
    'msvcrt.wscanf':( 'int', None, 'cdecl', 'msvcrt.wscanf', (('int', None),) ),
    }
