"""
Platform Support Modules

The vtrace platforms package is where the modules live that
implement platform specific functionality that is used behind
the scenes of the top level Trace object.  If you're not porting
vtrace to a new platform, or chasing a bug, you probably don't need
anything from here...
"""
# Copyright (C) 2007 Invisigoth - See LICENSE file for details
