
def analyze(vw):
    '''
    Analysis module which processes entries in VaSet "EntryPoints" as Functions
    This is a vital analysis module, only made into a module to allow for 
    control over when in the process it gets executed.
    '''
    vw.processEntryPoints()
