ord_names = {
    256:'??0_AFX_CHECKLIST_STATE@@QAE@XZ',
    257:'??0_AFX_COLOR_STATE@@QAE@XZ',
    258:'??0_AFX_CONTROLPOS@@QAE@XZ=_AfxBinaryCompatibleStubFunction',
    259:'??0_AFX_DAO_STATE@@QAE@XZ',
    260:'??0_AFX_EDIT_STATE@@QAE@XZ',
    261:'??0_AFX_OLE_STATE@@QAE@XZ',
    262:'??0_AFX_THREAD_STATE@@QAE@XZ',
    263:'??0_AFX_WIN_STATE@@QAE@XZ',
    264:'??0_AFXCTL_AMBIENT_CACHE@@QAE@XZ',
    265:'??0_AFXCTL_UIACTIVE_INFO@@QAE@PAUHMENU__@@PAUIOleInPlaceFrame@@@Z',
    266:'??0AFX_CLASSINIT@@QAE@PAUCRuntimeClass@@@Z=??0AFX_CLASSINIT_COMPAT@@QAE@PAUCRuntimeClass@@@Z',
    267:'??0AFX_DDPDATA@@QAE@PAXHH0IPBD@Z',
    268:'??0AFX_EXCEPTION_LINK@@QAE@XZ',
    269:'??0AFX_MODULE_STATE@@QAE@HP6GJPAUHWND__@@IIJ@ZK@Z',
    270:'??0AFX_MODULE_STATE@@QAE@HP6GJPAUHWND__@@IIJ@ZKH@Z',
    271:'??0AFX_MODULE_THREAD_STATE@@QAE@XZ',
    272:'??0CAnimateCtrl@@QAE@XZ',
    273:'??0CArchive@@QAE@PAVCFile@@IHPAX@Z',
    274:'??0CArchivePropExchange@@QAE@AAVCArchive@@@Z',
    275:'??0CArchiveStream@@QAE@PAVCArchive@@@Z',
    276:'??0CAsyncMonikerFile@@QAE@XZ',
    277:'??0CAsyncPropExchange@@QAE@K@Z',
    278:'??0CAsyncSocket@@QAE@XZ',
    279:'??0CBitmap@@QAE@XZ',
    280:'??0CBitmapButton@@QAE@XZ',
    281:'??0CBlobProperty@@QAE@PAX@Z',
    282:'??0CBrush@@QAE@HK@Z',
    283:'??0CBrush@@QAE@K@Z',
    284:'??0CBrush@@QAE@PAVCBitmap@@@Z',
    285:'??0CBrush@@QAE@XZ',
    286:'??0CButton@@QAE@XZ',
    287:'??0CByteArray@@QAE@XZ',
    288:'??0CCheckListBox@@QAE@XZ',
    289:'??0CClientDC@@QAE@PAVCWnd@@@Z',
    290:'??0CCmdTarget@@QAE@XZ',
    291:'??0CCmdUI@@QAE@XZ',
    292:'??0CColorButton@@QAE@XZ',
    293:'??0CColorDialog@@QAE@KKPAVCWnd@@@Z',
    294:'??0CColorPropPage@@QAE@XZ',
    295:'??0CComboBox@@QAE@XZ',
    296:'??0CCommandLineInfo@@QAE@XZ',
    297:'??0CConnectionPoint@@QAE@XZ',
    298:'??0CControlBar@@IAE@XZ',
    299:'??0CControlBarInfo@@QAE@XZ',
    300:'??0CControlDataSource@COleControl@@QAE@PAV1@@Z',
    301:'??0CControlFrameWnd@@QAE@PAVCOleControl@@@Z',
    302:'??0CCriticalSection@@QAE@XZ',
    303:'??0CCtrlView@@QAE@PBDK@Z',
    304:'??0CDaoDatabase@@QAE@PAVCDaoWorkspace@@@Z',
    305:'??0CDaoException@@QAE@XZ',
    306:'??0CDaoFieldExchange@@QAE@IPAVCDaoRecordset@@PAX@Z',
    307:'??0CDaoIndexFieldInfo@@QAE@XZ',
    308:'??0CDaoIndexInfo@@QAE@XZ',
    309:'??0CDaoQueryDef@@QAE@PAVCDaoDatabase@@@Z',
    310:'??0CDaoRecordset@@QAE@PAVCDaoDatabase@@@Z',
    311:'??0CDaoRecordView@@IAE@I@Z',
    312:'??0CDaoRecordView@@IAE@PBD@Z',
    313:'??0CDaoRelationFieldInfo@@QAE@XZ',
    314:'??0CDaoRelationInfo@@QAE@XZ',
    315:'??0CDaoTableDef@@QAE@PAVCDaoDatabase@@@Z',
    316:'??0CDaoWorkspace@@QAE@XZ',
    317:'??0CDatabase@@QAE@XZ',
    318:'??0CDataBoundProperty@@QAE@PAV0@JG@Z',
    319:'??0CDataExchange@@QAE@PAVCWnd@@H@Z',
    320:'??0CDataSourceControl@@QAE@PAVCOleControlSite@@@Z',
    321:'??0CDBException@@QAE@F@Z',
    322:'??0CDBVariant@@QAE@XZ',
    323:'??0CDC@@QAE@XZ',
    324:'??0CDialog@@QAE@IPAVCWnd@@@Z',
    325:'??0CDialog@@QAE@PBDPAVCWnd@@@Z',
    326:'??0CDialog@@QAE@XZ',
    327:'??0CDialogBar@@QAE@XZ',
    328:'??0CDialogTemplate@@QAE@PAX@Z',
    329:'??0CDialogTemplate@@QAE@PBUDLGTEMPLATE@@@Z',
    330:'??0CDocItem@@IAE@XZ',
    331:'??0CDockBar@@QAE@H@Z',
    332:'??0CDockContext@@QAE@PAVCControlBar@@@Z',
    333:'??0CDockState@@QAE@XZ',
    334:'??0CDocManager@@QAE@XZ',
    335:'??0CDocObjectServer@@QAE@PAVCOleServerDoc@@PAUIOleDocumentSite@@@Z',
    336:'??0CDocObjectServerItem@@IAE@PAVCOleServerDoc@@H@Z',
    337:'??0CDocTemplate@@IAE@IPAUCRuntimeClass@@00@Z',
    338:'??0CDocument@@QAE@XZ',
    339:'??0CDragListBox@@QAE@XZ',
    340:'??0CDumpContext@@QAE@PAVCFile@@@Z',
    341:'??0CDWordArray@@QAE@XZ',
    342:'??0CDynLinkLibrary@@QAE@AAUAFX_EXTENSION_MODULE@@H@Z',
    343:'??0CEdit@@QAE@XZ',
    344:'??0CEditView@@QAE@XZ',
    345:'??0CEnumArray@@QAE@IPBXIH@Z',
    346:'??0CEnumConnPoints@@QAE@PBXI@Z',
    347:'??0CEnumFormatEtc@@QAE@XZ',
    348:'??0CEvent@@QAE@HHPBDPAU_SECURITY_ATTRIBUTES@@@Z',
    349:'??0CException@@QAE@H@Z',
    350:'??0CException@@QAE@XZ',
    351:'??0CFieldExchange@@QAE@IPAVCRecordset@@PAX@Z',
    352:'??0CFile@@QAE@H@Z',
    353:'??0CFile@@QAE@PBDI@Z',
    354:'??0CFile@@QAE@XZ',
    355:'??0CFileDialog@@QAE@HPBD0K0PAVCWnd@@@Z',
    356:'??0CFileFind@@QAE@XZ',
    357:'??0CFindReplaceDialog@@QAE@XZ',
    358:'??0CFont@@QAE@XZ',
    359:'??0CFontComboBox@@QAE@XZ',
    360:'??0CFontDialog@@QAE@ABU_charformat@@KPAVCDC@@PAVCWnd@@@Z',
    361:'??0CFontDialog@@QAE@PAUtagLOGFONTA@@KPAVCDC@@PAVCWnd@@@Z',
    362:'??0CFontHolder@@QAE@PAUIPropertyNotifySink@@@Z',
    363:'??0CFontPropPage@@QAE@XZ',
    364:'??0CFormView@@IAE@I@Z',
    365:'??0CFormView@@IAE@PBD@Z',
    366:'??0CFrameWnd@@QAE@XZ',
    367:'??0CFtpConnection@@QAE@PAVCInternetSession@@PAXPBDK@Z',
    368:'??0CFtpConnection@@QAE@PAVCInternetSession@@PBD11KGH@Z',
    369:'??0CFtpFileFind@@QAE@PAVCFtpConnection@@K@Z',
    370:'??0CGdiObject@@QAE@XZ',
    371:'??0CGopherConnection@@QAE@PAVCInternetSession@@PAXPBDK@Z',
    372:'??0CGopherConnection@@QAE@PAVCInternetSession@@PBD11KG@Z',
    373:'??0CGopherFile@@IAE@PAX0PBDKK@Z',
    374:'??0CGopherFile@@IAE@PAXAAVCGopherLocator@@PAVCGopherConnection@@@Z',
    375:'??0CGopherFileFind@@QAE@PAVCGopherConnection@@K@Z',
    376:'??0CGopherLocator@@AAE@PBDK@Z',
    377:'??0CHandleMap@@QAE@PAUCRuntimeClass@@IH@Z',
    378:'??0CHeaderCtrl@@QAE@XZ',
    379:'??0CHotKeyCtrl@@QAE@XZ',
    380:'??0CHttpConnection@@QAE@PAVCInternetSession@@PAXPBDK@Z',
    381:'??0CHttpConnection@@QAE@PAVCInternetSession@@PBDG11K@Z',
    382:'??0CHttpFile@@IAE@PAX0PBD11K@Z',
    383:'??0CHttpFile@@IAE@PAXPBD1PAVCHttpConnection@@@Z',
    384:'??0CImageList@@QAE@XZ',
    385:'??0CInternetConnection@@QAE@PAVCInternetSession@@PBDGK@Z',
    386:'??0CInternetException@@QAE@K@Z',
    387:'??0CInternetFile@@IAE@PAX0PBD1KH@Z',
    388:'??0CInternetFile@@IAE@PAXPBDPAVCInternetConnection@@H@Z',
    389:'??0CInternetSession@@QAE@PBDKK00K@Z',
    390:'??0CListBox@@QAE@XZ',
    391:'??0CListCtrl@@QAE@XZ',
    392:'??0CListView@@QAE@XZ',
    393:'??0CLongBinary@@QAE@XZ',
    394:'??0CMapPtrToPtr@@QAE@H@Z',
    395:'??0CMapPtrToWord@@QAE@H@Z',
    396:'??0CMapStringToOb@@QAE@H@Z',
    397:'??0CMapStringToPtr@@QAE@H@Z',
    398:'??0CMapStringToString@@QAE@H@Z',
    399:'??0CMapWordToOb@@QAE@H@Z',
    400:'??0CMapWordToPtr@@QAE@H@Z',
    401:'??0CMDIChildWnd@@QAE@XZ',
    402:'??0CMDIFrameWnd@@QAE@XZ',
    403:'??0CMemFile@@QAE@I@Z',
    404:'??0CMemFile@@QAE@PAEII@Z',
    405:'??0CMenu@@QAE@XZ',
    406:'??0CMetaFileDC@@QAE@XZ',
    407:'??0CMiniDockFrameWnd@@QAE@XZ',
    408:'??0CMiniFrameWnd@@QAE@XZ',
    409:'??0CMirrorFile@@QAE@XZ',
    410:'??0CMonikerFile@@QAE@XZ',
    411:'??0CMultiDocTemplate@@QAE@IPAUCRuntimeClass@@00@Z',
    412:'??0CMultiLock@@QAE@QAPAVCSyncObject@@KH@Z',
    413:'??0CMutex@@QAE@HPBDPAU_SECURITY_ATTRIBUTES@@@Z',
    414:'??0CObArray@@QAE@XZ',
    415:'??0CObList@@QAE@H@Z',
    416:'??0CODBCFieldInfo@@QAE@XZ',
    417:'??0COleBusyDialog@@QAE@PAUHTASK__@@HKPAVCWnd@@@Z',
    418:'??0COleChangeIconDialog@@QAE@PAVCOleClientItem@@KPAVCWnd@@@Z',
    419:'??0COleChangeSourceDialog@@QAE@PAVCOleClientItem@@PAVCWnd@@@Z',
    420:'??0COleClientItem@@QAE@PAVCOleDocument@@@Z',
    421:'??0COleCmdUI@@QAE@PAU_tagOLECMD@@KPBU_GUID@@@Z',
    422:'??0COleCntrFrameWnd@@QAE@PAVCOleIPFrameWnd@@@Z',
    423:'??0COleControl@@QAE@XZ',
    424:'??0COleControlContainer@@QAE@PAVCWnd@@@Z',
    425:'??0COleControlLock@@QAE@ABU_GUID@@@Z',
    426:'??0COleControlSite@@QAE@PAVCOleControlContainer@@@Z',
    427:'??0COleConvertDialog@@QAE@PAVCOleClientItem@@KPAU_GUID@@PAVCWnd@@@Z',
    428:'??0COleCurrency@@QAE@JJ@Z',
    429:'??0COleDataObject@@QAE@XZ',
    430:'??0COleDataSource@@QAE@XZ',
    431:'??0COleDialog@@QAE@PAVCWnd@@@Z',
    432:'??0COleDispatchDriver@@QAE@ABV0@@Z',
    433:'??0COleDispatchDriver@@QAE@PAUIDispatch@@H@Z',
    434:'??0COleDispatchDriver@@QAE@XZ',
    435:'??0COleDispatchException@@QAE@PBDIG@Z',
    436:'??0COleDocIPFrameWnd@@QAE@XZ',
    437:'??0COleDocument@@QAE@XZ',
    438:'??0COleDropSource@@QAE@XZ',
    439:'??0COleDropTarget@@QAE@XZ',
    440:'??0COleFrameHook@@QAE@PAVCFrameWnd@@PAVCOleClientItem@@@Z',
    441:'??0COleInsertDialog@@QAE@KPAVCWnd@@@Z',
    442:'??0COleIPFrameWnd@@QAE@XZ',
    443:'??0COleLinkingDoc@@QAE@XZ',
    444:'??0COleLinksDialog@@QAE@PAVCOleDocument@@PAVCView@@KPAVCWnd@@@Z',
    445:'??0COleMessageFilter@@QAE@XZ',
    446:'??0COleObjectFactory@@QAE@ABU_GUID@@PAUCRuntimeClass@@HPBD@Z',
    447:'??0COlePasteSpecialDialog@@QAE@KPAVCOleDataObject@@PAVCWnd@@@Z',
    448:'??0COlePropertiesDialog@@QAE@PAVCOleClientItem@@IIPAVCWnd@@@Z',
    449:'??0COlePropertyPage@@QAE@II@Z',
    450:'??0COleResizeBar@@QAE@XZ',
    451:'??0COleSafeArray@@QAE@ABUtagSAFEARRAY@@G@Z',
    452:'??0COleSafeArray@@QAE@ABUtagVARIANT@@@Z',
    453:'??0COleSafeArray@@QAE@ABV0@@Z',
    454:'??0COleSafeArray@@QAE@PBUtagSAFEARRAY@@G@Z',
    455:'??0COleSafeArray@@QAE@PBUtagVARIANT@@@Z',
    456:'??0COleServerDoc@@QAE@XZ',
    457:'??0COleServerItem@@IAE@PAVCOleServerDoc@@H@Z',
    458:'??0COleStreamFile@@QAE@PAUIStream@@@Z',
    459:'??0COleTemplateServer@@QAE@XZ',
    460:'??0COleUILinkInfo@@QAE@PAVCOleDocument@@@Z',
    461:'??0COleUpdateDialog@@QAE@PAVCOleDocument@@HHPAVCWnd@@@Z',
    462:'??0COleVariant@@QAE@ABUtagVARIANT@@@Z',
    463:'??0COleVariant@@QAE@ABV0@@Z',
    464:'??0COleVariant@@QAE@FG@Z',
    465:'??0COleVariant@@QAE@JG@Z',
    466:'??0COleVariant@@QAE@PBDG@Z',
    467:'??0COleVariant@@QAE@PBUtagVARIANT@@@Z',
    468:'??0COleVariant@@QAE@XZ',
    469:'??0CPageSetupDialog@@QAE@KPAVCWnd@@@Z',
    470:'??0CPaintDC@@QAE@PAVCWnd@@@Z',
    471:'??0CPalette@@QAE@XZ',
    472:'??0CPen@@QAE@HHK@Z',
    473:'??0CPen@@QAE@HHPBUtagLOGBRUSH@@HPBK@Z',
    474:'??0CPen@@QAE@XZ',
    475:'??0CPictureHolder@@QAE@XZ',
    476:'??0CPicturePropPage@@QAE@XZ',
    477:'??0CPoint@@QAE@K@Z',
    478:'??0CPoint@@QAE@UtagPOINT@@@Z',
    479:'??0CPreviewDC@@QAE@XZ',
    480:'??0CPreviewView@@QAE@XZ',
    481:'??0CPrintDialog@@IAE@AAUtagPDA@@@Z',
    482:'??0CPrintDialog@@QAE@HKPAVCWnd@@@Z',
    483:'??0CPrintInfo@@QAE@XZ',
    484:'??0CPrintPreviewState@@QAE@XZ',
    485:'??0CProgressCtrl@@QAE@XZ',
    486:'??0CPropbagPropExchange@@QAE@PAUIPropertyBag@@PAUIErrorLog@@HH@Z',
    487:'??0CProperty@@QAE@KQAXK@Z',
    488:'??0CProperty@@QAE@XZ',
    489:'??0CPropertyPage@@QAE@II@Z',
    490:'??0CPropertyPage@@QAE@PBDI@Z',
    491:'??0CPropertyPage@@QAE@XZ',
    492:'??0CPropertySection@@QAE@U_GUID@@@Z',
    493:'??0CPropertySection@@QAE@XZ',
    494:'??0CPropertySet@@QAE@U_GUID@@@Z',
    495:'??0CPropertySet@@QAE@XZ',
    496:'??0CPropertySheet@@QAE@IPAVCWnd@@I@Z',
    497:'??0CPropertySheet@@QAE@PBDPAVCWnd@@I@Z',
    498:'??0CPropertySheet@@QAE@XZ',
    499:'??0CPropsetPropExchange@@QAE@AAVCPropertySection@@PAUIStorage@@H@Z',
    500:'??0CPtrArray@@QAE@XZ',
    501:'??0CPtrList@@QAE@H@Z',
    502:'??0CRecentFileList@@QAE@IPBD0HH@Z',
    503:'??0CRecordset@@QAE@PAVCDatabase@@@Z',
    504:'??0CRecordView@@IAE@I@Z',
    505:'??0CRecordView@@IAE@PBD@Z',
    506:'??0CRectTracker@@QAE@PBUtagRECT@@I@Z',
    507:'??0CRectTracker@@QAE@XZ',
    508:'??0CReObject@@QAE@PAVCRichEditCntrItem@@@Z',
    509:'??0CReObject@@QAE@XZ',
    510:'??0CResetPropExchange@@QAE@XZ',
    511:'??0CRgn@@QAE@XZ',
    512:'??0CRichEditCntrItem@@QAE@PAU_reobject@@PAVCRichEditDoc@@@Z',
    513:'??0CRichEditCtrl@@QAE@XZ',
    514:'??0CRichEditDoc@@IAE@XZ',
    515:'??0CRichEditView@@QAE@XZ',
    516:'??0CScrollBar@@QAE@XZ',
    517:'??0CScrollView@@IAE@XZ',
    518:'??0CSemaphore@@QAE@JJPBDPAU_SECURITY_ATTRIBUTES@@@Z',
    519:'??0CSharedFile@@QAE@II@Z',
    520:'??0CSingleDocTemplate@@QAE@IPAUCRuntimeClass@@00@Z',
    521:'??0CSingleLock@@QAE@PAVCSyncObject@@H@Z',
    522:'??0CSliderCtrl@@QAE@XZ',
    523:'??0CSocket@@QAE@XZ',
    524:'??0CSocketFile@@QAE@PAVCSocket@@H@Z',
    525:'??0CSocketWnd@@QAE@XZ',
    526:'??0CSpinButtonCtrl@@QAE@XZ',
    527:'??0CSplitterWnd@@QAE@XZ',
    528:'??0CStatic@@QAE@XZ',
    529:'??0CStatusBar@@QAE@XZ',
    530:'??0CStatusBarCtrl@@QAE@XZ',
    531:'??0CStdioFile@@QAE@PAU_iobuf@@@Z',
    532:'??0CStdioFile@@QAE@PBDI@Z',
    533:'??0CStdioFile@@QAE@XZ',
    534:'??0CStockPropPage@@QAE@II@Z',
    535:'??0CString@@QAE@ABV0@@Z',
    536:'??0CString@@QAE@DH@Z',
    537:'??0CString@@QAE@PBD@Z',
    538:'??0CString@@QAE@PBDH@Z',
    539:'??0CString@@QAE@PBG@Z',
    540:'??0CString@@QAE@XZ',
    541:'??0CStringArray@@QAE@XZ',
    542:'??0CStringList@@QAE@H@Z',
    543:'??0CSyncObject@@QAE@PBD@Z',
    544:'??0CTabCtrl@@QAE@XZ',
    545:'??0CTestCmdUI@@QAE@XZ',
    546:'??0CThreadSlotData@@QAE@XZ',
    547:'??0CTime@@QAE@ABU_FILETIME@@H@Z',
    548:'??0CTime@@QAE@ABU_SYSTEMTIME@@H@Z',
    549:'??0CTime@@QAE@ABV0@@Z',
    550:'??0CTime@@QAE@GGH@Z',
    551:'??0CTime@@QAE@HHHHHHH@Z',
    552:'??0CTime@@QAE@XZ',
    553:'??0CTimeSpan@@QAE@XZ',
    554:'??0CToolBar@@QAE@XZ',
    555:'??0CToolBarCtrl@@QAE@XZ',
    556:'??0CToolTipCtrl@@QAE@XZ',
    557:'??0CTreeCtrl@@QAE@XZ',
    558:'??0CTreeView@@QAE@XZ',
    559:'??0CUIntArray@@QAE@XZ',
    560:'??0CView@@IAE@XZ',
    561:'??0CWinApp@@QAE@PBD@Z',
    562:'??0CWindowDC@@QAE@PAVCWnd@@@Z',
    563:'??0CWindowlessDC@@QAE@PAUHDC__@@AAVCPoint@@@Z',
    564:'??0CWinThread@@QAE@P6AIPAX@Z0@Z',
    565:'??0CWinThread@@QAE@XZ',
    566:'??0CWnd@@AAE@PAUHWND__@@@Z',
    567:'??0CWnd@@QAE@XZ',
    568:'??0CWordArray@@QAE@XZ',
    569:'??0PAGE_INFO@CPreviewView@@QAE@XZ',
    570:'??0tagFONTOBJECT@@QAE@ABU0@@Z',
    571:'??1?$_CIP@UIBindCtx@@$1?IID_IBindCtx@@3U_GUID@@B@@QAE@XZ',
    572:'??1?$_CIP@UIBindHost@@$1?IID_IBindHost@@3U_GUID@@B@@QAE@XZ',
    573:'??1?$_CIP@UIBinding@@$1?IID_IBinding@@3U_GUID@@B@@QAE@XZ',
    574:'??1?$_CIP@UIMoniker@@$1?IID_IMoniker@@3U_GUID@@B@@QAE@XZ',
    575:'??1?$_CIP@UIServiceProvider@@$1?IID_IServiceProvider@@3U_GUID@@B@@QAE@XZ',
    576:'??1?$_CIP@UIStream@@$1?IID_IStream@@3U_GUID@@B@@QAE@XZ',
    577:'??1?$CIP@UIBindCtx@@$1?IID_IBindCtx@@3U_GUID@@B@@QAE@XZ',
    578:'??1?$CIP@UIBindHost@@$1?IID_IBindHost@@3U_GUID@@B@@QAE@XZ',
    579:'??1?$CIP@UIBinding@@$1?IID_IBinding@@3U_GUID@@B@@QAE@XZ',
    580:'??1?$CIP@UIMoniker@@$1?IID_IMoniker@@3U_GUID@@B@@QAE@XZ',
    581:'??1?$CIP@UIServiceProvider@@$1?IID_IServiceProvider@@3U_GUID@@B@@QAE@XZ',
    582:'??1?$CIP@UIStream@@$1?IID_IStream@@3U_GUID@@B@@QAE@XZ',
    583:'??1?$CThreadLocal@VAFX_MODULE_THREAD_STATE@@@@QAE@XZ',
    584:'??1_AFX_CHECKLIST_STATE@@UAE@XZ',
    585:'??1_AFX_CTL3D_STATE@@UAE@XZ',
    586:'??1_AFX_CTL3D_THREAD@@UAE@XZ',
    587:'??1_AFX_DAO_STATE@@UAE@XZ',
    588:'??1_AFX_DB_STATE@@UAE@XZ',
    589:'??1_AFX_EDIT_STATE@@UAE@XZ',
    590:'??1_AFX_EXTDLL_STATE@@UAE@XZ=_AfxBinaryCompatibleStubFunction',
    591:'??1_AFX_MAIL_STATE@@UAE@XZ',
    592:'??1_AFX_OLE_STATE@@UAE@XZ',
    593:'??1_AFX_RICHEDIT_STATE@@UAE@XZ',
    594:'??1_AFX_SOCK_STATE@@UAE@XZ',
    595:'??1_AFX_THREAD_STATE@@UAE@XZ',
    596:'??1_AFX_WIN_STATE@@UAE@XZ',
    597:'??1_AFXCTL_UIACTIVE_INFO@@QAE@XZ',
    598:'??1AFX_COM@@QAE@XZ=_AfxBinaryCompatibleStubFunction',
    599:'??1AFX_MAINTAIN_STATE@@QAE@XZ',
    600:'??1AFX_MODULE_STATE@@UAE@XZ',
    601:'??1AFX_MODULE_THREAD_STATE@@UAE@XZ',
    602:'??1CAnimateCtrl@@UAE@XZ',
    603:'??1CArchive@@QAE@XZ',
    604:'??1CAsyncMonikerFile@@UAE@XZ',
    605:'??1CAsyncSocket@@UAE@XZ',
    606:'??1CBitmap@@UAE@XZ',
    607:'??1CBitmapButton@@UAE@XZ',
    608:'??1CBrush@@UAE@XZ',
    609:'??1CButton@@UAE@XZ',
    610:'??1CByteArray@@UAE@XZ',
    611:'??1CCachedDataPathProperty@@UAE@XZ',
    612:'??1CCheckListBox@@UAE@XZ',
    613:'??1CClientDC@@UAE@XZ',
    614:'??1CCmdTarget@@UAE@XZ',
    615:'??1CColorButton@@UAE@XZ',
    616:'??1CComboBox@@UAE@XZ',
    617:'??1CCommandLineInfo@@UAE@XZ',
    618:'??1CCommonDialog@@UAE@XZ',
    619:'??1CConnectionPoint@@UAE@XZ',
    620:'??1CControlBar@@UAE@XZ',
    621:'??1CCriticalSection@@UAE@XZ',
    622:'??1CCtrlView@@UAE@XZ',
    623:'??1CDaoDatabase@@UAE@XZ',
    624:'??1CDaoException@@UAE@XZ',
    625:'??1CDaoFieldInfo@@QAE@XZ',
    626:'??1CDaoIndexFieldInfo@@QAE@XZ',
    627:'??1CDaoIndexInfo@@UAE@XZ',
    628:'??1CDaoQueryDef@@UAE@XZ',
    629:'??1CDaoRecordset@@UAE@XZ',
    630:'??1CDaoRecordView@@UAE@XZ',
    631:'??1CDaoRelationFieldInfo@@QAE@XZ',
    632:'??1CDaoRelationInfo@@UAE@XZ',
    633:'??1CDaoTableDef@@UAE@XZ',
    634:'??1CDaoWorkspace@@UAE@XZ',
    635:'??1CDatabase@@UAE@XZ',
    636:'??1CDataPathProperty@@UAE@XZ',
    637:'??1CDataSourceControl@@QAE@XZ',
    638:'??1CDBException@@UAE@XZ',
    639:'??1CDBVariant@@UAE@XZ',
    640:'??1CDC@@UAE@XZ',
    641:'??1CDialog@@UAE@XZ',
    642:'??1CDialogBar@@UAE@XZ',
    643:'??1CDialogTemplate@@QAE@XZ',
    644:'??1CDocItem@@UAE@XZ',
    645:'??1CDockBar@@UAE@XZ',
    646:'??1CDockContext@@QAE@XZ',
    647:'??1CDockState@@UAE@XZ',
    648:'??1CDocManager@@UAE@XZ',
    649:'??1CDocObjectServer@@UAE@XZ',
    650:'??1CDocObjectServerItem@@UAE@XZ',
    651:'??1CDocTemplate@@UAE@XZ',
    652:'??1CDocument@@UAE@XZ',
    653:'??1CDragListBox@@UAE@XZ',
    654:'??1CDWordArray@@UAE@XZ',
    655:'??1CDynLinkLibrary@@UAE@XZ',
    656:'??1CEdit@@UAE@XZ',
    657:'??1CEditView@@UAE@XZ',
    658:'??1CEnumArray@@UAE@XZ',
    659:'??1CEnumConnPoints@@UAE@XZ',
    660:'??1CEnumFormatEtc@@UAE@XZ',
    661:'??1CEnumOleVerb@@UAE@XZ',
    662:'??1CEnumUnknown@@UAE@XZ',
    663:'??1CEvent@@UAE@XZ',
    664:'??1CException@@UAE@XZ',
    665:'??1CFile@@UAE@XZ',
    666:'??1CFileDialog@@UAE@XZ',
    667:'??1CFileException@@UAE@XZ',
    668:'??1CFileFind@@UAE@XZ',
    669:'??1CFont@@UAE@XZ',
    670:'??1CFontComboBox@@UAE@XZ',
    671:'??1CFontDialog@@UAE@XZ',
    672:'??1CFontHolder@@QAE@XZ',
    673:'??1CFormView@@UAE@XZ',
    674:'??1CFrameWnd@@UAE@XZ',
    675:'??1CFtpConnection@@UAE@XZ',
    676:'??1CFtpFileFind@@UAE@XZ',
    677:'??1CGdiObject@@UAE@XZ',
    678:'??1CGopherConnection@@UAE@XZ',
    679:'??1CGopherFile@@UAE@XZ',
    680:'??1CGopherFileFind@@UAE@XZ',
    681:'??1CGopherLocator@@UAE@XZ',
    682:'??1CHeaderCtrl@@UAE@XZ',
    683:'??1CHotKeyCtrl@@UAE@XZ',
    684:'??1CHttpConnection@@UAE@XZ',
    685:'??1CHttpFile@@UAE@XZ',
    686:'??1CImageList@@UAE@XZ',
    687:'??1CInternetConnection@@UAE@XZ',
    688:'??1CInternetException@@UAE@XZ',
    689:'??1CInternetFile@@UAE@XZ',
    690:'??1CInternetSession@@UAE@XZ',
    691:'??1CItemDataSource@COleServerItem@@UAE@XZ',
    692:'??1CListBox@@UAE@XZ',
    693:'??1CListCtrl@@UAE@XZ',
    694:'??1CListView@@UAE@XZ',
    695:'??1CLongBinary@@UAE@XZ',
    696:'??1CMapPtrToPtr@@UAE@XZ',
    697:'??1CMapPtrToWord@@UAE@XZ',
    698:'??1CMapStringToOb@@UAE@XZ',
    699:'??1CMapStringToPtr@@UAE@XZ',
    700:'??1CMapStringToString@@UAE@XZ',
    701:'??1CMapWordToOb@@UAE@XZ',
    702:'??1CMapWordToPtr@@UAE@XZ',
    703:'??1CMemFile@@UAE@XZ',
    704:'??1CMenu@@UAE@XZ',
    705:'??1CMetaFileDC@@UAE@XZ',
    706:'??1CMiniFrameWnd@@UAE@XZ',
    707:'??1CMirrorFile@@UAE@XZ',
    708:'??1CMonikerFile@@UAE@XZ',
    709:'??1CMultiDocTemplate@@UAE@XZ',
    710:'??1CMultiLock@@QAE@XZ',
    711:'??1CMutex@@UAE@XZ',
    712:'??1CNoTrackObject@@UAE@XZ',
    713:'??1CObArray@@UAE@XZ',
    714:'??1CObject@@UAE@XZ',
    715:'??1CObList@@UAE@XZ',
    716:'??1CODBCFieldInfo@@QAE@XZ',
    717:'??1COleBusyDialog@@UAE@XZ',
    718:'??1COleChangeIconDialog@@UAE@XZ',
    719:'??1COleChangeSourceDialog@@UAE@XZ',
    720:'??1COleClientItem@@UAE@XZ',
    721:'??1COleCmdUI@@QAE@XZ',
    722:'??1COleCntrFrameWnd@@UAE@XZ',
    723:'??1COleControl@@UAE@XZ',
    724:'??1COleControlContainer@@UAE@XZ',
    725:'??1COleControlLock@@UAE@XZ',
    726:'??1COleControlSite@@UAE@XZ',
    727:'??1COleConvertDialog@@UAE@XZ',
    728:'??1COleDataObject@@QAE@XZ',
    729:'??1COleDataSource@@UAE@XZ',
    730:'??1COleDialog@@UAE@XZ',
    731:'??1COleDispatchDriver@@QAE@XZ',
    732:'??1COleDispatchException@@UAE@XZ',
    733:'??1COleDocIPFrameWnd@@UAE@XZ',
    734:'??1COleDocument@@UAE@XZ',
    735:'??1COleDropSource@@UAE@XZ',
    736:'??1COleDropTarget@@UAE@XZ',
    737:'??1COleFrameHook@@UAE@XZ',
    738:'??1COleInsertDialog@@UAE@XZ',
    739:'??1COleIPFrameWnd@@UAE@XZ',
    740:'??1COleLinkingDoc@@UAE@XZ',
    741:'??1COleLinksDialog@@UAE@XZ',
    742:'??1COleMessageFilter@@UAE@XZ',
    743:'??1COleObjectFactory@@UAE@XZ',
    744:'??1COlePasteSpecialDialog@@UAE@XZ',
    745:'??1COlePropertiesDialog@@UAE@XZ',
    746:'??1COlePropertyPage@@UAE@XZ',
    747:'??1COleResizeBar@@UAE@XZ',
    748:'??1COleServerDoc@@UAE@XZ',
    749:'??1COleServerItem@@UAE@XZ',
    750:'??1COleStreamFile@@UAE@XZ',
    751:'??1COleTemplateServer@@UAE@XZ',
    752:'??1COleUpdateDialog@@UAE@XZ',
    753:'??1COleVariant@@QAE@XZ',
    754:'??1CPageSetupDialog@@UAE@XZ',
    755:'??1CPaintDC@@UAE@XZ',
    756:'??1CPalette@@UAE@XZ',
    757:'??1CPen@@UAE@XZ',
    758:'??1CPictureHolder@@QAE@XZ',
    759:'??1CPicturePropPage@@UAE@XZ',
    760:'??1CPreviewDC@@UAE@XZ',
    761:'??1CPreviewView@@UAE@XZ',
    762:'??1CPrintDialog@@UAE@XZ',
    763:'??1CPrintInfo@@QAE@XZ',
    764:'??1CProcessLocalObject@@QAE@XZ',
    765:'??1CProgressCtrl@@UAE@XZ',
    766:'??1CPropbagPropExchange@@QAE@XZ',
    767:'??1CProperty@@QAE@XZ',
    768:'??1CPropertyPage@@UAE@XZ',
    769:'??1CPropertySection@@QAE@XZ',
    770:'??1CPropertySet@@QAE@XZ',
    771:'??1CPropertySheet@@UAE@XZ',
    772:'??1CPtrArray@@UAE@XZ',
    773:'??1CPtrList@@UAE@XZ',
    774:'??1CRecentFileList@@UAE@XZ',
    775:'??1CRecordset@@UAE@XZ',
    776:'??1CRecordView@@UAE@XZ',
    777:'??1CRectTracker@@UAE@XZ',
    778:'??1CReObject@@QAE@XZ',
    779:'??1CRgn@@UAE@XZ',
    780:'??1CRichEditCntrItem@@UAE@XZ',
    781:'??1CRichEditCtrl@@UAE@XZ',
    782:'??1CRichEditView@@UAE@XZ',
    783:'??1CScrollBar@@UAE@XZ',
    784:'??1CScrollView@@UAE@XZ',
    785:'??1CSemaphore@@UAE@XZ',
    786:'??1CSharedFile@@UAE@XZ',
    787:'??1CSimpleException@@UAE@XZ',
    788:'??1CSingleDocTemplate@@UAE@XZ',
    789:'??1CSizeComboBox@@UAE@XZ',
    790:'??1CSliderCtrl@@UAE@XZ',
    791:'??1CSocket@@UAE@XZ',
    792:'??1CSocketFile@@UAE@XZ',
    793:'??1CSpinButtonCtrl@@UAE@XZ',
    794:'??1CSplitterWnd@@UAE@XZ',
    795:'??1CStatic@@UAE@XZ',
    796:'??1CStatusBar@@UAE@XZ',
    797:'??1CStatusBarCtrl@@UAE@XZ',
    798:'??1CStdioFile@@UAE@XZ',
    799:'??1CStockPropPage@@UAE@XZ',
    800:'??1CString@@QAE@XZ',
    801:'??1CStringArray@@UAE@XZ',
    802:'??1CStringList@@UAE@XZ',
    803:'??1CSyncObject@@UAE@XZ',
    804:'??1CTabCtrl@@UAE@XZ',
    805:'??1CThreadLocalObject@@QAE@XZ',
    806:'??1CThreadSlotData@@QAE@XZ',
    807:'??1CToolBar@@UAE@XZ',
    808:'??1CToolBarCtrl@@UAE@XZ',
    809:'??1CToolTipCtrl@@UAE@XZ',
    810:'??1CTreeCtrl@@UAE@XZ',
    811:'??1CTreeView@@UAE@XZ',
    812:'??1CUIntArray@@UAE@XZ',
    813:'??1CView@@UAE@XZ',
    814:'??1CWaitCursor@@QAE@XZ',
    815:'??1CWinApp@@UAE@XZ',
    816:'??1CWindowDC@@UAE@XZ',
    817:'??1CWinThread@@UAE@XZ',
    818:'??1CWnd@@UAE@XZ',
    819:'??1CWordArray@@UAE@XZ',
    820:'??1tagFONTOBJECT@@QAE@XZ',
    821:'??1XEventConnPt@COleControl@@UAE@XZ',
    822:'??1XPropConnPt@COleControl@@UAE@XZ',
    823:'??2@YAPAXI@Z',
    824:'??2CNoTrackObject@@SGPAXI@Z',
    825:'??3@YAXPAX@Z',
    826:'??3CNoTrackObject@@SGXPAX@Z',
    827:'??3CObject@@SGXPAX@Z',
    828:'??4?$_CIP@UIBindHost@@$1?IID_IBindHost@@3U_GUID@@B@@QAEAAV0@PAUIBindHost@@@Z',
    829:'??4COleCurrency@@QAEABV0@ABUtagVARIANT@@@Z',
    830:'??4COleCurrency@@QAEABV0@ABV0@@Z',
    831:'??4COleCurrency@@QAEABV0@TtagCY@@@Z',
    832:'??4COleDateTime@@QAEABV0@ABJ@Z',
    833:'??4COleDateTime@@QAEABV0@ABU_FILETIME@@@Z',
    834:'??4COleDateTime@@QAEABV0@ABU_SYSTEMTIME@@@Z',
    835:'??4COleDateTime@@QAEABV0@ABUtagVARIANT@@@Z',
    836:'??4COleDateTime@@QAEABV0@N@Z',
    837:'??4COleDateTimeSpan@@QAEABV0@ABV0@@Z',
    838:'??4COleDateTimeSpan@@QAEABV0@N@Z',
    839:'??4COleDispatchDriver@@QAEABV0@ABV0@@Z',
    840:'??4COleSafeArray@@QAEAAV0@ABUtagVARIANT@@@Z',
    841:'??4COleSafeArray@@QAEAAV0@ABV0@@Z',
    842:'??4COleSafeArray@@QAEAAV0@ABVCOleVariant@@@Z',
    843:'??4COleSafeArray@@QAEAAV0@PBUtagVARIANT@@@Z',
    844:'??4COleVariant@@QAEABV0@ABUtagVARIANT@@@Z',
    845:'??4COleVariant@@QAEABV0@ABV0@@Z',
    846:'??4COleVariant@@QAEABV0@ABVCByteArray@@@Z',
    847:'??4COleVariant@@QAEABV0@ABVCLongBinary@@@Z',
    848:'??4COleVariant@@QAEABV0@ABVCOleCurrency@@@Z',
    849:'??4COleVariant@@QAEABV0@ABVCOleDateTime@@@Z',
    850:'??4COleVariant@@QAEABV0@ABVCString@@@Z',
    851:'??4COleVariant@@QAEABV0@E@Z',
    852:'??4COleVariant@@QAEABV0@F@Z',
    853:'??4COleVariant@@QAEABV0@J@Z',
    854:'??4COleVariant@@QAEABV0@M@Z',
    855:'??4COleVariant@@QAEABV0@N@Z',
    856:'??4COleVariant@@QAEABV0@PBUtagVARIANT@@@Z',
    857:'??4COleVariant@@QAEABV0@QBD@Z',
    858:'??4CString@@QAEABV0@ABV0@@Z',
    859:'??4CString@@QAEABV0@D@Z',
    860:'??4CString@@QAEABV0@PBD@Z',
    861:'??4CString@@QAEABV0@PBG@Z',
    862:'??5@YGAAVCArchive@@AAV0@AAPAVCByteArray@@@Z',
    863:'??5@YGAAVCArchive@@AAV0@AAPAVCDocItem@@@Z',
    864:'??5@YGAAVCArchive@@AAV0@AAPAVCDockState@@@Z',
    865:'??5@YGAAVCArchive@@AAV0@AAPAVCDWordArray@@@Z',
    866:'??5@YGAAVCArchive@@AAV0@AAPAVCMapStringToOb@@@Z',
    867:'??5@YGAAVCArchive@@AAV0@AAPAVCMapStringToString@@@Z',
    868:'??5@YGAAVCArchive@@AAV0@AAPAVCMapWordToOb@@@Z',
    869:'??5@YGAAVCArchive@@AAV0@AAPAVCObArray@@@Z',
    870:'??5@YGAAVCArchive@@AAV0@AAPAVCObList@@@Z',
    871:'??5@YGAAVCArchive@@AAV0@AAPAVCRichEditCntrItem@@@Z',
    872:'??5@YGAAVCArchive@@AAV0@AAPAVCStringArray@@@Z',
    873:'??5@YGAAVCArchive@@AAV0@AAPAVCStringList@@@Z',
    874:'??5@YGAAVCArchive@@AAV0@AAPAVCWordArray@@@Z',
    875:'??5@YGAAVCArchive@@AAV0@AAVCOleCurrency@@@Z',
    876:'??5@YGAAVCArchive@@AAV0@AAVCOleDateTime@@@Z',
    877:'??5@YGAAVCArchive@@AAV0@AAVCOleDateTimeSpan@@@Z',
    878:'??5@YGAAVCArchive@@AAV0@AAVCOleVariant@@@Z',
    879:'??5@YGAAVCArchive@@AAV0@AAVCString@@@Z',
    880:'??5@YGAAVCArchive@@AAV0@AAVCTime@@@Z',
    881:'??5@YGAAVCArchive@@AAV0@AAVCTimeSpan@@@Z',
    882:'??6@YGAAVCArchive@@AAV0@ABVCString@@@Z',
    883:'??6@YGAAVCArchive@@AAV0@VCOleCurrency@@@Z',
    884:'??6@YGAAVCArchive@@AAV0@VCOleDateTime@@@Z',
    885:'??6@YGAAVCArchive@@AAV0@VCOleDateTimeSpan@@@Z',
    886:'??6@YGAAVCArchive@@AAV0@VCOleVariant@@@Z',
    887:'??6@YGAAVCArchive@@AAV0@VCTime@@@Z',
    888:'??6@YGAAVCArchive@@AAV0@VCTimeSpan@@@Z',
    889:'??6CDumpContext@@QAEAAV0@ABVCObject@@@Z',
    890:'??6CDumpContext@@QAEAAV0@E@Z',
    891:'??6CDumpContext@@QAEAAV0@G@Z',
    892:'??6CDumpContext@@QAEAAV0@H@Z',
    893:'??6CDumpContext@@QAEAAV0@I@Z',
    894:'??6CDumpContext@@QAEAAV0@J@Z',
    895:'??6CDumpContext@@QAEAAV0@K@Z',
    896:'??6CDumpContext@@QAEAAV0@PBD@Z',
    897:'??6CDumpContext@@QAEAAV0@PBG@Z',
    898:'??6CDumpContext@@QAEAAV0@PBVCObject@@@Z',
    899:'??6CDumpContext@@QAEAAV0@PBX@Z',
    900:'??8COleSafeArray@@QBEHABUtagSAFEARRAY@@@Z',
    901:'??8COleSafeArray@@QBEHABUtagVARIANT@@@Z',
    902:'??8COleSafeArray@@QBEHABV0@@Z',
    903:'??8COleSafeArray@@QBEHABVCOleVariant@@@Z',
    904:'??8COleSafeArray@@QBEHPBUtagSAFEARRAY@@@Z',
    905:'??8COleSafeArray@@QBEHPBUtagVARIANT@@@Z',
    906:'??8COleVariant@@QBEHABUtagVARIANT@@@Z',
    907:'??8CTime@@QBEHV0@@Z',
    908:'??9CTime@@QBEHV0@@Z',
    909:'??ACMapPtrToPtr@@QAEAAPAXPAX@Z',
    910:'??ACMapPtrToWord@@QAEAAGPAX@Z',
    911:'??ACMapStringToOb@@QAEAAPAVCObject@@PBD@Z',
    912:'??ACMapStringToPtr@@QAEAAPAXPBD@Z',
    913:'??ACMapStringToString@@QAEAAVCString@@PBD@Z',
    914:'??ACMapWordToOb@@QAEAAPAVCObject@@G@Z',
    915:'??ACMapWordToPtr@@QAEAAPAXG@Z',
    916:'??DCOleCurrency@@QBE?AV0@J@Z',
    917:'??GCOleCurrency@@QBE?AV0@ABV0@@Z',
    918:'??GCOleCurrency@@QBE?AV0@XZ',
    919:'??GCOleDateTime@@QBE?AV0@ABVCOleDateTimeSpan@@@Z',
    920:'??GCOleDateTime@@QBE?AVCOleDateTimeSpan@@ABV0@@Z',
    921:'??GCOleDateTimeSpan@@QBE?AV0@ABV0@@Z',
    922:'??H@YG?AVCString@@ABV0@0@Z',
    923:'??H@YG?AVCString@@ABV0@D@Z',
    924:'??H@YG?AVCString@@ABV0@PBD@Z',
    925:'??H@YG?AVCString@@DABV0@@Z',
    926:'??H@YG?AVCString@@PBDABV0@@Z',
    927:'??HCOleCurrency@@QBE?AV0@ABV0@@Z',
    928:'??HCOleDateTime@@QBE?AV0@ABVCOleDateTimeSpan@@@Z',
    929:'??HCOleDateTimeSpan@@QBE?AV0@ABV0@@Z',
    930:'??KCOleCurrency@@QBE?AV0@J@Z',
    931:'??MCOleCurrency@@QBEHABV0@@Z',
    932:'??MCOleDateTime@@QBEHABV0@@Z',
    933:'??NCOleCurrency@@QBEHABV0@@Z',
    934:'??NCOleDateTime@@QBEHABV0@@Z',
    935:'??OCOleCurrency@@QBEHABV0@@Z',
    936:'??OCOleDateTime@@QBEHABV0@@Z',
    937:'??PCOleCurrency@@QBEHABV0@@Z',
    938:'??PCOleDateTime@@QBEHABV0@@Z',
    939:'??YCString@@QAEABV0@ABV0@@Z',
    940:'??YCString@@QAEABV0@D@Z',
    941:'??YCString@@QAEABV0@PBD@Z',
    942:'?_AfxAbortProc@@YGHPAUHDC__@@H@Z',
    943:'?_afxAdvApi@@3UAFX_ADVAPI_CALL@@A=_AfxBinaryCompatibleStubFunction',
    944:'?_afxAmbientCache@@3V?$CThreadLocal@V_AFXCTL_AMBIENT_CACHE@@@@A',
    945:'?_afxComDlg@@3UAFX_COMDLG_CALL@@A=_AfxBinaryCompatibleStubFunction',
    946:'?_afxCommCtrl@@3UAFX_COMMCTRL_CALL@@A=_AfxBinaryCompatibleStubFunction',
    947:'?_afxDBCS@@3HB',
    948:'?_afxODBC@@3UAFX_ODBC_CALL@@A=_AfxBinaryCompatibleStubFunction',
    949:'?_afxOLE@@3UAFX_OLE_CALL@@A=_AfxBinaryCompatibleStubFunction',
    950:'?_afxRichEditState@@3V?$CProcessLocal@V_AFX_RICHEDIT_STATE@@@@A',
    951:'?_afxShell@@3UAFX_SHELL_CALL@@A=_AfxBinaryCompatibleStubFunction',
    952:'?_afxSOCK@@3UAFX_SOCK_CALL@@A=_AfxBinaryCompatibleStubFunction',
    953:'?_afxUrlMon@@3UAFX_URLMON_CALL@@A=_AfxBinaryCompatibleStubFunction',
    954:'?_afxWinSpool@@3UAFX_WINSPOOL_CALL@@A=_AfxBinaryCompatibleStubFunction',
    955:'?_mbstowcsz@@YAHPAGPBDI@Z',
    956:'?_wcstombsz@@YAHPADPBGI@Z',
    957:'?Abort@CArchive@@QAEXXZ',
    958:'?Abort@CFile@@UAEXXZ',
    959:'?Abort@CInternetFile@@UAEXXZ',
    960:'?Abort@CMemFile@@UAEXXZ',
    961:'?Abort@CMirrorFile@@UAEXXZ',
    962:'?Abort@COleStreamFile@@UAEXXZ',
    963:'?Abort@CSocketFile@@UAEXXZ',
    964:'?Abort@CStdioFile@@UAEXXZ',
    965:'?AboutToDo@XNotifyDBEvents@COleControlSite@@UAGJKKQAUtagDBNOTIFYREASON@@@Z',
    966:'?Accept@CAsyncSocket@@UAEHAAV1@PAUsockaddr@@PAH@Z',
    967:'?Accept@CSocket@@UAEHAAVCAsyncSocket@@PAUsockaddr@@PAH@Z',
    968:'?AccessData@COleSafeArray@@QAEXPAPAX@Z',
    969:'?Activate@COleClientItem@@QAEXJPAVCView@@PAUtagMSG@@@Z',
    970:'?Activate@XPropertyPage@COlePropertyPage@@UAGJPAUHWND__@@PBUtagRECT@@H@Z',
    971:'?ActivateAs@COleClientItem@@UAEHPBDABU_GUID@@1@Z',
    972:'?ActivateAs@CRichEditCntrItem@@UAEHPBDABU_GUID@@1@Z',
    973:'?ActivateDocObject@CDocObjectServer@@QAEXXZ',
    974:'?ActivateDocObject@COleServerDoc@@QAEXXZ',
    975:'?ActivateFrame@CFrameWnd@@UAEXH@Z',
    976:'?ActivateFrame@CMDIChildWnd@@UAEXH@Z',
    977:'?ActivateInPlace@COleServerDoc@@QAEHXZ',
    978:'?ActivateNext@CSplitterWnd@@UAEXH@Z',
    979:'?ActivateTopParent@CWnd@@QAEXXZ',
    980:'?Add@CRecentFileList@@UAEXPBD@Z',
    981:'?AddBitmap@CToolBarCtrl@@QAEHHI@Z',
    982:'?AddBitmap@CToolBarCtrl@@QAEHHPAVCBitmap@@@Z',
    983:'?AddCachedData@COleClientItem@@IAEXPAVCOleDataSource@@@Z',
    984:'?AddConnPoint@CEnumConnPoints@@QAEXPAUIConnectionPoint@@@Z',
    985:'?AddDocTemplate@CDocManager@@UAEXPAVCDocTemplate@@@Z',
    986:'?AddDocTemplate@CWinApp@@QAEXPAVCDocTemplate@@@Z',
    987:'?AddDocument@CDocTemplate@@UAEXPAVCDocument@@@Z',
    988:'?AddDocument@CMultiDocTemplate@@UAEXPAVCDocument@@@Z',
    989:'?AddDocument@CSingleDocTemplate@@UAEXPAVCDocument@@@Z',
    990:'?AddFont@CFontComboBox@@QAEHPAUtagLOGFONTA@@K@Z',
    991:'?AddFormat@CEnumFormatEtc@@QAEXPBUtagFORMATETC@@@Z',
    992:'?AddFormat@COlePasteSpecialDialog@@QAEXABUtagFORMATETC@@PAD1K@Z',
    993:'?AddFormat@COlePasteSpecialDialog@@QAEXIKIHH@Z',
    994:'?AddFrameLevelUI@COleControl@@MAEXXZ',
    995:'?AddFrameWnd@CFrameWnd@@IAEXXZ',
    996:'?AddHead@CObList@@QAEPAU__POSITION@@PAVCObject@@@Z',
    997:'?AddHead@CObList@@QAEXPAV1@@Z',
    998:'?AddHead@CPtrList@@QAEPAU__POSITION@@PAX@Z',
    999:'?AddHead@CPtrList@@QAEXPAV1@@Z',
    1000:'?AddHead@CSimpleList@@QAEXPAX@Z',
    1001:'?AddHead@CStringList@@QAEPAU__POSITION@@PBD@Z',
    1002:'?AddHead@CStringList@@QAEXPAV1@@Z',
    1003:'?AddItem@COleDocument@@UAEXPAVCDocItem@@@Z',
    1004:'?AddLinkEntry@COlePasteSpecialDialog@@QAE?AW4tagOLEUIPASTEFLAG@@I@Z',
    1005:'?AddNew@CDaoRecordset@@UAEXXZ',
    1006:'?AddNew@CRecordset@@UAEXXZ',
    1007:'?AddOtherClipboardData@COleServerItem@@QAEXPAVCOleDataSource@@@Z',
    1008:'?AddPage@CPropertySheet@@QAEXPAVCPropertyPage@@@Z',
    1009:'?AddProperty@CPropertySection@@QAEXPAVCProperty@@@Z',
    1010:'?AddProperty@CPropertySet@@QAEXU_GUID@@PAVCProperty@@@Z',
    1011:'?AddRef@CArchiveStream@@UAGKXZ',
    1012:'?AddRef@CBlobProperty@@UAGKXZ',
    1013:'?AddRef@CInnerUnknown@@UAGKXZ',
    1014:'?AddRef@COleConnPtContainer@@UAGKXZ',
    1015:'?AddRef@COleDispatchImpl@@UAGKXZ',
    1016:'?AddRef@COleUILinkInfo@@UAGKXZ',
    1017:'?AddRef@XAdviseSink@COleClientItem@@UAGKXZ',
    1018:'?AddRef@XAmbientProps@COleControlSite@@UAGKXZ',
    1019:'?AddRef@XBoundObjectSite@COleControlSite@@UAGKXZ',
    1020:'?AddRef@XClassFactory@COleObjectFactory@@UAGKXZ',
    1021:'?AddRef@XConnPt@CConnectionPoint@@UAGKXZ',
    1022:'?AddRef@XDataObject@COleControl@@UAGKXZ',
    1023:'?AddRef@XDataObject@COleDataSource@@UAGKXZ',
    1024:'?AddRef@XDataObject@COleServerDoc@@UAGKXZ',
    1025:'?AddRef@XDataObject@COleServerItem@@UAGKXZ',
    1026:'?AddRef@XDropSource@COleDropSource@@UAGKXZ',
    1027:'?AddRef@XDropTarget@COleDropTarget@@UAGKXZ',
    1028:'?AddRef@XEnumVOID@CEnumArray@@UAGKXZ',
    1029:'?AddRef@XEventSink@COleControlSite@@UAGKXZ',
    1030:'?AddRef@XFontNotification@COleControl@@UAGKXZ',
    1031:'?AddRef@XMessageFilter@COleMessageFilter@@UAGKXZ',
    1032:'?AddRef@XNotifyDBEvents@COleControlSite@@UAGKXZ',
    1033:'?AddRef@XOleCache@COleControl@@UAGKXZ',
    1034:'?AddRef@XOleClientSite@COleClientItem@@UAGKXZ',
    1035:'?AddRef@XOleClientSite@COleControlSite@@UAGKXZ',
    1036:'?AddRef@XOleCommandTarget@CDocObjectServer@@UAGKXZ',
    1037:'?AddRef@XOleContainer@COleControlContainer@@UAGKXZ',
    1038:'?AddRef@XOleControl@COleControl@@UAGKXZ',
    1039:'?AddRef@XOleControlSite@COleControlSite@@UAGKXZ',
    1040:'?AddRef@XOleDocument@CDocObjectServer@@UAGKXZ',
    1041:'?AddRef@XOleDocumentView@CDocObjectServer@@UAGKXZ',
    1042:'?AddRef@XOleInPlaceActiveObject@COleControl@@UAGKXZ',
    1043:'?AddRef@XOleInPlaceActiveObject@COleServerDoc@@UAGKXZ',
    1044:'?AddRef@XOleInPlaceFrame@COleFrameHook@@UAGKXZ',
    1045:'?AddRef@XOleInPlaceObject@COleControl@@UAGKXZ',
    1046:'?AddRef@XOleInPlaceObject@COleServerDoc@@UAGKXZ',
    1047:'?AddRef@XOleIPFrame@COleControlContainer@@UAGKXZ',
    1048:'?AddRef@XOleIPSite@COleClientItem@@UAGKXZ',
    1049:'?AddRef@XOleIPSite@COleControlSite@@UAGKXZ',
    1050:'?AddRef@XOleItemContainer@COleLinkingDoc@@UAGKXZ',
    1051:'?AddRef@XOleObject@CDocObjectServer@@UAGKXZ',
    1052:'?AddRef@XOleObject@COleControl@@UAGKXZ',
    1053:'?AddRef@XOleObject@COleServerDoc@@UAGKXZ',
    1054:'?AddRef@XOleObject@COleServerItem@@UAGKXZ',
    1055:'?AddRef@XOleUIObjInfo@COlePropertiesDialog@@UAGKXZ',
    1056:'?AddRef@XPerPropertyBrowsing@COleControl@@UAGKXZ',
    1057:'?AddRef@XPersistFile@COleLinkingDoc@@UAGKXZ',
    1058:'?AddRef@XPersistMemory@COleControl@@UAGKXZ',
    1059:'?AddRef@XPersistPropertyBag@COleControl@@UAGKXZ',
    1060:'?AddRef@XPersistStorage@COleControl@@UAGKXZ',
    1061:'?AddRef@XPersistStorage@COleServerDoc@@UAGKXZ',
    1062:'?AddRef@XPersistStreamInit@COleControl@@UAGKXZ',
    1063:'?AddRef@XPointerInactive@COleControl@@UAGKXZ',
    1064:'?AddRef@XPrint@CDocObjectServer@@UAGKXZ',
    1065:'?AddRef@XPropertyNotifySink@COleControlSite@@UAGKXZ',
    1066:'?AddRef@XPropertyPage@COlePropertyPage@@UAGKXZ',
    1067:'?AddRef@XPropNotifySink@COlePropertyPage@@UAGKXZ',
    1068:'?AddRef@XProvideClassInfo@COleControl@@UAGKXZ',
    1069:'?AddRef@XQuickActivate@COleControl@@UAGKXZ',
    1070:'?AddRef@XRichEditOleCallback@CRichEditView@@UAGKXZ',
    1071:'?AddRef@XSpecifyPropertyPages@COleControl@@UAGKXZ',
    1072:'?AddRef@XViewObject@COleControl@@UAGKXZ',
    1073:'?AddReplaceBitmap@CToolBar@@QAEHPAUHBITMAP__@@@Z',
    1074:'?AddRequestHeaders@CHttpFile@@QAEHAAVCString@@K@Z',
    1075:'?AddRequestHeaders@CHttpFile@@QAEHPBDKH@Z',
    1076:'?AddSection@CPropertySet@@QAEPAVCPropertySection@@U_GUID@@@Z',
    1077:'?AddSection@CPropertySet@@QAEXPAVCPropertySection@@@Z',
    1078:'?AddSize@CSizeComboBox@@QAEHHJ@Z',
    1079:'?AddStandardFormats@COlePasteSpecialDialog@@QAEXH@Z',
    1080:'?AddString@CToolBarCtrl@@QAEHI@Z',
    1081:'?AddTail@CObList@@QAEPAU__POSITION@@PAVCObject@@@Z',
    1082:'?AddTail@CObList@@QAEXPAV1@@Z',
    1083:'?AddTail@CPtrList@@QAEPAU__POSITION@@PAX@Z',
    1084:'?AddTail@CPtrList@@QAEXPAV1@@Z',
    1085:'?AddTail@CStringList@@QAEPAU__POSITION@@PBD@Z',
    1086:'?AddTail@CStringList@@QAEXPAV1@@Z',
    1087:'?AddTool@CToolTipCtrl@@QAEHPAVCWnd@@IPBUtagRECT@@I@Z',
    1088:'?AddTool@CToolTipCtrl@@QAEHPAVCWnd@@PBDPBUtagRECT@@I@Z',
    1089:'?AddToRecentFileList@CWinApp@@UAEXPBD@Z',
    1090:'?AddView@CDocument@@QAEXPAVCView@@@Z',
    1091:'?AdjustCP@CMetaFileDC@@IAEXH@Z',
    1092:'?AdjustDialogPosition@CRichEditView@@QAEXPAVCDialog@@@Z',
    1093:'?AdjustRect@CRectTracker@@UAEXHPAUtagRECT@@@Z',
    1094:'?Advise@XConnPt@CConnectionPoint@@UAGJPAUIUnknown@@PAK@Z',
    1095:'?Advise@XOleObject@CDocObjectServer@@UAGJPAUIAdviseSink@@PAK@Z',
    1096:'?Advise@XOleObject@COleControl@@UAGJPAUIAdviseSink@@PAK@Z',
    1097:'?Advise@XOleObject@COleServerDoc@@UAGJPAUIAdviseSink@@PAK@Z',
    1098:'?Advise@XOleObject@COleServerItem@@UAGJPAUIAdviseSink@@PAK@Z',
    1099:'?AfxA2WHelper@@YGPAGPAGPBDH@Z',
    1100:'?AfxAbort@@YGXXZ',
    1101:'?AfxAllocTaskAnsiString@@YGPADPBD@Z',
    1102:'?AfxAllocTaskAnsiString@@YGPADPBG@Z',
    1103:'?AfxAllocTaskWideString@@YGPAGPBD@Z',
    1104:'?AfxAllocTaskWideString@@YGPAGPBG@Z',
    1105:'?AfxBeginThread@@YGPAVCWinThread@@P6AIPAX@Z0HIKPAU_SECURITY_ATTRIBUTES@@@Z',
    1106:'?AfxBeginThread@@YGPAVCWinThread@@PAUCRuntimeClass@@HIKPAU_SECURITY_ATTRIBUTES@@@Z',
    1107:'?AfxBSTR2ABSTR@@YGPAGPAG@Z',
    1108:'?AfxBSTR2CString@@YGXPAVCString@@PAG@Z',
    1109:'?AfxCallWndProc@@YGJPAVCWnd@@PAUHWND__@@IIJ@Z',
    1110:'?AfxCancelModes@@YGXPAUHWND__@@@Z',
    1111:'?AfxComparePath@@YGHPBD0@Z',
    1112:'?AfxCompareValueByRef@@YGHPAX0H@Z',
    1113:'?AfxConnectionAdvise@@YGHPAUIUnknown@@ABU_GUID@@0HPAK@Z',
    1114:'?AfxConnectionUnadvise@@YGHPAUIUnknown@@ABU_GUID@@0HK@Z',
    1115:'?AfxCopyValueByRef@@YGXPAX0PAJH@Z',
    1116:'?AfxCoreInitModule@@YGXXZ',
    1117:'?AfxCreateDC@@YGPAUHDC__@@PAX0@Z',
    1118:'?AfxCriticalInit@@YGHXZ',
    1119:'?AfxCriticalNewHandler@@YAHI@Z',
    1120:'?AfxCriticalTerm@@YGXXZ',
    1121:'?AfxCustomLogFont@@YGHIPAUtagLOGFONTA@@@Z',
    1122:'?AfxDaoCheck@@YGXJPBD0HHH@Z',
    1123:'?AfxDaoGetEngine@@YGPAU_DAODBEngine@@XZ',
    1124:'?AfxDaoInit@@YGXXZ',
    1125:'?AfxDaoTerm@@YGXXZ',
    1126:'?afxData@@3UAUX_DATA@@A',
    1127:'?AfxDeleteObject@@YGXPAPAX@Z',
    1128:'?AfxDevModeA2W@@YGPAU_devicemodeW@@PAU1@PAU_devicemodeA@@@Z',
    1129:'?AfxDevModeW2A@@YGPAU_devicemodeA@@PAU1@PAU_devicemodeW@@@Z',
    1130:'?AfxDlgProc@@YGHPAUHWND__@@IIJ@Z',
    1131:'?AfxDllCanUnloadNow@@YGJXZ',
    1132:'?AfxDllGetClassObject@@YGJABU_GUID@@0PAPAX@Z',
    1133:'?AfxDynamicDownCast@@YAPAVCObject@@PAUCRuntimeClass@@PAV1@@Z',
    1134:'?AfxEnableControlContainer@@YAXPAVCOccManager@@@Z',
    1135:'?AfxEnableWin31Compatibility@@YGXXZ',
    1136:'?AfxEnableWin40Compatibility@@YGXXZ',
    1137:'?AfxEndDeferRegisterClass@@YGHJ@Z',
    1138:'?AfxEndThread@@YGXIH@Z',
    1139:'?AfxEnumMetaFileProc@@YGHPAUHDC__@@PAUtagHANDLETABLE@@PAUtagMETARECORD@@HJ@Z',
    1140:'?AfxExtractSubString@@YGHAAVCString@@PBDHD@Z',
    1141:'?AfxFailMaxChars@@YGXPAVCDataExchange@@H@Z',
    1142:'?AfxFailRadio@@YGXPAVCDataExchange@@@Z',
    1143:'?AfxFieldText@@YGHPAVCDataExchange@@HPAXPAVCDaoRecordset@@@Z',
    1144:'?AfxFieldText@@YGHPAVCDataExchange@@HPAXPAVCRecordset@@@Z',
    1145:'?AfxFindMessageEntry@@YGPBUAFX_MSGMAP_ENTRY@@PBU1@III@Z',
    1146:'?AfxFindResourceHandle@@YGPAUHINSTANCE__@@PBD0@Z',
    1147:'?AfxFormatString1@@YGXAAVCString@@IPBD@Z',
    1148:'?AfxFormatString2@@YGXAAVCString@@IPBD1@Z',
    1149:'?AfxFormatStrings@@YGXAAVCString@@IPBQBDH@Z',
    1150:'?AfxFormatStrings@@YGXAAVCString@@PBDPBQBDH@Z',
    1151:'?AfxFreeLibrary@@YGHPAUHINSTANCE__@@@Z',
    1152:'?AfxFullPath@@YGHPADPBD@Z',
    1153:'?AfxGetAfxWndProc@@YGP6GJPAUHWND__@@IIJ@ZXZ',
    1154:'?AfxGetAppModuleState@@YGPAVAFX_MODULE_STATE@@XZ',
    1155:'?AfxGetClassIDFromString@@YGJPBDPAU_GUID@@@Z',
    1156:'?AfxGetCtl3dState@@YGPAV_AFX_CTL3D_STATE@@XZ',
    1157:'?AfxGetDaoState@@YAPAV_AFX_DAO_STATE@@XZ',
    1158:'?AfxGetEmptyString@@YGABVCString@@XZ',
    1159:'?AfxGetFieldInfo@@YAXPAU_DAOField@@AAUCDaoFieldInfo@@K@Z',
    1160:'?AfxGetFileName@@YGIPBDPADI@Z',
    1161:'?AfxGetFileTitle@@YGIPBDPADI@Z',
    1162:'?AfxGetIndexFieldInfo@@YAXPAU_DAOIndex@@AAUCDaoIndexInfo@@@Z',
    1163:'?AfxGetIndexFields@@YAXPAU_DAOIndex@@PAPAUDAOIndexFields@@@Z',
    1164:'?AfxGetIndexInfo@@YAXPAU_DAOIndex@@AAUCDaoIndexInfo@@K@Z',
    1165:'?AfxGetInProcServer@@YGHPBDAAVCString@@@Z',
    1166:'?AfxGetInternetHandleType@@YGKPAX@Z',
    1167:'?AfxGetModuleShortFileName@@YGXPAUHINSTANCE__@@AAVCString@@@Z',
    1168:'?AfxGetModuleState@@YGPAVAFX_MODULE_STATE@@XZ',
    1169:'?AfxGetModuleThreadState@@YGPAVAFX_MODULE_THREAD_STATE@@XZ',
    1170:'?AfxGetNewHandler@@YGP6AHI@ZXZ',
    1171:'?AfxGetParentOwner@@YGPAUHWND__@@PAU1@@Z',
    1172:'?AfxGetPropSheetFont@@YGHAAVCString@@AAGH@Z',
    1173:'?AfxGetRichEditState@@YAPAV_AFX_RICHEDIT_STATE@@XZ',
    1174:'?AfxGetRoot@@YGXPBDAAVCString@@@Z',
    1175:'?AfxGetThread@@YGPAVCWinThread@@XZ',
    1176:'?AfxGetThreadState@@YGPAV_AFX_THREAD_STATE@@XZ',
    1177:'?AfxGetTypeLibCache@@YGPAVCTypeLibCache@@PBU_GUID@@@Z',
    1178:'?AfxGlobalFree@@YGXPAX@Z',
    1179:'?AfxHelpEnabled@@YGHXZ',
    1180:'?AfxHookWindowCreate@@YGXPAVCWnd@@@Z',
    1181:'?AfxINetLoad@@YGXPAP6GHXZPBD@Z=_AfxBinaryCompatibleStubFunction',
    1182:'?AfxInitExtensionModule@@YGHAAUAFX_EXTENSION_MODULE@@PAUHINSTANCE__@@@Z',
    1183:'?AfxInitLocalData@@YGXPAUHINSTANCE__@@@Z',
    1184:'?AfxInitThread@@YGXXZ',
    1185:'?AfxInternetStatusCallback@@YGXPAXKK0K@Z',
    1186:'?AfxIsDescendant@@YGHPAUHWND__@@0@Z',
    1187:'?AfxIsValidAddress@@YGHPBXIH@Z',
    1188:'?AfxIsValidString@@YGHPBDH@Z',
    1189:'?AfxIsValidString@@YGHPBGH@Z',
    1190:'?AfxLoadDll@@YGPAUHINSTANCE__@@RAPAU1@PBD@Z=_AfxBinaryCompatibleStubFunction',
    1191:'?AfxLoadDll@@YGPAUHINSTANCE__@@RAPAU1@PBDPAP6GHXZ1@Z=_AfxBinaryCompatibleStubFunction',
    1192:'?AfxLoadField@@YGXAAVCRecordset@@IPAXPAJ@Z',
    1193:'?AfxLoadLibrary@@YGPAUHINSTANCE__@@PBD@Z',
    1194:'?AfxLoadString@@YGHIPADI@Z',
    1195:'?AfxLoadSysColorBitmap@@YGPAUHBITMAP__@@PAUHINSTANCE__@@PAUHRSRC__@@H@Z',
    1196:'?AfxLockGlobals@@YGXH@Z',
    1197:'?AfxLockTempMaps@@YGXXZ',
    1198:'?AfxMergeMenus@@YGPAUHMENU__@@PAU1@0PAJHH@Z',
    1199:'?AfxMessageBox@@YGHIII@Z',
    1200:'?AfxMessageBox@@YGHPBDII@Z',
    1201:'?AfxNewHandler@@YAHI@Z',
    1202:'?AfxOleCanExitApp@@YGHXZ',
    1203:'?AfxOleGetUserCtrl@@YGHXZ',
    1204:'?AfxOleHookProc@@YGIPAUHWND__@@IIJ@Z',
    1205:'?AfxOleInit@@YGHXZ',
    1206:'?AfxOleLockApp@@YGXXZ',
    1207:'?AfxOleLockControl@@YGHABU_GUID@@@Z',
    1208:'?AfxOleLockControl@@YGHPBD@Z',
    1209:'?AfxOleOnReleaseAllObjects@@YGXXZ',
    1210:'?AfxOleRegisterControlClass@@YGHPAUHINSTANCE__@@ABU_GUID@@PBDIIHK1GG@Z',
    1211:'?AfxOleRegisterHelper@@YGHPBQBD0HHPAUHKEY__@@@Z',
    1212:'?AfxOleRegisterPropertyPageClass@@YGHPAUHINSTANCE__@@ABU_GUID@@I@Z',
    1213:'?AfxOleRegisterPropertyPageClass@@YGHPAUHINSTANCE__@@ABU_GUID@@IH@Z',
    1214:'?AfxOleRegisterServerClass@@YGHABU_GUID@@PBD11W4OLE_APPTYPE@@PAPBD3H1@Z',
    1215:'?AfxOleRegisterServerClassCompat@@YGHABU_GUID@@PBD11W4OLE_APPTYPE@@PAPBD3@Z',
    1216:'?AfxOleRegisterTypeLib@@YGHPAUHINSTANCE__@@ABU_GUID@@PBD2@Z',
    1217:'?AfxOleSetEditMenu@@YGXPAVCOleClientItem@@PAVCMenu@@IIII@Z',
    1218:'?AfxOleSetUserCtrl@@YGXH@Z',
    1219:'?AfxOleTerm@@YGXH@Z',
    1220:'?AfxOleTermOrFreeLib@@YGXHH@Z',
    1221:'?AfxOleTypeMatchGuid@@YGHPAUITypeInfo@@PAUtagTYPEDESC@@ABU_GUID@@K@Z',
    1222:'?AfxOleUnlockAllControls@@YGXXZ',
    1223:'?AfxOleUnlockApp@@YGXXZ',
    1224:'?AfxOleUnlockControl@@YGHABU_GUID@@@Z',
    1225:'?AfxOleUnlockControl@@YGHPBD@Z',
    1226:'?AfxOleUnregisterClass@@YGHABU_GUID@@PBD@Z',
    1227:'?AfxOleUnregisterTypeLib@@YGHABU_GUID@@GGK@Z',
    1228:'?AfxParseURL@@YGHPBDAAKAAVCString@@2AAG@Z',
    1229:'?AfxPostQuitMessage@@YGXH@Z',
    1230:'?AfxPropPageCallback@@YGIPAUHWND__@@IPAU_PROPSHEETPAGEA@@@Z',
    1231:'?AfxPropSheetCallback@@YGHPAUHWND__@@IJ@Z',
    1232:'?AfxRegisterClass@@YGHPAUtagWNDCLASSA@@@Z',
    1233:'?AfxRegisterWndClass@@YGPBDIPAUHICON__@@PAUHBRUSH__@@0@Z',
    1234:'?AfxRepositionWindow@@YGXPAUAFX_SIZEPARENTPARAMS@@PAUHWND__@@PBUtagRECT@@@Z',
    1235:'?AfxResetMsgCache@@YGXXZ',
    1236:'?AfxResolveShortcut@@YGHPAVCWnd@@PBDPADH@Z',
    1237:'?AfxRFXBulkDefault@@YGXPAVCFieldExchange@@PBDPAXPAJHI@Z',
    1238:'?AfxSafeArrayInit@@YGXPAVCOleSafeArray@@@Z',
    1239:'?AfxSetCurrentRecord@@YGXPAJJF@Z',
    1240:'?AfxSetFieldInfo@@YAXPAU_DAOField@@AAUCDaoFieldInfo@@@Z',
    1241:'?AfxSetIndexFieldInfo@@YAXPAU_DAOIndex@@AAUCDaoIndexInfo@@@Z',
    1242:'?AfxSetIndexInfo@@YAXPAU_DAOIndex@@AAUCDaoIndexInfo@@@Z',
    1243:'?AfxSetModuleState@@YGPAVAFX_MODULE_STATE@@PAV1@@Z',
    1244:'?AfxSetNewHandler@@YGP6AHI@ZP6AHI@Z@Z',
    1245:'?AfxSetRecordCount@@YGXPAJJJHF@Z',
    1246:'?AfxSetWindowText@@YGXPAUHWND__@@PBD@Z',
    1247:'?AfxSocketInit@@YGHPAUWSAData@@@Z',
    1248:'?AfxSocketTerm@@YGXXZ',
    1249:'?AfxStoreField@@YGXAAVCRecordset@@IPAX@Z',
    1250:'?AfxStringFromCLSID@@YG?AVCString@@ABU_GUID@@@Z',
    1251:'?AfxTaskStringA2W@@YGPAGPBD@Z',
    1252:'?AfxTaskStringW2A@@YGPADPBG@Z',
    1253:'?AfxTermExtensionModule@@YGXAAUAFX_EXTENSION_MODULE@@H@Z',
    1254:'?AfxTermLocalData@@YGXPAUHINSTANCE__@@H@Z',
    1255:'?AfxTermThread@@YGXPAUHINSTANCE__@@@Z',
    1256:'?AfxTextFloatFormat@@YGXPAVCDataExchange@@HPAXNH@Z',
    1257:'?AfxTextMetricA2W@@YGPAUtagTEXTMETRICW@@PAU1@PAUtagTEXTMETRICA@@@Z',
    1258:'?AfxTextMetricW2A@@YGPAUtagTEXTMETRICA@@PAU1@PAUtagTEXTMETRICW@@@Z',
    1259:'?AfxThrowArchiveException@@YGXHPBD@Z',
    1260:'?AfxThrowDaoException@@YGXHJ@Z',
    1261:'?AfxThrowDBException@@YGXFPAVCDatabase@@PAX@Z',
    1262:'?AfxThrowFileException@@YGXHJPBD@Z',
    1263:'?AfxThrowInternetException@@YGXKK@Z',
    1264:'?AfxThrowLastCleanup@@YGXXZ',
    1265:'?AfxThrowMemoryException@@YGXXZ',
    1266:'?AfxThrowNotSupportedException@@YGXXZ',
    1267:'?AfxThrowOleDispatchException@@YGXGII@Z',
    1268:'?AfxThrowOleDispatchException@@YGXGPBDI@Z',
    1269:'?AfxThrowOleException@@YGXJ@Z',
    1270:'?AfxThrowResourceException@@YGXXZ',
    1271:'?AfxThrowUserException@@YGXXZ',
    1272:'?AfxThunkaccept@@YGIIPAUsockaddr@@PAH@Z',
    1273:'?AfxThunkbind@@YGHIPBUsockaddr@@H@Z',
    1274:'?AfxThunkChooseColor@@YGHPAUtagCHOOSECOLORA@@@Z',
    1275:'?AfxThunkChooseFont@@YGHPAUtagCHOOSEFONTA@@@Z',
    1276:'?AfxThunkClosePrinter@@YGHPAX@Z',
    1277:'?AfxThunkclosesocket@@YGHI@Z',
    1278:'?AfxThunkCLSIDFromProgID@@YGJPBGPAU_GUID@@@Z',
    1279:'?AfxThunkCLSIDFromString@@YGJPAGPAU_GUID@@@Z',
    1280:'?AfxThunkCoCreateInstance@@YGJABU_GUID@@PAPAUIUnknown@@K0PAPAX@Z',
    1281:'?AfxThunkCoDisconnectObject@@YGJPAUIUnknown@@K@Z',
    1282:'?AfxThunkCoFreeUnusedLibraries@@YGXXZ',
    1283:'?AfxThunkCoGetClassObject@@YGJABU_GUID@@KPAX0PAPAX@Z',
    1284:'?AfxThunkCoGetMalloc@@YGJKPAPAUIMalloc@@@Z',
    1285:'?AfxThunkCoInitialize@@YGJPAX@Z',
    1286:'?AfxThunkCoLockObjectExternal@@YGJPAUIUnknown@@HH@Z',
    1287:'?AfxThunkCommDlgExtendedError@@YGKXZ',
    1288:'?AfxThunkconnect@@YGHIPBUsockaddr@@H@Z',
    1289:'?AfxThunkCoRegisterClassObject@@YGJABU_GUID@@PAUIUnknown@@KKPAK@Z',
    1290:'?AfxThunkCoRegisterMessageFilter@@YGJPAUIMessageFilter@@PAPAU1@@Z',
    1291:'?AfxThunkCoRevokeClassObject@@YGJK@Z',
    1292:'?AfxThunkCoTaskMemAlloc@@YGPAXK@Z',
    1293:'?AfxThunkCoTaskMemFree@@YGXPAX@Z',
    1294:'?AfxThunkCoTreatAsClass@@YGJABU_GUID@@0@Z',
    1295:'?AfxThunkCoUninitialize@@YGXXZ',
    1296:'?AfxThunkCreateAsyncBindCtx@@YGJKPAUIBindStatusCallback@@PAUIEnumFORMATETC@@PAPAUIBindCtx@@@Z',
    1297:'?AfxThunkCreateBindCtx@@YGJKPAPAUIBindCtx@@@Z',
    1298:'?AfxThunkCreateDataAdviseHolder@@YGJPAPAUIDataAdviseHolder@@@Z',
    1299:'?AfxThunkCreateDataCache@@YGJPAUIUnknown@@ABU_GUID@@1PAPAX@Z',
    1300:'?AfxThunkCreateFileMoniker@@YGJPBGPAPAUIMoniker@@@Z',
    1301:'?AfxThunkCreateGenericComposite@@YGJPAUIMoniker@@0PAPAU1@@Z',
    1302:'?AfxThunkCreateILockBytesOnHGlobal@@YGJPAXHPAPAUILockBytes@@@Z',
    1303:'?AfxThunkCreateItemMoniker@@YGJPBG0PAPAUIMoniker@@@Z',
    1304:'?AfxThunkCreateMappedBitmap@@YGPAUHBITMAP__@@PAUHINSTANCE__@@HIPAU_COLORMAP@@H@Z',
    1305:'?AfxThunkCreateOleAdviseHolder@@YGJPAPAUIOleAdviseHolder@@@Z',
    1306:'?AfxThunkCreatePropertySheetPage@@YGPAU_PSP@@PBU_PROPSHEETPAGEA@@@Z',
    1307:'?AfxThunkCreateStreamOnHGlobal@@YGJPAXHPAPAUIStream@@@Z',
    1308:'?AfxThunkCreateURLMoniker@@YGJPAUIMoniker@@PBGPAPAU1@@Z',
    1309:'?AfxThunkDestroyPropertySheetPage@@YGHPAU_PSP@@@Z',
    1310:'?AfxThunkDocumentProperties@@YGJPAUHWND__@@PAXPADPAU_devicemodeA@@3K@Z',
    1311:'?AfxThunkDoDragDrop@@YGJPAUIDataObject@@PAUIDropSource@@KPAK@Z',
    1312:'?AfxThunkDosDateTimeToVariantTime@@YGHGGPAN@Z',
    1313:'?AfxThunkDragAcceptFiles@@YGXPAUHWND__@@H@Z',
    1314:'?AfxThunkDragFinish@@YGXPAUHDROP__@@@Z',
    1315:'?AfxThunkDragQueryFile@@YGIPAUHDROP__@@IPADI@Z',
    1316:'?AfxThunkExtractIcon@@YGPAUHICON__@@PAUHINSTANCE__@@PBDI@Z',
    1317:'?AfxThunkFindText@@YGPAUHWND__@@PAUtagFINDREPLACEA@@@Z',
    1318:'?AfxThunkGetFileSecurity@@YGHPBDKPAXKPAK@Z',
    1319:'?AfxThunkGetFileTitle@@YGFPBDPADG@Z',
    1320:'?AfxThunkGetHGlobalFromILockBytes@@YGJPAUILockBytes@@PAPAX@Z',
    1321:'?AfxThunkgethostbyname@@YGPAUhostent@@PBD@Z',
    1322:'?AfxThunkGetOpenFileName@@YGHPAUtagOFNA@@@Z',
    1323:'?AfxThunkgetpeername@@YGHIPAUsockaddr@@PAH@Z',
    1324:'?AfxThunkGetRunningObjectTable@@YGJKPAPAUIRunningObjectTable@@@Z',
    1325:'?AfxThunkGetSaveFileName@@YGHPAUtagOFNA@@@Z',
    1326:'?AfxThunkgetsockname@@YGHIPAUsockaddr@@PAH@Z',
    1327:'?AfxThunkgetsockopt@@YGHIHHPADPAH@Z',
    1328:'?AfxThunkhtonl@@YGKK@Z',
    1329:'?AfxThunkhtons@@YGGG@Z',
    1330:'?AfxThunkImageList_Add@@YGHPAU_IMAGELIST@@PAUHBITMAP__@@1@Z',
    1331:'?AfxThunkImageList_AddMasked@@YGHPAU_IMAGELIST@@PAUHBITMAP__@@K@Z',
    1332:'?AfxThunkImageList_BeginDrag@@YGHPAU_IMAGELIST@@HHH@Z',
    1333:'?AfxThunkImageList_Create@@YGPAU_IMAGELIST@@HHIHH@Z',
    1334:'?AfxThunkImageList_Destroy@@YGHPAU_IMAGELIST@@@Z',
    1335:'?AfxThunkImageList_DragEnter@@YGHPAUHWND__@@HH@Z',
    1336:'?AfxThunkImageList_DragLeave@@YGHPAUHWND__@@@Z',
    1337:'?AfxThunkImageList_DragMove@@YGHHH@Z',
    1338:'?AfxThunkImageList_DragShowNolock@@YGHH@Z',
    1339:'?AfxThunkImageList_Draw@@YGHPAU_IMAGELIST@@HPAUHDC__@@HHI@Z',
    1340:'?AfxThunkImageList_EndDrag@@YGXXZ',
    1341:'?AfxThunkImageList_GetBkColor@@YGKPAU_IMAGELIST@@@Z',
    1342:'?AfxThunkImageList_GetDragImage@@YGPAU_IMAGELIST@@PAUtagPOINT@@0@Z',
    1343:'?AfxThunkImageList_GetIcon@@YGPAUHICON__@@PAU_IMAGELIST@@HI@Z',
    1344:'?AfxThunkImageList_GetImageCount@@YGHPAU_IMAGELIST@@@Z',
    1345:'?AfxThunkImageList_GetImageInfo@@YGHPAU_IMAGELIST@@HPAU_IMAGEINFO@@@Z',
    1346:'?AfxThunkImageList_LoadImage@@YGPAU_IMAGELIST@@PAUHINSTANCE__@@PBDHHKII@Z',
    1347:'?AfxThunkImageList_Merge@@YGPAU_IMAGELIST@@PAU1@H0HHH@Z',
    1348:'?AfxThunkImageList_Read@@YGPAU_IMAGELIST@@PAUIStream@@@Z',
    1349:'?AfxThunkImageList_Remove@@YGHPAU_IMAGELIST@@H@Z',
    1350:'?AfxThunkImageList_Replace@@YGHPAU_IMAGELIST@@HPAUHBITMAP__@@1@Z',
    1351:'?AfxThunkImageList_ReplaceIcon@@YGHPAU_IMAGELIST@@HPAUHICON__@@@Z',
    1352:'?AfxThunkImageList_SetBkColor@@YGKPAU_IMAGELIST@@K@Z',
    1353:'?AfxThunkImageList_SetDragCursorImage@@YGHPAU_IMAGELIST@@HHH@Z',
    1354:'?AfxThunkImageList_SetOverlayImage@@YGHPAU_IMAGELIST@@HH@Z',
    1355:'?AfxThunkImageList_Write@@YGHPAU_IMAGELIST@@PAUIStream@@@Z',
    1356:'?AfxThunkinet_addr@@YGKPBD@Z',
    1357:'?AfxThunkinet_ntoa@@YGPADUin_addr@@@Z',
    1358:'?AfxThunkINetFtpCommand@@YGHPAXHKPBDK@Z',
    1359:'?AfxThunkINetFtpCreateDirectory@@YGHPAXPBD@Z',
    1360:'?AfxThunkINetFtpDeleteFile@@YGHPAXPBD@Z',
    1361:'?AfxThunkINetFtpFindFirstFile@@YGPAXPAXPBDPAU_WIN32_FIND_DATAA@@KK@Z',
    1362:'?AfxThunkINetFtpGetCurrentDirectory@@YGHPAXPBDPAK@Z',
    1363:'?AfxThunkINetFtpGetFile@@YGHPAXPBD1HKKK@Z',
    1364:'?AfxThunkINetFtpOpenFile@@YGPAXPAXPBDKKK@Z',
    1365:'?AfxThunkINetFtpPutFile@@YGHPAXPBD1KK@Z',
    1366:'?AfxThunkINetFtpRemoveDirectory@@YGHPAXPBD@Z',
    1367:'?AfxThunkINetFtpRenameFile@@YGHPAXPBD1@Z',
    1368:'?AfxThunkINetFtpSetCurrentDirectory@@YGHPAXPBD@Z',
    1369:'?AfxThunkINetGopherCreateLocator@@YGHPBDG00KPADPAK@Z',
    1370:'?AfxThunkINetGopherFindFirstFile@@YGPAXPAXPBD1PAUGOPHER_FIND_DATAA@@KK@Z',
    1371:'?AfxThunkINetGopherGetAttribute@@YGHPAXPBD1PAEKPAKP6GHPAUGOPHER_ATTRIBUTE_TYPE@@K@ZK@Z',
    1372:'?AfxThunkINetGopherGetLocatorType@@YGHPBDPAK@Z',
    1373:'?AfxThunkINetGopherOpenFile@@YGPAXPAXPBD1KK@Z',
    1374:'?AfxThunkINetHttpAddRequestHeaders@@YGHPAXPBDKK@Z',
    1375:'?AfxThunkINetHttpOpenRequest@@YGPAXPAXPBD111PAPBDKK@Z',
    1376:'?AfxThunkINetHttpQueryInfo@@YGHPAXK0PAK1@Z',
    1377:'?AfxThunkINetHttpSendRequest@@YGHPAXPBDK0K@Z',
    1378:'?AfxThunkINetInternetCloseHandle@@YGPAXPAX@Z',
    1379:'?AfxThunkINetInternetConnect@@YGPAXPAXPBDG11KKK@Z',
    1380:'?AfxThunkINetInternetErrorDlg@@YGKPAUHWND__@@PAXKKPAPAX@Z',
    1381:'?AfxThunkINetInternetFindNextFile@@YGHPAX0@Z',
    1382:'?AfxThunkINetInternetGetLastResponseInfo@@YGHPAKPAD0@Z',
    1383:'?AfxThunkINetInternetOpen@@YGPAXPBDK00K@Z',
    1384:'?AfxThunkINetInternetOpenUrl@@YGPAXPAXPBD1KKK@Z',
    1385:'?AfxThunkINetInternetQueryDataAvailable@@YGHPAXPAKKK@Z',
    1386:'?AfxThunkINetInternetQueryOption@@YGHPAXK0PAK@Z',
    1387:'?AfxThunkINetInternetReadFile@@YGPAXPAX0KPAK@Z',
    1388:'?AfxThunkINetInternetSetFilePointer@@YGKPAXJ0KK@Z',
    1389:'?AfxThunkINetInternetSetOption@@YGHPAXK0K@Z',
    1390:'?AfxThunkINetInternetSetOptionEx@@YGHPAXK0KK@Z',
    1391:'?AfxThunkINetInternetSetStatusCallback@@YGP6GXPAXKK0K@Z0P6GX0KK0K@Z@Z',
    1392:'?AfxThunkINetInternetWriteFile@@YGPAXPAXPBXKPAK@Z',
    1393:'?AfxThunkInitCommonControls@@YGXXZ',
    1394:'?AfxThunkioctlsocket@@YGHIJPAK@Z',
    1395:'?AfxThunkIsAccelerator@@YGHPAUHACCEL__@@HPAUtagMSG@@PAG@Z',
    1396:'?AfxThunkIsAsyncMoniker@@YGJPAUIMoniker@@@Z',
    1397:'?AfxThunkLBItemFromPt@@YGHPAUHWND__@@UtagPOINT@@H@Z',
    1398:'?AfxThunklisten@@YGHIH@Z',
    1399:'?AfxThunkLoadRegTypeLib@@YGJABU_GUID@@GGKPAPAUITypeLib@@@Z',
    1400:'?AfxThunkLoadTypeLib@@YGJPBGPAPAUITypeLib@@@Z',
    1401:'?AfxThunkMakeDragList@@YGHPAUHWND__@@@Z',
    1402:'?AfxThunkntohs@@YGGG@Z',
    1403:'?AfxThunkOleCreate@@YGJABU_GUID@@0KPAUtagFORMATETC@@PAUIOleClientSite@@PAUIStorage@@PAPAX@Z',
    1404:'?AfxThunkOleCreateDefaultHandler@@YGJABU_GUID@@PAUIUnknown@@0PAPAX@Z',
    1405:'?AfxThunkOleCreateFromData@@YGJPAUIDataObject@@ABU_GUID@@KPAUtagFORMATETC@@PAUIOleClientSite@@PAUIStorage@@PAPAX@Z',
    1406:'?AfxThunkOleCreateFromFile@@YGJABU_GUID@@PBG0KPAUtagFORMATETC@@PAUIOleClientSite@@PAUIStorage@@PAPAX@Z',
    1407:'?AfxThunkOleCreateLinkFromData@@YGJPAUIDataObject@@ABU_GUID@@KPAUtagFORMATETC@@PAUIOleClientSite@@PAUIStorage@@PAPAX@Z',
    1408:'?AfxThunkOleCreateLinkToFile@@YGJPBGABU_GUID@@KPAUtagFORMATETC@@PAUIOleClientSite@@PAUIStorage@@PAPAX@Z',
    1409:'?AfxThunkOleCreateMenuDescriptor@@YGPAXPAUHMENU__@@PAUtagOleMenuGroupWidths@@@Z',
    1410:'?AfxThunkOleCreateStaticFromData@@YGJPAUIDataObject@@ABU_GUID@@KPAUtagFORMATETC@@PAUIOleClientSite@@PAUIStorage@@PAPAX@Z',
    1411:'?AfxThunkOleDestroyMenuDescriptor@@YGJPAX@Z',
    1412:'?AfxThunkOleDuplicateData@@YGPAXPAXGI@Z',
    1413:'?AfxThunkOleFlushClipboard@@YGJXZ',
    1414:'?AfxThunkOleGetAutoConvert@@YGJABU_GUID@@PAU1@@Z',
    1415:'?AfxThunkOleGetClipboard@@YGJPAPAUIDataObject@@@Z',
    1416:'?AfxThunkOleGetIconOfClass@@YGPAXABU_GUID@@PAGH@Z',
    1417:'?AfxThunkOleInitialize@@YGJPAX@Z',
    1418:'?AfxThunkOleIsCurrentClipboard@@YGJPAUIDataObject@@@Z',
    1419:'?AfxThunkOleIsRunning@@YGHPAUIOleObject@@@Z',
    1420:'?AfxThunkOleLoad@@YGJPAUIStorage@@ABU_GUID@@PAUIOleClientSite@@PAPAX@Z',
    1421:'?AfxThunkOleLoadFromStream@@YGJPAUIStream@@ABU_GUID@@PAPAX@Z',
    1422:'?AfxThunkOleLockRunning@@YGJPAUIUnknown@@HH@Z',
    1423:'?AfxThunkOleQueryCreateFromData@@YGJPAUIDataObject@@@Z',
    1424:'?AfxThunkOleQueryLinkFromData@@YGJPAUIDataObject@@@Z',
    1425:'?AfxThunkOleRegEnumVerbs@@YGJABU_GUID@@PAPAUIEnumOLEVERB@@@Z',
    1426:'?AfxThunkOleRegGetMiscStatus@@YGJABU_GUID@@KPAK@Z',
    1427:'?AfxThunkOleRegGetUserType@@YGJABU_GUID@@KPAPAG@Z',
    1428:'?AfxThunkOleRun@@YGJPAUIUnknown@@@Z',
    1429:'?AfxThunkOleSave@@YGJPAUIPersistStorage@@PAUIStorage@@H@Z',
    1430:'?AfxThunkOleSaveToStream@@YGJPAUIPersistStream@@PAUIStream@@@Z',
    1431:'?AfxThunkOleSetClipboard@@YGJPAUIDataObject@@@Z',
    1432:'?AfxThunkOleSetContainedObject@@YGJPAUIUnknown@@H@Z',
    1433:'?AfxThunkOleSetMenuDescriptor@@YGJPAXPAUHWND__@@1PAUIOleInPlaceFrame@@PAUIOleInPlaceActiveObject@@@Z',
    1434:'?AfxThunkOleTranslateAccelerator@@YGJPAUIOleInPlaceFrame@@PAUtagOIFI@@PAUtagMSG@@@Z',
    1435:'?AfxThunkOleUIAddVerbMenu@@YGHPAUIOleObject@@PBDPAUHMENU__@@IIIHIPAPAU2@@Z',
    1436:'?AfxThunkOleUIBusy@@YGIPAUtagOLEUIBUSYA@@@Z',
    1437:'?AfxThunkOleUIChangeIcon@@YGIPAUtagOLEUICHANGEICONA@@@Z',
    1438:'?AfxThunkOleUIChangeSource@@YGIPAUtagOLEUICHANGESOURCEA@@@Z',
    1439:'?AfxThunkOleUIConvert@@YGIPAUtagOLEUICONVERTA@@@Z',
    1440:'?AfxThunkOleUIEditLinks@@YGIPAUtagOLEUIEDITLINKSA@@@Z',
    1441:'?AfxThunkOleUIInsertObject@@YGIPAUtagOLEUIINSERTOBJECTA@@@Z',
    1442:'?AfxThunkOleUIObjectProperties@@YGIPAUtagOLEUIOBJECTPROPSA@@@Z',
    1443:'?AfxThunkOleUIPasteSpecial@@YGIPAUtagOLEUIPASTESPECIALA@@@Z',
    1444:'?AfxThunkOleUIUpdateLinks@@YGHPAUIOleUILinkContainerA@@PAUHWND__@@PADH@Z',
    1445:'?AfxThunkOleUninitialize@@YGXXZ',
    1446:'?AfxThunkOpenPrinter@@YGHPADPAPAXPAU_PRINTER_DEFAULTSA@@@Z',
    1447:'?AfxThunkPageSetupDlg@@YGHPAUtagPSDA@@@Z',
    1448:'?AfxThunkPrintDlg@@YGHPAUtagPDA@@@Z',
    1449:'?AfxThunkPropertySheet@@YGHPBU_PROPSHEETHEADERA@@@Z',
    1450:'?AfxThunkReadClassStg@@YGJPAUIStorage@@PAU_GUID@@@Z',
    1451:'?AfxThunkReadClassStm@@YGJPAUIStream@@PAU_GUID@@@Z',
    1452:'?AfxThunkReadFmtUserTypeStg@@YGJPAUIStorage@@PAGPAPAG@Z',
    1453:'?AfxThunkrecv@@YGHIPADHH@Z',
    1454:'?AfxThunkrecvfrom@@YGHIPADHHPAUsockaddr@@PAH@Z',
    1455:'?AfxThunkRegCloseKey@@YGJPAUHKEY__@@@Z',
    1456:'?AfxThunkRegCreateKey@@YGJPAUHKEY__@@PBDPAPAU1@@Z',
    1457:'?AfxThunkRegCreateKeyEx@@YGJPAUHKEY__@@PBDKPADKKPAU_SECURITY_ATTRIBUTES@@PAPAU1@PAK@Z',
    1458:'?AfxThunkRegDeleteKey@@YGJPAUHKEY__@@PBD@Z',
    1459:'?AfxThunkRegDeleteValue@@YGJPAUHKEY__@@PBD@Z',
    1460:'?AfxThunkRegEnumKey@@YGJPAUHKEY__@@KPADK@Z',
    1461:'?AfxThunkRegisterBindStatusCallback@@YGJPAUIBindCtx@@PAUIBindStatusCallback@@PAPAU2@K@Z',
    1462:'?AfxThunkRegisterDragDrop@@YGJPAUHWND__@@PAUIDropTarget@@@Z',
    1463:'?AfxThunkRegisterTypeLib@@YGJPAUITypeLib@@PAG1@Z',
    1464:'?AfxThunkRegOpenKey@@YGJPAUHKEY__@@PBDPAPAU1@@Z',
    1465:'?AfxThunkRegOpenKeyEx@@YGJPAUHKEY__@@PBDKKPAPAU1@@Z',
    1466:'?AfxThunkRegQueryValue@@YGJPAUHKEY__@@PBDPADPAJ@Z',
    1467:'?AfxThunkRegQueryValueEx@@YGJPAUHKEY__@@PBDPAK2PAE2@Z',
    1468:'?AfxThunkRegSetValue@@YGJPAUHKEY__@@PBDK1K@Z',
    1469:'?AfxThunkRegSetValueEx@@YGJPAUHKEY__@@PBDKKPBEK@Z',
    1470:'?AfxThunkReleaseStgMedium@@YGXPAUtagSTGMEDIUM@@@Z',
    1471:'?AfxThunkReplaceText@@YGPAUHWND__@@PAUtagFINDREPLACEA@@@Z',
    1472:'?AfxThunkRevokeBindStatusCallback@@YGJPAUIBindCtx@@PAUIBindStatusCallback@@@Z',
    1473:'?AfxThunkRevokeDragDrop@@YGJPAUHWND__@@@Z',
    1474:'?AfxThunkSafeArrayAccessData@@YGJPAUtagSAFEARRAY@@PAPAX@Z',
    1475:'?AfxThunkSafeArrayAllocData@@YGJPAUtagSAFEARRAY@@@Z',
    1476:'?AfxThunkSafeArrayAllocDescriptor@@YGJIPAPAUtagSAFEARRAY@@@Z',
    1477:'?AfxThunkSafeArrayCopy@@YGJPAUtagSAFEARRAY@@PAPAU1@@Z',
    1478:'?AfxThunkSafeArrayCreate@@YGPAUtagSAFEARRAY@@GIPAUtagSAFEARRAYBOUND@@@Z',
    1479:'?AfxThunkSafeArrayDestroy@@YGJPAUtagSAFEARRAY@@@Z',
    1480:'?AfxThunkSafeArrayDestroyData@@YGJPAUtagSAFEARRAY@@@Z',
    1481:'?AfxThunkSafeArrayDestroyDescriptor@@YGJPAUtagSAFEARRAY@@@Z',
    1482:'?AfxThunkSafeArrayGetDim@@YGIPAUtagSAFEARRAY@@@Z',
    1483:'?AfxThunkSafeArrayGetElement@@YGJPAUtagSAFEARRAY@@PAJPAX@Z',
    1484:'?AfxThunkSafeArrayGetElemsize@@YGIPAUtagSAFEARRAY@@@Z',
    1485:'?AfxThunkSafeArrayGetLBound@@YGJPAUtagSAFEARRAY@@IPAJ@Z',
    1486:'?AfxThunkSafeArrayGetUBound@@YGJPAUtagSAFEARRAY@@IPAJ@Z',
    1487:'?AfxThunkSafeArrayLock@@YGJPAUtagSAFEARRAY@@@Z',
    1488:'?AfxThunkSafeArrayPtrOfIndex@@YGJPAUtagSAFEARRAY@@PAJPAPAX@Z',
    1489:'?AfxThunkSafeArrayPutElement@@YGJPAUtagSAFEARRAY@@PAJPAX@Z',
    1490:'?AfxThunkSafeArrayRedim@@YGJPAUtagSAFEARRAY@@PAUtagSAFEARRAYBOUND@@@Z',
    1491:'?AfxThunkSafeArrayUnaccessData@@YGJPAUtagSAFEARRAY@@@Z',
    1492:'?AfxThunkSafeArrayUnlock@@YGJPAUtagSAFEARRAY@@@Z',
    1493:'?AfxThunksend@@YGHIPBDHH@Z',
    1494:'?AfxThunksendto@@YGHIPBDHHPBUsockaddr@@H@Z',
    1495:'?AfxThunkSetConvertStg@@YGJPAUIStorage@@H@Z',
    1496:'?AfxThunkSetFileSecurity@@YGHPBDKPAX@Z',
    1497:'?AfxThunksetsockopt@@YGHIHHPBDH@Z',
    1498:'?AfxThunkSHGetFileInfo@@YGKPBDKPAU_SHFILEINFOA@@II@Z',
    1499:'?AfxThunkshutdown@@YGHIH@Z',
    1500:'?AfxThunksocket@@YGIHHH@Z',
    1501:'?AfxThunkSQLAllocConnect@@YGFPAXPAPAX@Z',
    1502:'?AfxThunkSQLAllocEnv@@YGFPAPAX@Z',
    1503:'?AfxThunkSQLAllocStmt@@YGFPAXPAPAX@Z',
    1504:'?AfxThunkSQLBindCol@@YGFPAXGF0JPAJ@Z',
    1505:'?AfxThunkSQLBindParameter@@YGFPAXGFFFKF0JPAJ@Z',
    1506:'?AfxThunkSQLCancel@@YGFPAX@Z',
    1507:'?AfxThunkSQLDescribeCol@@YGFPAXGPAEFPAF2PAK22@Z',
    1508:'?AfxThunkSQLDisconnect@@YGFPAX@Z',
    1509:'?AfxThunkSQLDriverConnect@@YGFPAXPAUHWND__@@PAEF2FPAFG@Z',
    1510:'?AfxThunkSQLError@@YGFPAX00PAEPAJ1FPAF@Z',
    1511:'?AfxThunkSQLExecDirect@@YGFPAXPAEJ@Z',
    1512:'?AfxThunkSQLExecute@@YGFPAX@Z',
    1513:'?AfxThunkSQLExtendedFetch@@YGFPAXGJPAKPAG@Z',
    1514:'?AfxThunkSQLFetch@@YGFPAX@Z',
    1515:'?AfxThunkSQLFreeConnect@@YGFPAX@Z',
    1516:'?AfxThunkSQLFreeEnv@@YGFPAX@Z',
    1517:'?AfxThunkSQLFreeStmt@@YGFPAXG@Z',
    1518:'?AfxThunkSQLGetCursorName@@YGFPAXPAEFPAF@Z',
    1519:'?AfxThunkSQLGetData@@YGFPAXGF0JPAJ@Z',
    1520:'?AfxThunkSQLGetFunctions@@YGFPAXGPAG@Z',
    1521:'?AfxThunkSQLGetInfo@@YGFPAXG0FPAF@Z',
    1522:'?AfxThunkSQLGetStmtOption@@YGFPAXG0@Z',
    1523:'?AfxThunkSQLMoreResults@@YGFPAX@Z',
    1524:'?AfxThunkSQLNumParams@@YGFPAXPAF@Z',
    1525:'?AfxThunkSQLNumResultCols@@YGFPAXPAF@Z',
    1526:'?AfxThunkSQLParamData@@YGFPAXPAPAX@Z',
    1527:'?AfxThunkSQLPrepare@@YGFPAXPAEJ@Z',
    1528:'?AfxThunkSQLPutData@@YGFPAX0J@Z',
    1529:'?AfxThunkSQLRowCount@@YGFPAXPAJ@Z',
    1530:'?AfxThunkSQLSetConnectOption@@YGFPAXGK@Z',
    1531:'?AfxThunkSQLSetPos@@YGFPAXGGG@Z',
    1532:'?AfxThunkSQLSetStmtOption@@YGFPAXGK@Z',
    1533:'?AfxThunkSQLTransact@@YGFPAX0G@Z',
    1534:'?AfxThunkStgCreateDocfile@@YGJPBGKKPAPAUIStorage@@@Z',
    1535:'?AfxThunkStgCreateDocfileOnILockBytes@@YGJPAUILockBytes@@KKPAPAUIStorage@@@Z',
    1536:'?AfxThunkStgIsStorageFile@@YGJPBG@Z',
    1537:'?AfxThunkStgIsStorageILockBytes@@YGJPAUILockBytes@@@Z',
    1538:'?AfxThunkStgOpenStorage@@YGJPBGPAUIStorage@@KPAPAGKPAPAU1@@Z',
    1539:'?AfxThunkStgOpenStorageOnILockBytes@@YGJPAUILockBytes@@PAUIStorage@@KPAPAGKPAPAU2@@Z',
    1540:'?AfxThunkStringFromCLSID@@YGJABU_GUID@@PAPAG@Z',
    1541:'?AfxThunkStringFromGUID2@@YGHABU_GUID@@PAGH@Z',
    1542:'?AfxThunkSysAllocString@@YGPAGPBG@Z',
    1543:'?AfxThunkSysAllocStringByteLen@@YGPAGPBDI@Z',
    1544:'?AfxThunkSysAllocStringLen@@YGPAGPBGI@Z',
    1545:'?AfxThunkSysFreeString@@YGXPAG@Z',
    1546:'?AfxThunkSysReAllocStringLen@@YGHPAPAGPBGI@Z',
    1547:'?AfxThunkSysStringByteLen@@YGIPAG@Z',
    1548:'?AfxThunkSysStringLen@@YGIPAG@Z',
    1549:'?AfxThunkVarBstrFromCy@@YGJTtagCY@@KKPAPAG@Z',
    1550:'?AfxThunkVarBstrFromDate@@YGJNKKPAPAG@Z',
    1551:'?AfxThunkVarCyFromStr@@YGJPAGKKPATtagCY@@@Z',
    1552:'?AfxThunkVarDateFromStr@@YGJPAGKKPAN@Z',
    1553:'?AfxThunkVariantChangeType@@YGJPAUtagVARIANT@@0GG@Z',
    1554:'?AfxThunkVariantClear@@YGJPAUtagVARIANT@@@Z',
    1555:'?AfxThunkVariantCopy@@YGJPAUtagVARIANT@@0@Z',
    1556:'?AfxThunkWriteClassStg@@YGJPAUIStorage@@ABU_GUID@@@Z',
    1557:'?AfxThunkWriteClassStm@@YGJPAUIStream@@ABU_GUID@@@Z',
    1558:'?AfxThunkWriteFmtUserTypeStg@@YGJPAUIStorage@@GPAG@Z',
    1559:'?AfxThunkWSAAsyncSelect@@YGHIPAUHWND__@@IJ@Z',
    1560:'?AfxThunkWSACleanup@@YGHXZ',
    1561:'?AfxThunkWSAGetLastError@@YGHXZ',
    1562:'?AfxThunkWSASetLastError@@YGXH@Z',
    1563:'?AfxThunkWSAStartup@@YGHGPAUWSAData@@@Z',
    1564:'?AfxTimeToFileTime@@YAXABVCTime@@PAU_FILETIME@@@Z',
    1565:'?AfxTlsAddRef@@YGXXZ',
    1566:'?AfxTlsRelease@@YGXXZ',
    1567:'?AfxTryCleanup@@YGXXZ',
    1568:'?AfxUnhookWindowCreate@@YGHXZ',
    1569:'?AfxUnlockGlobals@@YGXH@Z',
    1570:'?AfxUnlockTempMaps@@YGHH@Z',
    1571:'?AfxUnmergeMenus@@YGXPAUHMENU__@@00@Z',
    1572:'?AfxVariantInit@@YGXPAUtagVARIANT@@@Z',
    1573:'?AfxVerifyLicFile@@YGHPAUHINSTANCE__@@PBDPBGI@Z',
    1574:'?AfxW2AHelper@@YGPADPADPBGH@Z',
    1575:'?AfxWinInit@@YGHPAUHINSTANCE__@@0PADH@Z',
    1576:'?AfxWinMain@@YGHPAUHINSTANCE__@@0PADH@Z',
    1577:'?AfxWinTerm@@YGXXZ',
    1578:'?AfxWndProc@@YGJPAUHWND__@@IIJ@Z',
    1579:'?AfxWndProcBase@@YGJPAUHWND__@@IIJ@Z',
    1580:'?AfxWndProcDllOle@@YGJPAUHWND__@@IIJ@Z',
    1581:'?Alloc@CMemFile@@MAEPAEK@Z',
    1582:'?Alloc@CSharedFile@@MAEPAEK@Z',
    1583:'?AllocAndCacheFieldInfo@CRecordset@@IAEXXZ',
    1584:'?AllocBeforeWrite@CString@@IAEXH@Z',
    1585:'?AllocBuffer@CString@@IAEXH@Z',
    1586:'?AllocCache@CDaoRecordset@@IAEXXZ',
    1587:'?AllocCacheValue@CDaoFieldExchange@@SGXAAPAUCDaoFieldCache@@K@Z',
    1588:'?AllocConnect@CDatabase@@IAEXK@Z',
    1589:'?AllocCopy@CString@@IBEXAAV1@HHH@Z',
    1590:'?AllocData@COleSafeArray@@QAEXXZ',
    1591:'?AllocDatabase@CDaoRecordset@@IAEXXZ',
    1592:'?AllocDataCache@CRecordset@@QAEXXZ',
    1593:'?AllocDescriptor@COleSafeArray@@QAEXK@Z',
    1594:'?AllocElements@CControlBar@@QAEHHH@Z',
    1595:'?AllocElements@CStatusBar@@QAEHHH@Z',
    1596:'?AllocHstmt@CRecordset@@IAEHXZ',
    1597:'?AllocLongBinary@@YAXAAVCLongBinary@@K@Z',
    1598:'?AllocRowset@CRecordset@@IAEXXZ',
    1599:'?AllocSlot@CThreadSlotData@@QAEHXZ',
    1600:'?AllocStatusArrays@CRecordset@@QAEXXZ',
    1601:'?AllocSysString@CString@@QBEPAGXZ',
    1602:'?AllocValue@CProperty@@AAEPAXK@Z',
    1603:'?AmbientAppearance@COleControl@@QAEFXZ',
    1604:'?AmbientBackColor@COleControl@@QAEKXZ',
    1605:'?AmbientDisplayName@COleControl@@QAE?AVCString@@XZ',
    1606:'?AmbientFont@COleControl@@QAEPAUIFontDisp@@XZ',
    1607:'?AmbientForeColor@COleControl@@QAEKXZ',
    1608:'?AmbientLocaleID@COleControl@@QAEKXZ',
    1609:'?AmbientScaleUnits@COleControl@@QAE?AVCString@@XZ',
    1610:'?AmbientShowGrabHandles@COleControl@@QAEHXZ',
    1611:'?AmbientShowHatching@COleControl@@QAEHXZ',
    1612:'?AmbientTextAlign@COleControl@@QAEFXZ',
    1613:'?AmbientUIDead@COleControl@@QAEHXZ',
    1614:'?AmbientUserMode@COleControl@@QAEHXZ',
    1615:'?Append@CByteArray@@QAEHABV1@@Z',
    1616:'?Append@CDaoQueryDef@@UAEXXZ',
    1617:'?Append@CDaoTableDef@@UAEXXZ',
    1618:'?Append@CDaoWorkspace@@UAEXXZ',
    1619:'?Append@CDWordArray@@QAEHABV1@@Z',
    1620:'?Append@CObArray@@QAEHABV1@@Z',
    1621:'?Append@CPtrArray@@QAEHABV1@@Z',
    1622:'?Append@CStringArray@@QAEHABV1@@Z',
    1623:'?Append@CUIntArray@@QAEHABV1@@Z',
    1624:'?Append@CWordArray@@QAEHABV1@@Z',
    1625:'?AppendFilterAndSortSQL@CRecordset@@IAEXXZ',
    1626:'?AppendNames@CRecordset@@QAEIPAVCString@@PBD@Z',
    1627:'?AppendNamesValues@CRecordset@@QAEIPAXPAVCString@@PBD@Z',
    1628:'?AppendParamType@CDaoFieldExchange@@SGXAAVCString@@K@Z',
    1629:'?AppendValues@CRecordset@@QAEIPAXPAVCString@@PBD@Z',
    1630:'?Apply@XPropertyPage@COlePropertyPage@@UAGJXZ',
    1631:'?ApplyPrintDevice@COleDocument@@QAEHPBUtagDVTARGETDEVICE@@@Z',
    1632:'?ApplyPrintDevice@COleDocument@@QAEHPBUtagPDA@@@Z',
    1633:'?ApplyViewState@XOleDocumentView@CDocObjectServer@@UAGJPAUIStream@@@Z',
    1634:'?ArcTo@CDC@@QAEHHHHHHHHH@Z',
    1635:'?AssertValid@CObject@@UBEXXZ',
    1636:'?AssignCopy@CString@@IAEXHPBD@Z',
    1637:'?AssignInstance@CThreadSlotData@@QAEXPAUHINSTANCE__@@@Z',
    1638:'?AsyncSelect@CAsyncSocket@@QAEHJ@Z',
    1639:'?Attach@CAsyncSocket@@QAEHIJ@Z',
    1640:'?Attach@CDC@@QAEHPAUHDC__@@@Z',
    1641:'?Attach@CGdiObject@@QAEHPAX@Z',
    1642:'?Attach@CImageList@@QAEHPAU_IMAGELIST@@@Z',
    1643:'?Attach@CMemFile@@QAEXPAEII@Z',
    1644:'?Attach@CMenu@@QAEHPAUHMENU__@@@Z',
    1645:'?Attach@CMonikerFile@@IAEHPAUIMoniker@@PAUIBindHost@@PAUIBindStatusCallback@@PAUIBindCtx@@PAVCFileException@@@Z',
    1646:'?Attach@CMonikerFile@@IAEHPBDPAUIBindHost@@PAUIBindStatusCallback@@PAUIBindCtx@@PAVCFileException@@@Z',
    1647:'?Attach@COleDataObject@@QAEXPAUIDataObject@@H@Z',
    1648:'?Attach@COleSafeArray@@QAEXAAUtagVARIANT@@@Z',
    1649:'?Attach@COleStreamFile@@QAEXPAUIStream@@@Z',
    1650:'?Attach@COleVariant@@QAEXAAUtagVARIANT@@@Z',
    1651:'?Attach@CWnd@@QAEHPAUHWND__@@@Z',
    1652:'?AttachClipboard@COleDataObject@@QAEHXZ',
    1653:'?AttachControlSite@CWnd@@IAEXPAVCHandleMap@@@Z',
    1654:'?AttachControlSite@CWnd@@QAEXPAV1@@Z',
    1655:'?AttachDataObject@COleClientItem@@QBEXAAVCOleDataObject@@@Z',
    1656:'?AttachDispatch@COleDispatchDriver@@QAEXPAUIDispatch@@H@Z',
    1657:'?AttachHandle@CAsyncSocket@@SGXIPAV1@H@Z',
    1658:'?AttachOnSetup@CPrintDialog@@MAEPAV1@XZ',
    1659:'?AttachWindow@COleControlSite@@IAEXXZ',
    1660:'?AutoLoad@CBitmapButton@@QAEHIPAVCWnd@@@Z',
    1661:'?AuxQueueAdd@CSocket@@KGXIIJ@Z',
    1662:'?BeginBusyState@COleMessageFilter@@UAEXXZ',
    1663:'?BeginDrag@CDragListBox@@UAEHVCPoint@@@Z',
    1664:'?BeginEnumFormats@COleDataObject@@QAEXXZ',
    1665:'?BeginModalState@CFrameWnd@@UAEXXZ',
    1666:'?BeginModalState@CWnd@@UAEXXZ',
    1667:'?BeginTrans@CDaoWorkspace@@QAEXXZ',
    1668:'?BeginTrans@CDatabase@@QAEHXZ',
    1669:'?BeginWaitCursor@CCmdTarget@@QAEXXZ',
    1670:'?Bind@CAsyncSocket@@QAEHIPBD@Z',
    1671:'?BindColumns@CDataSourceControl@@UAEXXZ',
    1672:'?BindControls@COccManager@@IAEXPAVCWnd@@@Z',
    1673:'?BindDefaultProperty@COleControlSite@@UAEXJGPBDPAVCWnd@@@Z',
    1674:'?BindDefaultProperty@CWnd@@QAEXJGPBDPAV1@@Z',
    1675:'?BindFields@CDaoRecordset@@IAEXXZ',
    1676:'?BindFieldsForUpdate@CRecordset@@QAEXXZ',
    1677:'?BindFieldsToColumns@CRecordset@@QAEIXZ',
    1678:'?BindParameters@CDaoRecordset@@IAEXXZ',
    1679:'?BindParameters@CDatabase@@UAEXPAX@Z',
    1680:'?BindParams@CRecordset@@QAEIPAX@Z',
    1681:'?BindProp@CDataSourceControl@@UAEXPAVCDataBoundProperty@@H@Z',
    1682:'?BindProp@CDataSourceControl@@UAEXPAVCOleControlSite@@H@Z',
    1683:'?BindProperty@COleControlSite@@UAEXJPAVCWnd@@@Z',
    1684:'?BindProperty@CWnd@@QAEXJPAV1@@Z',
    1685:'?BoundPropertyChanged@COleControl@@QAEXJ@Z',
    1686:'?BoundPropertyRequestEdit@COleControl@@QAEHJ@Z',
    1687:'?BringToTop@CFrameWnd@@IAEXH@Z',
    1688:'?BuildErrorString@CDBException@@UAEXPAVCDatabase@@PAXH@Z',
    1689:'?BuildParameterList@CDaoRecordset@@IAEXXZ',
    1690:'?BuildPropPageArray@CPropertySheet@@UAEXXZ',
    1691:'?BuildSelectList@CDaoRecordset@@IAEXXZ',
    1692:'?BuildSelectSQL@CRecordset@@IAEXXZ',
    1693:'?BuildSharedMenu@COleControl@@MAEHXZ',
    1694:'?BuildSharedMenu@COleDocIPFrameWnd@@MAEHXZ',
    1695:'?BuildSharedMenu@COleIPFrameWnd@@MAEHXZ',
    1696:'?BuildSQL@CDaoRecordset@@IAEXXZ',
    1697:'?BuildSQL@CRecordset@@IAEXPBD@Z',
    1698:'?BuildUpdateSQL@CRecordset@@IAEXXZ',
    1699:'?ButtonDblClk@COleControl@@IAEXGIVCPoint@@@Z',
    1700:'?ButtonDown@COleControl@@IAEXGIVCPoint@@@Z',
    1701:'?ButtonUp@COleControl@@IAEXGIVCPoint@@@Z',
    1702:'?Cache@_AFXCTL_AMBIENT_CACHE@@QAEXPAUtagQACONTAINER@@@Z',
    1703:'?Cache@CTypeLibCache@@QAEXKPAUITypeLib@@@Z',
    1704:'?Cache@XOleCache@COleControl@@UAGJPAUtagFORMATETC@@KPAK@Z',
    1705:'?CacheData@COleDataSource@@QAEXGPAUtagSTGMEDIUM@@PAUtagFORMATETC@@@Z',
    1706:'?CacheGlobalData@COleDataSource@@QAEXGPAXPAUtagFORMATETC@@@Z',
    1707:'?CacheTypeInfo@CTypeLibCache@@QAEXKABU_GUID@@PAUITypeInfo@@@Z',
    1708:'?CalcBorders@CMiniFrameWnd@@SGXPAUtagRECT@@KK@Z',
    1709:'?CalcDynamicLayout@CControlBar@@UAE?AVCSize@@HK@Z',
    1710:'?CalcDynamicLayout@CToolBar@@UAE?AVCSize@@HK@Z',
    1711:'?CalcFixedLayout@CControlBar@@UAE?AVCSize@@HH@Z',
    1712:'?CalcFixedLayout@CDialogBar@@UAE?AVCSize@@HH@Z',
    1713:'?CalcFixedLayout@CDockBar@@UAE?AVCSize@@HH@Z',
    1714:'?CalcFixedLayout@CStatusBar@@UAE?AVCSize@@HH@Z',
    1715:'?CalcFixedLayout@CToolBar@@UAE?AVCSize@@HH@Z',
    1716:'?CalcInsideRect@CControlBar@@QBEXAAVCRect@@H@Z',
    1717:'?CalcInsideRect@CStatusBar@@QBEXAAVCRect@@H@Z',
    1718:'?CalcLayout@CToolBar@@IAE?AVCSize@@KH@Z',
    1719:'?CalcMinimumItemHeight@CCheckListBox@@IAEHXZ',
    1720:'?CalcPageDisplaySize@CPreviewView@@IAE?AVCSize@@XZ',
    1721:'?CalcScaleRatio@CPreviewView@@MAE?AVCSize@@V2@0@Z',
    1722:'?CalcSize@CToolBar@@IAE?AVCSize@@PAU_TBBUTTON@@H@Z',
    1723:'?CalcWindowRect@CEditView@@MAEXPAUtagRECT@@I@Z',
    1724:'?CalcWindowRect@CMiniFrameWnd@@UAEXPAUtagRECT@@I@Z',
    1725:'?CalcWindowRect@CScrollView@@UAEXPAUtagRECT@@I@Z',
    1726:'?CalcWindowRect@CView@@UAEXPAUtagRECT@@I@Z',
    1727:'?CalcWindowRect@CWnd@@UAEXPAUtagRECT@@I@Z',
    1728:'?CallMemberFunc@CCmdTarget@@IAEJPBUAFX_DISPMAP_ENTRY@@GPAUtagVARIANT@@PAUtagDISPPARAMS@@PAI@Z',
    1729:'?CanActivate@COleClientItem@@MAEHXZ',
    1730:'?CanActivate@CRichEditCntrItem@@UAEHXZ',
    1731:'?CanActivateNext@CSplitterWnd@@UAEHH@Z',
    1732:'?CanAppend@CDaoRecordset@@QBEHXZ',
    1733:'?CanBookmark@CDaoRecordset@@QAEHXZ',
    1734:'?CanBookmark@CRecordset@@QBEHXZ',
    1735:'?Cancel@CDatabase@@QAEXXZ',
    1736:'?Cancel@CRecordset@@QAEXXZ',
    1737:'?CancelBlockingCall@CSocket@@QAEXXZ',
    1738:'?CancelDrag@CDragListBox@@UAEXVCPoint@@@Z',
    1739:'?Cancelled@XNotifyDBEvents@COleControlSite@@UAGJKKQAUtagDBNOTIFYREASON@@@Z',
    1740:'?CancelLink@COleUILinkInfo@@UAGJK@Z',
    1741:'?CancelLoop@CDockContext@@QAEXXZ',
    1742:'?CancelToClose@CPropertyPage@@QAEXXZ',
    1743:'?CancelToolTips@CWnd@@SGXH@Z',
    1744:'?CancelUpdate@CDaoRecordset@@UAEXXZ',
    1745:'?CancelUpdate@CRecordset@@QAEXXZ',
    1746:'?CanCloseFrame@CDocument@@UAEHPAVCFrameWnd@@@Z',
    1747:'?CanCloseFrame@COleServerDoc@@UAEHPAVCFrameWnd@@@Z',
    1748:'?CanCreateFromData@COleClientItem@@SGHPBVCOleDataObject@@@Z',
    1749:'?CanCreateLinkFromData@COleClientItem@@SGHPBVCOleDataObject@@@Z',
    1750:'?CanDock@CDockContext@@QAEKXZ',
    1751:'?CanDock@CFrameWnd@@QAEKVCRect@@KPAPAVCDockBar@@@Z',
    1752:'?CanEnterHelpMode@CFrameWnd@@QAEHXZ',
    1753:'?CanInPlaceActivate@XOleIPSite@COleClientItem@@UAGJXZ',
    1754:'?CanInPlaceActivate@XOleIPSite@COleControlSite@@UAGJXZ',
    1755:'?CanPaste@COleClientItem@@SGHXZ',
    1756:'?CanPaste@CRichEditCtrl@@QBEHI@Z',
    1757:'?CanPaste@CRichEditView@@QBEHXZ',
    1758:'?CanPasteLink@COleClientItem@@SGHXZ',
    1759:'?CanRestart@CDaoRecordset@@QAEHXZ',
    1760:'?CanScroll@CDaoRecordset@@QBEHXZ',
    1761:'?CanTransact@CDaoDatabase@@QAEHXZ',
    1762:'?CanTransact@CDaoRecordset@@QAEHXZ',
    1763:'?CanUpdate@CDaoDatabase@@QAEHXZ',
    1764:'?CanUpdate@CDaoQueryDef@@QAEHXZ',
    1765:'?CanUpdate@CDaoRecordset@@QBEHXZ',
    1766:'?CanUpdate@CDaoTableDef@@QAEHXZ',
    1767:'?CenterOnPoint@CScrollView@@IAEXVCPoint@@@Z',
    1768:'?CenterWindow@CWnd@@QAEXPAV1@@Z',
    1769:'?ChangePicture@CPicturePropPage@@IAEXPAUIPicture@@@Z',
    1770:'?ChangeType@COleVariant@@QAEXGPAUtagVARIANT@@@Z',
    1771:'?CharToItem@CListBox@@UAEHII@Z',
    1772:'?CharToOemA@CString@@QAEXXZ',
    1773:'?Check@CDatabase@@UBEHF@Z',
    1774:'?Check@CRecordset@@UBEHF@Z',
    1775:'?CheckAutoCenter@CDialog@@UAEHXZ',
    1776:'?CheckAutoCenter@CWnd@@UAEHXZ',
    1777:'?CheckCount@CArchive@@QAEXXZ',
    1778:'?CheckDlgButton@COleControlContainer@@UAEXHI@Z',
    1779:'?CheckDlgButton@CWnd@@QAEXHI@Z',
    1780:'?CheckFromPoint@CCheckListBox@@IAEHVCPoint@@AAH@Z',
    1781:'?CheckGeneral@COleClientItem@@MAEXJ@Z',
    1782:'?CheckRadioButton@COleControlContainer@@UAEXHHH@Z',
    1783:'?CheckRadioButton@CWnd@@QAEXHHH@Z',
    1784:'?CheckRange@COleDateTime@@IAEXXZ',
    1785:'?CheckRange@COleDateTimeSpan@@QAEXXZ',
    1786:'?CheckRowsetCurrencyStatus@CRecordset@@QAEXGJ@Z',
    1787:'?CheckRowsetError@CRecordset@@UAEXF@Z',
    1788:'?classCAnimateCtrl@CAnimateCtrl@@2UCRuntimeClass@@B',
    1789:'?classCArchiveException@CArchiveException@@2UCRuntimeClass@@B',
    1790:'?classCAsyncMonikerFile@CAsyncMonikerFile@@2UCRuntimeClass@@B',
    1791:'?classCAsyncSocket@CAsyncSocket@@2UCRuntimeClass@@B',
    1792:'?classCBitmap@CBitmap@@2UCRuntimeClass@@B',
    1793:'?classCBitmapButton@CBitmapButton@@2UCRuntimeClass@@B',
    1794:'?classCBrush@CBrush@@2UCRuntimeClass@@B',
    1795:'?classCButton@CButton@@2UCRuntimeClass@@B',
    1796:'?classCByteArray@CByteArray@@2UCRuntimeClass@@A',
    1797:'?classCCheckListBox@CCheckListBox@@2UCRuntimeClass@@B',
    1798:'?classCClientDC@CClientDC@@2UCRuntimeClass@@B',
    1799:'?classCCmdTarget@CCmdTarget@@2UCRuntimeClass@@B',
    1800:'?classCColorDialog@CColorDialog@@2UCRuntimeClass@@B',
    1801:'?classCColorPropPage@CColorPropPage@@2UCRuntimeClass@@B',
    1802:'?classCComboBox@CComboBox@@2UCRuntimeClass@@B',
    1803:'?classCControlBar@CControlBar@@2UCRuntimeClass@@B',
    1804:'?classCCriticalSection@CCriticalSection@@2UCRuntimeClass@@B',
    1805:'?classCCtrlView@CCtrlView@@2UCRuntimeClass@@B',
    1806:'?classCDaoDatabase@CDaoDatabase@@2UCRuntimeClass@@B',
    1807:'?classCDaoException@CDaoException@@2UCRuntimeClass@@B',
    1808:'?classCDaoQueryDef@CDaoQueryDef@@2UCRuntimeClass@@B',
    1809:'?classCDaoRecordset@CDaoRecordset@@2UCRuntimeClass@@B',
    1810:'?classCDaoRecordView@CDaoRecordView@@2UCRuntimeClass@@B',
    1811:'?classCDaoTableDef@CDaoTableDef@@2UCRuntimeClass@@B',
    1812:'?classCDaoWorkspace@CDaoWorkspace@@2UCRuntimeClass@@B',
    1813:'?classCDatabase@CDatabase@@2UCRuntimeClass@@B',
    1814:'?classCDBException@CDBException@@2UCRuntimeClass@@B',
    1815:'?classCDC@CDC@@2UCRuntimeClass@@B',
    1816:'?classCDialog@CDialog@@2UCRuntimeClass@@B',
    1817:'?classCDialogBar@CDialogBar@@2UCRuntimeClass@@B',
    1818:'?classCDocItem@CDocItem@@2UCRuntimeClass@@A',
    1819:'?classCDockBar@CDockBar@@2UCRuntimeClass@@B',
    1820:'?classCDockState@CDockState@@2UCRuntimeClass@@A',
    1821:'?classCDocManager@CDocManager@@2UCRuntimeClass@@B',
    1822:'?classCDocObjectServer@CDocObjectServer@@2UCRuntimeClass@@B',
    1823:'?classCDocObjectServerItem@CDocObjectServerItem@@2UCRuntimeClass@@B',
    1824:'?classCDocTemplate@CDocTemplate@@2UCRuntimeClass@@B',
    1825:'?classCDocument@CDocument@@2UCRuntimeClass@@B',
    1826:'?classCDragListBox@CDragListBox@@2UCRuntimeClass@@B',
    1827:'?classCDWordArray@CDWordArray@@2UCRuntimeClass@@A',
    1828:'?classCDynLinkLibrary@CDynLinkLibrary@@2UCRuntimeClass@@B',
    1829:'?classCEdit@CEdit@@2UCRuntimeClass@@B',
    1830:'?classCEditView@CEditView@@2UCRuntimeClass@@B',
    1831:'?classCEvent@CEvent@@2UCRuntimeClass@@B',
    1832:'?classCException@CException@@2UCRuntimeClass@@B',
    1833:'?classCFile@CFile@@2UCRuntimeClass@@B',
    1834:'?classCFileDialog@CFileDialog@@2UCRuntimeClass@@B',
    1835:'?classCFileException@CFileException@@2UCRuntimeClass@@B',
    1836:'?classCFileFind@CFileFind@@2UCRuntimeClass@@B',
    1837:'?classCFindReplaceDialog@CFindReplaceDialog@@2UCRuntimeClass@@B',
    1838:'?classCFont@CFont@@2UCRuntimeClass@@B',
    1839:'?classCFontDialog@CFontDialog@@2UCRuntimeClass@@B',
    1840:'?classCFontPropPage@CFontPropPage@@2UCRuntimeClass@@B',
    1841:'?classCFormView@CFormView@@2UCRuntimeClass@@B',
    1842:'?classCFrameWnd@CFrameWnd@@2UCRuntimeClass@@B',
    1843:'?classCGdiObject@CGdiObject@@2UCRuntimeClass@@B',
    1844:'?classCHeaderCtrl@CHeaderCtrl@@2UCRuntimeClass@@B',
    1845:'?classCHotKeyCtrl@CHotKeyCtrl@@2UCRuntimeClass@@B',
    1846:'?classCImageList@CImageList@@2UCRuntimeClass@@B',
    1847:'?classCListBox@CListBox@@2UCRuntimeClass@@B',
    1848:'?classCListCtrl@CListCtrl@@2UCRuntimeClass@@B',
    1849:'?classCListView@CListView@@2UCRuntimeClass@@B',
    1850:'?classCLongBinary@CLongBinary@@2UCRuntimeClass@@B',
    1851:'?classCMapPtrToPtr@CMapPtrToPtr@@2UCRuntimeClass@@B',
    1852:'?classCMapPtrToWord@CMapPtrToWord@@2UCRuntimeClass@@B',
    1853:'?classCMapStringToOb@CMapStringToOb@@2UCRuntimeClass@@A',
    1854:'?classCMapStringToPtr@CMapStringToPtr@@2UCRuntimeClass@@B',
    1855:'?classCMapStringToString@CMapStringToString@@2UCRuntimeClass@@A',
    1856:'?classCMapWordToOb@CMapWordToOb@@2UCRuntimeClass@@A',
    1857:'?classCMapWordToPtr@CMapWordToPtr@@2UCRuntimeClass@@B',
    1858:'?classCMDIChildWnd@CMDIChildWnd@@2UCRuntimeClass@@B',
    1859:'?classCMDIFrameWnd@CMDIFrameWnd@@2UCRuntimeClass@@B',
    1860:'?classCMemFile@CMemFile@@2UCRuntimeClass@@B',
    1861:'?classCMemoryException@CMemoryException@@2UCRuntimeClass@@B',
    1862:'?classCMenu@CMenu@@2UCRuntimeClass@@B',
    1863:'?classCMetaFileDC@CMetaFileDC@@2UCRuntimeClass@@B',
    1864:'?classCMiniDockFrameWnd@CMiniDockFrameWnd@@2UCRuntimeClass@@B',
    1865:'?classCMiniFrameWnd@CMiniFrameWnd@@2UCRuntimeClass@@B',
    1866:'?classCMonikerFile@CMonikerFile@@2UCRuntimeClass@@B',
    1867:'?classCMultiDocTemplate@CMultiDocTemplate@@2UCRuntimeClass@@B',
    1868:'?classCMutex@CMutex@@2UCRuntimeClass@@B',
    1869:'?classCNotSupportedException@CNotSupportedException@@2UCRuntimeClass@@B',
    1870:'?classCObArray@CObArray@@2UCRuntimeClass@@A',
    1871:'?classCObject@CObject@@2UCRuntimeClass@@B',
    1872:'?classCObList@CObList@@2UCRuntimeClass@@A',
    1873:'?classCOleBusyDialog@COleBusyDialog@@2UCRuntimeClass@@B',
    1874:'?classCOleChangeIconDialog@COleChangeIconDialog@@2UCRuntimeClass@@B',
    1875:'?classCOleChangeSourceDialog@COleChangeSourceDialog@@2UCRuntimeClass@@B',
    1876:'?classCOleClientItem@COleClientItem@@2UCRuntimeClass@@B',
    1877:'?classCOleControl@COleControl@@2UCRuntimeClass@@B',
    1878:'?classCOleControlModule@COleControlModule@@2UCRuntimeClass@@B',
    1879:'?classCOleConvertDialog@COleConvertDialog@@2UCRuntimeClass@@B',
    1880:'?classCOleDialog@COleDialog@@2UCRuntimeClass@@B',
    1881:'?classCOleDispatchException@COleDispatchException@@2UCRuntimeClass@@B',
    1882:'?classCOleDocIPFrameWnd@COleDocIPFrameWnd@@2UCRuntimeClass@@B',
    1883:'?classCOleDocument@COleDocument@@2UCRuntimeClass@@B',
    1884:'?classCOleException@COleException@@2UCRuntimeClass@@B',
    1885:'?classCOleInsertDialog@COleInsertDialog@@2UCRuntimeClass@@B',
    1886:'?classCOleIPFrameWnd@COleIPFrameWnd@@2UCRuntimeClass@@B',
    1887:'?classCOleLinkingDoc@COleLinkingDoc@@2UCRuntimeClass@@B',
    1888:'?classCOleLinksDialog@COleLinksDialog@@2UCRuntimeClass@@B',
    1889:'?classCOleObjectFactory@COleObjectFactory@@2UCRuntimeClass@@B',
    1890:'?classCOlePasteSpecialDialog@COlePasteSpecialDialog@@2UCRuntimeClass@@B',
    1891:'?classCOlePropertiesDialog@COlePropertiesDialog@@2UCRuntimeClass@@B',
    1892:'?classCOlePropertyPage@COlePropertyPage@@2UCRuntimeClass@@B',
    1893:'?classCOleResizeBar@COleResizeBar@@2UCRuntimeClass@@B',
    1894:'?classCOleServerDoc@COleServerDoc@@2UCRuntimeClass@@B',
    1895:'?classCOleServerItem@COleServerItem@@2UCRuntimeClass@@B',
    1896:'?classCOleStreamFile@COleStreamFile@@2UCRuntimeClass@@B',
    1897:'?classCOleUpdateDialog@COleUpdateDialog@@2UCRuntimeClass@@B',
    1898:'?classCPageSetupDialog@CPageSetupDialog@@2UCRuntimeClass@@B',
    1899:'?classCPaintDC@CPaintDC@@2UCRuntimeClass@@B',
    1900:'?classCPalette@CPalette@@2UCRuntimeClass@@B',
    1901:'?classCPen@CPen@@2UCRuntimeClass@@B',
    1902:'?classCPicturePropPage@CPicturePropPage@@2UCRuntimeClass@@B',
    1903:'?classCPreviewDC@CPreviewDC@@2UCRuntimeClass@@B',
    1904:'?classCPreviewView@CPreviewView@@2UCRuntimeClass@@B',
    1905:'?classCPrintDialog@CPrintDialog@@2UCRuntimeClass@@B',
    1906:'?classCProgressCtrl@CProgressCtrl@@2UCRuntimeClass@@B',
    1907:'?classCPropertyPage@CPropertyPage@@2UCRuntimeClass@@B',
    1908:'?classCPropertySheet@CPropertySheet@@2UCRuntimeClass@@B',
    1909:'?classCPtrArray@CPtrArray@@2UCRuntimeClass@@B',
    1910:'?classCPtrList@CPtrList@@2UCRuntimeClass@@B',
    1911:'?classCRecordset@CRecordset@@2UCRuntimeClass@@B',
    1912:'?classCRecordView@CRecordView@@2UCRuntimeClass@@B',
    1913:'?classCResourceException@CResourceException@@2UCRuntimeClass@@B',
    1914:'?classCRgn@CRgn@@2UCRuntimeClass@@B',
    1915:'?classCRichEditCntrItem@CRichEditCntrItem@@2UCRuntimeClass@@A',
    1916:'?classCRichEditCtrl@CRichEditCtrl@@2UCRuntimeClass@@B',
    1917:'?classCRichEditDoc@CRichEditDoc@@2UCRuntimeClass@@B',
    1918:'?classCRichEditView@CRichEditView@@2UCRuntimeClass@@B',
    1919:'?classCScrollBar@CScrollBar@@2UCRuntimeClass@@B',
    1920:'?classCScrollView@CScrollView@@2UCRuntimeClass@@B',
    1921:'?classCSemaphore@CSemaphore@@2UCRuntimeClass@@B',
    1922:'?classCSharedFile@CSharedFile@@2UCRuntimeClass@@B',
    1923:'?classCSingleDocTemplate@CSingleDocTemplate@@2UCRuntimeClass@@B',
    1924:'?classCSliderCtrl@CSliderCtrl@@2UCRuntimeClass@@B',
    1925:'?classCSocket@CSocket@@2UCRuntimeClass@@B',
    1926:'?classCSocketFile@CSocketFile@@2UCRuntimeClass@@B',
    1927:'?classCSpinButtonCtrl@CSpinButtonCtrl@@2UCRuntimeClass@@B',
    1928:'?classCSplitterWnd@CSplitterWnd@@2UCRuntimeClass@@B',
    1929:'?classCStatic@CStatic@@2UCRuntimeClass@@B',
    1930:'?classCStatusBar@CStatusBar@@2UCRuntimeClass@@B',
    1931:'?classCStatusBarCtrl@CStatusBarCtrl@@2UCRuntimeClass@@B',
    1932:'?classCStdioFile@CStdioFile@@2UCRuntimeClass@@B',
    1933:'?classCStockPropPage@CStockPropPage@@2UCRuntimeClass@@B',
    1934:'?classCStringArray@CStringArray@@2UCRuntimeClass@@A',
    1935:'?classCStringList@CStringList@@2UCRuntimeClass@@A',
    1936:'?classCSyncObject@CSyncObject@@2UCRuntimeClass@@B',
    1937:'?classCTabCtrl@CTabCtrl@@2UCRuntimeClass@@B',
    1938:'?classCToolBar@CToolBar@@2UCRuntimeClass@@B',
    1939:'?classCToolBarCtrl@CToolBarCtrl@@2UCRuntimeClass@@B',
    1940:'?classCToolTipCtrl@CToolTipCtrl@@2UCRuntimeClass@@B',
    1941:'?classCTreeCtrl@CTreeCtrl@@2UCRuntimeClass@@B',
    1942:'?classCTreeView@CTreeView@@2UCRuntimeClass@@B',
    1943:'?classCUIntArray@CUIntArray@@2UCRuntimeClass@@B',
    1944:'?classCUserException@CUserException@@2UCRuntimeClass@@B',
    1945:'?classCView@CView@@2UCRuntimeClass@@B',
    1946:'?classCWinApp@CWinApp@@2UCRuntimeClass@@B',
    1947:'?classCWindowDC@CWindowDC@@2UCRuntimeClass@@B',
    1948:'?classCWinThread@CWinThread@@2UCRuntimeClass@@B',
    1949:'?classCWnd@CWnd@@2UCRuntimeClass@@B',
    1950:'?classCWordArray@CWordArray@@2UCRuntimeClass@@A',
    1951:'?Cleanup@CPropertyPage@@IAEXXZ',
    1952:'?CleanupObjectArray@COlePropertyPage@@IAEXXZ',
    1953:'?Clear@CDBVariant@@QAEXXZ',
    1954:'?Clear@CDockState@@QAEXXZ',
    1955:'?Clear@COleVariant@@QAEXXZ=?_ClearCompat@COleVariant@@QAEXXZ',
    1956:'?ClearDirtyFieldStatus@CDaoRecordset@@QAEXI@Z',
    1957:'?ClearDirtyFieldStatus@CRecordset@@QAEXK@Z',
    1958:'?ClearFieldStatus@CRecordset@@QAEXXZ',
    1959:'?ClearFieldStatusFlags@CDaoRecordset@@QAEXXZ',
    1960:'?ClearNullFieldStatus@CDaoRecordset@@QAEXI@Z',
    1961:'?ClearNullFieldStatus@CRecordset@@QAEXK@Z',
    1962:'?ClearNullParamStatus@CRecordset@@QAEXK@Z',
    1963:'?ClientToParent@COleControl@@UBEXPBUtagRECT@@PAUtagPOINT@@@Z',
    1964:'?ClipCaretRect@COleControl@@QAEHPAUtagRECT@@@Z',
    1965:'?ClipToPage@CPreviewDC@@QAEXXZ',
    1966:'?Clone@CArchiveStream@@UAGJPAPAUIStream@@@Z',
    1967:'?Clone@XEnumVOID@CEnumArray@@UAGJPAPAUIEnumVOID@@@Z',
    1968:'?Clone@XOleDocumentView@CDocObjectServer@@UAGJPAUIOleInPlaceSite@@PAPAUIOleDocumentView@@@Z',
    1969:'?Close@CArchive@@QAEXXZ',
    1970:'?Close@CAsyncMonikerFile@@UAEXXZ',
    1971:'?Close@CAsyncSocket@@UAEXXZ',
    1972:'?Close@CCachedDataPathProperty@@UAEXXZ',
    1973:'?Close@CDaoDatabase@@UAEXXZ',
    1974:'?Close@CDaoQueryDef@@UAEXXZ',
    1975:'?Close@CDaoRecordset@@UAEXXZ',
    1976:'?Close@CDaoTableDef@@UAEXXZ',
    1977:'?Close@CDaoWorkspace@@UAEXXZ',
    1978:'?Close@CDatabase@@UAEXXZ',
    1979:'?Close@CFile@@UAEXXZ',
    1980:'?Close@CFileFind@@QAEXXZ',
    1981:'?Close@CFtpConnection@@UAEXXZ',
    1982:'?Close@CGopherConnection@@UAEXXZ',
    1983:'?Close@CGopherFile@@UAEXXZ',
    1984:'?Close@CHttpConnection@@UAEXXZ',
    1985:'?Close@CHttpFile@@UAEXXZ',
    1986:'?Close@CInternetConnection@@MAEXXZ',
    1987:'?Close@CInternetFile@@UAEXXZ',
    1988:'?Close@CInternetSession@@UAEXXZ',
    1989:'?Close@CMemFile@@UAEXXZ',
    1990:'?Close@CMirrorFile@@UAEXXZ',
    1991:'?Close@CMonikerFile@@UAEXXZ',
    1992:'?Close@COleClientItem@@QAEXW4tagOLECLOSE@@@Z',
    1993:'?Close@COleStreamFile@@UAEXXZ',
    1994:'?Close@CRecordset@@UAEXXZ',
    1995:'?Close@CSocket@@UAEXXZ',
    1996:'?Close@CSocketFile@@UAEXXZ',
    1997:'?Close@CStdioFile@@UAEXXZ',
    1998:'?Close@XOleObject@CDocObjectServer@@UAGJK@Z',
    1999:'?Close@XOleObject@COleControl@@UAGJK@Z',
    2000:'?Close@XOleObject@COleServerDoc@@UAGJK@Z',
    2001:'?Close@XOleObject@COleServerItem@@UAGJK@Z',
    2002:'?CloseAllDocuments@CDocManager@@UAEXH@Z',
    2003:'?CloseAllDocuments@CDocTemplate@@UAEXH@Z',
    2004:'?CloseAllDocuments@CWinApp@@QAEXH@Z',
    2005:'?CloseContext@CFileFind@@MAEXXZ',
    2006:'?CloseContext@CFtpFileFind@@MAEXXZ',
    2007:'?CloseContext@CGopherFileFind@@MAEXXZ',
    2008:'?CloseView@XOleDocumentView@CDocObjectServer@@UAGJK@Z',
    2009:'?colGetFaceColor@CColorButton@@QAEKXZ',
    2010:'?commandMap@CCmdTarget@@1UAFX_OLECMDMAP@@B',
    2011:'?CommandToIndex@CStatusBar@@QBEHI@Z',
    2012:'?CommandToIndex@CToolBar@@QBEHI@Z',
    2013:'?Commit@CArchiveStream@@UAGJK@Z',
    2014:'?CommitItem@COleClientItem@@UAEXH@Z',
    2015:'?CommitItems@COleDocument@@QAEXH@Z',
    2016:'?CommitTrans@CDaoWorkspace@@QAEXXZ',
    2017:'?CommitTrans@CDatabase@@QAEHXZ',
    2018:'?CommonConstruct@CPropertyPage@@IAEXPBDI@Z',
    2019:'?CommonConstruct@CPropertySheet@@QAEXPAVCWnd@@I@Z',
    2020:'?CommonConstruct@CWinThread@@QAEXXZ',
    2021:'?CompactDatabase@CDaoWorkspace@@SGXPBD00H@Z',
    2022:'?CompactDatabase@CDaoWorkspace@@SGXPBD00H0@Z',
    2023:'?CompareItem@CComboBox@@UAEHPAUtagCOMPAREITEMSTRUCT@@@Z',
    2024:'?CompareItem@CListBox@@UAEHPAUtagCOMPAREITEMSTRUCT@@@Z',
    2025:'?CompareValue@CDaoFieldExchange@@SGHPAX0K@Z',
    2026:'?ComputeDeltas@CPreviewDC@@IAE?AVCSize@@AAHPBDAAIHIPAHHPAD30@Z',
    2027:'?ConcatCopy@CString@@IAEXHPBDH0@Z',
    2028:'?ConcatInPlace@CString@@IAEXHPBD@Z',
    2029:'?Connect@CAsyncSocket@@QAEHPBDI@Z',
    2030:'?Connect@CDatabase@@IAEHK@Z',
    2031:'?ConnectHelper@CAsyncSocket@@MAEHPBUsockaddr@@H@Z',
    2032:'?ConnectHelper@CSocket@@MAEHPBUsockaddr@@H@Z',
    2033:'?connectionMap@CCmdTarget@@1UAFX_CONNECTIONMAP@@B',
    2034:'?connectionMap@COleControl@@1UAFX_CONNECTIONMAP@@B',
    2035:'?ConnectSink@COleControlSite@@IAEKABU_GUID@@PAUIUnknown@@@Z',
    2036:'?ConnectTemplate@COleTemplateServer@@QAEXABU_GUID@@PAVCDocTemplate@@H@Z',
    2037:'?ConnectView@COleServerDoc@@IAEXPAVCWnd@@PAVCView@@@Z',
    2038:'?Construct@CPropertyPage@@QAEXII@Z',
    2039:'?Construct@CPropertyPage@@QAEXPBDI@Z',
    2040:'?Construct@CPropertySheet@@QAEXIPAVCWnd@@I@Z',
    2041:'?Construct@CPropertySheet@@QAEXPBDPAVCWnd@@I@Z',
    2042:'?Construct@CRectTracker@@IAEXXZ',
    2043:'?ConstructElements@@YGXPAVCOleVariant@@H@Z',
    2044:'?ConstructElements@@YGXPAVCString@@H@Z',
    2045:'?ContextSensitiveHelp@XOleInPlaceActiveObject@COleControl@@UAGJH@Z',
    2046:'?ContextSensitiveHelp@XOleInPlaceActiveObject@COleServerDoc@@UAGJH@Z',
    2047:'?ContextSensitiveHelp@XOleInPlaceFrame@COleFrameHook@@UAGJH@Z',
    2048:'?ContextSensitiveHelp@XOleInPlaceObject@COleControl@@UAGJH@Z',
    2049:'?ContextSensitiveHelp@XOleInPlaceObject@COleServerDoc@@UAGJH@Z',
    2050:'?ContextSensitiveHelp@XOleIPFrame@COleControlContainer@@UAGJH@Z',
    2051:'?ContextSensitiveHelp@XOleIPSite@COleClientItem@@UAGJH@Z',
    2052:'?ContextSensitiveHelp@XOleIPSite@COleControlSite@@UAGJH@Z',
    2053:'?ContextSensitiveHelp@XRichEditOleCallback@CRichEditView@@UAGJH@Z',
    2054:'?ContinueModal@CPropertySheet@@UAEHXZ',
    2055:'?ContinueModal@CWnd@@UAEHXZ',
    2056:'?ControlInfoChanged@COleControl@@QAEXXZ',
    2057:'?ConvertObject@XOleUIObjInfo@COlePropertiesDialog@@UAGJKABU_GUID@@@Z',
    2058:'?ConvertTo@COleClientItem@@UAEHABU_GUID@@@Z',
    2059:'?ConvertTo@CRichEditCntrItem@@UAEHABU_GUID@@@Z',
    2060:'?Copy@CByteArray@@QAEXABV1@@Z',
    2061:'?Copy@CDWordArray@@QAEXABV1@@Z',
    2062:'?Copy@CObArray@@QAEXABV1@@Z',
    2063:'?Copy@COleSafeArray@@QAEXPAPAUtagSAFEARRAY@@@Z',
    2064:'?Copy@CPtrArray@@QAEXABV1@@Z',
    2065:'?Copy@CStringArray@@QAEXABV1@@Z',
    2066:'?Copy@CUIntArray@@QAEXABV1@@Z',
    2067:'?Copy@CWordArray@@QAEXABV1@@Z',
    2068:'?CopyBeforeWrite@CString@@IAEXXZ',
    2069:'?CopyColumnID@CDataSourceControl@@QAEHPAUtagDBCOLUMNID@@PBU2@@Z',
    2070:'?CopyElements@@YGXPAVCOleVariant@@PBV1@H@Z',
    2071:'?CopyElements@@YGXPAVCString@@PBV1@H@Z',
    2072:'?CopyTo@CArchiveStream@@UAGJPAUIStream@@T_ULARGE_INTEGER@@PAT3@2@Z',
    2073:'?CopyToClipboard@COleClientItem@@QAEXH@Z',
    2074:'?CopyToClipboard@COleServerItem@@QAEXH@Z',
    2075:'?CopyValue@CDaoFieldExchange@@SGXPAX0K@Z',
    2076:'?Create@CAnimateCtrl@@QAEHKABUtagRECT@@PAVCWnd@@I@Z',
    2077:'?Create@CAsyncSocket@@QAEHIHJPBD@Z',
    2078:'?Create@CButton@@QAEHPBDKABUtagRECT@@PAVCWnd@@I@Z',
    2079:'?Create@CCheckListBox@@QAEHKABUtagRECT@@PAVCWnd@@I@Z',
    2080:'?Create@CComboBox@@QAEHKABUtagRECT@@PAVCWnd@@I@Z',
    2081:'?Create@CControlFrameWnd@@UAEHPBD@Z',
    2082:'?Create@CDaoDatabase@@UAEXPBD0H@Z',
    2083:'?Create@CDaoQueryDef@@UAEXPBD0@Z',
    2084:'?Create@CDaoTableDef@@UAEXPBDJ00@Z',
    2085:'?Create@CDaoWorkspace@@UAEXPBD00@Z',
    2086:'?Create@CDialog@@QAEHPBDPAVCWnd@@@Z',
    2087:'?Create@CDialogBar@@QAEHPAVCWnd@@PBDII@Z',
    2088:'?Create@CDockBar@@QAEHPAVCWnd@@KI@Z',
    2089:'?Create@CEdit@@QAEHKABUtagRECT@@PAVCWnd@@I@Z',
    2090:'?Create@CFindReplaceDialog@@QAEHHPBD0KPAVCWnd@@@Z',
    2091:'?Create@CFormView@@MAEHPBD0KABUtagRECT@@PAVCWnd@@IPAUCCreateContext@@@Z',
    2092:'?Create@CFrameWnd@@QAEHPBD0KABUtagRECT@@PAVCWnd@@0KPAUCCreateContext@@@Z',
    2093:'?Create@CHeaderCtrl@@QAEHKABUtagRECT@@PAVCWnd@@I@Z',
    2094:'?Create@CHotKeyCtrl@@QAEHKABUtagRECT@@PAVCWnd@@I@Z',
    2095:'?Create@CImageList@@QAEHAAV1@H0HHH@Z',
    2096:'?Create@CImageList@@QAEHHHIHH@Z',
    2097:'?Create@CImageList@@QAEHIHHK@Z',
    2098:'?Create@CImageList@@QAEHPBDHHK@Z',
    2099:'?Create@CListBox@@QAEHKABUtagRECT@@PAVCWnd@@I@Z',
    2100:'?Create@CListCtrl@@QAEHKABUtagRECT@@PAVCWnd@@I@Z',
    2101:'?Create@CMDIChildWnd@@UAEHPBD0KABUtagRECT@@PAVCMDIFrameWnd@@PAUCCreateContext@@@Z',
    2102:'?Create@CMiniDockFrameWnd@@UAEHPAVCWnd@@K@Z',
    2103:'?Create@CMiniFrameWnd@@QAEHPBD0KABUtagRECT@@PAVCWnd@@I@Z',
    2104:'?Create@COleResizeBar@@QAEHPAVCWnd@@KI@Z',
    2105:'?Create@COleSafeArray@@QAEXGKPAK@Z',
    2106:'?Create@COleSafeArray@@QAEXGKPAUtagSAFEARRAYBOUND@@@Z',
    2107:'?Create@CPlex@@SGPAU1@AAPAU1@II@Z',
    2108:'?Create@CProgressCtrl@@QAEHKABUtagRECT@@PAVCWnd@@I@Z',
    2109:'?Create@CPropertySheet@@QAEHPAVCWnd@@KK@Z',
    2110:'?Create@CReflectorWnd@@QAEHABVCRect@@PAUHWND__@@@Z',
    2111:'?Create@CRichEditCtrl@@QAEHKABUtagRECT@@PAVCWnd@@I@Z',
    2112:'?Create@CScrollBar@@QAEHKABUtagRECT@@PAVCWnd@@I@Z',
    2113:'?Create@CSliderCtrl@@QAEHKABUtagRECT@@PAVCWnd@@I@Z',
    2114:'?Create@CSpinButtonCtrl@@QAEHKABUtagRECT@@PAVCWnd@@I@Z',
    2115:'?Create@CSplitterWnd@@QAEHPAVCWnd@@HHUtagSIZE@@PAUCCreateContext@@KI@Z',
    2116:'?Create@CStatic@@QAEHPBDKABUtagRECT@@PAVCWnd@@I@Z',
    2117:'?Create@CStatusBar@@QAEHPAVCWnd@@KI@Z',
    2118:'?Create@CStatusBarCtrl@@QAEHKABUtagRECT@@PAVCWnd@@I@Z',
    2119:'?Create@CTabCtrl@@QAEHKABUtagRECT@@PAVCWnd@@I@Z',
    2120:'?Create@CToolBar@@QAEHPAVCWnd@@KI@Z',
    2121:'?Create@CToolBarCtrl@@QAEHKABUtagRECT@@PAVCWnd@@I@Z',
    2122:'?Create@CToolTipCtrl@@QAEHPAVCWnd@@K@Z',
    2123:'?Create@CTreeCtrl@@QAEHKABUtagRECT@@PAVCWnd@@I@Z',
    2124:'?Create@CWnd@@UAEHPBD0KABUtagRECT@@PAV1@IPAUCCreateContext@@@Z',
    2125:'?CreateBindContext@CMonikerFile@@IAEPAUIBindCtx@@PAVCFileException@@@Z',
    2126:'?CreateBindHost@CMonikerFile@@KAPAUIBindHost@@XZ',
    2127:'?CreateClient@CMDIFrameWnd@@UAEHPAUtagCREATESTRUCTA@@PAVCMenu@@@Z',
    2128:'?CreateClientItem@CRichEditDoc@@UBEPAVCRichEditCntrItem@@PAU_reobject@@@Z',
    2129:'?CreateCloneFrom@COleClientItem@@QAEHPBV1@@Z',
    2130:'?CreateCommon@CSplitterWnd@@IAEHPAVCWnd@@UtagSIZE@@KI@Z',
    2131:'?CreateConnectionArray@CConnectionPoint@@QAEXXZ',
    2132:'?CreateContainer@COccManager@@UAEPAVCOleControlContainer@@PAVCWnd@@@Z',
    2133:'?CreateControl@COleControlContainer@@QAEHPAVCWnd@@ABU_GUID@@PBDKABUtagRECT@@IPAVCFile@@HPAGPAPAVCOleControlSite@@@Z',
    2134:'?CreateControl@COleControlSite@@QAEJPAVCWnd@@ABU_GUID@@PBDKABUtagRECT@@IPAVCFile@@HPAG@Z',
    2135:'?CreateControl@CWnd@@QAEHABU_GUID@@PBDKABUtagRECT@@PAV1@IPAVCFile@@HPAG@Z',
    2136:'?CreateControl@CWnd@@QAEHPBD0KABUtagRECT@@PAV1@IPAVCFile@@HPAG@Z',
    2137:'?CreateControlWindow@COleControl@@MAEHPAUHWND__@@ABVCRect@@PBUtagRECT@@@Z',
    2138:'?CreateDIBPatternBrush@CBrush@@QAEHPAXI@Z',
    2139:'?CreateDirectoryA@CFtpConnection@@QAEHPBD@Z',
    2140:'?CreateDispatch@COleDispatchDriver@@QAEHABU_GUID@@PAVCOleException@@@Z',
    2141:'?CreateDispatch@COleDispatchDriver@@QAEHPBDPAVCOleException@@@Z',
    2142:'?CreateDlg@CWnd@@IAEHPBDPAV1@@Z',
    2143:'?CreateDlgControl@COccManager@@IAEPAUHWND__@@PAVCWnd@@PAU2@HPAUDLGITEMTEMPLATE@@GPAEK@Z',
    2144:'?CreateDlgControls@COccManager@@UAEHPAVCWnd@@PAXPAU_AFX_OCC_DIALOG_INFO@@@Z',
    2145:'?CreateDlgControls@COccManager@@UAEHPAVCWnd@@PBDPAU_AFX_OCC_DIALOG_INFO@@@Z',
    2146:'?CreateDlgIndirect@CWnd@@IAEHPBUDLGTEMPLATE@@PAV1@@Z',
    2147:'?CreateDlgIndirect@CWnd@@IAEHPBUDLGTEMPLATE@@PAV1@PAUHINSTANCE__@@@Z',
    2148:'?CreateDragImage@CListCtrl@@QAEPAVCImageList@@HPAUtagPOINT@@@Z',
    2149:'?CreateDragImage@CTreeCtrl@@QAEPAVCImageList@@PAU_TREEITEM@@@Z',
    2150:'?CreateEmpty@CPictureHolder@@QAEHXZ',
    2151:'?CreateEx@CMiniFrameWnd@@QAEHKPBD0KABUtagRECT@@PAVCWnd@@I@Z',
    2152:'?CreateEx@CWnd@@QAEHKPBD0KHHHHPAUHWND__@@PAUHMENU__@@PAX@Z',
    2153:'?CreateField@CDaoTableDef@@QAEXAAUCDaoFieldInfo@@@Z',
    2154:'?CreateField@CDaoTableDef@@QAEXPBDFJJ@Z',
    2155:'?CreateFloatingFrame@CFrameWnd@@QAEPAVCMiniDockFrameWnd@@K@Z',
    2156:'?CreateFrameWindow@COleControl@@MAEPAVCControlFrameWnd@@XZ',
    2157:'?CreateFromBitmap@CPictureHolder@@QAEHI@Z',
    2158:'?CreateFromBitmap@CPictureHolder@@QAEHPAUHBITMAP__@@PAUHPALETTE__@@H@Z',
    2159:'?CreateFromBitmap@CPictureHolder@@QAEHPAVCBitmap@@PAVCPalette@@H@Z',
    2160:'?CreateFromClipboard@COleClientItem@@QAEHW4tagOLERENDER@@GPAUtagFORMATETC@@@Z',
    2161:'?CreateFromData@COleClientItem@@QAEHPAVCOleDataObject@@W4tagOLERENDER@@GPAUtagFORMATETC@@@Z',
    2162:'?CreateFromFile@COleClientItem@@QAEHPBDABU_GUID@@W4tagOLERENDER@@GPAUtagFORMATETC@@@Z',
    2163:'?CreateFromIcon@CPictureHolder@@QAEHI@Z',
    2164:'?CreateFromIcon@CPictureHolder@@QAEHPAUHICON__@@H@Z',
    2165:'?CreateFromMetafile@CPictureHolder@@QAEHPAUHMETAFILE__@@HHH@Z',
    2166:'?CreateIndex@CDaoTableDef@@QAEXAAUCDaoIndexInfo@@@Z',
    2167:'?CreateIndirect@CDialog@@IAEHPAXPAVCWnd@@PAUHINSTANCE__@@@Z',
    2168:'?CreateIndirect@CDialog@@IAEHPBUDLGTEMPLATE@@PAVCWnd@@PAXPAUHINSTANCE__@@@Z',
    2169:'?CreateIndirect@CDialog@@QAEHPAXPAVCWnd@@@Z',
    2170:'?CreateIndirect@CDialog@@QAEHPBUDLGTEMPLATE@@PAVCWnd@@PAX@Z',
    2171:'?CreateInPlaceFrame@COleServerDoc@@MAEPAVCOleIPFrameWnd@@PAVCWnd@@@Z',
    2172:'?CreateInstance@AFX_COM@@QAEJABU_GUID@@PAUIUnknown@@0PAPAX@Z',
    2173:'?CreateInstance@XClassFactory@COleObjectFactory@@UAGJPAUIUnknown@@ABU_GUID@@PAPAX@Z',
    2174:'?CreateInstanceLic@XClassFactory@COleObjectFactory@@UAGJPAUIUnknown@@0ABU_GUID@@PAGPAPAX@Z',
    2175:'?CreateItem@COleInsertDialog@@QAEHPAVCOleClientItem@@@Z',
    2176:'?CreateItem@COlePasteSpecialDialog@@QAEHPAVCOleClientItem@@@Z',
    2177:'?CreateLinkFromClipboard@COleClientItem@@QAEHW4tagOLERENDER@@GPAUtagFORMATETC@@@Z',
    2178:'?CreateLinkFromData@COleClientItem@@QAEHPAVCOleDataObject@@W4tagOLERENDER@@GPAUtagFORMATETC@@@Z',
    2179:'?CreateLinkFromFile@COleClientItem@@QAEHPBDW4tagOLERENDER@@GPAUtagFORMATETC@@@Z',
    2180:'?CreateLocator@CGopherConnection@@QAE?AVCGopherLocator@@PBD0K@Z',
    2181:'?CreateLocator@CGopherConnection@@SA?AVCGopherLocator@@PBD@Z',
    2182:'?CreateLocator@CGopherConnection@@SA?AVCGopherLocator@@PBD00KG@Z',
    2183:'?CreateMemoryStream@COleStreamFile@@QAEHPAVCFileException@@@Z',
    2184:'?CreateNewDocument@CDocTemplate@@UAEPAVCDocument@@XZ',
    2185:'?CreateNewFrame@CDocTemplate@@UAEPAVCFrameWnd@@PAVCDocument@@PAV2@@Z',
    2186:'?CreateNewItem@COleClientItem@@QAEHABU_GUID@@W4tagOLERENDER@@GPAUtagFORMATETC@@@Z',
    2187:'?CreateObject@?$CProcessLocal@U_AFX_PROPPAGEFONTINFO@@@@SGPAVCNoTrackObject@@XZ',
    2188:'?CreateObject@?$CProcessLocal@V_AFX_BASE_MODULE_STATE@@@@SGPAVCNoTrackObject@@XZ',
    2189:'?CreateObject@?$CProcessLocal@V_AFX_CHECKLIST_STATE@@@@SGPAVCNoTrackObject@@XZ',
    2190:'?CreateObject@?$CProcessLocal@V_AFX_COLOR_STATE@@@@SGPAVCNoTrackObject@@XZ',
    2191:'?CreateObject@?$CProcessLocal@V_AFX_CTL3D_STATE@@@@SGPAVCNoTrackObject@@XZ',
    2192:'?CreateObject@?$CProcessLocal@V_AFX_DB_STATE@@@@SGPAVCNoTrackObject@@XZ',
    2193:'?CreateObject@?$CProcessLocal@V_AFX_EDIT_STATE@@@@SGPAVCNoTrackObject@@XZ',
    2194:'?CreateObject@?$CProcessLocal@V_AFX_EXTDLL_STATE@@@@SGPAVCNoTrackObject@@XZ=_AfxBinaryCompatibleStubFunction',
    2195:'?CreateObject@?$CProcessLocal@V_AFX_MAIL_STATE@@@@SGPAVCNoTrackObject@@XZ',
    2196:'?CreateObject@?$CProcessLocal@V_AFX_OLE_STATE@@@@SGPAVCNoTrackObject@@XZ',
    2197:'?CreateObject@?$CProcessLocal@V_AFX_RICHEDIT_STATE@@@@SGPAVCNoTrackObject@@XZ',
    2198:'?CreateObject@?$CProcessLocal@V_AFX_SOCK_STATE@@@@SGPAVCNoTrackObject@@XZ',
    2199:'?CreateObject@?$CProcessLocal@V_AFX_WIN_STATE@@@@SGPAVCNoTrackObject@@XZ',
    2200:'?CreateObject@?$CProcessLocal@VCOccManager@@@@SGPAVCNoTrackObject@@XZ',
    2201:'?CreateObject@?$CThreadLocal@V_AFX_CTL3D_THREAD@@@@SGPAVCNoTrackObject@@XZ',
    2202:'?CreateObject@?$CThreadLocal@V_AFX_THREAD_STATE@@@@SGPAVCNoTrackObject@@XZ',
    2203:'?CreateObject@?$CThreadLocal@V_AFXCTL_AMBIENT_CACHE@@@@SGPAVCNoTrackObject@@XZ',
    2204:'?CreateObject@?$CThreadLocal@VAFX_MODULE_THREAD_STATE@@@@SGPAVCNoTrackObject@@XZ',
    2205:'?CreateObject@CByteArray@@SGPAVCObject@@XZ',
    2206:'?CreateObject@CColorPropPage@@SGPAVCObject@@XZ',
    2207:'?CreateObject@CDC@@SGPAVCObject@@XZ',
    2208:'?CreateObject@CDocItem@@SGPAVCObject@@XZ',
    2209:'?CreateObject@CDockState@@SGPAVCObject@@XZ',
    2210:'?CreateObject@CDWordArray@@SGPAVCObject@@XZ',
    2211:'?CreateObject@CEditView@@SGPAVCObject@@XZ',
    2212:'?CreateObject@CFontPropPage@@SGPAVCObject@@XZ',
    2213:'?CreateObject@CFrameWnd@@SGPAVCObject@@XZ',
    2214:'?CreateObject@CGdiObject@@SGPAVCObject@@XZ',
    2215:'?CreateObject@CImageList@@SGPAVCObject@@XZ',
    2216:'?CreateObject@CListView@@SGPAVCObject@@XZ',
    2217:'?CreateObject@CMapStringToOb@@SGPAVCObject@@XZ',
    2218:'?CreateObject@CMapStringToString@@SGPAVCObject@@XZ',
    2219:'?CreateObject@CMapWordToOb@@SGPAVCObject@@XZ',
    2220:'?CreateObject@CMDIChildWnd@@SGPAVCObject@@XZ',
    2221:'?CreateObject@CMDIFrameWnd@@SGPAVCObject@@XZ',
    2222:'?CreateObject@CMenu@@SGPAVCObject@@XZ',
    2223:'?CreateObject@CMiniDockFrameWnd@@SGPAVCObject@@XZ',
    2224:'?CreateObject@CMiniFrameWnd@@SGPAVCObject@@XZ',
    2225:'?CreateObject@CObArray@@SGPAVCObject@@XZ',
    2226:'?CreateObject@CObList@@SGPAVCObject@@XZ',
    2227:'?CreateObject@COleDocIPFrameWnd@@SGPAVCObject@@XZ',
    2228:'?CreateObject@COleIPFrameWnd@@SGPAVCObject@@XZ',
    2229:'?CreateObject@CPicturePropPage@@SGPAVCObject@@XZ',
    2230:'?CreateObject@CPreviewView@@SGPAVCObject@@XZ',
    2231:'?CreateObject@CRichEditCntrItem@@SGPAVCObject@@XZ',
    2232:'?CreateObject@CRichEditView@@SGPAVCObject@@XZ',
    2233:'?CreateObject@CRuntimeClass@@QAEPAVCObject@@XZ',
    2234:'?CreateObject@CStringArray@@SGPAVCObject@@XZ',
    2235:'?CreateObject@CStringList@@SGPAVCObject@@XZ',
    2236:'?CreateObject@CTreeView@@SGPAVCObject@@XZ',
    2237:'?CreateObject@CWnd@@SGPAVCObject@@XZ',
    2238:'?CreateObject@CWordArray@@SGPAVCObject@@XZ',
    2239:'?CreateOleFont@COleControlContainer@@QAEXPAVCFont@@@Z',
    2240:'?CreateOleFrame@CDocTemplate@@QAEPAVCFrameWnd@@PAVCWnd@@PAVCDocument@@H@Z',
    2241:'?CreateOneDim@COleSafeArray@@QAEXGKPBXJ@Z',
    2242:'?CreateOrLoad@COleControlSite@@IAEJABU_GUID@@PAVCFile@@HPAG@Z',
    2243:'?CreatePointFont@CFont@@QAEHHPBDPAVCDC@@@Z',
    2244:'?CreatePointFontIndirect@CFont@@QAEHPBUtagLOGFONTA@@PAVCDC@@@Z',
    2245:'?CreatePrinterDC@CPageSetupDialog@@QAEPAUHDC__@@XZ',
    2246:'?CreatePrinterDC@CPrintDialog@@QAEPAUHDC__@@XZ',
    2247:'?CreatePrinterDC@CWinApp@@QAEHAAVCDC@@@Z',
    2248:'?CreateRelation@CDaoDatabase@@QAEXAAUCDaoRelationInfo@@@Z',
    2249:'?CreateRelation@CDaoDatabase@@QAEXPBD00J00@Z',
    2250:'?CreateScrollBarCtrl@CSplitterWnd@@UAEHKI@Z',
    2251:'?CreateSite@COccManager@@UAEPAVCOleControlSite@@PAVCOleControlContainer@@@Z',
    2252:'?CreateStatic@CSplitterWnd@@QAEHPAVCWnd@@HHKI@Z',
    2253:'?CreateStaticFromClipboard@COleClientItem@@QAEHW4tagOLERENDER@@GPAUtagFORMATETC@@@Z',
    2254:'?CreateStaticFromData@COleClientItem@@QAEHPAVCOleDataObject@@W4tagOLERENDER@@GPAUtagFORMATETC@@@Z',
    2255:'?CreateStream@COleStreamFile@@QAEHPAUIStorage@@PBDKPAVCFileException@@@Z',
    2256:'?CreateThread@CWinThread@@QAEHKIPAU_SECURITY_ATTRIBUTES@@@Z',
    2257:'?CreateTracker@COleControl@@IAEXHH@Z',
    2258:'?CreateView@CFrameWnd@@QAEPAVCWnd@@PAUCCreateContext@@I@Z',
    2259:'?CreateView@CSplitterWnd@@UAEHHHPAUCRuntimeClass@@UtagSIZE@@PAUCCreateContext@@@Z',
    2260:'?CreateView@XOleDocument@CDocObjectServer@@UAGJPAUIOleInPlaceSite@@PAUIStream@@KPAPAUIOleDocumentView@@@Z',
    2261:'?CreateWindowForSubclassedControl@COleControl@@IAEXXZ',
    2262:'?DAdvise@XDataObject@COleControl@@UAGJPAUtagFORMATETC@@KPAUIAdviseSink@@PAK@Z',
    2263:'?DAdvise@XDataObject@COleDataSource@@UAGJPAUtagFORMATETC@@KPAUIAdviseSink@@PAK@Z',
    2264:'?DAdvise@XDataObject@COleServerDoc@@UAGJPAUtagFORMATETC@@KPAUIAdviseSink@@PAK@Z',
    2265:'?DAdvise@XDataObject@COleServerItem@@UAGJPAUtagFORMATETC@@KPAUIAdviseSink@@PAK@Z',
    2266:'?DDP_Check@@YGXPAVCDataExchange@@HAAHPBD@Z',
    2267:'?DDP_EndCheck@@YGXPAVCDataExchange@@HPAHPBD@Z',
    2268:'?DDP_EndRadio@@YGXPAVCDataExchange@@HPAHPBD@Z',
    2269:'?DDP_EndText@@YGXPAVCDataExchange@@HPAEPBD@Z',
    2270:'?DDP_EndText@@YGXPAVCDataExchange@@HPAFPBD@Z',
    2271:'?DDP_EndText@@YGXPAVCDataExchange@@HPAHPBD@Z',
    2272:'?DDP_EndText@@YGXPAVCDataExchange@@HPAIPBD@Z',
    2273:'?DDP_EndText@@YGXPAVCDataExchange@@HPAJPBD@Z',
    2274:'?DDP_EndText@@YGXPAVCDataExchange@@HPAKPBD@Z',
    2275:'?DDP_EndText@@YGXPAVCDataExchange@@HPAMPBD@Z',
    2276:'?DDP_EndText@@YGXPAVCDataExchange@@HPANPBD@Z',
    2277:'?DDP_EndText@@YGXPAVCDataExchange@@HPAVCString@@PBD@Z',
    2278:'?DDP_PostProcessing@@YGXPAVCDataExchange@@@Z',
    2279:'?DDP_Radio@@YGXPAVCDataExchange@@HAAHPBD@Z',
    2280:'?DDP_Text@@YGXPAVCDataExchange@@HAAEPBD@Z',
    2281:'?DDP_Text@@YGXPAVCDataExchange@@HAAFPBD@Z',
    2282:'?DDP_Text@@YGXPAVCDataExchange@@HAAHPBD@Z',
    2283:'?DDP_Text@@YGXPAVCDataExchange@@HAAIPBD@Z',
    2284:'?DDP_Text@@YGXPAVCDataExchange@@HAAJPBD@Z',
    2285:'?DDP_Text@@YGXPAVCDataExchange@@HAAKPBD@Z',
    2286:'?DDP_Text@@YGXPAVCDataExchange@@HAAMPBD@Z',
    2287:'?DDP_Text@@YGXPAVCDataExchange@@HAANPBD@Z',
    2288:'?DDP_Text@@YGXPAVCDataExchange@@HAAVCString@@PBD@Z',
    2289:'?DDV_MaxChars@@YGXPAVCDataExchange@@ABVCString@@H@Z',
    2290:'?DDV_MinMaxByte@@YGXPAVCDataExchange@@EEE@Z',
    2291:'?DDV_MinMaxDouble@@YGXPAVCDataExchange@@ABNNN@Z',
    2292:'?DDV_MinMaxDWord@@YGXPAVCDataExchange@@KKK@Z',
    2293:'?DDV_MinMaxFloat@@YGXPAVCDataExchange@@ABMMM@Z',
    2294:'?DDV_MinMaxInt@@YGXPAVCDataExchange@@HHH@Z',
    2295:'?DDV_MinMaxLong@@YGXPAVCDataExchange@@JJJ@Z',
    2296:'?DDV_MinMaxShort@@YGXPAVCDataExchange@@FFF@Z',
    2297:'?DDV_MinMaxUInt@@YGXPAVCDataExchange@@III@Z',
    2298:'?DDX_CBIndex@@YGXPAVCDataExchange@@HAAH@Z',
    2299:'?DDX_CBString@@YGXPAVCDataExchange@@HAAVCString@@@Z',
    2300:'?DDX_CBStringExact@@YGXPAVCDataExchange@@HAAVCString@@@Z',
    2301:'?DDX_Check@@YGXPAVCDataExchange@@HAAH@Z',
    2302:'?DDX_Control@@YGXPAVCDataExchange@@HAAVCWnd@@@Z',
    2303:'?DDX_FieldCBIndex@@YGXPAVCDataExchange@@HAAHPAVCDaoRecordset@@@Z',
    2304:'?DDX_FieldCBIndex@@YGXPAVCDataExchange@@HAAHPAVCRecordset@@@Z',
    2305:'?DDX_FieldCBString@@YGXPAVCDataExchange@@HAAVCString@@PAVCDaoRecordset@@@Z',
    2306:'?DDX_FieldCBString@@YGXPAVCDataExchange@@HAAVCString@@PAVCRecordset@@@Z',
    2307:'?DDX_FieldCBStringExact@@YGXPAVCDataExchange@@HAAVCString@@PAVCDaoRecordset@@@Z',
    2308:'?DDX_FieldCBStringExact@@YGXPAVCDataExchange@@HAAVCString@@PAVCRecordset@@@Z',
    2309:'?DDX_FieldCheck@@YGXPAVCDataExchange@@HAAHPAVCDaoRecordset@@@Z',
    2310:'?DDX_FieldCheck@@YGXPAVCDataExchange@@HAAHPAVCRecordset@@@Z',
    2311:'?DDX_FieldLBIndex@@YGXPAVCDataExchange@@HAAHPAVCDaoRecordset@@@Z',
    2312:'?DDX_FieldLBIndex@@YGXPAVCDataExchange@@HAAHPAVCRecordset@@@Z',
    2313:'?DDX_FieldLBString@@YGXPAVCDataExchange@@HAAVCString@@PAVCDaoRecordset@@@Z',
    2314:'?DDX_FieldLBString@@YGXPAVCDataExchange@@HAAVCString@@PAVCRecordset@@@Z',
    2315:'?DDX_FieldLBStringExact@@YGXPAVCDataExchange@@HAAVCString@@PAVCDaoRecordset@@@Z',
    2316:'?DDX_FieldLBStringExact@@YGXPAVCDataExchange@@HAAVCString@@PAVCRecordset@@@Z',
    2317:'?DDX_FieldRadio@@YGXPAVCDataExchange@@HAAHPAVCDaoRecordset@@@Z',
    2318:'?DDX_FieldRadio@@YGXPAVCDataExchange@@HAAHPAVCRecordset@@@Z',
    2319:'?DDX_FieldScroll@@YGXPAVCDataExchange@@HAAHPAVCDaoRecordset@@@Z',
    2320:'?DDX_FieldScroll@@YGXPAVCDataExchange@@HAAHPAVCRecordset@@@Z',
    2321:'?DDX_FieldText@@YGXPAVCDataExchange@@HAAEPAVCDaoRecordset@@@Z',
    2322:'?DDX_FieldText@@YGXPAVCDataExchange@@HAAEPAVCRecordset@@@Z',
    2323:'?DDX_FieldText@@YGXPAVCDataExchange@@HAAFPAVCDaoRecordset@@@Z',
    2324:'?DDX_FieldText@@YGXPAVCDataExchange@@HAAHPAVCDaoRecordset@@@Z',
    2325:'?DDX_FieldText@@YGXPAVCDataExchange@@HAAHPAVCRecordset@@@Z',
    2326:'?DDX_FieldText@@YGXPAVCDataExchange@@HAAIPAVCRecordset@@@Z',
    2327:'?DDX_FieldText@@YGXPAVCDataExchange@@HAAJPAVCDaoRecordset@@@Z',
    2328:'?DDX_FieldText@@YGXPAVCDataExchange@@HAAJPAVCRecordset@@@Z',
    2329:'?DDX_FieldText@@YGXPAVCDataExchange@@HAAKPAVCDaoRecordset@@@Z',
    2330:'?DDX_FieldText@@YGXPAVCDataExchange@@HAAKPAVCRecordset@@@Z',
    2331:'?DDX_FieldText@@YGXPAVCDataExchange@@HAAMPAVCDaoRecordset@@@Z',
    2332:'?DDX_FieldText@@YGXPAVCDataExchange@@HAAMPAVCRecordset@@@Z',
    2333:'?DDX_FieldText@@YGXPAVCDataExchange@@HAANPAVCDaoRecordset@@@Z',
    2334:'?DDX_FieldText@@YGXPAVCDataExchange@@HAANPAVCRecordset@@@Z',
    2335:'?DDX_FieldText@@YGXPAVCDataExchange@@HAAVCOleCurrency@@PAVCDaoRecordset@@@Z',
    2336:'?DDX_FieldText@@YGXPAVCDataExchange@@HAAVCOleDateTime@@PAVCDaoRecordset@@@Z',
    2337:'?DDX_FieldText@@YGXPAVCDataExchange@@HAAVCString@@PAVCDaoRecordset@@@Z',
    2338:'?DDX_FieldText@@YGXPAVCDataExchange@@HAAVCString@@PAVCRecordset@@@Z',
    2339:'?DDX_LBIndex@@YGXPAVCDataExchange@@HAAH@Z',
    2340:'?DDX_LBString@@YGXPAVCDataExchange@@HAAVCString@@@Z',
    2341:'?DDX_LBStringExact@@YGXPAVCDataExchange@@HAAVCString@@@Z',
    2342:'?DDX_OCBool@@YGXPAVCDataExchange@@HJAAH@Z',
    2343:'?DDX_OCBoolRO@@YGXPAVCDataExchange@@HJAAH@Z',
    2344:'?DDX_OCColor@@YGXPAVCDataExchange@@HJAAK@Z',
    2345:'?DDX_OCColorRO@@YGXPAVCDataExchange@@HJAAK@Z',
    2346:'?DDX_OCFloat@@YGXPAVCDataExchange@@HJAAM@Z',
    2347:'?DDX_OCFloat@@YGXPAVCDataExchange@@HJAAN@Z',
    2348:'?DDX_OCFloatRO@@YGXPAVCDataExchange@@HJAAM@Z',
    2349:'?DDX_OCFloatRO@@YGXPAVCDataExchange@@HJAAN@Z',
    2350:'?DDX_OCInt@@YGXPAVCDataExchange@@HJAAH@Z',
    2351:'?DDX_OCInt@@YGXPAVCDataExchange@@HJAAJ@Z',
    2352:'?DDX_OCIntRO@@YGXPAVCDataExchange@@HJAAH@Z',
    2353:'?DDX_OCIntRO@@YGXPAVCDataExchange@@HJAAJ@Z',
    2354:'?DDX_OCShort@@YGXPAVCDataExchange@@HJAAF@Z',
    2355:'?DDX_OCShortRO@@YGXPAVCDataExchange@@HJAAF@Z',
    2356:'?DDX_OCText@@YGXPAVCDataExchange@@HJAAVCString@@@Z',
    2357:'?DDX_OCTextRO@@YGXPAVCDataExchange@@HJAAVCString@@@Z',
    2358:'?DDX_Radio@@YGXPAVCDataExchange@@HAAH@Z',
    2359:'?DDX_Scroll@@YGXPAVCDataExchange@@HAAH@Z',
    2360:'?DDX_Text@@YGXPAVCDataExchange@@HAAE@Z',
    2361:'?DDX_Text@@YGXPAVCDataExchange@@HAAF@Z',
    2362:'?DDX_Text@@YGXPAVCDataExchange@@HAAH@Z',
    2363:'?DDX_Text@@YGXPAVCDataExchange@@HAAI@Z',
    2364:'?DDX_Text@@YGXPAVCDataExchange@@HAAJ@Z',
    2365:'?DDX_Text@@YGXPAVCDataExchange@@HAAK@Z',
    2366:'?DDX_Text@@YGXPAVCDataExchange@@HAAM@Z',
    2367:'?DDX_Text@@YGXPAVCDataExchange@@HAAN@Z',
    2368:'?DDX_Text@@YGXPAVCDataExchange@@HAAVCOleCurrency@@@Z',
    2369:'?DDX_Text@@YGXPAVCDataExchange@@HAAVCOleDateTime@@@Z',
    2370:'?DDX_Text@@YGXPAVCDataExchange@@HAAVCString@@@Z',
    2371:'?Deactivate@COleClientItem@@QAEXXZ',
    2372:'?Deactivate@XPropertyPage@COlePropertyPage@@UAGJXZ',
    2373:'?DeactivateAndUndo@COleServerDoc@@QAEHXZ',
    2374:'?DeactivateAndUndo@XOleIPSite@COleClientItem@@UAGJXZ',
    2375:'?DeactivateAndUndo@XOleIPSite@COleControlSite@@UAGJXZ',
    2376:'?DeactivateUI@COleClientItem@@QAEXXZ',
    2377:'?Default@CDaoFieldExchange@@QAEXPBDPAXKK@Z',
    2378:'?Default@CFieldExchange@@QAEXPBDPAXPAJHII@Z',
    2379:'?Default@CWnd@@IAEJXZ',
    2380:'?DeflateRect@CRect@@QAEXHHHH@Z',
    2381:'?DeflateRect@CRect@@QAEXPBUtagRECT@@@Z',
    2382:'?DefWindowProcA@CMDIChildWnd@@MAEJIIJ@Z',
    2383:'?DefWindowProcA@CMDIFrameWnd@@MAEJIIJ@Z',
    2384:'?DefWindowProcA@COleControl@@MAEJIIJ@Z',
    2385:'?DefWindowProcA@CWnd@@MAEJIIJ@Z',
    2386:'?DelayRenderData@COleDataSource@@QAEXGPAUtagFORMATETC@@@Z',
    2387:'?DelayRenderFileData@COleDataSource@@QAEXGPAUtagFORMATETC@@@Z',
    2388:'?DelaySetData@COleDataSource@@QAEXGPAUtagFORMATETC@@@Z',
    2389:'?DelayShow@CControlBar@@UAEXH@Z',
    2390:'?DelayUpdateFrameMenu@CFrameWnd@@UAEXPAUHMENU__@@@Z',
    2391:'?DelayUpdateFrameMenu@CMDIFrameWnd@@UAEXPAUHMENU__@@@Z',
    2392:'?Delete@CDaoRecordset@@UAEXXZ',
    2393:'?Delete@CException@@QAEXXZ',
    2394:'?Delete@COleClientItem@@QAEXH@Z',
    2395:'?Delete@CRecordset@@UAEXXZ',
    2396:'?Delete@CWinThread@@UAEXXZ',
    2397:'?DeleteCacheValue@CDaoFieldExchange@@SGXPAUCDaoFieldCache@@K@Z',
    2398:'?DeleteColumn@CSplitterWnd@@UAEXH@Z',
    2399:'?DeleteContents@CDocument@@UAEXXZ',
    2400:'?DeleteContents@CEditView@@UAEXXZ',
    2401:'?DeleteContents@COleDocument@@UAEXXZ',
    2402:'?DeleteContents@COleServerDoc@@UAEXXZ',
    2403:'?DeleteContents@CRichEditDoc@@UAEXXZ',
    2404:'?DeleteContents@CRichEditView@@UAEXXZ',
    2405:'?DeleteDC@CDC@@QAEHXZ',
    2406:'?DeleteField@CDaoTableDef@@QAEXH@Z',
    2407:'?DeleteField@CDaoTableDef@@QAEXPBD@Z',
    2408:'?DeleteImageList@CImageList@@QAEHXZ',
    2409:'?DeleteIndex@CDaoTableDef@@QAEXH@Z',
    2410:'?DeleteIndex@CDaoTableDef@@QAEXPBD@Z',
    2411:'?DeleteItem@CComboBox@@UAEXPAUtagDELETEITEMSTRUCT@@@Z',
    2412:'?DeleteItem@CFontComboBox@@UAEXPAUtagDELETEITEMSTRUCT@@@Z',
    2413:'?DeleteItem@CListBox@@UAEXPAUtagDELETEITEMSTRUCT@@@Z',
    2414:'?DeleteObject@CGdiObject@@QAEHXZ',
    2415:'?DeleteObject@XRichEditOleCallback@CRichEditView@@UAGJPAUIOleObject@@@Z',
    2416:'?DeleteQueryDef@CDaoDatabase@@QAEXPBD@Z',
    2417:'?DeleteRelation@CDaoDatabase@@QAEXPBD@Z',
    2418:'?DeleteRow@CSplitterWnd@@UAEXH@Z',
    2419:'?DeleteTableDef@CDaoDatabase@@QAEXPBD@Z',
    2420:'?DeleteTemp@CHandleMap@@QAEXXZ',
    2421:'?DeleteTempMap@CDC@@SGXXZ',
    2422:'?DeleteTempMap@CGdiObject@@SGXXZ',
    2423:'?DeleteTempMap@CImageList@@SGXXZ',
    2424:'?DeleteTempMap@CMenu@@SGXXZ',
    2425:'?DeleteTempMap@CWnd@@SGXXZ',
    2426:'?DeleteUnmarkedItems@CRichEditDoc@@IBEXXZ',
    2427:'?DeleteValues@CThreadSlotData@@QAEXPAUCThreadData@@PAUHINSTANCE__@@@Z',
    2428:'?DeleteValues@CThreadSlotData@@QAEXPAUHINSTANCE__@@H@Z',
    2429:'?DeleteView@CSplitterWnd@@UAEXHH@Z',
    2430:'?DelRegTree@CWinApp@@QAEJPAUHKEY__@@ABVCString@@@Z',
    2431:'?DelTool@CToolTipCtrl@@QAEXPAVCWnd@@I@Z',
    2432:'?Destroy@COleSafeArray@@QAEXXZ',
    2433:'?DestroyControl@COleControlSite@@UAEHXZ',
    2434:'?DestroyData@COleSafeArray@@QAEXXZ',
    2435:'?DestroyDescriptor@COleSafeArray@@QAEXXZ',
    2436:'?DestroyDockBars@CFrameWnd@@QAEXXZ',
    2437:'?DestroyInPlaceFrame@COleServerDoc@@MAEXPAVCOleIPFrameWnd@@@Z',
    2438:'?DestroyMenu@CMenu@@QAEHXZ',
    2439:'?DestroySharedMenu@COleControl@@MAEXXZ',
    2440:'?DestroySharedMenu@COleDocIPFrameWnd@@MAEXXZ',
    2441:'?DestroySharedMenu@COleIPFrameWnd@@MAEXXZ',
    2442:'?DestroyToolTipCtrl@CToolTipCtrl@@QAEHXZ',
    2443:'?DestroyTracker@COleControl@@IAEXXZ',
    2444:'?DestroyWindow@CControlBar@@UAEHXZ',
    2445:'?DestroyWindow@CMDIChildWnd@@UAEHXZ',
    2446:'?DestroyWindow@CWnd@@UAEHXZ',
    2447:'?DestructElements@@YGXPAVCOleVariant@@H@Z',
    2448:'?DestructElements@@YGXPAVCString@@H@Z',
    2449:'?Detach@CAsyncSocket@@QAEIXZ',
    2450:'?Detach@CDC@@QAEPAUHDC__@@XZ',
    2451:'?Detach@CDialogTemplate@@QAEPAXXZ',
    2452:'?Detach@CGdiObject@@QAEPAXXZ',
    2453:'?Detach@CImageList@@QAEPAU_IMAGELIST@@XZ',
    2454:'?Detach@CMemFile@@QAEPAEXZ',
    2455:'?Detach@CMenu@@QAEPAUHMENU__@@XZ',
    2456:'?Detach@CMonikerFile@@QAEHPAVCFileException@@@Z',
    2457:'?Detach@COleDataObject@@QAEPAUIDataObject@@XZ',
    2458:'?Detach@COleSafeArray@@QAE?AUtagVARIANT@@XZ',
    2459:'?Detach@COleStreamFile@@QAEPAUIStream@@XZ',
    2460:'?Detach@COleVariant@@QAE?AUtagVARIANT@@XZ',
    2461:'?Detach@CSharedFile@@QAEPAXXZ',
    2462:'?Detach@CWindowlessDC@@QAEPAUHDC__@@XZ',
    2463:'?Detach@CWnd@@QAEPAUHWND__@@XZ',
    2464:'?DetachDispatch@COleDispatchDriver@@QAEPAUIDispatch@@XZ',
    2465:'?DetachHandle@CAsyncSocket@@SGXIH@Z',
    2466:'?DetachWindow@COleControlSite@@IAEXXZ',
    2467:'?DevModeChange@CWinApp@@QAEXPAD@Z',
    2468:'?DFX_Binary@@YGXPAVCDaoFieldExchange@@PBDAAVCByteArray@@HK@Z',
    2469:'?DFX_Bool@@YGXPAVCDaoFieldExchange@@PBDAAHK@Z',
    2470:'?DFX_Byte@@YGXPAVCDaoFieldExchange@@PBDAAEK@Z',
    2471:'?DFX_Currency@@YGXPAVCDaoFieldExchange@@PBDAAVCOleCurrency@@K@Z',
    2472:'?DFX_DateTime@@YGXPAVCDaoFieldExchange@@PBDAAVCOleDateTime@@K@Z',
    2473:'?DFX_Double@@YGXPAVCDaoFieldExchange@@PBDAANK@Z',
    2474:'?DFX_Long@@YGXPAVCDaoFieldExchange@@PBDAAJK@Z',
    2475:'?DFX_LongBinary@@YGXPAVCDaoFieldExchange@@PBDAAVCLongBinary@@KK@Z',
    2476:'?DFX_Short@@YGXPAVCDaoFieldExchange@@PBDAAFK@Z',
    2477:'?DFX_Single@@YGXPAVCDaoFieldExchange@@PBDAAMK@Z',
    2478:'?DFX_Text@@YGXPAVCDaoFieldExchange@@PBDAAVCString@@HK@Z',
    2479:'?DidEvent@XNotifyDBEvents@COleControlSite@@UAGJKKQAUtagDBNOTIFYREASON@@@Z',
    2480:'?DiscardUndoState@COleServerDoc@@QAEHXZ',
    2481:'?DiscardUndoState@XOleIPSite@COleClientItem@@UAGJXZ',
    2482:'?DiscardUndoState@XOleIPSite@COleControlSite@@UAGJXZ',
    2483:'?Disconnect@COleDispatchImpl@@UAEXXZ',
    2484:'?DisconnectSink@COleControlSite@@IAEXABU_GUID@@K@Z',
    2485:'?DisconnectViews@CDocument@@QAEXXZ',
    2486:'?dispatchMap@CCmdTarget@@1UAFX_DISPMAP@@B',
    2487:'?DispatchThreadMessage@CWinThread@@IAEXPAUtagMSG@@@Z',
    2488:'?DisplayError@COleControl@@UAEXJPBD00I@Z',
    2489:'?DoBeginPrinting@CDocObjectServer@@IAEXPAVCView@@PAVCDC@@PAUCPrintInfo@@@Z',
    2490:'?DoBulkFieldExchange@CRecordset@@UAEXPAVCFieldExchange@@@Z',
    2491:'?DoCallBack@CAsyncSocket@@SGXIJ@Z',
    2492:'?DoChangeIcon@COleChangeIconDialog@@QAEHPAVCOleClientItem@@@Z',
    2493:'?DockControlBar@CDockBar@@QAEXPAVCControlBar@@PBUtagRECT@@@Z',
    2494:'?DockControlBar@CFrameWnd@@QAEXPAVCControlBar@@IPBUtagRECT@@@Z',
    2495:'?DockControlBar@CFrameWnd@@QAEXPAVCControlBar@@PAVCDockBar@@PBUtagRECT@@@Z',
    2496:'?DoClick@COleControl@@QAEXXZ',
    2497:'?DoContextSensitiveHelp@COleFrameHook@@QAEHH@Z',
    2498:'?DoConvert@COleConvertDialog@@QAEHPAVCOleClientItem@@@Z',
    2499:'?DoDataExchange@CColorPropPage@@MAEXPAVCDataExchange@@@Z',
    2500:'?DoDataExchange@CFontPropPage@@MAEXPAVCDataExchange@@@Z',
    2501:'?DoDataExchange@CPicturePropPage@@MAEXPAVCDataExchange@@@Z',
    2502:'?DoDataExchange@CWnd@@MAEXPAVCDataExchange@@@Z',
    2503:'?DoDragDrop@COleClientItem@@QAEKPBUtagRECT@@VCPoint@@HK0@Z',
    2504:'?DoDragDrop@COleDataSource@@QAEKKPBUtagRECT@@PAVCOleDropSource@@@Z',
    2505:'?DoDragDrop@COleServerItem@@QAEKPBUtagRECT@@VCPoint@@HK0@Z',
    2506:'?DoEnableModeless@COleFrameHook@@QAEHH@Z',
    2507:'?DoEndPrinting@CDocObjectServer@@IAEXPAVCView@@PAVCDC@@PAUCPrintInfo@@@Z',
    2508:'?DoFieldExchange@CDaoRecordset@@UAEXPAVCDaoFieldExchange@@@Z',
    2509:'?DoFieldExchange@CRecordset@@UAEXPAVCFieldExchange@@@Z',
    2510:'?DoFileSave@CDocument@@UAEHXZ',
    2511:'?DoKeyboardSplit@CSplitterWnd@@UAEHXZ',
    2512:'?DoMessageBox@CWinApp@@UAEHPBDII@Z',
    2513:'?DoModal@CColorDialog@@UAEHXZ',
    2514:'?DoModal@CDialog@@UAEHXZ',
    2515:'?DoModal@CFileDialog@@UAEHXZ',
    2516:'?DoModal@CFontDialog@@UAEHXZ',
    2517:'?DoModal@COleBusyDialog@@UAEHXZ',
    2518:'?DoModal@COleChangeIconDialog@@UAEHXZ',
    2519:'?DoModal@COleChangeSourceDialog@@UAEHXZ',
    2520:'?DoModal@COleConvertDialog@@UAEHXZ',
    2521:'?DoModal@COleInsertDialog@@UAEHXZ',
    2522:'?DoModal@COleLinksDialog@@UAEHXZ',
    2523:'?DoModal@COlePasteSpecialDialog@@UAEHXZ',
    2524:'?DoModal@COlePropertiesDialog@@UAEHXZ',
    2525:'?DoModal@COleUpdateDialog@@UAEHXZ',
    2526:'?DoModal@CPageSetupDialog@@UAEHXZ',
    2527:'?DoModal@CPrintDialog@@UAEHXZ',
    2528:'?DoModal@CPropertySheet@@UAEHXZ',
    2529:'?DoOleVerb@CCmdTarget@@QAEHJPAUtagMSG@@PAUHWND__@@PBUtagRECT@@@Z',
    2530:'?DoPaint@CControlBar@@UAEXPAVCDC@@@Z',
    2531:'?DoPaint@CDockBar@@UAEXPAVCDC@@@Z',
    2532:'?DoPaste@CRichEditView@@QAEXAAVCOleDataObject@@GPAX@Z',
    2533:'?DoPrepareDC@CDocObjectServer@@IAEXPAVCView@@PAVCDC@@PAUCPrintInfo@@@Z',
    2534:'?DoPreparePrinting@CDocObjectServer@@IAEHPAVCView@@PAUCPrintInfo@@@Z',
    2535:'?DoPreparePrinting@CView@@QAEHPAUCPrintInfo@@@Z',
    2536:'?DoPrint@CDocObjectServer@@IAEXPAVCView@@PAVCDC@@PAUCPrintInfo@@@Z',
    2537:'?DoPrintDialog@CWinApp@@QAEHPAVCPrintDialog@@@Z',
    2538:'?DoPrintPreview@CView@@QAEHIPAV1@PAUCRuntimeClass@@PAUCPrintPreviewState@@@Z',
    2539:'?DoPromptFileName@CDocManager@@UAEHAAVCString@@IKHPAVCDocTemplate@@@Z',
    2540:'?DoPromptFileName@CWinApp@@QAEHAAVCString@@IKHPAVCDocTemplate@@@Z',
    2541:'?DoPropExchange@COleControl@@UAEXPAVCPropExchange@@@Z',
    2542:'?DoSave@CDocument@@UAEHPBDH@Z',
    2543:'?DoScroll@CSplitterWnd@@UAEHPAVCView@@IH@Z',
    2544:'?DoScrollBy@CSplitterWnd@@UAEHPAVCView@@VCSize@@H@Z',
    2545:'?DoSuperclassPaint@COleControl@@QAEXPAVCDC@@ABVCRect@@@Z',
    2546:'?DoUpdate@CCmdUI@@QAEHPAVCCmdTarget@@H@Z',
    2547:'?DoUpdate@COleCmdUI@@UAEHPAVCCmdTarget@@H@Z',
    2548:'?DoVerb@COleClientItem@@UAEHJPAVCView@@PAUtagMSG@@@Z',
    2549:'?DoVerb@COleControlSite@@UAEJJPAUtagMSG@@@Z',
    2550:'?DoVerb@XOleObject@CDocObjectServer@@UAGJJPAUtagMSG@@PAUIOleClientSite@@JPAUHWND__@@PBUtagRECT@@@Z',
    2551:'?DoVerb@XOleObject@COleControl@@UAGJJPAUtagMSG@@PAUIOleClientSite@@JPAUHWND__@@PBUtagRECT@@@Z',
    2552:'?DoVerb@XOleObject@COleServerDoc@@UAGJJPAUtagMSG@@PAUIOleClientSite@@JPAUHWND__@@PBUtagRECT@@@Z',
    2553:'?DoVerb@XOleObject@COleServerItem@@UAGJJPAUtagMSG@@PAUIOleClientSite@@JPAUHWND__@@PBUtagRECT@@@Z',
    2554:'?DoWaitCursor@CWinApp@@UAEXH@Z',
    2555:'?DoZoom@CPreviewView@@IAEXIVCPoint@@@Z',
    2556:'?DPtoHIMETRIC@CDC@@QBEXPAUtagSIZE@@@Z',
    2557:'?DPtoLP@CDC@@QBEXPAUtagSIZE@@@Z',
    2558:'?DragAcceptFiles@CWnd@@QAEXH@Z',
    2559:'?DragEnter@XDropTarget@COleDropTarget@@UAGJPAUIDataObject@@KU_POINTL@@PAK@Z',
    2560:'?Dragging@CDragListBox@@UAEIVCPoint@@@Z',
    2561:'?DragLeave@XDropTarget@COleDropTarget@@UAGJXZ',
    2562:'?DragOver@XDropTarget@COleDropTarget@@UAGJKU_POINTL@@PAK@Z',
    2563:'?Draw@COleClientItem@@QAEHPAVCDC@@PBUtagRECT@@W4tagDVASPECT@@@Z',
    2564:'?Draw@CRectTracker@@QBEXPAVCDC@@@Z',
    2565:'?Draw@XViewObject@COleControl@@UAGJKJPAXPAUtagDVTARGETDEVICE@@PAUHDC__@@2PBU_RECTL@@3P6GHK@ZK@Z',
    2566:'?Draw3dRect@CDC@@QAEXHHHHKK@Z',
    2567:'?Draw3dRect@CDC@@QAEXPBUtagRECT@@KK@Z',
    2568:'?DrawAllSplitBars@CSplitterWnd@@MAEXPAVCDC@@HH@Z',
    2569:'?DrawBorders@CControlBar@@QAEXPAVCDC@@AAVCRect@@@Z',
    2570:'?DrawContent@COleControl@@IAEXPAVCDC@@AAVCRect@@@Z',
    2571:'?DrawDragRect@CDC@@QAEXPBUtagRECT@@UtagSIZE@@01PAVCBrush@@2@Z',
    2572:'?DrawFocusRect@CDockContext@@QAEXH@Z',
    2573:'?DrawInsert@CDragListBox@@UAEXH@Z',
    2574:'?DrawItem@CBitmapButton@@MAEXPAUtagDRAWITEMSTRUCT@@@Z',
    2575:'?DrawItem@CButton@@UAEXPAUtagDRAWITEMSTRUCT@@@Z',
    2576:'?DrawItem@CCheckListBox@@UAEXPAUtagDRAWITEMSTRUCT@@@Z',
    2577:'?DrawItem@CColorButton@@MAEXPAUtagDRAWITEMSTRUCT@@@Z',
    2578:'?DrawItem@CComboBox@@UAEXPAUtagDRAWITEMSTRUCT@@@Z',
    2579:'?DrawItem@CFontComboBox@@UAEXPAUtagDRAWITEMSTRUCT@@@Z',
    2580:'?DrawItem@CHeaderCtrl@@UAEXPAUtagDRAWITEMSTRUCT@@@Z',
    2581:'?DrawItem@CListBox@@UAEXPAUtagDRAWITEMSTRUCT@@@Z',
    2582:'?DrawItem@CListCtrl@@UAEXPAUtagDRAWITEMSTRUCT@@@Z',
    2583:'?DrawItem@CListView@@UAEXPAUtagDRAWITEMSTRUCT@@@Z',
    2584:'?DrawItem@CMenu@@UAEXPAUtagDRAWITEMSTRUCT@@@Z',
    2585:'?DrawItem@CStatusBar@@UAEXPAUtagDRAWITEMSTRUCT@@@Z',
    2586:'?DrawItem@CStatusBarCtrl@@UAEXPAUtagDRAWITEMSTRUCT@@@Z',
    2587:'?DrawItem@CTabCtrl@@UAEXPAUtagDRAWITEMSTRUCT@@@Z',
    2588:'?DrawMetafile@COleControl@@IAEXPAVCDC@@AAVCRect@@@Z',
    2589:'?DrawSingle@CDragListBox@@QAEXH@Z',
    2590:'?DrawTextA@CDC@@UAEHPBDHPAUtagRECT@@I@Z',
    2591:'?DrawTextA@CMetaFileDC@@UAEHPBDHPAUtagRECT@@I@Z',
    2592:'?DrawTextA@CPreviewDC@@UAEHPBDHPAUtagRECT@@I@Z',
    2593:'?DrawTrackerRect@CRectTracker@@UAEXPBUtagRECT@@PAVCWnd@@PAVCDC@@1@Z',
    2594:'?Drop@XDropTarget@COleDropTarget@@UAGJPAUIDataObject@@KU_POINTL@@PAK@Z',
    2595:'?Dropped@CDragListBox@@UAEXHVCPoint@@@Z',
    2596:'?Dump@CObject@@UBEXAAVCDumpContext@@@Z',
    2597:'?DUnadvise@XDataObject@COleControl@@UAGJK@Z',
    2598:'?DUnadvise@XDataObject@COleDataSource@@UAGJK@Z',
    2599:'?DUnadvise@XDataObject@COleServerDoc@@UAGJK@Z',
    2600:'?DUnadvise@XDataObject@COleServerItem@@UAGJK@Z',
    2601:'?Duplicate@CFile@@UBEPAV1@XZ',
    2602:'?Duplicate@CInternetFile@@UBEPAVCFile@@XZ',
    2603:'?Duplicate@CMemFile@@UBEPAVCFile@@XZ',
    2604:'?Duplicate@COleStreamFile@@UBEPAVCFile@@XZ',
    2605:'?Duplicate@CSocketFile@@UBEPAVCFile@@XZ',
    2606:'?Duplicate@CStdioFile@@UBEPAVCFile@@XZ',
    2607:'?dwStyleDefault@CEditView@@2KB',
    2608:'?Edit@CDaoRecordset@@UAEXXZ',
    2609:'?Edit@CRecordset@@UAEXXZ',
    2610:'?EditProperty@XPropertyPage@COlePropertyPage@@UAGJJ@Z',
    2611:'?EditStreamCallBack@CRichEditView@@SGKKPAEJPAJ@Z',
    2612:'?Empty@CDBException@@QAEXXZ',
    2613:'?Empty@COleDataSource@@QAEXXZ',
    2614:'?Empty@CString@@QAEXXZ',
    2615:'?Enable@CCheckListBox@@QAEXHH@Z',
    2616:'?Enable@CCmdUI@@UAEXH@Z',
    2617:'?Enable@COleCmdUI@@UAEXH@Z',
    2618:'?Enable@CStatusCmdUI@@UAEXH@Z',
    2619:'?Enable@CTestCmdUI@@UAEXH@Z',
    2620:'?Enable@CToolCmdUI@@UAEXH@Z',
    2621:'?Enable3dControls@CWinApp@@IAEHXZ',
    2622:'?EnableAggregation@CCmdTarget@@QAEXXZ',
    2623:'?EnableAutomation@CCmdTarget@@QAEXXZ',
    2624:'?EnableBookmarks@CRecordset@@IAEXXZ',
    2625:'?EnableConnections@CCmdTarget@@QAEXXZ',
    2626:'?EnableDocking@CControlBar@@QAEXK@Z',
    2627:'?EnableDocking@CFrameWnd@@QAEXK@Z',
    2628:'?EnableDSC@COleControlSite@@UAEXXZ',
    2629:'?EnableModeless@CWinApp@@QAEXH@Z',
    2630:'?EnableModeless@XOleInPlaceActiveObject@COleControl@@UAGJH@Z',
    2631:'?EnableModeless@XOleInPlaceActiveObject@COleServerDoc@@UAGJH@Z',
    2632:'?EnableModeless@XOleInPlaceFrame@COleFrameHook@@UAGJH@Z',
    2633:'?EnableModeless@XOleIPFrame@COleControlContainer@@UAGJH@Z',
    2634:'?EnableScrollBarCtrl@CWnd@@QAEXHH@Z',
    2635:'?EnableShellOpen@CWinApp@@IAEXXZ',
    2636:'?EnableSimpleFrame@COleControl@@QAEXXZ',
    2637:'?EnableStackedTabs@CPropertySheet@@QAEXH@Z',
    2638:'?EnableStatusCallback@CInternetSession@@QAEHH@Z',
    2639:'?EnableToolTips@CWnd@@QAEHH@Z',
    2640:'?EnableTypeLib@CCmdTarget@@QAEXXZ',
    2641:'?EnableWindow@COleControlSite@@UAEHH@Z',
    2642:'?EnableWindow@CWnd@@QAEHH@Z',
    2643:'?EndBusyState@COleMessageFilter@@UAEXXZ',
    2644:'?EndDeferErrors@COleLinkingDoc@@QAEJJ@Z',
    2645:'?EndDialog@CDialog@@QAEXH@Z',
    2646:'?EndDialog@CPropertySheet@@QAEXH@Z',
    2647:'?EndDrag@CDockContext@@QAEXXZ',
    2648:'?EndModalLoop@CWnd@@UAEXH@Z',
    2649:'?EndModalState@CFrameWnd@@UAEXXZ',
    2650:'?EndModalState@CWnd@@UAEXXZ',
    2651:'?EndResize@CDockContext@@QAEXXZ',
    2652:'?EndWaitCursor@CCmdTarget@@QAEXXZ',
    2653:'?EnsureClipboardObject@COleDataObject@@QAEXXZ',
    2654:'?EnumAdvise@XOleObject@CDocObjectServer@@UAGJPAPAUIEnumSTATDATA@@@Z',
    2655:'?EnumAdvise@XOleObject@COleControl@@UAGJPAPAUIEnumSTATDATA@@@Z',
    2656:'?EnumAdvise@XOleObject@COleServerDoc@@UAGJPAPAUIEnumSTATDATA@@@Z',
    2657:'?EnumAdvise@XOleObject@COleServerItem@@UAGJPAPAUIEnumSTATDATA@@@Z',
    2658:'?EnumCache@XOleCache@COleControl@@UAGJPAPAUIEnumSTATDATA@@@Z',
    2659:'?EnumChildProc@COlePropertyPage@@KGHPAUHWND__@@J@Z',
    2660:'?EnumConnectionPoints@COleConnPtContainer@@UAGJPAPAUIEnumConnectionPoints@@@Z',
    2661:'?EnumConnections@XConnPt@CConnectionPoint@@UAGJPAPAUIEnumConnections@@@Z',
    2662:'?EnumControls@COlePropertyPage@@KGHPAUHWND__@@J@Z',
    2663:'?EnumDAdvise@XDataObject@COleControl@@UAGJPAPAUIEnumSTATDATA@@@Z',
    2664:'?EnumDAdvise@XDataObject@COleDataSource@@UAGJPAPAUIEnumSTATDATA@@@Z',
    2665:'?EnumDAdvise@XDataObject@COleServerDoc@@UAGJPAPAUIEnumSTATDATA@@@Z',
    2666:'?EnumDAdvise@XDataObject@COleServerItem@@UAGJPAPAUIEnumSTATDATA@@@Z',
    2667:'?EnumFontFamiliesCallBack@CFontPropPage@@KGHPAUtagENUMLOGFONTA@@PAUtagNEWTEXTMETRICA@@HJ@Z',
    2668:'?EnumFontFamiliesCallBack2@CFontPropPage@@KGHPAUtagENUMLOGFONTA@@PAUtagNEWTEXTMETRICA@@HJ@Z',
    2669:'?EnumFormatEtc@XDataObject@COleControl@@UAGJKPAPAUIEnumFORMATETC@@@Z',
    2670:'?EnumFormatEtc@XDataObject@COleDataSource@@UAGJKPAPAUIEnumFORMATETC@@@Z',
    2671:'?EnumFormatEtc@XDataObject@COleServerDoc@@UAGJKPAPAUIEnumFORMATETC@@@Z',
    2672:'?EnumFormatEtc@XDataObject@COleServerItem@@UAGJKPAPAUIEnumFORMATETC@@@Z',
    2673:'?EnumObjects@XOleContainer@COleControlContainer@@UAGJKPAPAUIEnumUnknown@@@Z',
    2674:'?EnumObjects@XOleItemContainer@COleLinkingDoc@@UAGJKPAPAUIEnumUnknown@@@Z',
    2675:'?EnumOleVerbs@CCmdTarget@@QAEHPAPAUIEnumOLEVERB@@@Z',
    2676:'?EnumVerbs@XOleObject@CDocObjectServer@@UAGJPAPAUIEnumOLEVERB@@@Z',
    2677:'?EnumVerbs@XOleObject@COleControl@@UAGJPAPAUIEnumOLEVERB@@@Z',
    2678:'?EnumVerbs@XOleObject@COleServerDoc@@UAGJPAPAUIEnumOLEVERB@@@Z',
    2679:'?EnumVerbs@XOleObject@COleServerItem@@UAGJPAPAUIEnumOLEVERB@@@Z',
    2680:'?EnumViews@XOleDocument@CDocObjectServer@@UAGJPAPAUIEnumOleDocumentViews@@PAPAUIOleDocumentView@@@Z',
    2681:'?EraseNonClient@CControlBar@@QAEXXZ',
    2682:'?ErrnoToException@CFileException@@SGHH@Z',
    2683:'?ErrorDlg@CHttpFile@@QAEKPAVCWnd@@KKPAPAX@Z',
    2684:'?Escape@CDC@@UAEHHHPBDPAX@Z',
    2685:'?Escape@CMetaFileDC@@UAEHHHPBDPAX@Z',
    2686:'?Escape@CPreviewDC@@UAEHHHPBDPAX@Z',
    2687:'?eventMap@COleControl@@1UAFX_EVENTMAP@@B',
    2688:'?eventsinkMap@CCmdTarget@@1UAFX_EVENTSINKMAP@@B',
    2689:'?ExchangeBlobProp@CArchivePropExchange@@UAEHPBDPAPAXPAX@Z',
    2690:'?ExchangeBlobProp@CAsyncPropExchange@@UAEHPBDPAPAXPAX@Z',
    2691:'?ExchangeBlobProp@CPropbagPropExchange@@UAEHPBDPAPAXPAX@Z',
    2692:'?ExchangeBlobProp@CPropsetPropExchange@@UAEHPBDPAPAXPAX@Z',
    2693:'?ExchangeBlobProp@CResetPropExchange@@UAEHPBDPAPAXPAX@Z',
    2694:'?ExchangeExtent@COleControl@@QAEHPAVCPropExchange@@@Z',
    2695:'?ExchangeFontProp@CArchivePropExchange@@UAEHPBDAAVCFontHolder@@PBUtagFONTDESC@@PAUIFontDisp@@@Z',
    2696:'?ExchangeFontProp@CAsyncPropExchange@@UAEHPBDAAVCFontHolder@@PBUtagFONTDESC@@PAUIFontDisp@@@Z',
    2697:'?ExchangeFontProp@CPropbagPropExchange@@UAEHPBDAAVCFontHolder@@PBUtagFONTDESC@@PAUIFontDisp@@@Z',
    2698:'?ExchangeFontProp@CPropsetPropExchange@@UAEHPBDAAVCFontHolder@@PBUtagFONTDESC@@PAUIFontDisp@@@Z',
    2699:'?ExchangeFontProp@CResetPropExchange@@UAEHPBDAAVCFontHolder@@PBUtagFONTDESC@@PAUIFontDisp@@@Z',
    2700:'?ExchangePersistentProp@CArchivePropExchange@@UAEHPBDPAPAUIUnknown@@ABU_GUID@@PAU2@@Z',
    2701:'?ExchangePersistentProp@CAsyncPropExchange@@UAEHPBDPAPAUIUnknown@@ABU_GUID@@PAU2@@Z',
    2702:'?ExchangePersistentProp@CPropbagPropExchange@@UAEHPBDPAPAUIUnknown@@ABU_GUID@@PAU2@@Z',
    2703:'?ExchangePersistentProp@CPropsetPropExchange@@UAEHPBDPAPAUIUnknown@@ABU_GUID@@PAU2@@Z',
    2704:'?ExchangePersistentProp@CResetPropExchange@@UAEHPBDPAPAUIUnknown@@ABU_GUID@@PAU2@@Z',
    2705:'?ExchangeProp@CArchivePropExchange@@UAEHPBDGPAXPBX@Z',
    2706:'?ExchangeProp@CAsyncPropExchange@@UAEHPBDGPAXPBX@Z',
    2707:'?ExchangeProp@CPropbagPropExchange@@UAEHPBDGPAXPBX@Z',
    2708:'?ExchangeProp@CPropsetPropExchange@@UAEHPBDGPAXPBX@Z',
    2709:'?ExchangeProp@CResetPropExchange@@UAEHPBDGPAXPBX@Z',
    2710:'?ExchangeStockProps@COleControl@@QAEXPAVCPropExchange@@@Z',
    2711:'?ExchangeVersion@CAsyncPropExchange@@UAEHAAKKH@Z',
    2712:'?ExchangeVersion@CPropExchange@@UAEHAAKKH@Z',
    2713:'?ExcludeClipRect@CDC@@QAEHHHHH@Z',
    2714:'?ExcludeClipRect@CDC@@QAEHPBUtagRECT@@@Z',
    2715:'?Exec@XOleCommandTarget@CDocObjectServer@@UAGJPBU_GUID@@KKPAUtagVARIANT@@1@Z',
    2716:'?Execute@CDaoDatabase@@QAEXPBDH@Z',
    2717:'?Execute@CDaoQueryDef@@UAEXH@Z',
    2718:'?ExecuteDlgInit@CWnd@@QAEHPAX@Z',
    2719:'?ExecuteDlgInit@CWnd@@QAEHPBD@Z',
    2720:'?ExecuteSetPosUpdate@CRecordset@@IAEXXZ',
    2721:'?ExecuteSQL@CDatabase@@QAEXPBD@Z',
    2722:'?ExecuteUpdateSQL@CRecordset@@IAEXXZ',
    2723:'?ExitHelpMode@CFrameWnd@@UAEXXZ',
    2724:'?ExitInstance@COleControlModule@@UAEHXZ',
    2725:'?ExitInstance@CWinApp@@UAEHXZ',
    2726:'?ExitInstance@CWinThread@@UAEHXZ',
    2727:'?ExternalAddRef@CCmdTarget@@QAEKXZ',
    2728:'?ExternalDisconnect@CCmdTarget@@QAEXXZ',
    2729:'?ExternalQueryInterface@CCmdTarget@@QAEKPBXPAPAX@Z',
    2730:'?ExternalRelease@CCmdTarget@@QAEKXZ',
    2731:'?ExtTextOutA@CDC@@UAEHHHIPBUtagRECT@@PBDIPAH@Z',
    2732:'?ExtTextOutA@CMetaFileDC@@UAEHHHIPBUtagRECT@@PBDIPAH@Z',
    2733:'?ExtTextOutA@CPreviewDC@@UAEHHHIPBUtagRECT@@PBDIPAH@Z',
    2734:'?factory@CColorPropPage@@1VCColorPropPageFactory@1@A',
    2735:'?factory@CFontPropPage@@1VCFontPropPageFactory@1@A',
    2736:'?factory@CPicturePropPage@@1VCPicturePropPageFactory@1@A',
    2737:'?Fail@CDataExchange@@QAEXXZ',
    2738:'?FailedToDo@XNotifyDBEvents@COleControlSite@@UAGJKKQAUtagDBNOTIFYREASON@@@Z',
    2739:'?FetchData@CRecordset@@QAEFGJPAK@Z',
    2740:'?FillBuffer@CArchive@@QAEXI@Z',
    2741:'?FillCache@CDaoRecordset@@QAEXPAJPAVCOleVariant@@@Z',
    2742:'?FillDatabaseInfo@CDaoWorkspace@@IAEXPAUDAODatabase@@AAUCDaoDatabaseInfo@@K@Z',
    2743:'?FillErrorInfo@CDaoException@@IAEXXZ',
    2744:'?FillFacenameList@CFontPropPage@@IAEXXZ',
    2745:'?FillInLogFont@CFontDialog@@QAEKABU_charformat@@@Z',
    2746:'?FillInToolInfo@CToolTipCtrl@@QBEXAAUtagTOOLINFOA@@PAVCWnd@@I@Z',
    2747:'?FillOutsideRect@CScrollView@@QAEXPAVCDC@@PAVCBrush@@@Z',
    2748:'?FillParameterInfo@CDaoQueryDef@@IAEXPAUDAOParameter@@AAUCDaoParameterInfo@@K@Z',
    2749:'?FillPropnameList@CStockPropPage@@IAEXABU_GUID@@HAAVCComboBox@@@Z',
    2750:'?FillQueryDefInfo@CDaoDatabase@@IAEXPAU_DAOQueryDef@@AAUCDaoQueryDefInfo@@K@Z',
    2751:'?FillRelationInfo@CDaoDatabase@@IAEXPAU_DAORelation@@AAUCDaoRelationInfo@@K@Z',
    2752:'?FillSizeList@CFontPropPage@@IAEXXZ',
    2753:'?FillSolidRect@CDC@@QAEXHHHHK@Z',
    2754:'?FillSolidRect@CDC@@QAEXPBUtagRECT@@K@Z',
    2755:'?FillSysColors@CColorPropPage@@IAEXXZ',
    2756:'?FillTableDefInfo@CDaoDatabase@@IAEXPAU_DAOTableDef@@AAUCDaoTableDefInfo@@K@Z',
    2757:'?FillVariant@CDaoFieldExchange@@SGXPAXKPAPAVCOleVariant@@@Z',
    2758:'?FillWorkspaceInfo@CDaoWorkspace@@IAEXPAUDAOWorkspace@@AAUCDaoWorkspaceInfo@@K@Z',
    2759:'?FilterToolTipMessage@CWnd@@QAEXPAUtagMSG@@@Z',
    2760:'?Find@CDaoRecordset@@UAEHJPBD@Z',
    2761:'?Find@CObList@@QBEPAU__POSITION@@PAVCObject@@PAU2@@Z',
    2762:'?Find@CPtrList@@QBEPAU__POSITION@@PAXPAU2@@Z',
    2763:'?Find@CString@@QBEHD@Z',
    2764:'?Find@CString@@QBEHPBD@Z',
    2765:'?Find@CStringList@@QBEPAU__POSITION@@PBDPAU2@@Z',
    2766:'?FindAndSelect@CRichEditView@@QAEJKAAU_findtextexa@@@Z',
    2767:'?FindBar@CDockBar@@QAEHPAVCControlBar@@H@Z',
    2768:'?FindConnectionPoint@COleConnPtContainer@@UAGJABU_GUID@@PAPAUIConnectionPoint@@@Z',
    2769:'?FindDlgItem@@YAPAVCWnd@@PAV1@K@Z=_AfxBinaryCompatibleStubFunction',
    2770:'?FindFile@CFileFind@@UAEHPBDK@Z',
    2771:'?FindFile@CFtpFileFind@@UAEHPBDK@Z',
    2772:'?FindFile@CGopherFileFind@@UAEHAAVCGopherLocator@@PBDK@Z',
    2773:'?FindFile@CGopherFileFind@@UAEHPBDK@Z',
    2774:'?FindFirst@CDaoRecordset@@QAEHPBD@Z',
    2775:'?FindIndex@CObList@@QBEPAU__POSITION@@H@Z',
    2776:'?FindIndex@CPtrList@@QBEPAU__POSITION@@H@Z',
    2777:'?FindIndex@CStringList@@QBEPAU__POSITION@@H@Z',
    2778:'?FindItem@COleControlContainer@@UBEPAVCOleControlSite@@I@Z',
    2779:'?FindLast@CDaoRecordset@@QAEHPBD@Z',
    2780:'?FindNext@CDaoRecordset@@QAEHPBD@Z',
    2781:'?FindNextFileA@CFileFind@@UAEHXZ',
    2782:'?FindNextFileA@CFtpFileFind@@UAEHXZ',
    2783:'?FindNextFileA@CGopherFileFind@@UAEHXZ',
    2784:'?FindOneOf@CString@@QBEHPBD@Z',
    2785:'?FindPageRect@CPreviewView@@IAEHAAVCPoint@@AAI@Z',
    2786:'?FindPrev@CDaoRecordset@@QAEHPBD@Z',
    2787:'?FindSQLToken@CRecordset@@SGPBDPBD0@Z',
    2788:'?FindTextA@CEditView@@QAEHPBDHH@Z',
    2789:'?FindTextA@CRichEditView@@QAEHPAV_AFX_RICHEDIT_STATE@@@Z',
    2790:'?FindTextA@CRichEditView@@QAEHPBDHH@Z',
    2791:'?FindTextSimple@CRichEditView@@QAEHPAV_AFX_RICHEDIT_STATE@@@Z',
    2792:'?FindTextSimple@CRichEditView@@QAEHPBDHH@Z',
    2793:'?FinishCreate@COleClientItem@@MAEHJ@Z',
    2794:'?FireError@COleControl@@QAEXJPBDI@Z',
    2795:'?FireEvent@COleControl@@QAAXJPAEZZ',
    2796:'?FireEvent@XNotifyDBEvents@COleControlSite@@QAEJKKQAUtagDBNOTIFYREASON@@W4DSCSTATE@@@Z',
    2797:'?FireEventV@COleControl@@IAEXJPAEPAD@Z',
    2798:'?Fixup@CDaoRecordset@@IAEXXZ',
    2799:'?Fixups@CRecordset@@QAEXXZ',
    2800:'?FloatControlBar@CFrameWnd@@QAEXPAVCControlBar@@VCPoint@@K@Z',
    2801:'?Flush@CArchive@@QAEXXZ',
    2802:'?Flush@CDumpContext@@QAEXXZ',
    2803:'?Flush@CFile@@UAEXXZ',
    2804:'?Flush@CInternetFile@@UAEXXZ',
    2805:'?Flush@CMemFile@@UAEXXZ',
    2806:'?Flush@COleStreamFile@@UAEXXZ',
    2807:'?Flush@CSocketFile@@UAEXXZ',
    2808:'?Flush@CStdioFile@@UAEXXZ',
    2809:'?FlushClipboard@COleDataSource@@SGXXZ',
    2810:'?FlushResultSet@CRecordset@@QBEHXZ',
    2811:'?Format@COleCurrency@@QBE?AVCString@@KK@Z',
    2812:'?Format@COleDateTime@@QBE?AVCString@@I@Z',
    2813:'?Format@COleDateTime@@QBE?AVCString@@KK@Z',
    2814:'?Format@COleDateTime@@QBE?AVCString@@PBD@Z',
    2815:'?Format@COleDateTimeSpan@@QBE?AVCString@@I@Z',
    2816:'?Format@COleDateTimeSpan@@QBE?AVCString@@PBD@Z',
    2817:'?Format@CString@@QAAXIZZ',
    2818:'?Format@CString@@QAAXPBDZZ',
    2819:'?Format@CTime@@QBE?AVCString@@I@Z',
    2820:'?Format@CTime@@QBE?AVCString@@PBD@Z',
    2821:'?Format@CTimeSpan@@QBE?AVCString@@I@Z',
    2822:'?Format@CTimeSpan@@QBE?AVCString@@PBD@Z',
    2823:'?FormatGmt@CTime@@QBE?AVCString@@I@Z',
    2824:'?FormatGmt@CTime@@QBE?AVCString@@PBD@Z',
    2825:'?FormatMessageA@CString@@QAAXIZZ',
    2826:'?FormatMessageA@CString@@QAAXPBDZZ',
    2827:'?FormatV@CString@@QAEXPBDPAD@Z',
    2828:'?ForwardActivationMsg@COleControl@@IAEXPAUtagMSG@@@Z',
    2829:'?Free@CDatabase@@IAEXXZ',
    2830:'?Free@CMemFile@@MAEXPAE@Z',
    2831:'?Free@CSharedFile@@MAEXPAE@Z',
    2832:'?FreeAssoc@CMapPtrToPtr@@IAEXPAUCAssoc@1@@Z',
    2833:'?FreeAssoc@CMapPtrToWord@@IAEXPAUCAssoc@1@@Z',
    2834:'?FreeAssoc@CMapStringToOb@@IAEXPAUCAssoc@1@@Z',
    2835:'?FreeAssoc@CMapStringToPtr@@IAEXPAUCAssoc@1@@Z',
    2836:'?FreeAssoc@CMapStringToString@@IAEXPAUCAssoc@1@@Z',
    2837:'?FreeAssoc@CMapWordToOb@@IAEXPAUCAssoc@1@@Z',
    2838:'?FreeAssoc@CMapWordToPtr@@IAEXPAUCAssoc@1@@Z',
    2839:'?FreeCache@CDaoRecordset@@IAEXXZ',
    2840:'?FreeDataCache@CRecordset@@QAEXXZ',
    2841:'?FreeDataChain@CPlex@@QAEXXZ',
    2842:'?FreeExtra@CByteArray@@QAEXXZ',
    2843:'?FreeExtra@CDWordArray@@QAEXXZ',
    2844:'?FreeExtra@CObArray@@QAEXXZ',
    2845:'?FreeExtra@CPtrArray@@QAEXXZ',
    2846:'?FreeExtra@CString@@QAEXXZ',
    2847:'?FreeExtra@CStringArray@@QAEXXZ',
    2848:'?FreeExtra@CUIntArray@@QAEXXZ',
    2849:'?FreeExtra@CWordArray@@QAEXXZ',
    2850:'?FreeNode@CObList@@IAEXPAUCNode@1@@Z',
    2851:'?FreeNode@CPtrList@@IAEXPAUCNode@1@@Z',
    2852:'?FreeNode@CStringList@@IAEXPAUCNode@1@@Z',
    2853:'?FreeRowset@CRecordset@@IAEXXZ',
    2854:'?FreeSlot@CThreadSlotData@@QAEXH@Z',
    2855:'?FreeValue@CProperty@@AAEXXZ',
    2856:'?Freeze@XViewObject@COleControl@@UAGJKJPAXPAK@Z',
    2857:'?FreezeEvents@XOleControl@COleControl@@UAGJH@Z',
    2858:'?FreezeLink@COleClientItem@@UAEHXZ',
    2859:'?FromHandle@CDC@@SGPAV1@PAUHDC__@@@Z',
    2860:'?FromHandle@CGdiObject@@SGPAV1@PAX@Z',
    2861:'?FromHandle@CHandleMap@@QAEPAVCObject@@PAX@Z',
    2862:'?FromHandle@CImageList@@SGPAV1@PAU_IMAGELIST@@@Z',
    2863:'?FromHandle@CMenu@@SGPAV1@PAUHMENU__@@@Z',
    2864:'?FromHandle@CWnd@@SGPAV1@PAUHWND__@@@Z',
    2865:'?FromHandlePermanent@CImageList@@SGPAV1@PAU_IMAGELIST@@@Z',
    2866:'?FromHandlePermanent@CMenu@@SGPAV1@PAUHMENU__@@@Z',
    2867:'?FromHandlePermanent@CWnd@@SGPAV1@PAUHWND__@@@Z',
    2868:'?FromIDispatch@CCmdTarget@@SGPAV1@PAUIDispatch@@@Z',
    2869:'?Get@CProperty@@QAEPAXPAK@Z',
    2870:'?Get@CProperty@@QAEPAXXZ',
    2871:'?Get@CPropertySection@@QAEPAXK@Z',
    2872:'?Get@CPropertySection@@QAEPAXKPAK@Z',
    2873:'?Get@CPropertySet@@QAEPAXU_GUID@@K@Z',
    2874:'?Get@CPropertySet@@QAEPAXU_GUID@@KPAK@Z',
    2875:'?GetAbsolutePosition@CDaoRecordset@@QAEJXZ',
    2876:'?GetActivationPolicy@COleControl@@UAEKXZ',
    2877:'?GetActivationPolicy@XPointerInactive@COleControl@@UAGJPAK@Z',
    2878:'?GetActiveDocument@CFrameWnd@@UAEPAVCDocument@@XZ',
    2879:'?GetActiveFrame@CFrameWnd@@UAEPAV1@XZ',
    2880:'?GetActiveFrame@CMDIFrameWnd@@UAEPAVCFrameWnd@@XZ',
    2881:'?GetActiveIndex@CPropertySheet@@QBEHXZ',
    2882:'?GetActivePage@CPropertySheet@@QBEPAVCPropertyPage@@XZ',
    2883:'?GetActivePane@CSplitterWnd@@UAEPAVCWnd@@PAH0@Z',
    2884:'?GetActiveView@CFrameWnd@@QBEPAVCView@@XZ',
    2885:'?GetAdvise@XViewObject@COleControl@@UAGJPAK0PAPAUIAdviseSink@@@Z',
    2886:'?GetAmbientDispatchDriver@COleControl@@IAEPAVCOleDispatchDriver@@XZ',
    2887:'?GetAmbientProp@COleControlContainer@@UAEHPAVCOleControlSite@@JPAUtagVARIANT@@@Z',
    2888:'?GetAmbientProperty@COleControl@@QAEHJGPAX@Z',
    2889:'?GetAppearance@COleControl@@QAEFXZ',
    2890:'?GetAppRegistryKey@CWinApp@@QAEPAUHKEY__@@XZ',
    2891:'?GetAssocAt@CMapPtrToPtr@@IBEPAUCAssoc@1@PAXAAI@Z',
    2892:'?GetAssocAt@CMapPtrToWord@@IBEPAUCAssoc@1@PAXAAI@Z',
    2893:'?GetAssocAt@CMapStringToOb@@IBEPAUCAssoc@1@PBDAAI@Z',
    2894:'?GetAssocAt@CMapStringToPtr@@IBEPAUCAssoc@1@PBDAAI@Z',
    2895:'?GetAssocAt@CMapStringToString@@IBEPAUCAssoc@1@PBDAAI@Z',
    2896:'?GetAssocAt@CMapWordToOb@@IBEPAUCAssoc@1@GAAI@Z',
    2897:'?GetAssocAt@CMapWordToPtr@@IBEPAUCAssoc@1@GAAI@Z',
    2898:'?GetAt@CStringArray@@QBE?AVCString@@H@Z',
    2899:'?GetAttribute@CGopherConnection@@QAEHAAVCGopherLocator@@VCString@@AAV3@@Z',
    2900:'?GetAttributes@CDaoTableDef@@QAEJXZ',
    2901:'?GetBackColor@COleControl@@QAEKXZ',
    2902:'?GetBarInfo@CControlBar@@QAEXPAVCControlBarInfo@@@Z',
    2903:'?GetBarInfo@CDockBar@@QAEXPAVCControlBarInfo@@@Z',
    2904:'?GetBindInfo@CAsyncMonikerFile@@MBEKXZ',
    2905:'?GetBlob@CBlobProperty@@QAEPAXXZ',
    2906:'?GetBookmark@CDaoRecordset@@QAE?AVCOleVariant@@XZ',
    2907:'?GetBookmark@CRecordset@@QAEXAAVCDBVariant@@@Z',
    2908:'?GetBorder@XOleInPlaceFrame@COleFrameHook@@UAGJPAUtagRECT@@@Z',
    2909:'?GetBorder@XOleIPFrame@COleControlContainer@@UAGJPAUtagRECT@@@Z',
    2910:'?GetBorders@CStatusBarCtrl@@QBEHAAH00@Z',
    2911:'?GetBorderStyle@COleControl@@QAEFXZ',
    2912:'?GetBoundClientRow@CDataSourceControl@@QAEJXZ',
    2913:'?GetBoundFieldIndex@CRecordset@@QAEHPAX@Z',
    2914:'?GetBoundParamIndex@CRecordset@@QAEHPAX@Z',
    2915:'?GetBuffer@CString@@QAEPADH@Z',
    2916:'?GetBufferLength@CEditView@@QBEIXZ',
    2917:'?GetBufferPtr@CFile@@UAEIIIPAPAX0@Z',
    2918:'?GetBufferPtr@CMemFile@@UAEIIIPAPAX0@Z',
    2919:'?GetBufferSetLength@CString@@QAEPADH@Z',
    2920:'?GetButtonInfo@CToolBar@@QBEXHAAI0AAH@Z',
    2921:'?GetButtonStyle@CToolBar@@QBEIH@Z',
    2922:'?GetButtonText@CToolBar@@QBE?AVCString@@H@Z',
    2923:'?GetButtonText@CToolBar@@QBEXHAAVCString@@@Z',
    2924:'?GetByteOrder@CPropertySet@@QAEGXZ',
    2925:'?GetCachedExtent@COleClientItem@@QAEHPAUtagSIZE@@W4tagDVASPECT@@@Z',
    2926:'?GetCacheEntry@COleDataSource@@IAEPAUAFX_DATACACHE_ENTRY@@PAUtagFORMATETC@@W4tagDATADIR@@@Z',
    2927:'?GetCacheSize@CDaoRecordset@@QAEJXZ',
    2928:'?GetCacheStart@CDaoRecordset@@QAE?AVCOleVariant@@XZ',
    2929:'?GetCacheValue@CDaoFieldExchange@@SGPAUCDaoFieldCache@@PAVCDaoRecordset@@PAX@Z',
    2930:'?GetCanonicalFormatEtc@XDataObject@COleControl@@UAGJPAUtagFORMATETC@@0@Z',
    2931:'?GetCanonicalFormatEtc@XDataObject@COleDataSource@@UAGJPAUtagFORMATETC@@0@Z',
    2932:'?GetCanonicalFormatEtc@XDataObject@COleServerDoc@@UAGJPAUtagFORMATETC@@0@Z',
    2933:'?GetCanonicalFormatEtc@XDataObject@COleServerItem@@UAGJPAUtagFORMATETC@@0@Z',
    2934:'?GetCapture@COleControl@@QAEPAVCWnd@@XZ',
    2935:'?GetCharFormat@CFontDialog@@QBEXAAU_charformat@@@Z',
    2936:'?GetCharFormatSelection@CRichEditView@@QAEAAU_charformat@@XZ',
    2937:'?GetCheck@CCheckListBox@@QAEHH@Z',
    2938:'?GetCheckedRadioButton@CWnd@@QAEHHH@Z',
    2939:'?GetClassID@CBlobProperty@@UAGJPAU_GUID@@@Z',
    2940:'?GetClassID@CColorPropPage@@UAEJPAU_GUID@@@Z',
    2941:'?GetClassID@CFontPropPage@@UAEJPAU_GUID@@@Z',
    2942:'?GetClassID@COleClientItem@@QBEXPAU_GUID@@@Z',
    2943:'?GetClassID@CPicturePropPage@@UAEJPAU_GUID@@@Z',
    2944:'?GetClassID@CPropertySet@@QAE?AU_GUID@@XZ',
    2945:'?GetClassID@XPersistFile@COleLinkingDoc@@UAGJPAU_GUID@@@Z',
    2946:'?GetClassID@XPersistMemory@COleControl@@UAGJPAU_GUID@@@Z',
    2947:'?GetClassID@XPersistPropertyBag@COleControl@@UAGJPAU_GUID@@@Z',
    2948:'?GetClassID@XPersistStorage@COleControl@@UAGJPAU_GUID@@@Z',
    2949:'?GetClassID@XPersistStorage@COleServerDoc@@UAGJPAU_GUID@@@Z',
    2950:'?GetClassID@XPersistStreamInit@COleControl@@UAGJPAU_GUID@@@Z',
    2951:'?GetClassInfoA@XProvideClassInfo@COleControl@@UAGJPAPAUITypeInfo@@@Z',
    2952:'?GetClassObject@AFX_COM@@QAEJABU_GUID@@0PAPAX@Z',
    2953:'?GetClientOffset@COleControl@@UBEXPAJ0@Z',
    2954:'?GetClientRect@COleControl@@UBEXPAUtagRECT@@@Z',
    2955:'?GetClientSite@COleClientItem@@MAEPAUIOleClientSite@@XZ',
    2956:'?GetClientSite@COleControl@@QAEPAUIOleClientSite@@XZ',
    2957:'?GetClientSite@CRichEditCntrItem@@UAEPAUIOleClientSite@@XZ',
    2958:'?GetClientSite@XOleObject@CDocObjectServer@@UAGJPAPAUIOleClientSite@@@Z',
    2959:'?GetClientSite@XOleObject@COleControl@@UAGJPAPAUIOleClientSite@@@Z',
    2960:'?GetClientSite@XOleObject@COleServerDoc@@UAGJPAPAUIOleClientSite@@@Z',
    2961:'?GetClientSite@XOleObject@COleServerItem@@UAGJPAPAUIOleClientSite@@@Z',
    2962:'?GetClipboardData@COleClientItem@@QAEXPAVCOleDataSource@@HPAUtagPOINT@@PAUtagSIZE@@@Z',
    2963:'?GetClipboardData@COleServerItem@@QAEXPAVCOleDataSource@@HPAUtagPOINT@@PAUtagSIZE@@@Z',
    2964:'?GetClipboardData@CRichEditView@@MAEJPAU_charrange@@KPAUIDataObject@@PAPAU3@@Z',
    2965:'?GetClipboardData@XOleObject@CDocObjectServer@@UAGJKPAPAUIDataObject@@@Z',
    2966:'?GetClipboardData@XOleObject@COleControl@@UAGJKPAPAUIDataObject@@@Z',
    2967:'?GetClipboardData@XOleObject@COleServerDoc@@UAGJKPAPAUIDataObject@@@Z',
    2968:'?GetClipboardData@XOleObject@COleServerItem@@UAGJKPAPAUIDataObject@@@Z',
    2969:'?GetClipboardData@XRichEditOleCallback@CRichEditView@@UAGJPAU_charrange@@KPAPAUIDataObject@@@Z',
    2970:'?GetClipboardOwner@COleDataSource@@SGPAV1@XZ',
    2971:'?GetClipBox@CDC@@UBEHPAUtagRECT@@@Z',
    2972:'?GetClipBox@CMetaFileDC@@UBEHPAUtagRECT@@@Z',
    2973:'?GetColorProp@CColorPropPage@@IAEHPAVCDataExchange@@PAKPBD@Z',
    2974:'?GetColorSet@XViewObject@COleControl@@UAGJKJPAXPAUtagDVTARGETDEVICE@@PAUHDC__@@PAPAUtagLOGPALETTE@@@Z',
    2975:'?GetColumnInfo@CSplitterWnd@@QBEXHAAH0@Z',
    2976:'?GetCommandMap@CCmdTarget@@MBEPBUAFX_OLECMDMAP@@XZ',
    2977:'?GetConnect@CDaoDatabase@@QAE?AVCString@@XZ',
    2978:'?GetConnect@CDaoQueryDef@@QAE?AVCString@@XZ',
    2979:'?GetConnect@CDaoTableDef@@QAE?AVCString@@XZ',
    2980:'?GetConnectInfo@CDatabase@@IAEXXZ',
    2981:'?GetConnectionCount@CConnectionPoint@@QAEHXZ',
    2982:'?GetConnectionHook@CCmdTarget@@MAEPAUIConnectionPoint@@ABU_GUID@@@Z',
    2983:'?GetConnectionHook@COleControl@@MAEPAUIConnectionPoint@@ABU_GUID@@@Z',
    2984:'?GetConnectionInterface@XConnPt@CConnectionPoint@@UAGJPAU_GUID@@@Z',
    2985:'?GetConnectionMap@CCmdTarget@@MBEPBUAFX_CONNECTIONMAP@@XZ',
    2986:'?GetConnectionMap@COleControl@@MBEPBUAFX_CONNECTIONMAP@@XZ',
    2987:'?GetConnectionPointContainer@XConnPt@CConnectionPoint@@UAGJPAPAUIConnectionPointContainer@@@Z',
    2988:'?GetConnections@CConnectionPoint@@QAEPBVCPtrArray@@XZ',
    2989:'?GetContainer@CConnectionPoint@@UAEPAUIConnectionPointContainer@@XZ',
    2990:'?GetContainer@COleDocument@@UAEPAUIOleItemContainer@@XZ',
    2991:'?GetContainer@COleLinkingDoc@@UAEPAUIOleItemContainer@@XZ',
    2992:'?GetContainer@XOleClientSite@COleClientItem@@UAGJPAPAUIOleContainer@@@Z',
    2993:'?GetContainer@XOleClientSite@COleControlSite@@UAGJPAPAUIOleContainer@@@Z',
    2994:'?GetContentExtent@XQuickActivate@COleControl@@UAGJPAUtagSIZE@@@Z',
    2995:'?GetContextMenu@CRichEditView@@MAEPAUHMENU__@@GPAUIOleObject@@PAU_charrange@@@Z',
    2996:'?GetContextMenu@XRichEditOleCallback@CRichEditView@@UAGJGPAUIOleObject@@PAU_charrange@@PAPAUHMENU__@@@Z',
    2997:'?GetControlBar@CFrameWnd@@QAEPAVCControlBar@@I@Z',
    2998:'?GetControlFlags@COleControl@@UAEKXZ',
    2999:'?GetControlInfo@COleControlSite@@QAEXXZ',
    3000:'?GetControlInfo@XOleControl@COleControl@@UAGJPAUtagCONTROLINFO@@@Z',
    3001:'?GetControllingFrame@CDocObjectServer@@QBEPAVCOleDocIPFrameWnd@@XZ',
    3002:'?GetControllingUnknown@CCmdTarget@@QAEPAUIUnknown@@XZ',
    3003:'?GetControlSize@COleControl@@QAEXPAH0@Z',
    3004:'?GetControlStatus@COlePropertyPage@@QAEHI@Z',
    3005:'?GetControlUnknown@CWnd@@QAEPAUIUnknown@@XZ',
    3006:'?GetConvertInfo@XOleUIObjInfo@COlePropertiesDialog@@UAGJKPAU_GUID@@PAG0PAPAU3@PAI@Z',
    3007:'?GetCopies@CPrintDialog@@QBEHXZ',
    3008:'?GetCount@CPropertySection@@QAEKXZ',
    3009:'?GetCount@CPropertySet@@QAEKXZ',
    3010:'?GetCreationTime@CFileFind@@UBEHAAVCTime@@@Z',
    3011:'?GetCreationTime@CFileFind@@UBEHPAU_FILETIME@@@Z',
    3012:'?GetCreationTime@CGopherFileFind@@UBEHAAVCTime@@@Z',
    3013:'?GetCreationTime@CGopherFileFind@@UBEHPAU_FILETIME@@@Z',
    3014:'?GetCurFile@XPersistFile@COleLinkingDoc@@UAGJPAPAG@Z',
    3015:'?GetCurrentDirectoryA@CFtpConnection@@QBEHAAVCString@@@Z',
    3016:'?GetCurrentDirectoryA@CFtpConnection@@QBEHPADPAK@Z',
    3017:'?GetCurrentDirectoryAsURL@CFtpConnection@@QBEHAAVCString@@@Z',
    3018:'?GetCurrentDirectoryAsURL@CFtpConnection@@QBEHPADPAK@Z',
    3019:'?GetCurrentFont@CFontDialog@@QAEXPAUtagLOGFONTA@@@Z',
    3020:'?GetCurrentIndex@CDaoRecordset@@QAE?AVCString@@XZ',
    3021:'?GetCurrentMessage@CWnd@@KGPBUtagMSG@@XZ',
    3022:'?GetCurrentName@CFontComboBox@@QAE?AVCString@@XZ',
    3023:'?GetCursor@CDataBoundProperty@@QAEPAUICursor@@XZ=?GetCursor@CDataBoundProperty@@QAEPAUIUnknown@@XZ',
    3024:'?GetCursor@CDataSourceControl@@UAEPAUICursor@@XZ=?GetCursor@CDataSourceControl@@UAEPAUIUnknown@@XZ',
    3025:'?GetCursor@COleControlSite@@IAEJJPAPAUICursor@@PAPAX@Z=?GetCursor@COleControlSite@@IAEJJPAPAUIUnknown@@PAPAX@Z',
    3026:'?GetCursor@XBoundObjectSite@COleControlSite@@UAGJJPAPAUICursor@@PAPAX@Z',
    3027:'?GetData@COleDataObject@@QAEHGPAUtagSTGMEDIUM@@PAUtagFORMATETC@@@Z',
    3028:'?GetData@CProcessLocalObject@@QAEPAVCNoTrackObject@@P6GPAV2@XZ@Z',
    3029:'?GetData@CRecordset@@SGJPAVCDatabase@@PAXFF1HF@Z',
    3030:'?GetData@CThreadLocalObject@@QAEPAVCNoTrackObject@@P6GPAV2@XZ@Z',
    3031:'?GetData@XDataObject@COleControl@@UAGJPAUtagFORMATETC@@PAUtagSTGMEDIUM@@@Z',
    3032:'?GetData@XDataObject@COleDataSource@@UAGJPAUtagFORMATETC@@PAUtagSTGMEDIUM@@@Z',
    3033:'?GetData@XDataObject@COleServerDoc@@UAGJPAUtagFORMATETC@@PAUtagSTGMEDIUM@@@Z',
    3034:'?GetData@XDataObject@COleServerItem@@UAGJPAUtagFORMATETC@@PAUtagSTGMEDIUM@@@Z',
    3035:'?GetDataAndFixupNulls@CDaoRecordset@@QAEXXZ',
    3036:'?GetDatabaseCount@CDaoWorkspace@@QAEFXZ',
    3037:'?GetDatabaseInfo@CDaoWorkspace@@QAEXHAAUCDaoDatabaseInfo@@K@Z',
    3038:'?GetDatabaseInfo@CDaoWorkspace@@QAEXPBDAAUCDaoDatabaseInfo@@K@Z',
    3039:'?GetDatabaseName@CDatabase@@QBE?AVCString@@XZ',
    3040:'?GetDataBuffer@CRecordset@@SGPAXAAVCDBVariant@@FPAHFK@Z',
    3041:'?GetDataHere@XDataObject@COleControl@@UAGJPAUtagFORMATETC@@PAUtagSTGMEDIUM@@@Z',
    3042:'?GetDataHere@XDataObject@COleDataSource@@UAGJPAUtagFORMATETC@@PAUtagSTGMEDIUM@@@Z',
    3043:'?GetDataHere@XDataObject@COleServerDoc@@UAGJPAUtagFORMATETC@@PAUtagSTGMEDIUM@@@Z',
    3044:'?GetDataHere@XDataObject@COleServerItem@@UAGJPAUtagFORMATETC@@PAUtagSTGMEDIUM@@@Z',
    3045:'?GetDataNA@CThreadLocalObject@@QAEPAVCNoTrackObject@@XZ',
    3046:'?GetDataObject@COleServerItem@@QAEPAUIDataObject@@XZ',
    3047:'?GetDataSource@COleControl@@IAEPAVCControlDataSource@1@XZ',
    3048:'?GetDateCreated@CDaoQueryDef@@QAE?AVCOleDateTime@@XZ',
    3049:'?GetDateCreated@CDaoRecordset@@QAE?AVCOleDateTime@@XZ',
    3050:'?GetDateCreated@CDaoTableDef@@QAE?AVCOleDateTime@@XZ',
    3051:'?GetDateLastUpdated@CDaoQueryDef@@QAE?AVCOleDateTime@@XZ',
    3052:'?GetDateLastUpdated@CDaoRecordset@@QAE?AVCOleDateTime@@XZ',
    3053:'?GetDateLastUpdated@CDaoTableDef@@QAE?AVCOleDateTime@@XZ',
    3054:'?GetDay@COleDateTime@@QBEHXZ',
    3055:'?GetDayOfWeek@COleDateTime@@QBEHXZ',
    3056:'?GetDayOfYear@COleDateTime@@QBEHXZ',
    3057:'?GetDC@COleControl@@QAEPAVCDC@@PBUtagRECT@@K@Z',
    3058:'?GetDefaultAccelerator@CDocument@@UAEPAUHACCEL__@@XZ',
    3059:'?GetDefaultAccelerator@CFrameWnd@@UAEPAUHACCEL__@@XZ',
    3060:'?GetDefaultAccelerator@COleServerDoc@@MAEPAUHACCEL__@@XZ',
    3061:'?GetDefaultCharFormat@CRichEditCtrl@@QBEKAAU_charformat@@@Z',
    3062:'?GetDefaultConnect@CRecordset@@UAE?AVCString@@XZ',
    3063:'?GetDefaultDBName@CDaoRecordset@@UAE?AVCString@@XZ',
    3064:'?GetDefaultFieldType@CRecordset@@SGFF@Z',
    3065:'?GetDefaultMenu@CDocument@@UAEPAUHMENU__@@XZ',
    3066:'?GetDefaultMenu@COleServerDoc@@MAEPAUHMENU__@@XZ',
    3067:'?GetDefaults@CPrintDialog@@QAEHXZ',
    3068:'?GetDefaultSQL@CDaoRecordset@@UAE?AVCString@@XZ',
    3069:'?GetDefaultSQL@CRecordset@@UAE?AVCString@@XZ',
    3070:'?GetDefBtnCode@COccManager@@SAKPAVCWnd@@@Z',
    3071:'?GetDefBtnCode@COleControlSite@@QAEKXZ',
    3072:'?GetDescendantWindow@CWnd@@SGPAV1@PAUHWND__@@HH@Z',
    3073:'?GetDeviceName@CPageSetupDialog@@QBE?AVCString@@XZ',
    3074:'?GetDeviceName@CPrintDialog@@QBE?AVCString@@XZ',
    3075:'?GetDeviceScrollPosition@CScrollView@@QBE?AVCPoint@@XZ',
    3076:'?GetDeviceScrollSizes@CScrollView@@QBEXAAHAAUtagSIZE@@11@Z',
    3077:'?GetDevMode@CPageSetupDialog@@QBEPAU_devicemodeA@@XZ',
    3078:'?GetDevMode@CPrintDialog@@QBEPAU_devicemodeA@@XZ',
    3079:'?GetDispatchIID@CCmdTarget@@UAEHPAU_GUID@@@Z',
    3080:'?GetDispatchIID@COleControl@@MAEHPAU_GUID@@@Z',
    3081:'?GetDispatchMap@CCmdTarget@@MBEPBUAFX_DISPMAP@@XZ',
    3082:'?GetDispatchMap@COleControlContainer@@MBEPBUAFX_DISPMAP@@XZ',
    3083:'?GetDispEntry@CCmdTarget@@IAGPBUAFX_DISPMAP_ENTRY@@J@Z',
    3084:'?GetDisplayName@CRecentFileList@@QBEHAAVCString@@HPBDHH@Z',
    3085:'?GetDisplayString@CFontHolder@@QAEHAAVCString@@@Z',
    3086:'?GetDisplayString@CPictureHolder@@QAEHAAVCString@@@Z',
    3087:'?GetDisplayString@XPerPropertyBrowsing@COleControl@@UAGJJPAPAG@Z',
    3088:'?GetDlgCtrlID@COleControlSite@@UBEHXZ',
    3089:'?GetDlgCtrlID@CWnd@@QBEHXZ',
    3090:'?GetDlgItem@COleControlContainer@@UBEPAVCWnd@@H@Z',
    3091:'?GetDlgItem@COleControlContainer@@UBEXHPAPAUHWND__@@@Z',
    3092:'?GetDlgItem@CWnd@@QBEPAV1@H@Z',
    3093:'?GetDlgItem@CWnd@@QBEXHPAPAUHWND__@@@Z',
    3094:'?GetDlgItemInt@COleControlContainer@@UBEIHPAHH@Z',
    3095:'?GetDlgItemInt@CWnd@@QBEIHPAHH@Z',
    3096:'?GetDlgItemTextA@COleControlContainer@@UBEHHPADH@Z',
    3097:'?GetDlgItemTextA@CWnd@@QBEHHAAVCString@@@Z',
    3098:'?GetDlgItemTextA@CWnd@@QBEHHPADH@Z',
    3099:'?GetDockBar@CDockContext@@QAEPAVCDockBar@@K@Z',
    3100:'?GetDockedControlBar@CDockBar@@IBEPAVCControlBar@@H@Z',
    3101:'?GetDockedCount@CDockBar@@QBEHXZ',
    3102:'?GetDockedVisibleCount@CDockBar@@UBEHXZ',
    3103:'?GetDockingFrame@CControlBar@@QBEPAVCFrameWnd@@XZ',
    3104:'?GetDockState@CFrameWnd@@QBEXAAVCDockState@@@Z',
    3105:'?GetDocMiscStatus@XOleDocument@CDocObjectServer@@UAGJPAK@Z',
    3106:'?GetDocObjectServer@COleServerDoc@@MAEPAVCDocObjectServer@@PAUIOleDocumentSite@@@Z',
    3107:'?GetDocString@CDocTemplate@@UBEHAAVCString@@W4DocStringIndex@1@@Z',
    3108:'?GetDocument@XOleDocumentView@CDocObjectServer@@UAGJPAPAUIUnknown@@@Z',
    3109:'?GetDragDropEffect@XRichEditOleCallback@CRichEditView@@UAGJHKPAK@Z',
    3110:'?GetDriverName@CPageSetupDialog@@QBE?AVCString@@XZ',
    3111:'?GetDriverName@CPrintDialog@@QBE?AVCString@@XZ',
    3112:'?GetDropTarget@XOleInPlaceObject@COleControl@@UAGJPAPAUIDropTarget@@@Z',
    3113:'?GetDSCCursor@CWnd@@QAEPAUIUnknown@@XZ',
    3114:'?GetEditMode@CDaoRecordset@@QAEFXZ',
    3115:'?GetElement@COleSafeArray@@QAEXPAJPAX@Z',
    3116:'?GetEmbeddedItem@COleServerDoc@@QAEPAVCOleServerItem@@XZ',
    3117:'?GetEmbeddedItemData@COleClientItem@@IAEXPAUtagSTGMEDIUM@@@Z',
    3118:'?GetEmbedSourceData@COleServerItem@@QAEXPAUtagSTGMEDIUM@@@Z',
    3119:'?GetEnabled@COleControl@@QAEHXZ',
    3120:'?GetEntryCount@CCmdTarget@@KGIPBUAFX_DISPMAP@@@Z',
    3121:'?GetErrorCount@CDaoException@@QAEFXZ',
    3122:'?GetErrorInfo@CDaoException@@QAEXH@Z',
    3123:'?GetErrorMessage@CArchiveException@@UAEHPADIPAI@Z',
    3124:'?GetErrorMessage@CDaoException@@UAEHPADIPAI@Z',
    3125:'?GetErrorMessage@CDBException@@UAEHPADIPAI@Z',
    3126:'?GetErrorMessage@CException@@UAEHPADIPAI@Z',
    3127:'?GetErrorMessage@CFileException@@UAEHPADIPAI@Z',
    3128:'?GetErrorMessage@CInternetException@@UAEHPADIPAI@Z',
    3129:'?GetErrorMessage@COleDispatchException@@UAEHPADIPAI@Z',
    3130:'?GetErrorMessage@COleException@@UAEHPADIPAI@Z',
    3131:'?GetErrorMessage@CSimpleException@@UAEHPADIPAI@Z',
    3132:'?GetEventIID@COleControlSite@@QAEHPAU_GUID@@@Z',
    3133:'?GetEventMap@COleControl@@MBEPBUAFX_EVENTMAP@@XZ',
    3134:'?GetEventMapEntry@COleControl@@IBEPBUAFX_EVENTMAP_ENTRY@@PBDPAJ@Z',
    3135:'?GetEventSinkEntry@CCmdTarget@@IAGPBUAFX_EVENTSINKMAP_ENTRY@@IPAUAFX_EVENT@@@Z',
    3136:'?GetEventSinkMap@CCmdTarget@@MBEPBUAFX_EVENTSINKMAP@@XZ',
    3137:'?GetExStyle@COleControlSite@@UBEKXZ',
    3138:'?GetExStyle@CWnd@@QBEKXZ',
    3139:'?GetExtendedControl@COleControl@@QAEPAUIDispatch@@XZ',
    3140:'?GetExtendedControl@XOleControlSite@COleControlSite@@UAGJPAPAUIDispatch@@@Z',
    3141:'?GetExtent@COleClientItem@@QAEHPAUtagSIZE@@W4tagDVASPECT@@@Z',
    3142:'?GetExtent@XOleObject@CDocObjectServer@@UAGJKPAUtagSIZE@@@Z',
    3143:'?GetExtent@XOleObject@COleControl@@UAGJKPAUtagSIZE@@@Z',
    3144:'?GetExtent@XOleObject@COleServerDoc@@UAGJKPAUtagSIZE@@@Z',
    3145:'?GetExtent@XOleObject@COleServerItem@@UAGJKPAUtagSIZE@@@Z',
    3146:'?GetExtent@XViewObject@COleControl@@UAGJKJPAUtagDVTARGETDEVICE@@PAUtagSIZE@@@Z',
    3147:'?GetExtraConnectionPoints@CCmdTarget@@MAEHPAVCPtrArray@@@Z',
    3148:'?GetExtraConnectionPoints@COleControl@@MAEHPAVCPtrArray@@@Z',
    3149:'?GetFaceName@CFontDialog@@QBE?AVCString@@XZ',
    3150:'?GetFieldCount@CDaoQueryDef@@QAEFXZ',
    3151:'?GetFieldCount@CDaoRecordset@@QAEFXZ',
    3152:'?GetFieldCount@CDaoTableDef@@QAEFXZ',
    3153:'?GetFieldIndex@CDaoRecordset@@IAEHPAX@Z',
    3154:'?GetFieldIndexByName@CRecordset@@QAEFPBD@Z',
    3155:'?GetFieldInfo@CDaoQueryDef@@QAEXHAAUCDaoFieldInfo@@K@Z',
    3156:'?GetFieldInfo@CDaoQueryDef@@QAEXPBDAAUCDaoFieldInfo@@K@Z',
    3157:'?GetFieldInfo@CDaoRecordset@@QAEXHAAUCDaoFieldInfo@@K@Z',
    3158:'?GetFieldInfo@CDaoRecordset@@QAEXPBDAAUCDaoFieldInfo@@K@Z',
    3159:'?GetFieldInfo@CDaoTableDef@@QAEXHAAUCDaoFieldInfo@@K@Z',
    3160:'?GetFieldInfo@CDaoTableDef@@QAEXPBDAAUCDaoFieldInfo@@K@Z',
    3161:'?GetFieldLength@CDaoRecordset@@QAEKH@Z',
    3162:'?GetFieldLengthBuffer@CRecordset@@QAEPAJKH@Z',
    3163:'?GetFieldStatus@CRecordset@@QAEEK@Z',
    3164:'?GetFieldValue@CDaoRecordset@@UAE?AVCOleVariant@@H@Z',
    3165:'?GetFieldValue@CDaoRecordset@@UAE?AVCOleVariant@@PBD@Z',
    3166:'?GetFieldValue@CDaoRecordset@@UAEXHAAVCOleVariant@@@Z',
    3167:'?GetFieldValue@CDaoRecordset@@UAEXPBDAAVCOleVariant@@@Z',
    3168:'?GetFieldValue@CRecordset@@QAEXFAAVCDBVariant@@F@Z',
    3169:'?GetFieldValue@CRecordset@@QAEXFAAVCString@@@Z',
    3170:'?GetFieldValue@CRecordset@@QAEXPBDAAVCDBVariant@@F@Z',
    3171:'?GetFieldValue@CRecordset@@QAEXPBDAAVCString@@@Z',
    3172:'?GetFile@CDocument@@UAEPAVCFile@@PBDIPAVCFileException@@@Z',
    3173:'?GetFile@CFtpConnection@@QAEHPBD0HKKK@Z',
    3174:'?GetFileData@COleDataObject@@QAEPAVCFile@@GPAUtagFORMATETC@@@Z',
    3175:'?GetFileExt@CFileDialog@@QBE?AVCString@@XZ',
    3176:'?GetFileName@CFile@@UBE?AVCString@@XZ',
    3177:'?GetFileName@CFileDialog@@QBE?AVCString@@XZ',
    3178:'?GetFileName@CFileFind@@UBE?AVCString@@XZ',
    3179:'?GetFileName@CGopherFileFind@@UBE?AVCString@@XZ',
    3180:'?GetFilePath@CFile@@UBE?AVCString@@XZ',
    3181:'?GetFilePath@CFileFind@@UBE?AVCString@@XZ',
    3182:'?GetFilePath@CGopherFileFind@@UBE?AVCString@@XZ',
    3183:'?GetFileTitle@CFile@@UBE?AVCString@@XZ',
    3184:'?GetFileTitle@CFileDialog@@QBE?AVCString@@XZ',
    3185:'?GetFileTitle@CFileFind@@UBE?AVCString@@XZ',
    3186:'?GetFileTitle@CGopherFileFind@@UBE?AVCString@@XZ',
    3187:'?GetFileTypeString@COleServerDoc@@MAEHAAVCString@@@Z',
    3188:'?GetFileURL@CFileFind@@UBE?AVCString@@XZ',
    3189:'?GetFileURL@CFtpFileFind@@UBE?AVCString@@XZ',
    3190:'?GetFileURL@CGopherFileFind@@UBE?AVCString@@XZ',
    3191:'?GetFileURL@CHttpFile@@UBE?AVCString@@XZ',
    3192:'?GetFindString@CFindReplaceDialog@@QBE?AVCString@@XZ',
    3193:'?GetFirstDocPosition@CMultiDocTemplate@@UBEPAU__POSITION@@XZ',
    3194:'?GetFirstDocPosition@CSingleDocTemplate@@UBEPAU__POSITION@@XZ',
    3195:'?GetFirstDocTemplatePosition@CDocManager@@UBEPAU__POSITION@@XZ',
    3196:'?GetFirstDocTemplatePosition@CWinApp@@QBEPAU__POSITION@@XZ',
    3197:'?GetFirstFrame@COleDocument@@QAEPAVCFrameWnd@@XZ',
    3198:'?GetFirstViewPosition@CDocument@@UBEPAU__POSITION@@XZ',
    3199:'?GetFocus@COleControl@@QAEPAVCWnd@@XZ',
    3200:'?GetFolderPath@CFileDialog@@QBE?AVCString@@XZ',
    3201:'?GetFont@CDialogTemplate@@QBEHAAVCString@@AAG@Z',
    3202:'?GetFont@CDialogTemplate@@SAHPBUDLGTEMPLATE@@AAVCString@@AAG@Z',
    3203:'?GetFont@COleControl@@QAEPAUIFontDisp@@XZ',
    3204:'?GetFontDispatch@CFontHolder@@QAEPAUIFontDisp@@XZ',
    3205:'?GetFontHandle@CFontHolder@@QAEPAUHFONT__@@JJ@Z',
    3206:'?GetFontHandle@CFontHolder@@QAEPAUHFONT__@@XZ',
    3207:'?GetFontItem@CFontComboBox@@QAEPAUFONTITEM_PPG@@H@Z',
    3208:'?GetFontProps@CFontPropPage@@IAEHPAVCDataExchange@@PAUtagFONTOBJECT@@PBDPAUtagMERGEOBJECT@@@Z',
    3209:'?GetFontSizeField@CDialogTemplate@@KAPAEPBUDLGTEMPLATE@@@Z',
    3210:'?GetFontTextMetrics@COleControl@@QAEXPAUtagTEXTMETRICA@@AAVCFontHolder@@@Z',
    3211:'?GetFontType@CFontComboBox@@QAEKH@Z',
    3212:'?GetForeColor@COleControl@@QAEKXZ',
    3213:'?GetFormatID@CPropertySection@@QAE?AU_GUID@@XZ',
    3214:'?GetFormatVersion@CPropertySet@@QAEGXZ',
    3215:'?GetFtpConnection@CInternetSession@@QAEPAVCFtpConnection@@PBD00GH@Z',
    3216:'?GetGlobalData@COleDataObject@@QAEPAXGPAUtagFORMATETC@@@Z',
    3217:'?GetGmtTm@CTime@@QBEPAUtm@@PAU2@@Z',
    3218:'?GetGopherConnection@CInternetSession@@QAEPAVCGopherConnection@@PBD00G@Z',
    3219:'?GetGUID@XProvideClassInfo@COleControl@@UAGJKPAU_GUID@@@Z',
    3220:'?GetHalftoneBrush@CDC@@SGPAVCBrush@@XZ',
    3221:'?GetHandleMask@CRectTracker@@UBEIXZ',
    3222:'?GetHandleRect@CRectTracker@@IBEXHPAVCRect@@@Z',
    3223:'?GetHandleSize@CRectTracker@@MBEHPBUtagRECT@@@Z',
    3224:'?GetHeight@CSizeComboBox@@QAEJH@Z',
    3225:'?GetHitRect@CSplitterWnd@@MAEXHAAVCRect@@@Z',
    3226:'?GetHotKey@CHotKeyCtrl@@QBEXAAG0@Z',
    3227:'?GetHour@COleDateTime@@QBEHXZ',
    3228:'?GetHours@COleDateTimeSpan@@QBEJXZ',
    3229:'?GetHttpConnection@CInternetSession@@QAEPAVCHttpConnection@@PBDG00@Z',
    3230:'?GetHwnd@COleControl@@QAEIXZ',
    3231:'?GetIconicMetafile@COleClientItem@@QAEPAXXZ',
    3232:'?GetIconWndClass@CFrameWnd@@IAEPBDKI@Z',
    3233:'?GetID@COleControlSite@@QAEIXZ',
    3234:'?GetID@CProperty@@QAEKXZ',
    3235:'?GetID@CPropertySection@@QAEHPBDPAK@Z',
    3236:'?GetIDataObject@COleDataObject@@QAEPAUIDataObject@@H@Z',
    3237:'?GetIDispatch@CCmdTarget@@QAEPAUIDispatch@@H@Z',
    3238:'?GetIDsOfNames@COleDispatchImpl@@UAGJABU_GUID@@PAPAGIKPAJ@Z',
    3239:'?GetIDsOfNames@XAmbientProps@COleControlSite@@UAGJABU_GUID@@PAPAGIKPAJ@Z',
    3240:'?GetIDsOfNames@XEventSink@COleControlSite@@UAGJABU_GUID@@PAPAGIKPAJ@Z',
    3241:'?GetIID@XEventConnPt@COleControl@@UAEABU_GUID@@XZ',
    3242:'?GetIID@XPropConnPt@COleControl@@UAEABU_GUID@@XZ',
    3243:'?GetIndexCount@CDaoRecordset@@QAEFXZ',
    3244:'?GetIndexCount@CDaoTableDef@@QAEFXZ',
    3245:'?GetIndexInfo@CDaoRecordset@@QAEXHAAUCDaoIndexInfo@@K@Z',
    3246:'?GetIndexInfo@CDaoRecordset@@QAEXPBDAAUCDaoIndexInfo@@K@Z',
    3247:'?GetIndexInfo@CDaoTableDef@@QAEXHAAUCDaoIndexInfo@@K@Z',
    3248:'?GetIndexInfo@CDaoTableDef@@QAEXPBDAAUCDaoIndexInfo@@K@Z',
    3249:'?GetIniPath@CDaoWorkspace@@SG?AVCString@@XZ',
    3250:'?GetInPlaceActiveItem@COleDocument@@UAEPAVCOleClientItem@@PAVCWnd@@@Z',
    3251:'?GetInPlaceActiveItem@CRichEditDoc@@UAEPAVCOleClientItem@@PAVCWnd@@@Z',
    3252:'?GetInPlaceActiveItem@CRichEditView@@QBEPAVCRichEditCntrItem@@XZ',
    3253:'?GetInPlaceContext@XRichEditOleCallback@CRichEditView@@UAGJPAPAUIOleInPlaceFrame@@PAPAUIOleInPlaceUIWindow@@PAUtagOIFI@@@Z',
    3254:'?GetInPlaceMenu@COleIPFrameWnd@@MAEPAUHMENU__@@XZ',
    3255:'?GetInPlaceSite@XOleDocumentView@CDocObjectServer@@UAGJPAPAUIOleInPlaceSite@@@Z',
    3256:'?GetInPlaceWindow@COleClientItem@@QAEPAVCWnd@@XZ',
    3257:'?GetInsideRect@CSplitterWnd@@MBEXAAVCRect@@@Z',
    3258:'?GetInterface@CCmdTarget@@QAEPAUIUnknown@@PBX@Z',
    3259:'?GetInterfaceHook@CCmdTarget@@UAEPAUIUnknown@@PBX@Z',
    3260:'?GetInterfaceHook@COleControl@@MAEPAUIUnknown@@PBX@Z',
    3261:'?GetInterfaceHook@COleServerDoc@@MAEPAUIUnknown@@PBX@Z',
    3262:'?GetInterfaceMap@CCmdTarget@@MBEPBUAFX_INTERFACEMAP@@XZ',
    3263:'?GetInterfaceMap@CDocObjectServer@@MBEPBUAFX_INTERFACEMAP@@XZ',
    3264:'?GetInterfaceMap@CEnumConnPoints@@MBEPBUAFX_INTERFACEMAP@@XZ',
    3265:'?GetInterfaceMap@CEnumFormatEtc@@MBEPBUAFX_INTERFACEMAP@@XZ',
    3266:'?GetInterfaceMap@CEnumOleVerb@@MBEPBUAFX_INTERFACEMAP@@XZ',
    3267:'?GetInterfaceMap@CEnumUnknown@@MBEPBUAFX_INTERFACEMAP@@XZ',
    3268:'?GetInterfaceMap@COleClientItem@@MBEPBUAFX_INTERFACEMAP@@XZ',
    3269:'?GetInterfaceMap@COleControl@@MBEPBUAFX_INTERFACEMAP@@XZ',
    3270:'?GetInterfaceMap@COleControlContainer@@MBEPBUAFX_INTERFACEMAP@@XZ',
    3271:'?GetInterfaceMap@COleControlSite@@MBEPBUAFX_INTERFACEMAP@@XZ',
    3272:'?GetInterfaceMap@COleDataSource@@MBEPBUAFX_INTERFACEMAP@@XZ',
    3273:'?GetInterfaceMap@COleDropSource@@MBEPBUAFX_INTERFACEMAP@@XZ',
    3274:'?GetInterfaceMap@COleDropTarget@@MBEPBUAFX_INTERFACEMAP@@XZ',
    3275:'?GetInterfaceMap@COleFrameHook@@MBEPBUAFX_INTERFACEMAP@@XZ',
    3276:'?GetInterfaceMap@COleLinkingDoc@@MBEPBUAFX_INTERFACEMAP@@XZ',
    3277:'?GetInterfaceMap@COleMessageFilter@@MBEPBUAFX_INTERFACEMAP@@XZ',
    3278:'?GetInterfaceMap@COleObjectFactory@@MBEPBUAFX_INTERFACEMAP@@XZ',
    3279:'?GetInterfaceMap@COlePropertyPage@@MBEPBUAFX_INTERFACEMAP@@XZ',
    3280:'?GetInterfaceMap@COleServerDoc@@MBEPBUAFX_INTERFACEMAP@@XZ',
    3281:'?GetInterfaceMap@COleServerItem@@MBEPBUAFX_INTERFACEMAP@@XZ',
    3282:'?GetInterfaceMap@CRichEditView@@MBEPBUAFX_INTERFACEMAP@@XZ',
    3283:'?GetIRichEditOle@CRichEditCtrl@@QBEPAUIRichEditOle@@XZ',
    3284:'?GetIsolateODBCTrans@CDaoWorkspace@@QAEHXZ',
    3285:'?GetItemClipRect@COleServerDoc@@QBEXPAUtagRECT@@@Z',
    3286:'?GetItemData@CListCtrl@@QBEKH@Z',
    3287:'?GetItemData@CTreeCtrl@@QBEKPAU_TREEITEM@@@Z',
    3288:'?GetItemID@CStatusBar@@QBEIH@Z',
    3289:'?GetItemID@CToolBar@@QBEIH@Z',
    3290:'?GetItemImage@CTreeCtrl@@QBEHPAU_TREEITEM@@AAH1@Z',
    3291:'?GetItemName@COleClientItem@@QBEXPAD@Z',
    3292:'?GetItemPosition@COleServerDoc@@QBEXPAUtagRECT@@@Z',
    3293:'?GetItemRect@CListCtrl@@QBEHHPAUtagRECT@@I@Z',
    3294:'?GetItemRect@CStatusBar@@QBEXHPAUtagRECT@@@Z',
    3295:'?GetItemRect@CToolBar@@UBEXHPAUtagRECT@@@Z',
    3296:'?GetItemRect@CTreeCtrl@@QBEHPAU_TREEITEM@@PAUtagRECT@@H@Z',
    3297:'?GetItemState@CTreeCtrl@@QBEIPAU_TREEITEM@@I@Z',
    3298:'?GetItemStorage@COleClientItem@@UAEXXZ',
    3299:'?GetItemStorageCompound@COleClientItem@@QAEXXZ',
    3300:'?GetItemStorageFlat@COleClientItem@@QAEXXZ',
    3301:'?GetItemText@CListCtrl@@QBE?AVCString@@HH@Z',
    3302:'?GetItemText@CListCtrl@@QBEHHHPADH@Z',
    3303:'?GetItemText@CTreeCtrl@@QBE?AVCString@@PAU_TREEITEM@@@Z',
    3304:'?GetLastAccessTime@CFileFind@@UBEHAAVCTime@@@Z',
    3305:'?GetLastAccessTime@CFileFind@@UBEHPAU_FILETIME@@@Z',
    3306:'?GetLastAccessTime@CGopherFileFind@@UBEHAAVCTime@@@Z',
    3307:'?GetLastAccessTime@CGopherFileFind@@UBEHPAU_FILETIME@@@Z',
    3308:'?GetLastModifiedBookmark@CDaoRecordset@@QAE?AVCOleVariant@@XZ',
    3309:'?GetLastUpdate@COleUILinkInfo@@UAGJKPAU_FILETIME@@@Z',
    3310:'?GetLastWriteTime@CFileFind@@UBEHAAVCTime@@@Z',
    3311:'?GetLastWriteTime@CFileFind@@UBEHPAU_FILETIME@@@Z',
    3312:'?GetLastWriteTime@CGopherFileFind@@UBEHAAVCTime@@@Z',
    3313:'?GetLastWriteTime@CGopherFileFind@@UBEHPAU_FILETIME@@@Z',
    3314:'?GetLBFetchSize@CRecordset@@MAEJJ@Z',
    3315:'?GetLBound@COleSafeArray@@QAEXKPAJ@Z',
    3316:'?GetLBReallocSize@CRecordset@@MAEJJ@Z',
    3317:'?GetLBText@CComboBox@@QBEXHAAVCString@@@Z',
    3318:'?GetLength@CFile@@UBEKXZ',
    3319:'?GetLength@CFileFind@@QBEKXZ',
    3320:'?GetLength@CGopherFileFind@@UBEKXZ',
    3321:'?GetLength@CInternetFile@@UBEKXZ',
    3322:'?GetLength@COleStreamFile@@UBEKXZ',
    3323:'?GetLength@CSocketFile@@UBEKXZ',
    3324:'?GetLength64@CFileFind@@QBE_JXZ',
    3325:'?GetLength64@CGopherFileFind@@UBE_JXZ',
    3326:'?GetLicenseKey@COleObjectFactory@@MAEHKPAPAG@Z',
    3327:'?GetLicInfo@XClassFactory@COleObjectFactory@@UAGJPAUtagLICINFO@@@Z',
    3328:'?GetLine@CRichEditCtrl@@QBEHHPAD@Z',
    3329:'?GetLine@CRichEditCtrl@@QBEHHPADH@Z',
    3330:'?GetLinkSource@COleUILinkInfo@@UAGJKPAPADPAK00PAH2@Z',
    3331:'?GetLinkSourceData@COleClientItem@@IAEHPAUtagSTGMEDIUM@@@Z',
    3332:'?GetLinkSourceData@COleServerItem@@QAEHPAUtagSTGMEDIUM@@@Z',
    3333:'?GetLinkUpdateOptions@COleClientItem@@QAE?AW4tagOLEUPDATE@@XZ',
    3334:'?GetLinkUpdateOptions@COleUILinkInfo@@UAGJKPAK@Z',
    3335:'?GetList@CPropertySection@@QAEPAVCPtrList@@XZ',
    3336:'?GetList@CPropertySet@@QAEPAVCPtrList@@XZ',
    3337:'?GetLocalTm@CTime@@QBEPAUtm@@PAU2@@Z',
    3338:'?GetLocator@CGopherFileFind@@QBE?AVCGopherLocator@@XZ',
    3339:'?GetLockingMode@CDaoRecordset@@QAEHXZ',
    3340:'?GetLogFont@CFontComboBox@@QAEPAUtagLOGFONTA@@H@Z',
    3341:'?GetLoginTimeout@CDaoWorkspace@@SGFXZ',
    3342:'?GetLongBinaryData@CFieldExchange@@QAEXHAAVCLongBinary@@PAJ@Z',
    3343:'?GetLongBinaryDataAndCleanup@CRecordset@@SGXPAVCDatabase@@PAXFJPAPAXHAAVCDBVariant@@F@Z',
    3344:'?GetLongBinarySize@CFieldExchange@@QAEJH@Z',
    3345:'?GetLongCharDataAndCleanup@CRecordset@@SGXPAVCDatabase@@PAXFJPAPAXHAAVCString@@F@Z',
    3346:'?GetMainWnd@CWinThread@@UAEPAVCWnd@@XZ',
    3347:'?GetMargins@CPageSetupDialog@@QBEXPAUtagRECT@@0@Z',
    3348:'?GetMaxConnections@CConnectionPoint@@UAEHXZ',
    3349:'?GetMDIFrame@CMDIChildWnd@@QAEPAVCMDIFrameWnd@@XZ',
    3350:'?GetMessageBar@CFrameWnd@@UAEPAVCWnd@@XZ',
    3351:'?GetMessageBar@CMDIChildWnd@@MAEPAVCWnd@@XZ',
    3352:'?GetMessageMap@CCheckListBox@@MBEPBUAFX_MSGMAP@@XZ',
    3353:'?GetMessageMap@CCmdTarget@@MBEPBUAFX_MSGMAP@@XZ',
    3354:'?GetMessageMap@CColorDialog@@MBEPBUAFX_MSGMAP@@XZ',
    3355:'?GetMessageMap@CColorPropPage@@MBEPBUAFX_MSGMAP@@XZ',
    3356:'?GetMessageMap@CCommonDialog@@MBEPBUAFX_MSGMAP@@XZ',
    3357:'?GetMessageMap@CControlBar@@MBEPBUAFX_MSGMAP@@XZ',
    3358:'?GetMessageMap@CControlFrameWnd@@MBEPBUAFX_MSGMAP@@XZ',
    3359:'?GetMessageMap@CCtrlView@@MBEPBUAFX_MSGMAP@@XZ',
    3360:'?GetMessageMap@CDaoRecordView@@MBEPBUAFX_MSGMAP@@XZ',
    3361:'?GetMessageMap@CDialog@@MBEPBUAFX_MSGMAP@@XZ',
    3362:'?GetMessageMap@CDialogBar@@MBEPBUAFX_MSGMAP@@XZ',
    3363:'?GetMessageMap@CDockBar@@MBEPBUAFX_MSGMAP@@XZ',
    3364:'?GetMessageMap@CDocObjectServer@@MBEPBUAFX_MSGMAP@@XZ',
    3365:'?GetMessageMap@CDocument@@MBEPBUAFX_MSGMAP@@XZ',
    3366:'?GetMessageMap@CEditView@@MBEPBUAFX_MSGMAP@@XZ',
    3367:'?GetMessageMap@CFontPropPage@@MBEPBUAFX_MSGMAP@@XZ',
    3368:'?GetMessageMap@CFormView@@MBEPBUAFX_MSGMAP@@XZ',
    3369:'?GetMessageMap@CFrameWnd@@MBEPBUAFX_MSGMAP@@XZ',
    3370:'?GetMessageMap@CListCtrl@@MBEPBUAFX_MSGMAP@@XZ',
    3371:'?GetMessageMap@CListView@@MBEPBUAFX_MSGMAP@@XZ',
    3372:'?GetMessageMap@CMDIChildWnd@@MBEPBUAFX_MSGMAP@@XZ',
    3373:'?GetMessageMap@CMDIFrameWnd@@MBEPBUAFX_MSGMAP@@XZ',
    3374:'?GetMessageMap@CMiniDockFrameWnd@@MBEPBUAFX_MSGMAP@@XZ',
    3375:'?GetMessageMap@CMiniFrameWnd@@MBEPBUAFX_MSGMAP@@XZ',
    3376:'?GetMessageMap@COleControl@@MBEPBUAFX_MSGMAP@@XZ',
    3377:'?GetMessageMap@COleDocIPFrameWnd@@MBEPBUAFX_MSGMAP@@XZ',
    3378:'?GetMessageMap@COleIPFrameWnd@@MBEPBUAFX_MSGMAP@@XZ',
    3379:'?GetMessageMap@COlePropertyPage@@MBEPBUAFX_MSGMAP@@XZ',
    3380:'?GetMessageMap@COleResizeBar@@MBEPBUAFX_MSGMAP@@XZ',
    3381:'?GetMessageMap@COleServerDoc@@MBEPBUAFX_MSGMAP@@XZ',
    3382:'?GetMessageMap@CPicturePropPage@@MBEPBUAFX_MSGMAP@@XZ',
    3383:'?GetMessageMap@CPreviewView@@MBEPBUAFX_MSGMAP@@XZ',
    3384:'?GetMessageMap@CPrintDialog@@MBEPBUAFX_MSGMAP@@XZ',
    3385:'?GetMessageMap@CPropertyPage@@MBEPBUAFX_MSGMAP@@XZ',
    3386:'?GetMessageMap@CPropertySheet@@MBEPBUAFX_MSGMAP@@XZ',
    3387:'?GetMessageMap@CRecordView@@MBEPBUAFX_MSGMAP@@XZ',
    3388:'?GetMessageMap@CRichEditView@@MBEPBUAFX_MSGMAP@@XZ',
    3389:'?GetMessageMap@CScrollView@@MBEPBUAFX_MSGMAP@@XZ',
    3390:'?GetMessageMap@CSocketWnd@@MBEPBUAFX_MSGMAP@@XZ',
    3391:'?GetMessageMap@CSplitterWnd@@MBEPBUAFX_MSGMAP@@XZ',
    3392:'?GetMessageMap@CStatusBar@@MBEPBUAFX_MSGMAP@@XZ',
    3393:'?GetMessageMap@CStockPropPage@@MBEPBUAFX_MSGMAP@@XZ',
    3394:'?GetMessageMap@CTabCtrl@@MBEPBUAFX_MSGMAP@@XZ',
    3395:'?GetMessageMap@CToolBar@@MBEPBUAFX_MSGMAP@@XZ',
    3396:'?GetMessageMap@CToolBarCtrl@@MBEPBUAFX_MSGMAP@@XZ',
    3397:'?GetMessageMap@CToolTipCtrl@@MBEPBUAFX_MSGMAP@@XZ',
    3398:'?GetMessageMap@CTreeCtrl@@MBEPBUAFX_MSGMAP@@XZ',
    3399:'?GetMessageMap@CTreeView@@MBEPBUAFX_MSGMAP@@XZ',
    3400:'?GetMessageMap@CView@@MBEPBUAFX_MSGMAP@@XZ',
    3401:'?GetMessageMap@CWinApp@@MBEPBUAFX_MSGMAP@@XZ',
    3402:'?GetMessageMap@CWnd@@MBEPBUAFX_MSGMAP@@XZ',
    3403:'?GetMessageString@CFrameWnd@@UBEXIAAVCString@@@Z',
    3404:'?GetMessageString@COleControl@@UBEXIAAVCString@@@Z',
    3405:'?GetMetaData@CDataSourceControl@@QAEJXZ',
    3406:'?GetMetafileData@COleControl@@IAEHPAUtagFORMATETC@@PAUtagSTGMEDIUM@@@Z',
    3407:'?GetMetafileData@COleServerItem@@MAEHPAUtagFORMATETC@@PAUtagSTGMEDIUM@@@Z',
    3408:'?GetMinute@COleDateTime@@QBEHXZ',
    3409:'?GetMinutes@COleDateTimeSpan@@QBEJXZ',
    3410:'?GetMiscStatus@XOleObject@CDocObjectServer@@UAGJKPAK@Z',
    3411:'?GetMiscStatus@XOleObject@COleControl@@UAGJKPAK@Z',
    3412:'?GetMiscStatus@XOleObject@COleServerDoc@@UAGJKPAK@Z',
    3413:'?GetMiscStatus@XOleObject@COleServerItem@@UAGJKPAK@Z',
    3414:'?GetModifyPointers@CRectTracker@@IAEXHPAPAH0PAH1@Z',
    3415:'?GetMoniker@COleDocument@@UAEPAUIMoniker@@W4tagOLEGETMONIKER@@@Z',
    3416:'?GetMoniker@COleLinkingDoc@@UAEPAUIMoniker@@W4tagOLEGETMONIKER@@@Z',
    3417:'?GetMoniker@COleServerDoc@@UAEPAUIMoniker@@W4tagOLEGETMONIKER@@@Z',
    3418:'?GetMoniker@COleServerItem@@QAEPAUIMoniker@@W4tagOLEGETMONIKER@@@Z',
    3419:'?GetMoniker@XOleClientSite@COleClientItem@@UAGJKKPAPAUIMoniker@@@Z',
    3420:'?GetMoniker@XOleClientSite@COleControlSite@@UAGJKKPAPAUIMoniker@@@Z',
    3421:'?GetMoniker@XOleObject@CDocObjectServer@@UAGJKKPAPAUIMoniker@@@Z',
    3422:'?GetMoniker@XOleObject@COleControl@@UAGJKKPAPAUIMoniker@@@Z',
    3423:'?GetMoniker@XOleObject@COleServerDoc@@UAGJKKPAPAUIMoniker@@@Z',
    3424:'?GetMoniker@XOleObject@COleServerItem@@UAGJKKPAPAUIMoniker@@@Z',
    3425:'?GetMonth@COleDateTime@@QBEHXZ',
    3426:'?GetName@CDaoDatabase@@QAE?AVCString@@XZ',
    3427:'?GetName@CDaoQueryDef@@QAE?AVCString@@XZ',
    3428:'?GetName@CDaoRecordset@@QAE?AVCString@@XZ',
    3429:'?GetName@CDaoTableDef@@QAE?AVCString@@XZ',
    3430:'?GetName@CDaoWorkspace@@QAE?AVCString@@XZ',
    3431:'?GetNaturalExtent@XViewObject@COleControl@@UAGJKJPAUtagDVTARGETDEVICE@@PAUHDC__@@PAUtagExtentInfo@@PAUtagSIZE@@@Z',
    3432:'?GetNewItemNumber@COleClientItem@@QAEKXZ',
    3433:'?GetNewStorage@XRichEditOleCallback@CRichEditView@@UAGJPAPAUIStorage@@@Z',
    3434:'?GetNext@CDataBoundProperty@@QAEPAV1@XZ',
    3435:'?GetNextAssoc@CMapPtrToPtr@@QBEXAAPAU__POSITION@@AAPAX1@Z',
    3436:'?GetNextAssoc@CMapPtrToWord@@QBEXAAPAU__POSITION@@AAPAXAAG@Z',
    3437:'?GetNextAssoc@CMapStringToOb@@QBEXAAPAU__POSITION@@AAVCString@@AAPAVCObject@@@Z',
    3438:'?GetNextAssoc@CMapStringToPtr@@QBEXAAPAU__POSITION@@AAVCString@@AAPAX@Z',
    3439:'?GetNextAssoc@CMapStringToString@@QBEXAAPAU__POSITION@@AAVCString@@1@Z',
    3440:'?GetNextAssoc@CMapWordToOb@@QBEXAAPAU__POSITION@@AAGAAPAVCObject@@@Z',
    3441:'?GetNextAssoc@CMapWordToPtr@@QBEXAAPAU__POSITION@@AAGAAPAX@Z',
    3442:'?GetNextClientItem@COleDocument@@QBEPAVCOleClientItem@@AAPAU__POSITION@@@Z',
    3443:'?GetNextConnection@CConnectionPoint@@QBEPAUIUnknown@@AAPAU__POSITION@@@Z',
    3444:'?GetNextDoc@CMultiDocTemplate@@UBEPAVCDocument@@AAPAU__POSITION@@@Z',
    3445:'?GetNextDoc@CSingleDocTemplate@@UBEPAVCDocument@@AAPAU__POSITION@@@Z',
    3446:'?GetNextDocTemplate@CDocManager@@UBEPAVCDocTemplate@@AAPAU__POSITION@@@Z',
    3447:'?GetNextDocTemplate@CWinApp@@QBEPAVCDocTemplate@@AAPAU__POSITION@@@Z',
    3448:'?GetNextFormat@COleDataObject@@QAEHPAUtagFORMATETC@@@Z',
    3449:'?GetNextItem@COleDocument@@UBEPAVCDocItem@@AAPAU__POSITION@@@Z',
    3450:'?GetNextItemOfKind@COleDocument@@IBEPAVCDocItem@@AAPAU__POSITION@@PAUCRuntimeClass@@@Z',
    3451:'?GetNextLink@COleUILinkInfo@@UAGKK@Z',
    3452:'?GetNextPathName@CFileDialog@@QBE?AVCString@@AAPAU__POSITION@@@Z',
    3453:'?GetNextServerItem@COleDocument@@QBEPAVCOleServerItem@@AAPAU__POSITION@@@Z',
    3454:'?GetNextView@CDocument@@UBEPAVCView@@AAPAU__POSITION@@@Z',
    3455:'?GetNotifier@CFindReplaceDialog@@SGPAV1@J@Z',
    3456:'?GetNotSupported@CCmdTarget@@QAEXXZ',
    3457:'?GetNotSupported@COleControl@@QAEXXZ',
    3458:'?GetObjectA@CHttpFile@@QBE?AVCString@@XZ',
    3459:'?GetObjectA@XOleItemContainer@COleLinkingDoc@@UAGJPAGKPAUIBindCtx@@ABU_GUID@@PAPAX@Z',
    3460:'?GetObjectArray@COlePropertyPage@@QAEPAPAUIDispatch@@PAK@Z',
    3461:'?GetObjectDescriptorData@COleClientItem@@IAEXPAUtagPOINT@@PAUtagSIZE@@PAUtagSTGMEDIUM@@@Z',
    3462:'?GetObjectDescriptorData@COleServerItem@@QAEXPAUtagPOINT@@PAUtagSIZE@@PAUtagSTGMEDIUM@@@Z',
    3463:'?GetObjectInfo@XOleUIObjInfo@COlePropertiesDialog@@UAGJKPAKPAPAD111@Z',
    3464:'?GetObjectSchema@CArchive@@QAEIXZ',
    3465:'?GetObjectStorage@XOleItemContainer@COleLinkingDoc@@UAGJPAGPAUIBindCtx@@ABU_GUID@@PAPAX@Z',
    3466:'?GetODBCFieldInfo@CRecordset@@QAEXFAAUCODBCFieldInfo@@@Z',
    3467:'?GetODBCFieldInfo@CRecordset@@QAEXPBDAAUCODBCFieldInfo@@@Z',
    3468:'?GetODBCTimeout@CDaoQueryDef@@QAEFXZ',
    3469:'?GetOleObject@COleServerItem@@QAEPAUIOleObject@@XZ',
    3470:'?GetOneDimSize@COleSafeArray@@QAEKXZ',
    3471:'?GetOpenDocumentCount@CDocManager@@UAEHXZ',
    3472:'?GetOpenDocumentCount@CWinApp@@QAEHXZ',
    3473:'?GetOSVersion@CPropertySet@@QAEKXZ',
    3474:'?GetOuterWindow@COleControl@@IBEPAVCWnd@@XZ',
    3475:'?GetPageCount@CPropertySheet@@QBEHXZ',
    3476:'?GetPageIndex@CPropertySheet@@QAEHPAVCPropertyPage@@@Z',
    3477:'?GetPageInfo@XPrint@CDocObjectServer@@UAGJPAJ0@Z',
    3478:'?GetPageInfo@XPropertyPage@COlePropertyPage@@UAGJPAUtagPROPPAGEINFO@@@Z',
    3479:'?GetPages@XSpecifyPropertyPages@COleControl@@UAGJPAUtagCAUUID@@@Z',
    3480:'?GetPageSite@COlePropertyPage@@QAEPAUIPropertyPageSite@@XZ',
    3481:'?GetPane@CSplitterWnd@@QBEPAVCWnd@@HH@Z',
    3482:'?GetPaneInfo@CStatusBar@@QBEXHAAI0AAH@Z',
    3483:'?GetPaneStyle@CStatusBar@@QBEIH@Z',
    3484:'?GetPaneText@CStatusBar@@QBE?AVCString@@H@Z',
    3485:'?GetPaneText@CStatusBar@@QBEXHAAVCString@@@Z',
    3486:'?GetParaFormat@CRichEditCtrl@@QBEKAAU_paraformat@@@Z',
    3487:'?GetParaFormatSelection@CRichEditView@@QAEAAU_paraformat@@XZ',
    3488:'?GetParameterCount@CDaoQueryDef@@QAEFXZ',
    3489:'?GetParameterInfo@CDaoQueryDef@@QAEXHAAUCDaoParameterInfo@@K@Z',
    3490:'?GetParameterInfo@CDaoQueryDef@@QAEXPBDAAUCDaoParameterInfo@@K@Z',
    3491:'?GetParamValue@CDaoQueryDef@@UAE?AVCOleVariant@@H@Z',
    3492:'?GetParamValue@CDaoQueryDef@@UAE?AVCOleVariant@@PBD@Z',
    3493:'?GetParamValue@CDaoRecordset@@UAE?AVCOleVariant@@H@Z',
    3494:'?GetParamValue@CDaoRecordset@@UAE?AVCOleVariant@@PBD@Z',
    3495:'?GetParentFrame@CWnd@@QBEPAVCFrameWnd@@XZ',
    3496:'?GetParentOwner@CWnd@@QBEPAV1@XZ',
    3497:'?GetParentSplitter@CView@@SGPAVCSplitterWnd@@PBVCWnd@@H@Z',
    3498:'?GetPath@CDataPathProperty@@QBE?AVCString@@XZ',
    3499:'?GetPathName@CFileDialog@@QBE?AVCString@@XZ',
    3500:'?GetPeerName@CAsyncSocket@@QAEHAAVCString@@AAI@Z',
    3501:'?GetPercentPosition@CDaoRecordset@@QAEMXZ',
    3502:'?GetPictureDispatch@CPictureHolder@@QAEPAUIPictureDisp@@XZ',
    3503:'?GetPictureProp@CPicturePropPage@@IAEHPAVCDataExchange@@PAPAUIPictureDisp@@PBD@Z',
    3504:'?GetPointSize@CSizeComboBox@@QAEXAATtagCY@@@Z',
    3505:'?GetPortName@CPageSetupDialog@@QBE?AVCString@@XZ',
    3506:'?GetPortName@CPrintDialog@@QBE?AVCString@@XZ',
    3507:'?GetPosition@CFile@@UBEKXZ',
    3508:'?GetPosition@CMemFile@@UBEKXZ',
    3509:'?GetPosition@COleStreamFile@@UBEKXZ',
    3510:'?GetPosition@CSocketFile@@UBEKXZ',
    3511:'?GetPosition@CStdioFile@@UBEKXZ',
    3512:'?GetPredefinedStrings@XPerPropertyBrowsing@COleControl@@UAGJJPAUtagCALPOLESTR@@PAUtagCADWORD@@@Z',
    3513:'?GetPredefinedValue@XPerPropertyBrowsing@COleControl@@UAGJJKPAUtagVARIANT@@@Z',
    3514:'?GetPrimarySelectedItem@COleDocument@@UAEPAVCOleClientItem@@PAVCView@@@Z',
    3515:'?GetPrimarySelectedItem@CRichEditDoc@@UAEPAVCOleClientItem@@PAVCView@@@Z',
    3516:'?GetPrintDeviceInfo@COleClientItem@@IAEHPAPAUIOleCache@@PAPAUtagDVTARGETDEVICE@@PAK@Z',
    3517:'?GetPrinterDeviceDefaults@CWinApp@@QAEHPAUtagPDA@@@Z',
    3518:'?GetPrinterFont@CEditView@@QBEPAVCFont@@XZ',
    3519:'?GetPriority@CAsyncMonikerFile@@MBEJXZ',
    3520:'?GetProfileBinary@CWinApp@@QAEHPBD0PAPAEPAI@Z',
    3521:'?GetProfileIntA@CWinApp@@QAEIPBD0H@Z',
    3522:'?GetProfileStringA@CWinApp@@QAE?AVCString@@PBD00@Z',
    3523:'?GetPropCheck@COlePropertyPage@@QAEHPBDPAH@Z',
    3524:'?GetProperty@COleControlSite@@UBEXJGPAX@Z',
    3525:'?GetProperty@COleDispatchDriver@@QBEXJGPAX@Z',
    3526:'?GetProperty@CPropertySection@@QAEPAVCProperty@@K@Z',
    3527:'?GetProperty@CPropertySet@@QAEPAVCProperty@@U_GUID@@K@Z',
    3528:'?GetProperty@CWnd@@QBEXJGPAX@Z',
    3529:'?GetPropIndex@COlePropertyPage@@QAEHPBDPAH@Z',
    3530:'?GetPropPageIDs@COleControl@@MAEPAU_GUID@@AAK@Z',
    3531:'?GetPropRadio@COlePropertyPage@@QAEHPBDPAH@Z',
    3532:'?GetPropsetData@COleControl@@IAEHPAUtagFORMATETC@@PAUtagSTGMEDIUM@@ABU_GUID@@@Z',
    3533:'?GetPropText@COlePropertyPage@@QAEHPBDPAE@Z',
    3534:'?GetPropText@COlePropertyPage@@QAEHPBDPAF@Z',
    3535:'?GetPropText@COlePropertyPage@@QAEHPBDPAH@Z',
    3536:'?GetPropText@COlePropertyPage@@QAEHPBDPAI@Z',
    3537:'?GetPropText@COlePropertyPage@@QAEHPBDPAJ@Z',
    3538:'?GetPropText@COlePropertyPage@@QAEHPBDPAK@Z',
    3539:'?GetPropText@COlePropertyPage@@QAEHPBDPAM@Z',
    3540:'?GetPropText@COlePropertyPage@@QAEHPBDPAN@Z',
    3541:'?GetPropText@COlePropertyPage@@QAEHPBDPAVCString@@@Z',
    3542:'?GetQueryDefCount@CDaoDatabase@@QAEFXZ',
    3543:'?GetQueryDefInfo@CDaoDatabase@@QAEXHAAUCDaoQueryDefInfo@@K@Z',
    3544:'?GetQueryDefInfo@CDaoDatabase@@QAEXPBDAAUCDaoQueryDefInfo@@K@Z',
    3545:'?GetQueryTimeout@CDaoDatabase@@QAEFXZ',
    3546:'?GetRange@CSliderCtrl@@QBEXAAH0@Z',
    3547:'?GetRange@CSpinButtonCtrl@@QBEXAAH0@Z',
    3548:'?GetRawValue@CProperty@@QAEPAXXZ',
    3549:'?GetReadyState@COleControl@@QAEJXZ',
    3550:'?GetRecordCount@CDaoRecordset@@QAEJXZ',
    3551:'?GetRecordCount@CDaoTableDef@@QAEJXZ',
    3552:'?GetRecordsAffected@CDaoDatabase@@QAEJXZ',
    3553:'?GetRecordsAffected@CDaoQueryDef@@QAEJXZ',
    3554:'?GetRect@XOleDocumentView@CDocObjectServer@@UAGJPAUtagRECT@@@Z',
    3555:'?GetRect@XViewObject@COleControl@@UAGJKPAU_RECTL@@@Z',
    3556:'?GetRectInContainer@COleControl@@QAEHPAUtagRECT@@@Z',
    3557:'?GetRelationCount@CDaoDatabase@@QAEFXZ',
    3558:'?GetRelationInfo@CDaoDatabase@@QAEXHAAUCDaoRelationInfo@@K@Z',
    3559:'?GetRelationInfo@CDaoDatabase@@QAEXPBDAAUCDaoRelationInfo@@K@Z',
    3560:'?GetReplaceString@CFindReplaceDialog@@QBE?AVCString@@XZ',
    3561:'?GetReturnsRecords@CDaoQueryDef@@QAEHXZ',
    3562:'?GetRoot@CFileFind@@UBE?AVCString@@XZ',
    3563:'?GetRoot@CGopherFileFind@@UBE?AVCString@@XZ',
    3564:'?GetRoutingFrame@CCmdTarget@@IAEPAVCFrameWnd@@XZ',
    3565:'?GetRoutingView@CCmdTarget@@IAEPAVCView@@XZ',
    3566:'?GetRowInfo@CSplitterWnd@@QBEXHAAH0@Z',
    3567:'?GetRuntimeClass@CAnimateCtrl@@UBEPAUCRuntimeClass@@XZ',
    3568:'?GetRuntimeClass@CArchiveException@@UBEPAUCRuntimeClass@@XZ',
    3569:'?GetRuntimeClass@CAsyncMonikerFile@@UBEPAUCRuntimeClass@@XZ',
    3570:'?GetRuntimeClass@CAsyncSocket@@UBEPAUCRuntimeClass@@XZ',
    3571:'?GetRuntimeClass@CBitmap@@UBEPAUCRuntimeClass@@XZ',
    3572:'?GetRuntimeClass@CBitmapButton@@UBEPAUCRuntimeClass@@XZ',
    3573:'?GetRuntimeClass@CBrush@@UBEPAUCRuntimeClass@@XZ',
    3574:'?GetRuntimeClass@CButton@@UBEPAUCRuntimeClass@@XZ',
    3575:'?GetRuntimeClass@CByteArray@@UBEPAUCRuntimeClass@@XZ',
    3576:'?GetRuntimeClass@CCachedDataPathProperty@@UBEPAUCRuntimeClass@@XZ',
    3577:'?GetRuntimeClass@CCheckListBox@@UBEPAUCRuntimeClass@@XZ',
    3578:'?GetRuntimeClass@CClientDC@@UBEPAUCRuntimeClass@@XZ',
    3579:'?GetRuntimeClass@CCmdTarget@@UBEPAUCRuntimeClass@@XZ',
    3580:'?GetRuntimeClass@CColorDialog@@UBEPAUCRuntimeClass@@XZ',
    3581:'?GetRuntimeClass@CColorPropPage@@UBEPAUCRuntimeClass@@XZ',
    3582:'?GetRuntimeClass@CComboBox@@UBEPAUCRuntimeClass@@XZ',
    3583:'?GetRuntimeClass@CControlBar@@UBEPAUCRuntimeClass@@XZ',
    3584:'?GetRuntimeClass@CCriticalSection@@UBEPAUCRuntimeClass@@XZ',
    3585:'?GetRuntimeClass@CCtrlView@@UBEPAUCRuntimeClass@@XZ',
    3586:'?GetRuntimeClass@CDaoDatabase@@UBEPAUCRuntimeClass@@XZ',
    3587:'?GetRuntimeClass@CDaoException@@UBEPAUCRuntimeClass@@XZ',
    3588:'?GetRuntimeClass@CDaoQueryDef@@UBEPAUCRuntimeClass@@XZ',
    3589:'?GetRuntimeClass@CDaoRecordset@@UBEPAUCRuntimeClass@@XZ',
    3590:'?GetRuntimeClass@CDaoRecordView@@UBEPAUCRuntimeClass@@XZ',
    3591:'?GetRuntimeClass@CDaoTableDef@@UBEPAUCRuntimeClass@@XZ',
    3592:'?GetRuntimeClass@CDaoWorkspace@@UBEPAUCRuntimeClass@@XZ',
    3593:'?GetRuntimeClass@CDatabase@@UBEPAUCRuntimeClass@@XZ',
    3594:'?GetRuntimeClass@CDataPathProperty@@UBEPAUCRuntimeClass@@XZ',
    3595:'?GetRuntimeClass@CDBException@@UBEPAUCRuntimeClass@@XZ',
    3596:'?GetRuntimeClass@CDC@@UBEPAUCRuntimeClass@@XZ',
    3597:'?GetRuntimeClass@CDialog@@UBEPAUCRuntimeClass@@XZ',
    3598:'?GetRuntimeClass@CDialogBar@@UBEPAUCRuntimeClass@@XZ',
    3599:'?GetRuntimeClass@CDocItem@@UBEPAUCRuntimeClass@@XZ',
    3600:'?GetRuntimeClass@CDockBar@@UBEPAUCRuntimeClass@@XZ',
    3601:'?GetRuntimeClass@CDockState@@UBEPAUCRuntimeClass@@XZ',
    3602:'?GetRuntimeClass@CDocManager@@UBEPAUCRuntimeClass@@XZ',
    3603:'?GetRuntimeClass@CDocObjectServer@@UBEPAUCRuntimeClass@@XZ',
    3604:'?GetRuntimeClass@CDocObjectServerItem@@UBEPAUCRuntimeClass@@XZ',
    3605:'?GetRuntimeClass@CDocTemplate@@UBEPAUCRuntimeClass@@XZ',
    3606:'?GetRuntimeClass@CDocument@@UBEPAUCRuntimeClass@@XZ',
    3607:'?GetRuntimeClass@CDragListBox@@UBEPAUCRuntimeClass@@XZ',
    3608:'?GetRuntimeClass@CDWordArray@@UBEPAUCRuntimeClass@@XZ',
    3609:'?GetRuntimeClass@CDynLinkLibrary@@UBEPAUCRuntimeClass@@XZ',
    3610:'?GetRuntimeClass@CEdit@@UBEPAUCRuntimeClass@@XZ',
    3611:'?GetRuntimeClass@CEditView@@UBEPAUCRuntimeClass@@XZ',
    3612:'?GetRuntimeClass@CEvent@@UBEPAUCRuntimeClass@@XZ',
    3613:'?GetRuntimeClass@CException@@UBEPAUCRuntimeClass@@XZ',
    3614:'?GetRuntimeClass@CFile@@UBEPAUCRuntimeClass@@XZ',
    3615:'?GetRuntimeClass@CFileDialog@@UBEPAUCRuntimeClass@@XZ',
    3616:'?GetRuntimeClass@CFileException@@UBEPAUCRuntimeClass@@XZ',
    3617:'?GetRuntimeClass@CFileFind@@UBEPAUCRuntimeClass@@XZ',
    3618:'?GetRuntimeClass@CFindReplaceDialog@@UBEPAUCRuntimeClass@@XZ',
    3619:'?GetRuntimeClass@CFont@@UBEPAUCRuntimeClass@@XZ',
    3620:'?GetRuntimeClass@CFontDialog@@UBEPAUCRuntimeClass@@XZ',
    3621:'?GetRuntimeClass@CFontPropPage@@UBEPAUCRuntimeClass@@XZ',
    3622:'?GetRuntimeClass@CFormView@@UBEPAUCRuntimeClass@@XZ',
    3623:'?GetRuntimeClass@CFrameWnd@@UBEPAUCRuntimeClass@@XZ',
    3624:'?GetRuntimeClass@CFtpConnection@@UBEPAUCRuntimeClass@@XZ',
    3625:'?GetRuntimeClass@CFtpFileFind@@UBEPAUCRuntimeClass@@XZ',
    3626:'?GetRuntimeClass@CGdiObject@@UBEPAUCRuntimeClass@@XZ',
    3627:'?GetRuntimeClass@CGopherConnection@@UBEPAUCRuntimeClass@@XZ',
    3628:'?GetRuntimeClass@CGopherFile@@UBEPAUCRuntimeClass@@XZ',
    3629:'?GetRuntimeClass@CGopherFileFind@@UBEPAUCRuntimeClass@@XZ',
    3630:'?GetRuntimeClass@CHeaderCtrl@@UBEPAUCRuntimeClass@@XZ',
    3631:'?GetRuntimeClass@CHotKeyCtrl@@UBEPAUCRuntimeClass@@XZ',
    3632:'?GetRuntimeClass@CHttpConnection@@UBEPAUCRuntimeClass@@XZ',
    3633:'?GetRuntimeClass@CHttpFile@@UBEPAUCRuntimeClass@@XZ',
    3634:'?GetRuntimeClass@CImageList@@UBEPAUCRuntimeClass@@XZ',
    3635:'?GetRuntimeClass@CInternetConnection@@UBEPAUCRuntimeClass@@XZ',
    3636:'?GetRuntimeClass@CInternetException@@UBEPAUCRuntimeClass@@XZ',
    3637:'?GetRuntimeClass@CInternetFile@@UBEPAUCRuntimeClass@@XZ',
    3638:'?GetRuntimeClass@CInternetSession@@UBEPAUCRuntimeClass@@XZ',
    3639:'?GetRuntimeClass@CListBox@@UBEPAUCRuntimeClass@@XZ',
    3640:'?GetRuntimeClass@CListCtrl@@UBEPAUCRuntimeClass@@XZ',
    3641:'?GetRuntimeClass@CListView@@UBEPAUCRuntimeClass@@XZ',
    3642:'?GetRuntimeClass@CLongBinary@@UBEPAUCRuntimeClass@@XZ',
    3643:'?GetRuntimeClass@CMapPtrToPtr@@UBEPAUCRuntimeClass@@XZ',
    3644:'?GetRuntimeClass@CMapPtrToWord@@UBEPAUCRuntimeClass@@XZ',
    3645:'?GetRuntimeClass@CMapStringToOb@@UBEPAUCRuntimeClass@@XZ',
    3646:'?GetRuntimeClass@CMapStringToPtr@@UBEPAUCRuntimeClass@@XZ',
    3647:'?GetRuntimeClass@CMapStringToString@@UBEPAUCRuntimeClass@@XZ',
    3648:'?GetRuntimeClass@CMapWordToOb@@UBEPAUCRuntimeClass@@XZ',
    3649:'?GetRuntimeClass@CMapWordToPtr@@UBEPAUCRuntimeClass@@XZ',
    3650:'?GetRuntimeClass@CMDIChildWnd@@UBEPAUCRuntimeClass@@XZ',
    3651:'?GetRuntimeClass@CMDIFrameWnd@@UBEPAUCRuntimeClass@@XZ',
    3652:'?GetRuntimeClass@CMemFile@@UBEPAUCRuntimeClass@@XZ',
    3653:'?GetRuntimeClass@CMemoryException@@UBEPAUCRuntimeClass@@XZ',
    3654:'?GetRuntimeClass@CMenu@@UBEPAUCRuntimeClass@@XZ',
    3655:'?GetRuntimeClass@CMetaFileDC@@UBEPAUCRuntimeClass@@XZ',
    3656:'?GetRuntimeClass@CMiniDockFrameWnd@@UBEPAUCRuntimeClass@@XZ',
    3657:'?GetRuntimeClass@CMiniFrameWnd@@UBEPAUCRuntimeClass@@XZ',
    3658:'?GetRuntimeClass@CMonikerFile@@UBEPAUCRuntimeClass@@XZ',
    3659:'?GetRuntimeClass@CMultiDocTemplate@@UBEPAUCRuntimeClass@@XZ',
    3660:'?GetRuntimeClass@CMutex@@UBEPAUCRuntimeClass@@XZ',
    3661:'?GetRuntimeClass@CNotSupportedException@@UBEPAUCRuntimeClass@@XZ',
    3662:'?GetRuntimeClass@CObArray@@UBEPAUCRuntimeClass@@XZ',
    3663:'?GetRuntimeClass@CObject@@UBEPAUCRuntimeClass@@XZ',
    3664:'?GetRuntimeClass@CObList@@UBEPAUCRuntimeClass@@XZ',
    3665:'?GetRuntimeClass@COleBusyDialog@@UBEPAUCRuntimeClass@@XZ',
    3666:'?GetRuntimeClass@COleChangeIconDialog@@UBEPAUCRuntimeClass@@XZ',
    3667:'?GetRuntimeClass@COleChangeSourceDialog@@UBEPAUCRuntimeClass@@XZ',
    3668:'?GetRuntimeClass@COleClientItem@@UBEPAUCRuntimeClass@@XZ',
    3669:'?GetRuntimeClass@COleControl@@UBEPAUCRuntimeClass@@XZ',
    3670:'?GetRuntimeClass@COleControlModule@@UBEPAUCRuntimeClass@@XZ',
    3671:'?GetRuntimeClass@COleConvertDialog@@UBEPAUCRuntimeClass@@XZ',
    3672:'?GetRuntimeClass@COleDialog@@UBEPAUCRuntimeClass@@XZ',
    3673:'?GetRuntimeClass@COleDispatchException@@UBEPAUCRuntimeClass@@XZ',
    3674:'?GetRuntimeClass@COleDocIPFrameWnd@@UBEPAUCRuntimeClass@@XZ',
    3675:'?GetRuntimeClass@COleDocument@@UBEPAUCRuntimeClass@@XZ',
    3676:'?GetRuntimeClass@COleException@@UBEPAUCRuntimeClass@@XZ',
    3677:'?GetRuntimeClass@COleInsertDialog@@UBEPAUCRuntimeClass@@XZ',
    3678:'?GetRuntimeClass@COleIPFrameWnd@@UBEPAUCRuntimeClass@@XZ',
    3679:'?GetRuntimeClass@COleLinkingDoc@@UBEPAUCRuntimeClass@@XZ',
    3680:'?GetRuntimeClass@COleLinksDialog@@UBEPAUCRuntimeClass@@XZ',
    3681:'?GetRuntimeClass@COleObjectFactory@@UBEPAUCRuntimeClass@@XZ',
    3682:'?GetRuntimeClass@COlePasteSpecialDialog@@UBEPAUCRuntimeClass@@XZ',
    3683:'?GetRuntimeClass@COlePropertiesDialog@@UBEPAUCRuntimeClass@@XZ',
    3684:'?GetRuntimeClass@COlePropertyPage@@UBEPAUCRuntimeClass@@XZ',
    3685:'?GetRuntimeClass@COleResizeBar@@UBEPAUCRuntimeClass@@XZ',
    3686:'?GetRuntimeClass@COleServerDoc@@UBEPAUCRuntimeClass@@XZ',
    3687:'?GetRuntimeClass@COleServerItem@@UBEPAUCRuntimeClass@@XZ',
    3688:'?GetRuntimeClass@COleStreamFile@@UBEPAUCRuntimeClass@@XZ',
    3689:'?GetRuntimeClass@COleUpdateDialog@@UBEPAUCRuntimeClass@@XZ',
    3690:'?GetRuntimeClass@CPageSetupDialog@@UBEPAUCRuntimeClass@@XZ',
    3691:'?GetRuntimeClass@CPaintDC@@UBEPAUCRuntimeClass@@XZ',
    3692:'?GetRuntimeClass@CPalette@@UBEPAUCRuntimeClass@@XZ',
    3693:'?GetRuntimeClass@CPen@@UBEPAUCRuntimeClass@@XZ',
    3694:'?GetRuntimeClass@CPicturePropPage@@UBEPAUCRuntimeClass@@XZ',
    3695:'?GetRuntimeClass@CPreviewDC@@UBEPAUCRuntimeClass@@XZ',
    3696:'?GetRuntimeClass@CPreviewView@@UBEPAUCRuntimeClass@@XZ',
    3697:'?GetRuntimeClass@CPrintDialog@@UBEPAUCRuntimeClass@@XZ',
    3698:'?GetRuntimeClass@CProgressCtrl@@UBEPAUCRuntimeClass@@XZ',
    3699:'?GetRuntimeClass@CPropertyPage@@UBEPAUCRuntimeClass@@XZ',
    3700:'?GetRuntimeClass@CPropertySheet@@UBEPAUCRuntimeClass@@XZ',
    3701:'?GetRuntimeClass@CPtrArray@@UBEPAUCRuntimeClass@@XZ',
    3702:'?GetRuntimeClass@CPtrList@@UBEPAUCRuntimeClass@@XZ',
    3703:'?GetRuntimeClass@CRecordset@@UBEPAUCRuntimeClass@@XZ',
    3704:'?GetRuntimeClass@CRecordView@@UBEPAUCRuntimeClass@@XZ',
    3705:'?GetRuntimeClass@CResourceException@@UBEPAUCRuntimeClass@@XZ',
    3706:'?GetRuntimeClass@CRgn@@UBEPAUCRuntimeClass@@XZ',
    3707:'?GetRuntimeClass@CRichEditCntrItem@@UBEPAUCRuntimeClass@@XZ',
    3708:'?GetRuntimeClass@CRichEditCtrl@@UBEPAUCRuntimeClass@@XZ',
    3709:'?GetRuntimeClass@CRichEditDoc@@UBEPAUCRuntimeClass@@XZ',
    3710:'?GetRuntimeClass@CRichEditView@@UBEPAUCRuntimeClass@@XZ',
    3711:'?GetRuntimeClass@CScrollBar@@UBEPAUCRuntimeClass@@XZ',
    3712:'?GetRuntimeClass@CScrollView@@UBEPAUCRuntimeClass@@XZ',
    3713:'?GetRuntimeClass@CSemaphore@@UBEPAUCRuntimeClass@@XZ',
    3714:'?GetRuntimeClass@CSharedFile@@UBEPAUCRuntimeClass@@XZ',
    3715:'?GetRuntimeClass@CSingleDocTemplate@@UBEPAUCRuntimeClass@@XZ',
    3716:'?GetRuntimeClass@CSliderCtrl@@UBEPAUCRuntimeClass@@XZ',
    3717:'?GetRuntimeClass@CSocket@@UBEPAUCRuntimeClass@@XZ',
    3718:'?GetRuntimeClass@CSocketFile@@UBEPAUCRuntimeClass@@XZ',
    3719:'?GetRuntimeClass@CSpinButtonCtrl@@UBEPAUCRuntimeClass@@XZ',
    3720:'?GetRuntimeClass@CSplitterWnd@@UBEPAUCRuntimeClass@@XZ',
    3721:'?GetRuntimeClass@CStatic@@UBEPAUCRuntimeClass@@XZ',
    3722:'?GetRuntimeClass@CStatusBar@@UBEPAUCRuntimeClass@@XZ',
    3723:'?GetRuntimeClass@CStatusBarCtrl@@UBEPAUCRuntimeClass@@XZ',
    3724:'?GetRuntimeClass@CStdioFile@@UBEPAUCRuntimeClass@@XZ',
    3725:'?GetRuntimeClass@CStockPropPage@@UBEPAUCRuntimeClass@@XZ',
    3726:'?GetRuntimeClass@CStringArray@@UBEPAUCRuntimeClass@@XZ',
    3727:'?GetRuntimeClass@CStringList@@UBEPAUCRuntimeClass@@XZ',
    3728:'?GetRuntimeClass@CSyncObject@@UBEPAUCRuntimeClass@@XZ',
    3729:'?GetRuntimeClass@CTabCtrl@@UBEPAUCRuntimeClass@@XZ',
    3730:'?GetRuntimeClass@CToolBar@@UBEPAUCRuntimeClass@@XZ',
    3731:'?GetRuntimeClass@CToolBarCtrl@@UBEPAUCRuntimeClass@@XZ',
    3732:'?GetRuntimeClass@CToolTipCtrl@@UBEPAUCRuntimeClass@@XZ',
    3733:'?GetRuntimeClass@CTreeCtrl@@UBEPAUCRuntimeClass@@XZ',
    3734:'?GetRuntimeClass@CTreeView@@UBEPAUCRuntimeClass@@XZ',
    3735:'?GetRuntimeClass@CUIntArray@@UBEPAUCRuntimeClass@@XZ',
    3736:'?GetRuntimeClass@CUserException@@UBEPAUCRuntimeClass@@XZ',
    3737:'?GetRuntimeClass@CView@@UBEPAUCRuntimeClass@@XZ',
    3738:'?GetRuntimeClass@CWinApp@@UBEPAUCRuntimeClass@@XZ',
    3739:'?GetRuntimeClass@CWindowDC@@UBEPAUCRuntimeClass@@XZ',
    3740:'?GetRuntimeClass@CWindowlessDC@@UBEPAUCRuntimeClass@@XZ',
    3741:'?GetRuntimeClass@CWinThread@@UBEPAUCRuntimeClass@@XZ',
    3742:'?GetRuntimeClass@CWnd@@UBEPAUCRuntimeClass@@XZ',
    3743:'?GetRuntimeClass@CWordArray@@UBEPAUCRuntimeClass@@XZ',
    3744:'?GetSafeOwner@CWnd@@SGPAV1@PAV1@PAPAUHWND__@@@Z',
    3745:'?GetSavedCustomColors@CColorDialog@@SGPAKXZ',
    3746:'?GetScreenName@CGopherFileFind@@QBE?AVCString@@XZ',
    3747:'?GetScreenSize@CDockState@@QAE?AVCSize@@XZ',
    3748:'?GetScrollBarCtrl@CView@@UBEPAVCScrollBar@@H@Z',
    3749:'?GetScrollBarCtrl@CWnd@@UBEPAVCScrollBar@@H@Z',
    3750:'?GetScrollBarSizes@CScrollView@@IAEXAAVCSize@@@Z',
    3751:'?GetScrollBarState@CScrollView@@IAEXVCSize@@AAV2@1AAVCPoint@@H@Z',
    3752:'?GetScrollInfo@CWnd@@QAEHHPAUtagSCROLLINFO@@I@Z',
    3753:'?GetScrollLimit@CWnd@@QAEHH@Z',
    3754:'?GetScrollPos@CWnd@@QBEHH@Z',
    3755:'?GetScrollPosition@CScrollView@@QBE?AVCPoint@@XZ',
    3756:'?GetScrollRange@CWnd@@QBEXHPAH0@Z',
    3757:'?GetScrollStyle@CSplitterWnd@@QBEKXZ',
    3758:'?GetSecond@COleDateTime@@QBEHXZ',
    3759:'?GetSeconds@COleDateTimeSpan@@QBEJXZ',
    3760:'?GetSection@CPropertySet@@QAEPAVCPropertySection@@U_GUID@@@Z',
    3761:'?GetSectionKey@CWinApp@@QAEPAUHKEY__@@PBD@Z',
    3762:'?GetSectionName@CPropertySection@@QAEPBDXZ',
    3763:'?GetSel@CRichEditCtrl@@QBEXAAJ0@Z',
    3764:'?GetSelectedItem@CRichEditView@@QBEPAVCRichEditCntrItem@@XZ',
    3765:'?GetSelectedText@CEditView@@QBEXAAVCString@@@Z',
    3766:'?GetSelection@CSliderCtrl@@QBEXAAH0@Z',
    3767:'?GetSelectionCharFormat@CRichEditCtrl@@QBEKAAU_charformat@@@Z',
    3768:'?GetSelectionType@COleConvertDialog@@QBEIXZ',
    3769:'?GetSelectionType@COleInsertDialog@@QBEIXZ',
    3770:'?GetSelectionType@COlePasteSpecialDialog@@QBEIXZ',
    3771:'?GetSelText@CRichEditCtrl@@QBE?AVCString@@XZ',
    3772:'?GetServerName@CInternetConnection@@QBE?AVCString@@XZ',
    3773:'?GetSize@CPropertySection@@QAEKXZ',
    3774:'?GetSizeInDialogUnits@CDialogTemplate@@QBEXPAUtagSIZE@@@Z',
    3775:'?GetSizeInPixels@CDialogTemplate@@QBEXPAUtagSIZE@@@Z',
    3776:'?GetSizeMax@CBlobProperty@@UAGJPAT_ULARGE_INTEGER@@@Z',
    3777:'?GetSizeMax@XPersistMemory@COleControl@@UAGJPAK@Z',
    3778:'?GetSizeMax@XPersistStreamInit@COleControl@@UAGJPAT_ULARGE_INTEGER@@@Z',
    3779:'?GetSizingParent@CSplitterWnd@@IAEPAVCWnd@@XZ',
    3780:'?GetSockName@CAsyncSocket@@QAEHAAVCString@@AAI@Z',
    3781:'?GetSourceTableName@CDaoTableDef@@QAE?AVCString@@XZ',
    3782:'?GetSQL@CDaoQueryDef@@QAE?AVCString@@XZ',
    3783:'?GetSQL@CDaoRecordset@@QBE?AVCString@@XZ',
    3784:'?GetStackSize@CCmdTarget@@KGIPBEG@Z',
    3785:'?GetStandardProp@CCmdTarget@@IAEXPBUAFX_DISPMAP_ENTRY@@PAUtagVARIANT@@PAI@Z',
    3786:'?GetStartPosition@CConnectionPoint@@QBEPAU__POSITION@@XZ',
    3787:'?GetStartPosition@COleDocument@@UBEPAU__POSITION@@XZ',
    3788:'?GetStartPosition@CRichEditDoc@@UBEPAU__POSITION@@XZ',
    3789:'?GetStatus@CFile@@QBEHAAUCFileStatus@@@Z',
    3790:'?GetStatus@CFile@@SGHPBDAAUCFileStatus@@@Z',
    3791:'?GetStatus@CMemFile@@QBEHAAUCFileStatus@@@Z',
    3792:'?GetStatus@COleStreamFile@@QBEHAAUCFileStatus@@@Z',
    3793:'?GetStockTextMetrics@COleControl@@QAEXPAUtagTEXTMETRICA@@@Z',
    3794:'?GetStorageName@COleStreamFile@@UBE?BVCString@@XZ',
    3795:'?GetStream@COleStreamFile@@QBEPAUIStream@@XZ',
    3796:'?GetStyle@COleControlSite@@UBEKXZ',
    3797:'?GetStyle@CWnd@@QBEKXZ',
    3798:'?GetSuperWndProcAddr@CWnd@@MAEPAP6GJPAUHWND__@@IIJ@ZXZ',
    3799:'?GetTableDefCount@CDaoDatabase@@QAEFXZ',
    3800:'?GetTableDefInfo@CDaoDatabase@@QAEXHAAUCDaoTableDefInfo@@K@Z',
    3801:'?GetTableDefInfo@CDaoDatabase@@QAEXPBDAAUCDaoTableDefInfo@@K@Z',
    3802:'?GetTemplateSize@CDialogTemplate@@KAIPBUDLGTEMPLATE@@@Z',
    3803:'?GetText@CListBox@@QBEXHAAVCString@@@Z',
    3804:'?GetText@COleControl@@QAEPAGXZ',
    3805:'?GetText@CStatusBarCtrl@@QBE?AVCString@@HPAH@Z',
    3806:'?GetText@CStatusBarCtrl@@QBEHPBDHPAH@Z',
    3807:'?GetText@CToolTipCtrl@@QBEXAAVCString@@PAVCWnd@@I@Z',
    3808:'?GetTextLen@CRecordset@@SGHFK@Z',
    3809:'?GetTextLength@CStatusBarCtrl@@QBEHHPAH@Z',
    3810:'?GetTickCount@COleDateTime@@SG?AV1@XZ',
    3811:'?GetTickCount@CTime@@SG?AV1@XZ',
    3812:'?GetToolInfo@CToolTipCtrl@@QBEHAAVCToolInfo@@PAVCWnd@@I@Z',
    3813:'?GetTopLevelFrame@CWnd@@QBEPAVCFrameWnd@@XZ',
    3814:'?GetTopLevelOwner@CWnd@@QBEPAV1@XZ',
    3815:'?GetTopLevelParent@CWnd@@QBEPAV1@XZ',
    3816:'?GetTrueClientSize@CScrollView@@IAEHAAVCSize@@0@Z',
    3817:'?GetTrueRect@CRectTracker@@QBEXPAUtagRECT@@@Z',
    3818:'?GetType@CDaoQueryDef@@QAEFXZ',
    3819:'?GetType@CDaoRecordset@@QAEFXZ',
    3820:'?GetType@CPictureHolder@@QAEFXZ',
    3821:'?GetType@CProperty@@QAEKXZ',
    3822:'?GetTypeInfo@COleDispatchImpl@@UAGJIKPAPAUITypeInfo@@@Z',
    3823:'?GetTypeInfo@XAmbientProps@COleControlSite@@UAGJIKPAPAUITypeInfo@@@Z',
    3824:'?GetTypeInfo@XEventSink@COleControlSite@@UAGJIKPAPAUITypeInfo@@@Z',
    3825:'?GetTypeInfoCount@CCmdTarget@@UAEIXZ',
    3826:'?GetTypeInfoCount@COleDispatchImpl@@UAGJPAI@Z',
    3827:'?GetTypeInfoCount@XAmbientProps@COleControlSite@@UAGJPAI@Z',
    3828:'?GetTypeInfoCount@XEventSink@COleControlSite@@UAGJPAI@Z',
    3829:'?GetTypeInfoOfGuid@CCmdTarget@@QAEJKABU_GUID@@PAPAUITypeInfo@@@Z',
    3830:'?GetTypeLib@CCmdTarget@@UAEJKPAPAUITypeLib@@@Z',
    3831:'?GetTypeLibCache@CCmdTarget@@UAEPAVCTypeLibCache@@XZ',
    3832:'?GetUBound@COleSafeArray@@QAEXKPAJ@Z',
    3833:'?GetUserClassID@XOleObject@CDocObjectServer@@UAGJPAU_GUID@@@Z',
    3834:'?GetUserClassID@XOleObject@COleControl@@UAGJPAU_GUID@@@Z',
    3835:'?GetUserClassID@XOleObject@COleServerDoc@@UAGJPAU_GUID@@@Z',
    3836:'?GetUserClassID@XOleObject@COleServerItem@@UAGJPAU_GUID@@@Z',
    3837:'?GetUserNameA@CDaoWorkspace@@QAE?AVCString@@XZ',
    3838:'?GetUserType@COleClientItem@@QAEXW4tagUSERCLASSTYPE@@AAVCString@@@Z',
    3839:'?GetUserType@COleControl@@IAEXPAD@Z',
    3840:'?GetUserType@XOleObject@CDocObjectServer@@UAGJKPAPAG@Z',
    3841:'?GetUserType@XOleObject@COleControl@@UAGJKPAPAG@Z',
    3842:'?GetUserType@XOleObject@COleServerDoc@@UAGJKPAPAG@Z',
    3843:'?GetUserType@XOleObject@COleServerItem@@UAGJKPAPAG@Z',
    3844:'?GetValidationRule@CDaoRecordset@@QAE?AVCString@@XZ',
    3845:'?GetValidationRule@CDaoTableDef@@QAE?AVCString@@XZ',
    3846:'?GetValidationText@CDaoRecordset@@QAE?AVCString@@XZ',
    3847:'?GetValidationText@CDaoTableDef@@QAE?AVCString@@XZ',
    3848:'?GetValueAt@CMapPtrToPtr@@QBEPAXPAX@Z',
    3849:'?GetVerb@CHttpFile@@QBE?AVCString@@XZ',
    3850:'?GetVersion@CDaoDatabase@@QAE?AVCString@@XZ',
    3851:'?GetVersion@CDaoWorkspace@@SG?AVCString@@XZ',
    3852:'?GetVersion@CDockState@@QAEKXZ',
    3853:'?GetView@CRichEditDoc@@UBEPAVCRichEditView@@XZ',
    3854:'?GetViewInfo@XOleUIObjInfo@COlePropertiesDialog@@UAGJKPAPAXPAKPAH@Z',
    3855:'?GetViewStatus@XViewObject@COleControl@@UAGJPAK@Z',
    3856:'?GetWindow@XOleInPlaceActiveObject@COleControl@@UAGJPAPAUHWND__@@@Z',
    3857:'?GetWindow@XOleInPlaceActiveObject@COleServerDoc@@UAGJPAPAUHWND__@@@Z',
    3858:'?GetWindow@XOleInPlaceFrame@COleFrameHook@@UAGJPAPAUHWND__@@@Z',
    3859:'?GetWindow@XOleInPlaceObject@COleControl@@UAGJPAPAUHWND__@@@Z',
    3860:'?GetWindow@XOleInPlaceObject@COleServerDoc@@UAGJPAPAUHWND__@@@Z',
    3861:'?GetWindow@XOleIPFrame@COleControlContainer@@UAGJPAPAUHWND__@@@Z',
    3862:'?GetWindow@XOleIPSite@COleClientItem@@UAGJPAPAUHWND__@@@Z',
    3863:'?GetWindow@XOleIPSite@COleControlSite@@UAGJPAPAUHWND__@@@Z',
    3864:'?GetWindowContext@CRichEditCntrItem@@QAEJPAPAUIOleInPlaceFrame@@PAPAUIOleInPlaceUIWindow@@PAUtagOIFI@@@Z',
    3865:'?GetWindowContext@CRichEditView@@QAEJPAPAUIOleInPlaceFrame@@PAPAUIOleInPlaceUIWindow@@PAUtagOIFI@@@Z',
    3866:'?GetWindowContext@XOleIPSite@COleClientItem@@UAGJPAPAUIOleInPlaceFrame@@PAPAUIOleInPlaceUIWindow@@PAUtagRECT@@2PAUtagOIFI@@@Z',
    3867:'?GetWindowContext@XOleIPSite@COleControlSite@@UAGJPAPAUIOleInPlaceFrame@@PAPAUIOleInPlaceUIWindow@@PAUtagRECT@@2PAUtagOIFI@@@Z',
    3868:'?GetWindowlessDropTarget@COleControl@@UAEPAUIDropTarget@@XZ',
    3869:'?GetWindowMenuPopup@CMDIFrameWnd@@UAEPAUHMENU__@@PAU2@@Z',
    3870:'?GetWindowPlacement@CWnd@@QBEHPAUtagWINDOWPLACEMENT@@@Z',
    3871:'?GetWindowTextA@COleControlSite@@UBEHPADH@Z',
    3872:'?GetWindowTextA@COleControlSite@@UBEXAAVCString@@@Z',
    3873:'?GetWindowTextA@CWnd@@QBEHPADH@Z',
    3874:'?GetWindowTextA@CWnd@@QBEXAAVCString@@@Z',
    3875:'?GetWindowTextLengthA@COleControlSite@@UBEHXZ',
    3876:'?GetWindowTextLengthA@CWnd@@QBEHXZ',
    3877:'?GetWorkspaceCount@CDaoWorkspace@@QAEFXZ',
    3878:'?GetWorkspaceInfo@CDaoWorkspace@@QAEXHAAUCDaoWorkspaceInfo@@K@Z',
    3879:'?GetWorkspaceInfo@CDaoWorkspace@@QAEXPBDAAUCDaoWorkspaceInfo@@K@Z',
    3880:'?GetYear@COleDateTime@@QBEHXZ',
    3881:'?GetZoomFactor@COleServerDoc@@QBEHPAUtagSIZE@@0PBUtagRECT@@@Z',
    3882:'?GiveFeedback@COleDropSource@@UAEJK@Z',
    3883:'?GiveFeedback@XDropSource@COleDropSource@@UAGJK@Z',
    3884:'?GrayCtlColor@CWnd@@SGHPAUHDC__@@PAUHWND__@@IPAUHBRUSH__@@K@Z',
    3885:'?GrayStringA@CDC@@UAEHPAVCBrush@@P6GHPAUHDC__@@JH@ZJHHHHH@Z',
    3886:'?GrayStringA@CPreviewDC@@UAEHPAVCBrush@@P6GHPAUHDC__@@JH@ZJHHHHH@Z',
    3887:'?GrowFile@CMemFile@@MAEXK@Z',
    3888:'?guid@CColorPropPage@@2U_GUID@@B',
    3889:'?guid@CFontPropPage@@2U_GUID@@B',
    3890:'?guid@CPicturePropPage@@2U_GUID@@B',
    3891:'?HandleFloatingSysCommand@CWnd@@QAEHIJ@Z',
    3892:'?HandleInComingCall@XMessageFilter@COleMessageFilter@@UAGKKPAUHTASK__@@KPAUtagINTERFACEINFO@@@Z',
    3893:'?HandleInitDialog@CDialog@@IAEJIJ@Z',
    3894:'?HandleInitDialog@CDialogBar@@IAEJIJ@Z',
    3895:'?HandleInitDialog@CFormView@@IAEJIJ@Z',
    3896:'?HandleInitDialog@CPropertySheet@@IAEJIJ@Z',
    3897:'?HandleSetFont@CDialog@@IAEJIJ@Z',
    3898:'?HandsOffStorage@XPersistStorage@COleControl@@UAGJXZ',
    3899:'?HandsOffStorage@XPersistStorage@COleServerDoc@@UAGJXZ',
    3900:'?HasBlankItems@COleDocument@@QBEHXZ',
    3901:'?HasFont@CDialogTemplate@@QBEHXZ',
    3902:'?HashKey@@YGIABUtagVARIANT@@@Z',
    3903:'?HashKey@@YGIPBD@Z',
    3904:'?Help@XPropertyPage@COlePropertyPage@@UAGJPBG@Z',
    3905:'?HexDump@CDumpContext@@QAEXPBDPAEHH@Z',
    3906:'?HideApplication@CWinApp@@QAEXXZ',
    3907:'?HideControl@CFileDialog@@QAEXH@Z',
    3908:'?HIMETRICtoDP@CDC@@QBEXPAUtagSIZE@@@Z',
    3909:'?HIMETRICtoLP@CDC@@QBEXPAUtagSIZE@@@Z',
    3910:'?HitTest@CListCtrl@@QBEHVCPoint@@PAI@Z',
    3911:'?HitTest@CRectTracker@@QBEHVCPoint@@@Z',
    3912:'?HitTest@CSplitterWnd@@MBEHVCPoint@@@Z',
    3913:'?HitTest@CToolTipCtrl@@QBEHPAVCWnd@@VCPoint@@PAUtagTOOLINFOA@@@Z',
    3914:'?HitTest@CTreeCtrl@@QBEPAU_TREEITEM@@VCPoint@@PAI@Z',
    3915:'?HitTestHandles@CRectTracker@@IBEHVCPoint@@@Z',
    3916:'?IdFromRowCol@CSplitterWnd@@QBEHHH@Z',
    3917:'?Idle@CDaoWorkspace@@SGXH@Z',
    3918:'?IgnoreApply@COlePropertyPage@@QAEXI@Z',
    3919:'?IgnoreWindowMessage@COleControl@@IAEHIIJPAJ@Z',
    3920:'?InflateRect@CRect@@QAEXHHHH@Z',
    3921:'?InflateRect@CRect@@QAEXPBUtagRECT@@@Z',
    3922:'?InitApplication@CWinApp@@UAEHXZ',
    3923:'?InitCache@XOleCache@COleControl@@UAGJPAUIDataObject@@@Z',
    3924:'?InitControlContainer@CWnd@@IAEHXZ',
    3925:'?InitDatabasesCollection@CDaoWorkspace@@IAEXXZ',
    3926:'?InitDialogInfo@CPropertyPage@@IAEPBUDLGTEMPLATE@@PBU2@@Z',
    3927:'?InitErrorsCollection@CDaoException@@IAEXXZ',
    3928:'?InitFieldsCollection@CDaoQueryDef@@IAEXXZ',
    3929:'?InitFieldsCollection@CDaoRecordset@@IAEXXZ',
    3930:'?InitFieldsCollection@CDaoTableDef@@IAEXXZ',
    3931:'?InitFromData@XOleObject@CDocObjectServer@@UAGJPAUIDataObject@@HK@Z',
    3932:'?InitFromData@XOleObject@COleControl@@UAGJPAUIDataObject@@HK@Z',
    3933:'?InitFromData@XOleObject@COleServerDoc@@UAGJPAUIDataObject@@HK@Z',
    3934:'?InitFromData@XOleObject@COleServerItem@@UAGJPAUIDataObject@@HK@Z',
    3935:'?InitHashTable@CMapPtrToPtr@@QAEXIH@Z',
    3936:'?InitHashTable@CMapPtrToWord@@QAEXIH@Z',
    3937:'?InitHashTable@CMapStringToOb@@QAEXIH@Z',
    3938:'?InitHashTable@CMapStringToPtr@@QAEXIH@Z',
    3939:'?InitHashTable@CMapStringToString@@QAEXIH@Z',
    3940:'?InitHashTable@CMapWordToOb@@QAEXIH@Z',
    3941:'?InitHashTable@CMapWordToPtr@@QAEXIH@Z',
    3942:'?Initialize@CDataSourceControl@@QAEJXZ',
    3943:'?Initialize@CMiniFrameWnd@@SAXXZ',
    3944:'?InitializeEngine@CDaoWorkspace@@KAXXZ',
    3945:'?InitializeFont@CFontHolder@@QAEXPBUtagFONTDESC@@PAUIDispatch@@@Z',
    3946:'?InitializeIIDs@COleControl@@QAEXPBU_GUID@@0@Z',
    3947:'?InitializeReplace@CEditView@@IAEHXZ',
    3948:'?InitialUpdateFrame@CDocTemplate@@UAEXPAVCFrameWnd@@PAVCDocument@@H@Z',
    3949:'?InitialUpdateFrame@CFrameWnd@@QAEXPAVCDocument@@H@Z',
    3950:'?InitIndexesCollection@CDaoRecordset@@IAEXXZ',
    3951:'?InitIndexesCollection@CDaoTableDef@@IAEXXZ',
    3952:'?InitInstance@COleControlModule@@UAEHXZ',
    3953:'?InitInstance@CWinApp@@UAEHXZ',
    3954:'?InitInstance@CWinThread@@UAEHXZ',
    3955:'?InitLoop@CDockContext@@QAEXXZ',
    3956:'?InitModalIndirect@CDialog@@QAEHPAXPAVCWnd@@@Z',
    3957:'?InitModalIndirect@CDialog@@QAEHPBUDLGTEMPLATE@@PAVCWnd@@PAX@Z',
    3958:'?InitNew@XPersistMemory@COleControl@@UAGJXZ',
    3959:'?InitNew@XPersistPropertyBag@COleControl@@UAGJXZ',
    3960:'?InitNew@XPersistStorage@COleControl@@UAGJPAUIStorage@@@Z',
    3961:'?InitNew@XPersistStorage@COleServerDoc@@UAGJPAUIStorage@@@Z',
    3962:'?InitNew@XPersistStreamInit@COleControl@@UAGJXZ',
    3963:'?InitParametersCollection@CDaoQueryDef@@IAEXXZ',
    3964:'?InitQueryDefsCollection@CDaoDatabase@@IAEXXZ',
    3965:'?InitRecord@CRecordset@@QAEXXZ',
    3966:'?InitRelationsCollection@CDaoDatabase@@IAEXXZ',
    3967:'?InitStockEventMask@COleControl@@IAEXXZ',
    3968:'?InitStockPropMask@COleControl@@IAEXXZ',
    3969:'?InitString@CSimpleException@@QAEXXZ',
    3970:'?InitTableDefsCollection@CDaoDatabase@@IAEXXZ',
    3971:'?InitWorkspace@CDaoDatabase@@IAEXXZ',
    3972:'?InitWorkspacesCollection@CDaoWorkspace@@IAEXXZ',
    3973:'?InPlaceDeactivate@XOleInPlaceObject@COleControl@@UAGJXZ',
    3974:'?InPlaceDeactivate@XOleInPlaceObject@COleServerDoc@@UAGJXZ',
    3975:'?Insert@CDockBar@@IAEHPAVCControlBar@@VCRect@@VCPoint@@@Z',
    3976:'?InsertAfter@CObList@@QAEPAU__POSITION@@PAU2@PAVCObject@@@Z',
    3977:'?InsertAfter@CPtrList@@QAEPAU__POSITION@@PAU2@PAX@Z',
    3978:'?InsertAfter@CStringList@@QAEPAU__POSITION@@PAU2@PBD@Z',
    3979:'?InsertAt@CByteArray@@QAEXHEH@Z',
    3980:'?InsertAt@CByteArray@@QAEXHPAV1@@Z',
    3981:'?InsertAt@CDWordArray@@QAEXHKH@Z',
    3982:'?InsertAt@CDWordArray@@QAEXHPAV1@@Z',
    3983:'?InsertAt@CObArray@@QAEXHPAV1@@Z',
    3984:'?InsertAt@CObArray@@QAEXHPAVCObject@@H@Z',
    3985:'?InsertAt@CPtrArray@@QAEXHPAV1@@Z',
    3986:'?InsertAt@CPtrArray@@QAEXHPAXH@Z',
    3987:'?InsertAt@CStringArray@@QAEXHPAV1@@Z',
    3988:'?InsertAt@CStringArray@@QAEXHPBDH@Z',
    3989:'?InsertAt@CUIntArray@@QAEXHIH@Z',
    3990:'?InsertAt@CUIntArray@@QAEXHPAV1@@Z',
    3991:'?InsertAt@CWordArray@@QAEXHGH@Z',
    3992:'?InsertAt@CWordArray@@QAEXHPAV1@@Z',
    3993:'?InsertBefore@CObList@@QAEPAU__POSITION@@PAU2@PAVCObject@@@Z',
    3994:'?InsertBefore@CPtrList@@QAEPAU__POSITION@@PAU2@PAX@Z',
    3995:'?InsertBefore@CStringList@@QAEPAU__POSITION@@PAU2@PBD@Z',
    3996:'?InsertColumn@CListCtrl@@QAEHHPBDHHH@Z',
    3997:'?InsertFileAsObject@CRichEditView@@QAEXPBD@Z',
    3998:'?InsertItem@CListCtrl@@QAEHIHPBDIIHJ@Z',
    3999:'?InsertItem@CRichEditView@@QAEJPAVCRichEditCntrItem@@@Z',
    4000:'?InsertItem@CTreeCtrl@@QAEPAU_TREEITEM@@IPBDHHIIJPAU2@1@Z',
    4001:'?InsertMenus@XOleInPlaceFrame@COleFrameHook@@UAGJPAUHMENU__@@PAUtagOleMenuGroupWidths@@@Z',
    4002:'?InsertMenus@XOleIPFrame@COleControlContainer@@UAGJPAUHMENU__@@PAUtagOleMenuGroupWidths@@@Z',
    4003:'?interfaceMap@CCmdTarget@@1UAFX_INTERFACEMAP@@B',
    4004:'?interfaceMap@CDocObjectServer@@1UAFX_INTERFACEMAP@@B',
    4005:'?interfaceMap@COleClientItem@@1UAFX_INTERFACEMAP@@B',
    4006:'?interfaceMap@COleControl@@1UAFX_INTERFACEMAP@@B',
    4007:'?interfaceMap@COleDataSource@@1UAFX_INTERFACEMAP@@B',
    4008:'?interfaceMap@COleDropSource@@1UAFX_INTERFACEMAP@@B',
    4009:'?interfaceMap@COleDropTarget@@1UAFX_INTERFACEMAP@@B',
    4010:'?interfaceMap@COleFrameHook@@1UAFX_INTERFACEMAP@@B',
    4011:'?interfaceMap@COleLinkingDoc@@1UAFX_INTERFACEMAP@@B',
    4012:'?interfaceMap@COleMessageFilter@@1UAFX_INTERFACEMAP@@B',
    4013:'?interfaceMap@COleObjectFactory@@1UAFX_INTERFACEMAP@@B',
    4014:'?interfaceMap@COlePropertyPage@@1UAFX_INTERFACEMAP@@B',
    4015:'?interfaceMap@COleServerDoc@@1UAFX_INTERFACEMAP@@B',
    4016:'?interfaceMap@COleServerItem@@1UAFX_INTERFACEMAP@@B',
    4017:'?interfaceMap@CRichEditView@@1UAFX_INTERFACEMAP@@B',
    4018:'?InternalGetFont@COleControl@@QAEAAVCFontHolder@@XZ',
    4019:'?InternalGetText@COleControl@@QAEABVCString@@XZ',
    4020:'?InternalQueryInterface@CCmdTarget@@QAEKPBXPAPAX@Z',
    4021:'?InternalRelease@CCmdTarget@@QAEKXZ',
    4022:'?IntersectClipRect@CDC@@QAEHHHHH@Z',
    4023:'?IntersectClipRect@CDC@@QAEHPBUtagRECT@@@Z',
    4024:'?InvalidateCheck@CCheckListBox@@IAEXH@Z',
    4025:'?InvalidateControl@COleControl@@QAEXPBUtagRECT@@H@Z',
    4026:'?InvalidateItem@CCheckListBox@@IAEXH@Z',
    4027:'?InvalidateRgn@COleControl@@QAEXPAVCRgn@@H@Z',
    4028:'?InvertSysMenu@CMiniFrameWnd@@IAEXXZ',
    4029:'?Invoke@COleDispatchImpl@@UAGJJABU_GUID@@KGPAUtagDISPPARAMS@@PAUtagVARIANT@@PAUtagEXCEPINFO@@PAI@Z',
    4030:'?Invoke@XAmbientProps@COleControlSite@@UAGJJABU_GUID@@KGPAUtagDISPPARAMS@@PAUtagVARIANT@@PAUtagEXCEPINFO@@PAI@Z',
    4031:'?Invoke@XEventSink@COleControlSite@@UAGJJABU_GUID@@KGPAUtagDISPPARAMS@@PAUtagVARIANT@@PAUtagEXCEPINFO@@PAI@Z',
    4032:'?InvokeHelper@COleControlSite@@UAAXJGGPAXPBEZZ',
    4033:'?InvokeHelper@COleDispatchDriver@@QAAXJGGPAXPBEZZ',
    4034:'?InvokeHelper@CWnd@@QAAXJGGPAXPBEZZ',
    4035:'?InvokeHelperV@COleControlSite@@UAEXJGGPAXPBEPAD@Z',
    4036:'?InvokeHelperV@COleDispatchDriver@@QAEXJGGPAXPBEPAD@Z',
    4037:'?IsBlank@CDocItem@@UBEHXZ',
    4038:'?IsBlank@COleServerItem@@MBEHXZ',
    4039:'?IsBOF@CDaoRecordset@@QBEHXZ',
    4040:'?IsChildPane@CSplitterWnd@@QAEHPAVCWnd@@PAH1@Z',
    4041:'?IsConnected@COleServerItem@@QBEHXZ',
    4042:'?IsDataAvailable@COleDataObject@@QAEHGPAUtagFORMATETC@@@Z',
    4043:'?IsDefaultButton@COleControlSite@@QAEHXZ',
    4044:'?IsDeleted@CDaoRecordset@@QBEHXZ',
    4045:'?IsDerivedFrom@CRuntimeClass@@QBEHPBU1@@Z',
    4046:'?IsDialogMessageA@COccManager@@UAEHPAVCWnd@@PAUtagMSG@@@Z',
    4047:'?IsDialogMessageA@CWnd@@QAEHPAUtagMSG@@@Z',
    4048:'?IsDirty@CBlobProperty@@UAGJXZ',
    4049:'?IsDirty@XPersistFile@COleLinkingDoc@@UAGJXZ',
    4050:'?IsDirty@XPersistMemory@COleControl@@UAGJXZ',
    4051:'?IsDirty@XPersistStorage@COleControl@@UAGJXZ',
    4052:'?IsDirty@XPersistStorage@COleServerDoc@@UAGJXZ',
    4053:'?IsDirty@XPersistStreamInit@COleControl@@UAGJXZ',
    4054:'?IsDlgButtonChecked@COleControlContainer@@UBEIH@Z',
    4055:'?IsDlgButtonChecked@CWnd@@QBEIH@Z',
    4056:'?IsDockBar@CControlBar@@UBEHXZ',
    4057:'?IsDockBar@CDockBar@@UBEHXZ',
    4058:'?IsDots@CFileFind@@UBEHXZ',
    4059:'?IsDots@CGopherFileFind@@UBEHXZ',
    4060:'?IsEnabled@CCheckListBox@@QAEHH@Z',
    4061:'?IsEOF@CDaoRecordset@@QBEHXZ',
    4062:'?IsFieldDirty@CDaoRecordset@@QAEHPAX@Z',
    4063:'?IsFieldDirty@CRecordset@@QAEHPAX@Z',
    4064:'?IsFieldNull@CDaoRecordset@@QAEHPAX@Z',
    4065:'?IsFieldNull@CRecordset@@QAEHPAX@Z',
    4066:'?IsFieldNullable@CDaoRecordset@@QAEHPAX@Z',
    4067:'?IsFieldNullable@CRecordset@@QAEHPAX@Z',
    4068:'?IsFieldNullable@CRecordset@@QBEHK@Z',
    4069:'?IsFieldStatusDirty@CDaoRecordset@@QAEHI@Z',
    4070:'?IsFieldStatusDirty@CRecordset@@QBEHK@Z',
    4071:'?IsFieldStatusNull@CDaoRecordset@@QAEHI@Z',
    4072:'?IsFieldStatusNull@CRecordset@@QBEHK@Z',
    4073:'?IsFieldStatusNullable@CDaoRecordset@@QAEHI@Z',
    4074:'?IsFieldStatusNullableKnown@CDaoRecordset@@QAEHI@Z',
    4075:'?IsFieldType@CFieldExchange@@QAEHPAI@Z',
    4076:'?IsFloating@CControlBar@@QBEHXZ',
    4077:'?IsFrameWnd@CFrameWnd@@UBEHXZ',
    4078:'?IsFrameWnd@CWnd@@UBEHXZ',
    4079:'?IsIdleMessage@CWinThread@@UAEHPAUtagMSG@@@Z',
    4080:'?IsInvokeAllowed@CCmdTarget@@UAEHJ@Z',
    4081:'?IsInvokeAllowed@COleControl@@MAEHJ@Z',
    4082:'?IsJoin@CRecordset@@SGHPBD@Z',
    4083:'?IsKindOf@CObject@@QBEHPBUCRuntimeClass@@@Z',
    4084:'?IsLabelControl@COccManager@@SAHPAVCWnd@@@Z',
    4085:'?IsLicenseValid@COleObjectFactory@@QAEHXZ',
    4086:'?IsMatch@CDaoRecordset@@QAEHXZ',
    4087:'?IsMatchingMnemonic@COccManager@@SAHPAVCWnd@@PAUtagMSG@@@Z',
    4088:'?IsMatchingMnemonic@COleControlSite@@QAEHPAUtagMSG@@@Z',
    4089:'?IsModified@CDocument@@UAEHXZ',
    4090:'?IsModified@COleClientItem@@QBEHXZ',
    4091:'?IsModified@COleControl@@QAEHXZ',
    4092:'?IsModified@COlePropertyPage@@QAEHXZ',
    4093:'?IsModified@CRichEditDoc@@UAEHXZ',
    4094:'?IsNullValue@CDaoFieldExchange@@SGHPAXK@Z',
    4095:'?IsOnFirstRecord@CDaoRecordView@@QAEHXZ',
    4096:'?IsOnFirstRecord@CRecordView@@QAEHXZ',
    4097:'?IsOnLastRecord@CDaoRecordView@@QAEHXZ',
    4098:'?IsOnLastRecord@CRecordView@@QAEHXZ',
    4099:'?IsOpen@CRecordset@@QBEHXZ',
    4100:'?IsPageDirty@XPropertyPage@COlePropertyPage@@UAGJXZ',
    4101:'?IsParamStatusNull@CRecordset@@QBEHK@Z',
    4102:'?IsRecordsetUpdatable@CRecordset@@IAEHXZ',
    4103:'?IsRegistered@COleObjectFactory@@UBEHXZ',
    4104:'?IsResultExpected@CCmdTarget@@QAEHXZ',
    4105:'?IsRichEditFormat@CRichEditView@@SAHG@Z',
    4106:'?IsRunning@XOleItemContainer@COleLinkingDoc@@UAGJPAG@Z',
    4107:'?IsSelected@CRichEditView@@MBEHPBVCObject@@@Z',
    4108:'?IsSelected@CView@@UBEHPBVCObject@@@Z',
    4109:'?IsSelectQueryUpdatable@CRecordset@@QAEHPBD@Z',
    4110:'?IsSerializable@CObject@@QBEHXZ',
    4111:'?IsSignificantMessage@COleMessageFilter@@UAEHPAUtagMSG@@@Z',
    4112:'?IsSQLUpdatable@CRecordset@@QAEHPBD@Z',
    4113:'?IsSubclassedControl@COleControl@@UAEHXZ',
    4114:'?IsTopParentActive@CWnd@@QBEHXZ',
    4115:'?IsTracking@CFrameWnd@@QBEHXZ',
    4116:'?IsUpToDate@XOleObject@CDocObjectServer@@UAGJXZ',
    4117:'?IsUpToDate@XOleObject@COleControl@@UAGJXZ',
    4118:'?IsUpToDate@XOleObject@COleServerDoc@@UAGJXZ',
    4119:'?IsUpToDate@XOleObject@COleServerItem@@UAGJXZ',
    4120:'?IsValidOperation@CDaoFieldExchange@@QAEHXZ',
    4121:'?IsVisible@CControlBar@@UBEHXZ',
    4122:'?IsWindowEnabled@COleControlSite@@UBEHXZ',
    4123:'?IsWindowEnabled@CWnd@@QBEHXZ',
    4124:'?ItemFromPoint@CListBox@@QBEIVCPoint@@AAH@Z',
    4125:'?ItemHasChildren@CTreeCtrl@@QBEHPAU_TREEITEM@@@Z',
    4126:'?KeyDown@COleControl@@IAEXPAG@Z',
    4127:'?KeyUp@COleControl@@IAEXPAG@Z',
    4128:'?KillSocket@CAsyncSocket@@SGXIPAV1@@Z',
    4129:'?Left@CString@@QBE?AV1@H@Z',
    4130:'?LineIndex@CRichEditCtrl@@QBEHH@Z',
    4131:'?LineLength@CRichEditCtrl@@QBEHH@Z',
    4132:'?LineScroll@CRichEditCtrl@@QAEXHH@Z',
    4133:'?LineTo@CDC@@QAEHHH@Z',
    4134:'?lMaxSize@CRichEditView@@2KA',
    4135:'?Load@CBlobProperty@@UAGJPAUIStream@@@Z',
    4136:'?Load@CDialogTemplate@@QAEHPBD@Z',
    4137:'?Load@COleControl@@QAEXPBDAAVCDataPathProperty@@@Z',
    4138:'?Load@CRuntimeClass@@SGPAU1@AAVCArchive@@PAI@Z',
    4139:'?Load@XPersistFile@COleLinkingDoc@@UAGJPBGK@Z',
    4140:'?Load@XPersistMemory@COleControl@@UAGJPAXK@Z',
    4141:'?Load@XPersistPropertyBag@COleControl@@UAGJPAUIPropertyBag@@PAUIErrorLog@@@Z',
    4142:'?Load@XPersistStorage@COleControl@@UAGJPAUIStorage@@@Z',
    4143:'?Load@XPersistStorage@COleServerDoc@@UAGJPAUIStorage@@@Z',
    4144:'?Load@XPersistStreamInit@COleControl@@UAGJPAUIStream@@@Z',
    4145:'?LoadAccelTable@CFrameWnd@@QAEHPBD@Z',
    4146:'?LoadBarState@CFrameWnd@@QAEXPBD@Z',
    4147:'?LoadBitmapA@CToolBar@@QAEHPBD@Z',
    4148:'?LoadBitmaps@CBitmapButton@@QAEHPBD000@Z',
    4149:'?LoadFields@CDaoRecordset@@IAEXXZ',
    4150:'?LoadFields@CRecordset@@QAEXXZ',
    4151:'?LoadFrame@CFrameWnd@@UAEHIKPAVCWnd@@PAUCCreateContext@@@Z',
    4152:'?LoadFrame@CMDIChildWnd@@UAEHIKPAVCWnd@@PAUCCreateContext@@@Z',
    4153:'?LoadFrame@CMDIFrameWnd@@UAEHIKPAVCWnd@@PAUCCreateContext@@@Z',
    4154:'?LoadFrame@COleIPFrameWnd@@UAEHIKPAVCWnd@@PAUCCreateContext@@@Z',
    4155:'?LoadFromStorage@COleDocument@@MAEXXZ',
    4156:'?LoadState@CControlBarInfo@@QAEHPBDHPAVCDockState@@@Z',
    4157:'?LoadState@CDockState@@QAEXPBD@Z',
    4158:'?LoadState@COleControl@@IAEJPAUIStream@@@Z',
    4159:'?LoadStdProfileSettings@CWinApp@@IAEXI@Z',
    4160:'?LoadStringA@CString@@QAEHI@Z',
    4161:'?LoadTemplate@CDocTemplate@@UAEXXZ',
    4162:'?LoadTemplate@CMultiDocTemplate@@UAEXXZ',
    4163:'?LoadToolBar@CToolBar@@QAEHPBD@Z',
    4164:'?Lock@CCriticalSection@@UAEHK@Z',
    4165:'?Lock@CMultiLock@@QAEKKHK@Z',
    4166:'?Lock@COleSafeArray@@QAEXXZ',
    4167:'?Lock@CSingleLock@@QAEHK@Z',
    4168:'?Lock@CSyncObject@@UAEHK@Z',
    4169:'?Lock@CTypeLibCache@@QAEXXZ',
    4170:'?LockBuffer@CEditView@@QBEPBDXZ',
    4171:'?LockBuffer@CString@@QAEPADXZ',
    4172:'?LockContainer@XOleContainer@COleControlContainer@@UAGJH@Z',
    4173:'?LockContainer@XOleItemContainer@COleLinkingDoc@@UAGJH@Z',
    4174:'?LockExternal@COleLinkingDoc@@IAEXHH@Z',
    4175:'?LockInPlaceActive@COleControl@@QAEHH@Z',
    4176:'?LockInPlaceActive@XOleControlSite@COleControlSite@@UAGJH@Z',
    4177:'?LockRange@CFile@@UAEXKK@Z',
    4178:'?LockRange@CInternetFile@@UAEXKK@Z',
    4179:'?LockRange@CMemFile@@UAEXKK@Z',
    4180:'?LockRange@COleStreamFile@@UAEXKK@Z',
    4181:'?LockRange@CSocketFile@@UAEXKK@Z',
    4182:'?LockRange@CStdioFile@@UAEXKK@Z',
    4183:'?LockRegion@CArchiveStream@@UAGJT_ULARGE_INTEGER@@0K@Z',
    4184:'?LockServer@XClassFactory@COleObjectFactory@@UAGJH@Z',
    4185:'?Lookup@CMapPtrToPtr@@QBEHPAXAAPAX@Z',
    4186:'?Lookup@CMapPtrToWord@@QBEHPAXAAG@Z',
    4187:'?Lookup@CMapStringToOb@@QBEHPBDAAPAVCObject@@@Z',
    4188:'?Lookup@CMapStringToPtr@@QBEHPBDAAPAX@Z',
    4189:'?Lookup@CMapStringToString@@QBEHPBDAAVCString@@@Z',
    4190:'?Lookup@CMapWordToOb@@QBEHGAAPAVCObject@@@Z',
    4191:'?Lookup@CMapWordToPtr@@QBEHGAAPAX@Z',
    4192:'?Lookup@COleDataSource@@IBEPAUAFX_DATACACHE_ENTRY@@PAUtagFORMATETC@@W4tagDATADIR@@@Z',
    4193:'?Lookup@CTypeLibCache@@QAEHKPAPAUITypeLib@@@Z',
    4194:'?LookupHandle@CAsyncSocket@@SGPAV1@IH@Z',
    4195:'?LookupItem@CRichEditDoc@@QBEPAVCRichEditCntrItem@@PAUIOleObject@@@Z',
    4196:'?LookupKey@CMapStringToOb@@QBEHPBDAAPBD@Z',
    4197:'?LookupKey@CMapStringToPtr@@QBEHPBDAAPBD@Z',
    4198:'?LookupKey@CMapStringToString@@QBEHPBDAAPBD@Z',
    4199:'?LookupTypeInfo@CTypeLibCache@@QAEHKABU_GUID@@PAPAUITypeInfo@@@Z',
    4200:'?LPtoDP@CDC@@QBEXPAUtagSIZE@@@Z',
    4201:'?LPtoHIMETRIC@CDC@@QBEXPAUtagSIZE@@@Z',
    4202:'?MakeLower@CString@@QAEXXZ',
    4203:'?MakeReverse@CString@@QAEXXZ',
    4204:'?MakeUpper@CString@@QAEXXZ',
    4205:'?MapObject@CArchive@@QAEXPBVCObject@@@Z',
    4206:'?MapPropertyToPage@XPerPropertyBrowsing@COleControl@@UAGJJPAU_GUID@@@Z',
    4207:'?MapResult@COleDialog@@QAEHI@Z',
    4208:'?MapWizardResult@CPropertyPage@@IAEJJ@Z',
    4209:'?MarkForAddNew@CDaoRecordset@@IAEXXZ',
    4210:'?MarkForAddNew@CRecordset@@QAEXXZ',
    4211:'?MarkForEdit@CDaoRecordset@@IAEXXZ',
    4212:'?MarkForUpdate@CRecordset@@QAEXXZ',
    4213:'?MarkItemsClear@CRichEditDoc@@IBEXXZ',
    4214:'?MatchDocType@CDocTemplate@@UAE?AW4Confidence@1@PBDAAPAVCDocument@@@Z',
    4215:'?MatchesMask@CFileFind@@UBEHK@Z',
    4216:'?MDIGetActive@CMDIFrameWnd@@QBEPAVCMDIChildWnd@@PAH@Z',
    4217:'?MeasureItem@CCheckListBox@@UAEXPAUtagMEASUREITEMSTRUCT@@@Z',
    4218:'?MeasureItem@CComboBox@@UAEXPAUtagMEASUREITEMSTRUCT@@@Z',
    4219:'?MeasureItem@CListBox@@UAEXPAUtagMEASUREITEMSTRUCT@@@Z',
    4220:'?MeasureItem@CMenu@@UAEXPAUtagMEASUREITEMSTRUCT@@@Z',
    4221:'?MemberIDFromName@CCmdTarget@@KGJPBUAFX_DISPMAP@@PBD@Z',
    4222:'?Memcpy@CMemFile@@MAEPAEPAEPBEI@Z',
    4223:'?MessageBoxA@COlePropertyPage@@QAEHPBD0I@Z',
    4224:'?MessageBoxA@CWnd@@QAEHPBD0I@Z',
    4225:'?messageMap@CCheckListBox@@1UAFX_MSGMAP@@B',
    4226:'?messageMap@CCmdTarget@@1UAFX_MSGMAP@@B',
    4227:'?messageMap@CColorDialog@@1UAFX_MSGMAP@@B',
    4228:'?messageMap@CColorPropPage@@1UAFX_MSGMAP@@B',
    4229:'?messageMap@CCommonDialog@@1UAFX_MSGMAP@@B',
    4230:'?messageMap@CControlBar@@1UAFX_MSGMAP@@B',
    4231:'?messageMap@CControlFrameWnd@@1UAFX_MSGMAP@@B',
    4232:'?messageMap@CCtrlView@@1UAFX_MSGMAP@@B',
    4233:'?messageMap@CDaoRecordView@@1UAFX_MSGMAP@@B',
    4234:'?messageMap@CDialog@@1UAFX_MSGMAP@@B',
    4235:'?messageMap@CDialogBar@@1UAFX_MSGMAP@@B',
    4236:'?messageMap@CDockBar@@1UAFX_MSGMAP@@B',
    4237:'?messageMap@CDocObjectServer@@1UAFX_MSGMAP@@B',
    4238:'?messageMap@CDocument@@1UAFX_MSGMAP@@B',
    4239:'?messageMap@CEditView@@1UAFX_MSGMAP@@B',
    4240:'?messageMap@CFontPropPage@@1UAFX_MSGMAP@@B',
    4241:'?messageMap@CFormView@@1UAFX_MSGMAP@@B',
    4242:'?messageMap@CFrameWnd@@1UAFX_MSGMAP@@B',
    4243:'?messageMap@CListCtrl@@1UAFX_MSGMAP@@B',
    4244:'?messageMap@CListView@@1UAFX_MSGMAP@@B',
    4245:'?messageMap@CMDIChildWnd@@1UAFX_MSGMAP@@B',
    4246:'?messageMap@CMDIFrameWnd@@1UAFX_MSGMAP@@B',
    4247:'?messageMap@CMiniDockFrameWnd@@1UAFX_MSGMAP@@B',
    4248:'?messageMap@CMiniFrameWnd@@1UAFX_MSGMAP@@B',
    4249:'?messageMap@COleControl@@1UAFX_MSGMAP@@B',
    4250:'?messageMap@COleDocIPFrameWnd@@1UAFX_MSGMAP@@B',
    4251:'?messageMap@COleIPFrameWnd@@1UAFX_MSGMAP@@B',
    4252:'?messageMap@COlePropertyPage@@1UAFX_MSGMAP@@B',
    4253:'?messageMap@COleResizeBar@@1UAFX_MSGMAP@@B',
    4254:'?messageMap@COleServerDoc@@1UAFX_MSGMAP@@B',
    4255:'?messageMap@CPicturePropPage@@1UAFX_MSGMAP@@B',
    4256:'?messageMap@CPreviewView@@1UAFX_MSGMAP@@B',
    4257:'?messageMap@CPrintDialog@@1UAFX_MSGMAP@@B',
    4258:'?messageMap@CPropertyPage@@1UAFX_MSGMAP@@B',
    4259:'?messageMap@CPropertySheet@@1UAFX_MSGMAP@@B',
    4260:'?messageMap@CRecordView@@1UAFX_MSGMAP@@B',
    4261:'?messageMap@CRichEditView@@1UAFX_MSGMAP@@B',
    4262:'?messageMap@CScrollView@@1UAFX_MSGMAP@@B',
    4263:'?messageMap@CSocketWnd@@1UAFX_MSGMAP@@B',
    4264:'?messageMap@CSplitterWnd@@1UAFX_MSGMAP@@B',
    4265:'?messageMap@CStatusBar@@1UAFX_MSGMAP@@B',
    4266:'?messageMap@CStockPropPage@@1UAFX_MSGMAP@@B',
    4267:'?messageMap@CTabCtrl@@1UAFX_MSGMAP@@B',
    4268:'?messageMap@CToolBar@@1UAFX_MSGMAP@@B',
    4269:'?messageMap@CToolBarCtrl@@1UAFX_MSGMAP@@B',
    4270:'?messageMap@CToolTipCtrl@@1UAFX_MSGMAP@@B',
    4271:'?messageMap@CTreeCtrl@@1UAFX_MSGMAP@@B',
    4272:'?messageMap@CTreeView@@1UAFX_MSGMAP@@B',
    4273:'?messageMap@CView@@1UAFX_MSGMAP@@B',
    4274:'?messageMap@CWinApp@@1UAFX_MSGMAP@@B',
    4275:'?messageMap@CWnd@@1UAFX_MSGMAP@@B',
    4276:'?MessagePending@XMessageFilter@COleMessageFilter@@UAGKPAUHTASK__@@KK@Z',
    4277:'?Mid@CString@@QBE?AV1@H@Z',
    4278:'?Mid@CString@@QBE?AV1@HH@Z',
    4279:'?MirrorAttributes@CPreviewDC@@IAEXXZ',
    4280:'?MirrorFont@CPreviewDC@@IAEXXZ',
    4281:'?MirrorMappingMode@CPreviewDC@@IAEXH@Z',
    4282:'?MirrorViewportOrg@CPreviewDC@@IAEXXZ',
    4283:'?ModifyStyle@COleControlSite@@UAEHKKI@Z',
    4284:'?ModifyStyle@CWnd@@QAEHKKI@Z',
    4285:'?ModifyStyle@CWnd@@SGHPAUHWND__@@KKI@Z',
    4286:'?ModifyStyleEx@COleControlSite@@UAEHKKI@Z',
    4287:'?ModifyStyleEx@CWnd@@QAEHKKI@Z',
    4288:'?ModifyStyleEx@CWnd@@SGHPAUHWND__@@KKI@Z',
    4289:'?Move@CDaoRecordset@@UAEXJ@Z',
    4290:'?Move@CDockContext@@QAEXVCPoint@@@Z',
    4291:'?Move@CRecordset@@UAEXJG@Z',
    4292:'?Move@XPropertyPage@COlePropertyPage@@UAGJPBUtagRECT@@@Z',
    4293:'?MoveFirst@CDaoRecordset@@QAEXXZ',
    4294:'?MoveLast@CDaoRecordset@@QAEXXZ',
    4295:'?MoveNext@CDaoRecordset@@QAEXXZ',
    4296:'?MovePrev@CDaoRecordset@@QAEXXZ',
    4297:'?MoveTo@CDC@@QAE?AVCPoint@@HH@Z',
    4298:'?MoveWindow@COleControlSite@@UAEXHHHHH@Z',
    4299:'?MoveWindow@CWnd@@QAEXHHHHH@Z',
    4300:'?MulDiv@CRect@@QBE?AV1@HH@Z',
    4301:'?nDragDelay@COleDropSource@@1IA',
    4302:'?nDragMinDist@COleDropSource@@1IA',
    4303:'?NegotiateBorderSpace@CFrameWnd@@UAEHIPAUtagRECT@@@Z',
    4304:'?NewAssoc@CMapPtrToPtr@@IAEPAUCAssoc@1@XZ',
    4305:'?NewAssoc@CMapPtrToWord@@IAEPAUCAssoc@1@XZ',
    4306:'?NewAssoc@CMapStringToOb@@IAEPAUCAssoc@1@XZ',
    4307:'?NewAssoc@CMapStringToPtr@@IAEPAUCAssoc@1@XZ',
    4308:'?NewAssoc@CMapStringToString@@IAEPAUCAssoc@1@XZ',
    4309:'?NewAssoc@CMapWordToOb@@IAEPAUCAssoc@1@XZ',
    4310:'?NewAssoc@CMapWordToPtr@@IAEPAUCAssoc@1@XZ',
    4311:'?NewNode@CObList@@IAEPAUCNode@1@PAU21@0@Z',
    4312:'?NewNode@CPtrList@@IAEPAUCNode@1@PAU21@0@Z',
    4313:'?NewNode@CStringList@@IAEPAUCNode@1@PAU21@0@Z',
    4314:'?Next@XEnumVOID@CEnumArray@@UAGJKPAXPAK@Z',
    4315:'?nMaxSize@CEditView@@2IB',
    4316:'?NormalizeHit@CRectTracker@@QBEHH@Z',
    4317:'?NormalizeRect@CRect@@QAEXXZ',
    4318:'?Notify@CDataBoundProperty@@QAEXXZ',
    4319:'?NotifyAllInPlace@COleFrameHook@@QAEHHP81@AEHH@Z@Z',
    4320:'?NotifyAllItems@COleServerDoc@@IAEXW4OLE_NOTIFICATION@@K@Z',
    4321:'?NotifyClient@COleServerItem@@QAEXW4OLE_NOTIFICATION@@K@Z',
    4322:'?NotifyFloatingWindows@CFrameWnd@@QAEXK@Z',
    4323:'?NotifyRename@COleServerDoc@@QAEXPBD@Z',
    4324:'?nScrollDelay@COleDropTarget@@1IA',
    4325:'?nScrollInset@COleDropTarget@@1HA',
    4326:'?nScrollInterval@COleDropTarget@@1IA',
    4327:'?OemToCharA@CString@@QAEXXZ',
    4328:'?OffsetClipRgn@CDC@@QAEHHH@Z',
    4329:'?OffsetClipRgn@CDC@@QAEHUtagSIZE@@@Z',
    4330:'?OffsetViewportOrg@CDC@@UAE?AVCPoint@@HH@Z',
    4331:'?OffsetViewportOrg@CMetaFileDC@@UAE?AVCPoint@@HH@Z',
    4332:'?OffsetViewportOrg@CPreviewDC@@UAE?AVCPoint@@HH@Z',
    4333:'?OffsetWindowOrg@CDC@@QAE?AVCPoint@@HH@Z',
    4334:'?OKToDo@XNotifyDBEvents@COleControlSite@@UAGJKKQAUtagDBNOTIFYREASON@@@Z',
    4335:'?OnAccept@CAsyncSocket@@MAEXH@Z',
    4336:'?OnActivate@CControlFrameWnd@@IAEXIPAVCWnd@@H@Z',
    4337:'?OnActivate@CFrameWnd@@IAEXIPAVCWnd@@H@Z',
    4338:'?OnActivate@COleClientItem@@UAEXXZ',
    4339:'?OnActivate@COleFrameHook@@UAEXH@Z',
    4340:'?OnActivateFrame@CFormView@@MAEXIPAVCFrameWnd@@@Z',
    4341:'?OnActivateFrame@CView@@MAEXIPAVCFrameWnd@@@Z',
    4342:'?OnActivateInPlace@COleControl@@MAEJHPAUtagMSG@@@Z',
    4343:'?OnActivateTopLevel@CFrameWnd@@IAEJIJ@Z',
    4344:'?OnActivateTopLevel@CWnd@@IAEJIJ@Z',
    4345:'?OnActivateUI@COleClientItem@@UAEXXZ',
    4346:'?OnActivateView@CDocObjectServer@@MAEJXZ',
    4347:'?OnActivateView@CFormView@@MAEXHPAVCView@@0@Z',
    4348:'?OnActivateView@CPreviewView@@MAEXHPAVCView@@0@Z',
    4349:'?OnActivateView@CView@@MAEXHPAV1@0@Z',
    4350:'?OnAddTool@CToolTipCtrl@@IAEJIJ@Z',
    4351:'?OnAdvise@CConnectionPoint@@UAEXH@Z',
    4352:'?OnAdvise@XEventConnPt@COleControl@@UAEXH@Z',
    4353:'?OnAmbientProperty@CWnd@@UAEHPAVCOleControlSite@@JPAUtagVARIANT@@@Z',
    4354:'?OnAmbientPropertyChange@COleControl@@UAEXJ@Z',
    4355:'?OnAmbientPropertyChange@XOleControl@COleControl@@UAGJJ@Z',
    4356:'?OnAppearanceChanged@COleControl@@UAEXXZ',
    4357:'?OnAppExit@CWinApp@@IAEXXZ',
    4358:'?OnApply@CPropertyPage@@UAEHXZ',
    4359:'?OnApplyScale@COlePropertiesDialog@@UAEHPAVCOleClientItem@@HH@Z',
    4360:'?OnApplyViewState@CDocObjectServer@@MAEXAAVCArchive@@@Z',
    4361:'?OnBackColorChanged@COleControl@@UAEXXZ',
    4362:'?OnBarCheck@CFrameWnd@@QAEHI@Z',
    4363:'?OnBarCheck@COleIPFrameWnd@@IAEHI@Z',
    4364:'?OnBarStyleChange@CControlBar@@UAEXKK@Z',
    4365:'?OnBarStyleChange@CStatusBar@@UAEXKK@Z',
    4366:'?OnBarStyleChange@CToolBar@@UAEXKK@Z',
    4367:'?OnBeginDrag@COleDropSource@@UAEHPAVCWnd@@@Z',
    4368:'?OnBeginPrinting@CEditView@@MAEXPAVCDC@@PAUCPrintInfo@@@Z',
    4369:'?OnBeginPrinting@CRichEditView@@MAEXPAVCDC@@PAUCPrintInfo@@@Z',
    4370:'?OnBeginPrinting@CView@@MAEXPAVCDC@@PAUCPrintInfo@@@Z',
    4371:'?OnBorderStyleChanged@COleControl@@UAEXXZ',
    4372:'?OnBrowse@CPicturePropPage@@IAEXXZ',
    4373:'?OnBullet@CRichEditView@@IAEXXZ',
    4374:'?OnBusyDialog@COleMessageFilter@@UAEHPAUHTASK__@@@Z',
    4375:'?OnCancel@CCommonDialog@@MAEXXZ',
    4376:'?OnCancel@CDialog@@MAEXXZ',
    4377:'?OnCancel@CPropertyPage@@UAEXXZ',
    4378:'?OnCancelEditCntr@CRichEditView@@IAEXXZ',
    4379:'?OnCancelMode@COleControl@@IAEXXZ',
    4380:'?OnCancelMode@CSplitterWnd@@IAEXXZ',
    4381:'?OnCancelMode@CWnd@@IAEXXZ',
    4382:'?OnChange@COleClientItem@@UAEXW4OLE_NOTIFICATION@@K@Z',
    4383:'?OnChanged@XFontNotification@COleControl@@UAGJJ@Z',
    4384:'?OnChanged@XPropertyNotifySink@COleControlSite@@UAGJJ@Z',
    4385:'?OnChanged@XPropNotifySink@COlePropertyPage@@UAGJJ@Z',
    4386:'?OnChangedRect@CRectTracker@@UAEXABVCRect@@@Z',
    4387:'?OnChangedViewList@CDocument@@UAEXXZ',
    4388:'?OnChangeItemPosition@COleClientItem@@MAEHABVCRect@@@Z',
    4389:'?OnChangeItemPosition@CRichEditCntrItem@@MAEHABVCRect@@@Z',
    4390:'?OnChar@COleControl@@IAEXIII@Z',
    4391:'?OnCharBold@CRichEditView@@IAEXXZ',
    4392:'?OnCharEffect@CRichEditView@@QAEXKK@Z',
    4393:'?OnCharItalic@CRichEditView@@IAEXXZ',
    4394:'?OnCharToItem@CWnd@@IAEHIPAVCListBox@@I@Z',
    4395:'?OnCharUnderline@CRichEditView@@IAEXXZ',
    4396:'?OnChildNotify@CButton@@MAEHIIJPAJ@Z',
    4397:'?OnChildNotify@CCheckListBox@@MAEHIIJPAJ@Z',
    4398:'?OnChildNotify@CComboBox@@MAEHIIJPAJ@Z',
    4399:'?OnChildNotify@CDragListBox@@MAEHIIJPAJ@Z',
    4400:'?OnChildNotify@CHeaderCtrl@@MAEHIIJPAJ@Z',
    4401:'?OnChildNotify@CListBox@@MAEHIIJPAJ@Z',
    4402:'?OnChildNotify@CListCtrl@@MAEHIIJPAJ@Z',
    4403:'?OnChildNotify@CListView@@MAEHIIJPAJ@Z',
    4404:'?OnChildNotify@CStatusBar@@MAEHIIJPAJ@Z',
    4405:'?OnChildNotify@CStatusBarCtrl@@MAEHIIJPAJ@Z',
    4406:'?OnChildNotify@CTabCtrl@@MAEHIIJPAJ@Z',
    4407:'?OnChildNotify@CWnd@@MAEHIIJPAJ@Z',
    4408:'?OnClear@CPicturePropPage@@IAEXXZ',
    4409:'?OnClick@COleControl@@UAEXG@Z',
    4410:'?OnClone@CEnumArray@@MAEPAV1@XZ',
    4411:'?OnClose@CAsyncSocket@@MAEXH@Z',
    4412:'?OnClose@CControlFrameWnd@@IAEXXZ',
    4413:'?OnClose@CFrameWnd@@IAEXXZ',
    4414:'?OnClose@CMiniDockFrameWnd@@QAEXXZ',
    4415:'?OnClose@COleControl@@UAEXK@Z',
    4416:'?OnClose@COleServerDoc@@MAEXW4tagOLECLOSE@@@Z',
    4417:'?OnClose@CPropertySheet@@IAEXXZ',
    4418:'?OnClose@XAdviseSink@COleClientItem@@UAGXXZ',
    4419:'?OnCloseDocument@CDocObjectServer@@UAEXXZ',
    4420:'?OnCloseDocument@CDocument@@UAEXXZ',
    4421:'?OnCloseDocument@COleDocument@@UAEXXZ',
    4422:'?OnCloseDocument@COleLinkingDoc@@UAEXXZ',
    4423:'?OnCloseDocument@COleServerDoc@@UAEXXZ',
    4424:'?OnCmdMsg@CCmdTarget@@UAEHIHPAXPAUAFX_CMDHANDLERINFO@@@Z',
    4425:'?OnCmdMsg@CDialog@@UAEHIHPAXPAUAFX_CMDHANDLERINFO@@@Z',
    4426:'?OnCmdMsg@CDocument@@UAEHIHPAXPAUAFX_CMDHANDLERINFO@@@Z',
    4427:'?OnCmdMsg@CFrameWnd@@UAEHIHPAXPAUAFX_CMDHANDLERINFO@@@Z',
    4428:'?OnCmdMsg@CMDIFrameWnd@@UAEHIHPAXPAUAFX_CMDHANDLERINFO@@@Z',
    4429:'?OnCmdMsg@COleCntrFrameWnd@@UAEHIHPAXPAUAFX_CMDHANDLERINFO@@@Z',
    4430:'?OnCmdMsg@COleDocument@@UAEHIHPAXPAUAFX_CMDHANDLERINFO@@@Z',
    4431:'?OnCmdMsg@CPropertySheet@@UAEHIHPAXPAUAFX_CMDHANDLERINFO@@@Z',
    4432:'?OnCmdMsg@CView@@UAEHIHPAXPAUAFX_CMDHANDLERINFO@@@Z',
    4433:'?OnColorDefault@CRichEditView@@IAEXXZ',
    4434:'?OnColorOK@CColorDialog@@MAEHXZ',
    4435:'?OnColorPick@CRichEditView@@IAEXK@Z',
    4436:'?OnCommand@CFrameWnd@@MAEHIJ@Z',
    4437:'?OnCommand@CMDIFrameWnd@@MAEHIJ@Z',
    4438:'?OnCommand@COlePropertyPage@@MAEHIJ@Z',
    4439:'?OnCommand@CPropertySheet@@UAEHIJ@Z',
    4440:'?OnCommand@CSplitterWnd@@MAEHIJ@Z',
    4441:'?OnCommand@CWnd@@MAEHIJ@Z',
    4442:'?OnCommandHelp@CDialog@@IAEJIJ@Z',
    4443:'?OnCommandHelp@CFrameWnd@@IAEJIJ@Z',
    4444:'?OnCommandHelp@CMDIFrameWnd@@IAEJIJ@Z',
    4445:'?OnCommandHelp@CPropertySheet@@IAEJIJ@Z',
    4446:'?OnCompareItem@CWnd@@IAEHHPAUtagCOMPAREITEMSTRUCT@@@Z',
    4447:'?OnConnect@CAsyncSocket@@MAEXH@Z',
    4448:'?OnContextHelp@CFrameWnd@@QAEXXZ',
    4449:'?OnContextHelp@COleFrameHook@@UAEHH@Z',
    4450:'?OnContextHelp@COleIPFrameWnd@@IAEXXZ',
    4451:'?OnContextHelp@CWinApp@@IAEXXZ',
    4452:'?OnControlInfoChanged@XOleControlSite@COleControlSite@@UAGJXZ',
    4453:'?OnCreate@CCheckListBox@@IAEHPAUtagCREATESTRUCTA@@@Z',
    4454:'?OnCreate@CControlBar@@QAEHPAUtagCREATESTRUCTA@@@Z',
    4455:'?OnCreate@CEditView@@IAEHPAUtagCREATESTRUCTA@@@Z',
    4456:'?OnCreate@CFormView@@IAEHPAUtagCREATESTRUCTA@@@Z',
    4457:'?OnCreate@CFrameWnd@@IAEHPAUtagCREATESTRUCTA@@@Z',
    4458:'?OnCreate@CMDIChildWnd@@IAEHPAUtagCREATESTRUCTA@@@Z',
    4459:'?OnCreate@COleControl@@IAEHPAUtagCREATESTRUCTA@@@Z',
    4460:'?OnCreate@COleIPFrameWnd@@IAEHPAUtagCREATESTRUCTA@@@Z',
    4461:'?OnCreate@CPreviewView@@IAEHPAUtagCREATESTRUCTA@@@Z',
    4462:'?OnCreate@CRichEditView@@IAEHPAUtagCREATESTRUCTA@@@Z',
    4463:'?OnCreate@CToolBarCtrl@@IAEHPAUtagCREATESTRUCTA@@@Z',
    4464:'?OnCreate@CView@@IAEHPAUtagCREATESTRUCTA@@@Z',
    4465:'?OnCreateAggregates@CCmdTarget@@UAEHXZ',
    4466:'?OnCreateAggregates@COleControl@@MAEHXZ',
    4467:'?OnCreateClient@CFrameWnd@@MAEHPAUtagCREATESTRUCTA@@PAUCCreateContext@@@Z',
    4468:'?OnCreateClient@CMDIFrameWnd@@UAEHPAUtagCREATESTRUCTA@@PAUCCreateContext@@@Z',
    4469:'?OnCreateControlBars@COleIPFrameWnd@@UAEHPAVCFrameWnd@@0@Z',
    4470:'?OnCreateControlBars@COleIPFrameWnd@@UAEHPAVCWnd@@0@Z',
    4471:'?OnCreateHelper@CFrameWnd@@IAEHPAUtagCREATESTRUCTA@@PAUCCreateContext@@@Z',
    4472:'?OnCreateObject@COleObjectFactory@@MAEPAVCCmdTarget@@XZ',
    4473:'?OnCreateObject@COleTemplateServer@@MAEPAVCCmdTarget@@XZ',
    4474:'?OnCtlColor@CColorDialog@@IAEPAUHBRUSH__@@PAVCDC@@PAVCWnd@@I@Z',
    4475:'?OnCtlColor@CControlBar@@QAEPAUHBRUSH__@@PAVCDC@@PAVCWnd@@I@Z',
    4476:'?OnCtlColor@CDialog@@IAEPAUHBRUSH__@@PAVCDC@@PAVCWnd@@I@Z',
    4477:'?OnCtlColor@COlePropertyPage@@IAEPAUHBRUSH__@@PAVCDC@@PAVCWnd@@I@Z',
    4478:'?OnCtlColor@CPropertyPage@@IAEPAUHBRUSH__@@PAVCDC@@PAVCWnd@@I@Z',
    4479:'?OnCtlColor@CPropertySheet@@IAEPAUHBRUSH__@@PAVCDC@@PAVCWnd@@I@Z',
    4480:'?OnCtlColor@CWnd@@IAEPAUHBRUSH__@@PAVCDC@@PAV1@I@Z',
    4481:'?OnDataAvailable@CAsyncMonikerFile@@MAEXKK@Z',
    4482:'?OnDataAvailable@CCachedDataPathProperty@@UAEXKK@Z',
    4483:'?OnDataChange@COleClientItem@@MAEXPAUtagFORMATETC@@PAUtagSTGMEDIUM@@@Z',
    4484:'?OnDataChange@XAdviseSink@COleClientItem@@UAGXPAUtagFORMATETC@@PAUtagSTGMEDIUM@@@Z',
    4485:'?OnDDECommand@CDocManager@@UAEHPAD@Z',
    4486:'?OnDDECommand@CWinApp@@UAEHPAD@Z',
    4487:'?OnDDEExecute@CFrameWnd@@IAEJIJ@Z',
    4488:'?OnDDEInitiate@CFrameWnd@@IAEJIJ@Z',
    4489:'?OnDDETerminate@CFrameWnd@@IAEJIJ@Z',
    4490:'?OnDeactivate@COleClientItem@@UAEXXZ',
    4491:'?OnDeactivate@COleServerDoc@@UAEXXZ',
    4492:'?OnDeactivateAndUndo@COleClientItem@@MAEXXZ',
    4493:'?OnDeactivateUI@COleClientItem@@UAEXH@Z',
    4494:'?OnDeactivateUI@COleServerDoc@@UAEXH@Z',
    4495:'?OnDeactivateUI@CRichEditCntrItem@@UAEXH@Z',
    4496:'?OnDeleteItem@CWnd@@IAEXHPAUtagDELETEITEMSTRUCT@@@Z',
    4497:'?OnDestroy@CControlBar@@QAEXXZ',
    4498:'?OnDestroy@CEditView@@IAEXXZ',
    4499:'?OnDestroy@CFrameWnd@@IAEXXZ',
    4500:'?OnDestroy@CMDIChildWnd@@IAEXXZ',
    4501:'?OnDestroy@CMDIFrameWnd@@IAEXXZ',
    4502:'?OnDestroy@COleControl@@IAEXXZ',
    4503:'?OnDestroy@COleIPFrameWnd@@IAEXXZ',
    4504:'?OnDestroy@CRichEditView@@IAEXXZ',
    4505:'?OnDestroy@CTabCtrl@@IAEXXZ',
    4506:'?OnDestroy@CTreeCtrl@@QAEXXZ',
    4507:'?OnDestroy@CTreeView@@QAEXXZ',
    4508:'?OnDestroy@CView@@IAEXXZ',
    4509:'?OnDevModeChange@CRichEditView@@IAEXPAD@Z',
    4510:'?OnDevModeChange@CWnd@@IAEXPAD@Z',
    4511:'?OnDisableModal@CToolTipCtrl@@IAEJIJ@Z',
    4512:'?OnDiscardUndoState@COleClientItem@@MAEXXZ',
    4513:'?OnDisplayChange@CSplitterWnd@@IAEXXZ',
    4514:'?OnDisplayChange@CWnd@@IAEJIJ@Z',
    4515:'?OnDisplayPageNumber@CPreviewView@@MAEXII@Z',
    4516:'?OnDocActivate@COleFrameHook@@UAEHH@Z',
    4517:'?OnDocWindowActivate@COleServerDoc@@MAEXH@Z',
    4518:'?OnDocWindowActivate@XOleInPlaceActiveObject@COleControl@@UAGJH@Z',
    4519:'?OnDocWindowActivate@XOleInPlaceActiveObject@COleServerDoc@@UAGJH@Z',
    4520:'?OnDoVerb@CDocObjectServerItem@@MAEXJ@Z',
    4521:'?OnDoVerb@COleControl@@UAEHJPAUtagMSG@@PAUHWND__@@PBUtagRECT@@@Z',
    4522:'?OnDoVerb@COleServerItem@@UAEXJ@Z',
    4523:'?OnDragEnter@COleDropTarget@@UAEKPAVCWnd@@PAVCOleDataObject@@KVCPoint@@@Z',
    4524:'?OnDragEnter@CView@@UAEKPAVCOleDataObject@@KVCPoint@@@Z',
    4525:'?OnDragLeave@COleDropTarget@@UAEXPAVCWnd@@@Z',
    4526:'?OnDragLeave@CView@@UAEXXZ',
    4527:'?OnDragList@CWnd@@IAEJIJ@Z',
    4528:'?OnDragOver@COleDropTarget@@UAEKPAVCWnd@@PAVCOleDataObject@@KVCPoint@@@Z',
    4529:'?OnDragOver@CView@@UAEKPAVCOleDataObject@@KVCPoint@@@Z',
    4530:'?OnDragScroll@COleDropTarget@@UAEKPAVCWnd@@KVCPoint@@@Z',
    4531:'?OnDragScroll@CView@@UAEKKVCPoint@@@Z',
    4532:'?OnDraw@CCtrlView@@MAEXPAVCDC@@@Z',
    4533:'?OnDraw@CFormView@@MAEXPAVCDC@@@Z',
    4534:'?OnDraw@COleControl@@UAEXPAVCDC@@ABVCRect@@1@Z',
    4535:'?OnDraw@CPreviewView@@MAEXPAVCDC@@@Z',
    4536:'?OnDraw@CView@@MAEXPAVCDC@@@Z',
    4537:'?OnDrawEx@COleServerItem@@UAEHPAVCDC@@W4tagDVASPECT@@AAVCSize@@@Z',
    4538:'?OnDrawItem@CWnd@@IAEXHPAUtagDRAWITEMSTRUCT@@@Z',
    4539:'?OnDrawMetafile@COleControl@@UAEXPAVCDC@@ABVCRect@@@Z',
    4540:'?OnDrawPage@CPageSetupDialog@@UAEIPAVCDC@@IPAUtagRECT@@@Z',
    4541:'?OnDrawSplitter@CSplitterWnd@@MAEXPAVCDC@@W4ESplitType@1@ABVCRect@@@Z',
    4542:'?OnDrop@COleDropTarget@@UAEHPAVCWnd@@PAVCOleDataObject@@KVCPoint@@@Z',
    4543:'?OnDrop@CView@@UAEHPAVCOleDataObject@@KVCPoint@@@Z',
    4544:'?OnDropEx@COleDropTarget@@UAEKPAVCWnd@@PAVCOleDataObject@@KKVCPoint@@@Z',
    4545:'?OnDropEx@CView@@UAEKPAVCOleDataObject@@KKVCPoint@@@Z',
    4546:'?OnDropFiles@CFrameWnd@@IAEXPAUHDROP__@@@Z',
    4547:'?OnDropFiles@CRichEditView@@IAEXPAUHDROP__@@@Z',
    4548:'?OnEdit@COleControl@@UAEHPAUtagMSG@@PAUHWND__@@PBUtagRECT@@@Z',
    4549:'?OnEditChange@CEditView@@IAEHXZ',
    4550:'?OnEditchangeFontstyles@CFontPropPage@@IAEXXZ',
    4551:'?OnEditChangeIcon@COleDocument@@IAEXXZ',
    4552:'?OnEditClear@CEditView@@IAEXXZ',
    4553:'?OnEditClear@CRichEditView@@IAEXXZ',
    4554:'?OnEditConvert@COleDocument@@IAEXXZ',
    4555:'?OnEditCopy@CEditView@@IAEXXZ',
    4556:'?OnEditCopy@CRichEditView@@IAEXXZ',
    4557:'?OnEditCut@CEditView@@IAEXXZ',
    4558:'?OnEditCut@CRichEditView@@IAEXXZ',
    4559:'?OnEditFind@CEditView@@IAEXXZ',
    4560:'?OnEditFind@CRichEditView@@IAEXXZ',
    4561:'?OnEditFindReplace@CEditView@@IAEXH@Z',
    4562:'?OnEditFindReplace@CRichEditView@@IAEXH@Z',
    4563:'?OnEditLinks@COleDocument@@IAEXXZ',
    4564:'?OnEditPaste@CEditView@@IAEXXZ',
    4565:'?OnEditPaste@CRichEditView@@IAEXXZ',
    4566:'?OnEditPasteSpecial@CRichEditView@@IAEXXZ',
    4567:'?OnEditProperties@CRichEditView@@IAEXXZ',
    4568:'?OnEditProperty@CColorPropPage@@MAEHJ@Z',
    4569:'?OnEditProperty@CFontPropPage@@MAEHJ@Z',
    4570:'?OnEditProperty@COlePropertyPage@@UAEHJ@Z',
    4571:'?OnEditProperty@CPicturePropPage@@MAEHJ@Z',
    4572:'?OnEditProperty@CStockPropPage@@IAEHJAAVCComboBox@@@Z',
    4573:'?OnEditRepeat@CEditView@@IAEXXZ',
    4574:'?OnEditRepeat@CRichEditView@@IAEXXZ',
    4575:'?OnEditReplace@CEditView@@IAEXXZ',
    4576:'?OnEditReplace@CRichEditView@@IAEXXZ',
    4577:'?OnEditSelectAll@CEditView@@IAEXXZ',
    4578:'?OnEditSelectAll@CRichEditView@@IAEXXZ',
    4579:'?OnEditUndo@CEditView@@IAEXXZ',
    4580:'?OnEditUndo@CRichEditView@@IAEXXZ',
    4581:'?OnEditupdateFontnames@CFontPropPage@@IAEXXZ',
    4582:'?OnEditupdateFontsizes@CFontPropPage@@IAEXXZ',
    4583:'?OnEnable@CFrameWnd@@IAEXH@Z',
    4584:'?OnEnabledChanged@COleControl@@UAEXXZ',
    4585:'?OnEnableModeless@COleFrameHook@@UAEXH@Z',
    4586:'?OnEndPrinting@CEditView@@MAEXPAVCDC@@PAUCPrintInfo@@@Z',
    4587:'?OnEndPrinting@CRichEditView@@MAEXPAVCDC@@PAUCPrintInfo@@@Z',
    4588:'?OnEndPrinting@CView@@MAEXPAVCDC@@PAUCPrintInfo@@@Z',
    4589:'?OnEndPrintPreview@CView@@MAEXPAVCDC@@PAUCPrintInfo@@UtagPOINT@@PAVCPreviewView@@@Z',
    4590:'?OnEndSession@CFrameWnd@@IAEXH@Z',
    4591:'?OnEnterIdle@CFrameWnd@@IAEXIPAVCWnd@@@Z',
    4592:'?OnEnterIdle@COleControl@@IAEXIPAVCWnd@@@Z',
    4593:'?OnEnterIdle@CWnd@@IAEXIPAV1@@Z',
    4594:'?OnEnumVerbs@COleControl@@UAEHPAPAUIEnumOLEVERB@@@Z',
    4595:'?OnEraseBkgnd@CFrameWnd@@IAEHPAVCDC@@@Z',
    4596:'?OnEraseBkgnd@COleResizeBar@@IAEHPAVCDC@@@Z',
    4597:'?OnEraseBkgnd@CPreviewView@@IAEHPAVCDC@@@Z',
    4598:'?OnEraseBkgnd@CToolBar@@IAEHPAVCDC@@@Z',
    4599:'?OnEraseBkgnd@CWnd@@IAEHPAVCDC@@@Z',
    4600:'?OnEvent@CCmdTarget@@QAEHIPAUAFX_EVENT@@PAUAFX_CMDHANDLERINFO@@@Z',
    4601:'?OnEvent@COccManager@@UAEHPAVCCmdTarget@@IPAUAFX_EVENT@@PAUAFX_CMDHANDLERINFO@@@Z',
    4602:'?OnEvent@COleControlSite@@IAEHPAUAFX_EVENT@@@Z',
    4603:'?OnEventAdvise@COleControl@@UAEXH@Z',
    4604:'?OnExecOleCmd@CDocObjectServer@@IAGJPBU_GUID@@KKPAUtagVARIANT@@1@Z',
    4605:'?OnExecOleCmd@COleServerDoc@@MAEJPBU_GUID@@KKPAUtagVARIANT@@1@Z',
    4606:'?OnFileClose@CDocument@@IAEXXZ',
    4607:'?OnFileNameChange@CFileDialog@@MAEXXZ',
    4608:'?OnFileNameOK@CFileDialog@@MAEHXZ',
    4609:'?OnFileNew@CDocManager@@UAEXXZ',
    4610:'?OnFileNew@CWinApp@@IAEXXZ',
    4611:'?OnFileOpen@CDocManager@@UAEXXZ',
    4612:'?OnFileOpen@CWinApp@@IAEXXZ',
    4613:'?OnFilePrint@CView@@IAEXXZ',
    4614:'?OnFilePrintPreview@CView@@IAEXXZ',
    4615:'?OnFilePrintSetup@CWinApp@@IAEXXZ',
    4616:'?OnFileSave@CDocument@@IAEXXZ',
    4617:'?OnFileSaveAs@CDocument@@IAEXXZ',
    4618:'?OnFileSaveCopyAs@COleServerDoc@@IAEXXZ',
    4619:'?OnFileSendMail@CDocument@@IAEXXZ',
    4620:'?OnFileSendMail@COleDocument@@IAEXXZ',
    4621:'?OnFileUpdate@COleServerDoc@@IAEXXZ',
    4622:'?OnFinalRelease@CCmdTarget@@UAEXXZ',
    4623:'?OnFinalRelease@CDocument@@UAEXXZ',
    4624:'?OnFinalRelease@COleControl@@MAEXXZ',
    4625:'?OnFinalRelease@COlePropertyPage@@MAEXXZ',
    4626:'?OnFinalRelease@COleServerItem@@UAEXXZ',
    4627:'?OnFinalRelease@CWnd@@UAEXXZ',
    4628:'?OnFindEmbeddedItem@COleLinkingDoc@@MAEPAVCOleClientItem@@PBD@Z',
    4629:'?OnFindNext@CEditView@@MAEXPBDHH@Z',
    4630:'?OnFindNext@CRichEditView@@MAEXPBDHHH@Z',
    4631:'?OnFindReplaceCmd@CEditView@@IAEJIJ@Z',
    4632:'?OnFindReplaceCmd@CRichEditView@@IAEJIJ@Z',
    4633:'?OnFloatStatus@CMiniFrameWnd@@QAEJIJ@Z',
    4634:'?OnFocus@XOleControlSite@COleControlSite@@UAGJH@Z',
    4635:'?OnFolderChange@CFileDialog@@MAEXXZ',
    4636:'?OnFontChanged@COleControl@@UAEXXZ',
    4637:'?OnForeColorChanged@COleControl@@UAEXXZ',
    4638:'?OnFormatFont@CRichEditView@@IAEXXZ',
    4639:'?OnFrameClose@COleControl@@MAEXXZ',
    4640:'?OnFrameWindowActivate@COleServerDoc@@MAEXH@Z',
    4641:'?OnFrameWindowActivate@XOleInPlaceActiveObject@COleControl@@UAGJH@Z',
    4642:'?OnFrameWindowActivate@XOleInPlaceActiveObject@COleServerDoc@@UAGJH@Z',
    4643:'?OnFreezeEvents@COleControl@@UAEXH@Z',
    4644:'?OnGetCheckPosition@CCheckListBox@@UAE?AVCRect@@V2@0@Z',
    4645:'?OnGetClipboardData@COleClientItem@@UAEPAVCOleDataSource@@HPAUtagPOINT@@PAUtagSIZE@@@Z',
    4646:'?OnGetClipboardData@COleServerItem@@UAEPAVCOleDataSource@@HPAUtagPOINT@@PAUtagSIZE@@@Z',
    4647:'?OnGetClipRect@COleClientItem@@MAEXAAVCRect@@@Z',
    4648:'?OnGetColorSet@COleControl@@UAEHPAUtagDVTARGETDEVICE@@PAUHDC__@@PAPAUtagLOGPALETTE@@@Z',
    4649:'?OnGetControlInfo@COleControl@@UAEXPAUtagCONTROLINFO@@@Z',
    4650:'?OnGetDisplayString@COleControl@@UAEHJAAVCString@@@Z',
    4651:'?OnGetDlgCode@COleControl@@IAEIXZ',
    4652:'?OnGetEmbeddedItem@CRichEditDoc@@MAEPAVCOleServerItem@@XZ',
    4653:'?OnGetExtent@COleServerItem@@UAEHW4tagDVASPECT@@AAVCSize@@@Z',
    4654:'?OnGetInPlaceMenu@COleControl@@UAEPAUHMENU__@@XZ',
    4655:'?OnGetItemPosition@COleClientItem@@MAEXAAVCRect@@@Z',
    4656:'?OnGetLinkedItem@COleLinkingDoc@@MAEPAVCOleServerItem@@PBD@Z',
    4657:'?OnGetLinkedItem@COleServerDoc@@UAEPAVCOleServerItem@@PBD@Z',
    4658:'?OnGetMinMaxInfo@CMiniFrameWnd@@QAEXPAUtagMINMAXINFO@@@Z',
    4659:'?OnGetNaturalExtent@COleControl@@UAEHKJPAUtagDVTARGETDEVICE@@PAUHDC__@@PAUtagExtentInfo@@PAUtagSIZE@@@Z',
    4660:'?OnGetPredefinedStrings@COleControl@@UAEHJPAVCStringArray@@PAVCDWordArray@@@Z',
    4661:'?OnGetPredefinedValue@COleControl@@UAEHJKPAUtagVARIANT@@@Z',
    4662:'?OnGetText@CMiniFrameWnd@@QAEJIJ@Z',
    4663:'?OnGetText@CStatusBar@@IAEJIJ@Z',
    4664:'?OnGetTextLength@CMiniFrameWnd@@QAEJIJ@Z',
    4665:'?OnGetTextLength@CStatusBar@@IAEJIJ@Z',
    4666:'?OnGetViewExtent@COleControl@@UAEHKJPAUtagDVTARGETDEVICE@@PAUtagSIZE@@@Z',
    4667:'?OnGetViewRect@COleControl@@UAEHKPAU_RECTL@@@Z',
    4668:'?OnGetViewStatus@COleControl@@UAEKXZ',
    4669:'?OnGetWindowContext@COleClientItem@@MAEHPAPAVCFrameWnd@@0PAUtagOIFI@@@Z',
    4670:'?OnGrayCtlColor@CWnd@@QAEPAUHBRUSH__@@PAVCDC@@PAV1@I@Z',
    4671:'?OnHelp@CFrameWnd@@IAEXXZ',
    4672:'?OnHelp@COlePropertyPage@@UAEHPBD@Z',
    4673:'?OnHelp@CWinApp@@IAEXXZ',
    4674:'?OnHelp@CWnd@@QAEXXZ',
    4675:'?OnHelpFinder@CWinApp@@IAEXXZ',
    4676:'?OnHelpFinder@CWnd@@QAEXXZ',
    4677:'?OnHelpHitTest@CControlBar@@QAEJIJ@Z',
    4678:'?OnHelpHitTest@CDialog@@IAEJIJ@Z',
    4679:'?OnHelpHitTest@CFrameWnd@@IAEJIJ@Z',
    4680:'?OnHelpIndex@CWinApp@@IAEXXZ',
    4681:'?OnHelpIndex@CWnd@@QAEXXZ',
    4682:'?OnHelpInfo@CCommonDialog@@IAEHPAUtagHELPINFO@@@Z',
    4683:'?OnHelpInfo@CWnd@@IAEHPAUtagHELPINFO@@@Z',
    4684:'?OnHelpUsing@CWinApp@@IAEXXZ',
    4685:'?OnHelpUsing@CWnd@@QAEXXZ',
    4686:'?OnHide@CDocObjectServerItem@@MAEXXZ',
    4687:'?OnHide@COleControl@@MAEJXZ',
    4688:'?OnHide@COleServerItem@@MAEXXZ',
    4689:'?OnHideToolBars@COleControl@@UAEXXZ',
    4690:'?OnHScroll@CFrameWnd@@IAEXIIPAVCScrollBar@@@Z',
    4691:'?OnHScroll@CPreviewView@@IAEXIIPAVCScrollBar@@@Z',
    4692:'?OnHScroll@CScrollView@@QAEXIIPAVCScrollBar@@@Z',
    4693:'?OnHScroll@CSplitterWnd@@IAEXIIPAVCScrollBar@@@Z',
    4694:'?OnHScroll@CWnd@@IAEXIIPAVCScrollBar@@@Z',
    4695:'?OnIdle@CDocTemplate@@UAEXXZ',
    4696:'?OnIdle@CDocument@@UAEXXZ',
    4697:'?OnIdle@COleDocument@@UAEXXZ',
    4698:'?OnIdle@CWinApp@@UAEHJ@Z',
    4699:'?OnIdle@CWinThread@@UAEHJ@Z',
    4700:'?OnIdleUpdateCmdUI@CControlBar@@QAEJIJ@Z',
    4701:'?OnIdleUpdateCmdUI@CFrameWnd@@IAEXXZ',
    4702:'?OnIdleUpdateCmdUI@CMDIFrameWnd@@IAEXXZ',
    4703:'?OnIdleUpdateCmdUI@COleCntrFrameWnd@@QAEXXZ',
    4704:'?OnIdleUpdateCmdUI@COleIPFrameWnd@@IAEXXZ',
    4705:'?OnInactiveMouseMove@COleControl@@UAEXPBUtagRECT@@JJK@Z',
    4706:'?OnInactiveMouseMove@XPointerInactive@COleControl@@UAGJPBUtagRECT@@JJK@Z',
    4707:'?OnInactiveSetCursor@COleControl@@UAEHPBUtagRECT@@JJKH@Z',
    4708:'?OnInactiveSetCursor@XPointerInactive@COleControl@@UAGJPBUtagRECT@@JJKH@Z',
    4709:'?OnInitDialog@CColorPropPage@@MAEHXZ',
    4710:'?OnInitDialog@CDialog@@UAEHXZ',
    4711:'?OnInitDialog@CFontPropPage@@MAEHXZ',
    4712:'?OnInitDialog@COlePropertiesDialog@@UAEHXZ',
    4713:'?OnInitDialog@COlePropertyPage@@UAEHXZ',
    4714:'?OnInitDialog@CPicturePropPage@@MAEHXZ',
    4715:'?OnInitDialog@CPropertySheet@@UAEHXZ',
    4716:'?OnInitDone@CFileDialog@@MAEXXZ',
    4717:'?OnInitFromData@COleServerItem@@UAEHPAVCOleDataObject@@H@Z',
    4718:'?OnInitialUpdate@CControlBar@@QAEXXZ',
    4719:'?OnInitialUpdate@CDaoRecordView@@UAEXXZ',
    4720:'?OnInitialUpdate@CFormView@@UAEXXZ',
    4721:'?OnInitialUpdate@CRecordView@@UAEXXZ',
    4722:'?OnInitialUpdate@CRichEditView@@MAEXXZ',
    4723:'?OnInitialUpdate@CView@@UAEXXZ',
    4724:'?OnInitMenuPopup@CFrameWnd@@IAEXPAVCMenu@@IH@Z',
    4725:'?OnInitMenuPopup@COleControl@@IAEXPAVCMenu@@IH@Z',
    4726:'?OnInPlaceActivate@XOleIPSite@COleClientItem@@UAGJXZ',
    4727:'?OnInPlaceActivate@XOleIPSite@COleControlSite@@UAGJXZ',
    4728:'?OnInPlaceDeactivate@XOleIPSite@COleClientItem@@UAGJXZ',
    4729:'?OnInPlaceDeactivate@XOleIPSite@COleControlSite@@UAGJXZ',
    4730:'?OnInsertMenus@COleClientItem@@UAEXPAVCMenu@@PAUtagOleMenuGroupWidths@@@Z',
    4731:'?OnInsertObject@CRichEditView@@IAEXXZ',
    4732:'?OnInvertTracker@CSplitterWnd@@MAEXABVCRect@@@Z',
    4733:'?OnKey@CDockContext@@QAEXHH@Z',
    4734:'?OnKeyDown@CCheckListBox@@IAEXIII@Z',
    4735:'?OnKeyDown@COleControl@@IAEXIII@Z',
    4736:'?OnKeyDown@CRichEditView@@IAEXIII@Z',
    4737:'?OnKeyDown@CSplitterWnd@@IAEXIII@Z',
    4738:'?OnKeyDownEvent@COleControl@@UAEXGG@Z',
    4739:'?OnKeyPressEvent@COleControl@@UAEXG@Z',
    4740:'?OnKeyUp@COleControl@@IAEXIII@Z',
    4741:'?OnKeyUpEvent@COleControl@@UAEXGG@Z',
    4742:'?OnKillActive@CPropertyPage@@UAEHXZ',
    4743:'?OnKillFocus@COleControl@@IAEXPAVCWnd@@@Z',
    4744:'?OnLBAddString@CCheckListBox@@IAEJIJ@Z',
    4745:'?OnLBFindString@CCheckListBox@@IAEJIJ@Z',
    4746:'?OnLBFindStringExact@CCheckListBox@@IAEJIJ@Z',
    4747:'?OnLBGetItemData@CCheckListBox@@IAEJIJ@Z',
    4748:'?OnLBGetText@CCheckListBox@@IAEJIJ@Z',
    4749:'?OnLBInsertString@CCheckListBox@@IAEJIJ@Z',
    4750:'?OnLBSelChangedNotify@CFileDialog@@MAEXIII@Z',
    4751:'?OnLBSelectString@CCheckListBox@@IAEJIJ@Z',
    4752:'?OnLBSetItemData@CCheckListBox@@IAEJIJ@Z',
    4753:'?OnLBSetItemHeight@CCheckListBox@@IAEJIJ@Z',
    4754:'?OnLButtonDblClk@CCheckListBox@@IAEXIVCPoint@@@Z',
    4755:'?OnLButtonDblClk@CControlBar@@QAEXIVCPoint@@@Z',
    4756:'?OnLButtonDblClk@COleControl@@IAEXIVCPoint@@@Z',
    4757:'?OnLButtonDblClk@CSplitterWnd@@IAEXIVCPoint@@@Z',
    4758:'?OnLButtonDown@CCheckListBox@@IAEXIVCPoint@@@Z',
    4759:'?OnLButtonDown@CControlBar@@QAEXIVCPoint@@@Z',
    4760:'?OnLButtonDown@COleControl@@IAEXIVCPoint@@@Z',
    4761:'?OnLButtonDown@COleResizeBar@@IAEXIVCPoint@@@Z',
    4762:'?OnLButtonDown@CPreviewView@@IAEXIVCPoint@@@Z',
    4763:'?OnLButtonDown@CSplitterWnd@@IAEXIVCPoint@@@Z',
    4764:'?OnLButtonUp@CMiniFrameWnd@@QAEXIVCPoint@@@Z',
    4765:'?OnLButtonUp@COleControl@@IAEXIVCPoint@@@Z',
    4766:'?OnLButtonUp@CSplitterWnd@@IAEXIVCPoint@@@Z',
    4767:'?OnLowResource@CAsyncMonikerFile@@MAEXXZ',
    4768:'?OnMapPropertyToPage@COleControl@@UAEHJPAU_GUID@@PAH@Z',
    4769:'?OnMButtonDblClk@COleControl@@IAEXIVCPoint@@@Z',
    4770:'?OnMButtonDown@COleControl@@IAEXIVCPoint@@@Z',
    4771:'?OnMButtonUp@COleControl@@IAEXIVCPoint@@@Z',
    4772:'?OnMDIActivate@CMDIChildWnd@@IAEXHPAVCWnd@@0@Z',
    4773:'?OnMDIWindowCmd@CMDIFrameWnd@@IAEHI@Z',
    4774:'?OnMeasureItem@CWnd@@IAEXHPAUtagMEASUREITEMSTRUCT@@@Z',
    4775:'?OnMenuChar@CMDIFrameWnd@@IAEJIIPAVCMenu@@@Z',
    4776:'?OnMenuSelect@CFrameWnd@@IAEXIIPAUHMENU__@@@Z',
    4777:'?OnMenuSelect@COleControl@@IAEXIIPAUHMENU__@@@Z',
    4778:'?OnMessagePending@COleMessageFilter@@UAEHPBUtagMSG@@@Z',
    4779:'?OnMessagePending@CSocket@@MAEHXZ',
    4780:'?OnMnemonic@COleControl@@UAEXPAUtagMSG@@@Z',
    4781:'?OnMnemonic@XOleControl@COleControl@@UAGJPAUtagMSG@@@Z',
    4782:'?OnMouseActivate@CControlBar@@QAEHPAVCWnd@@II@Z',
    4783:'?OnMouseActivate@CMDIChildWnd@@IAEHPAVCWnd@@II@Z',
    4784:'?OnMouseActivate@CMiniDockFrameWnd@@QAEHPAVCWnd@@II@Z',
    4785:'?OnMouseActivate@COleControl@@IAEHPAVCWnd@@II@Z',
    4786:'?OnMouseActivate@CView@@IAEHPAVCWnd@@II@Z',
    4787:'?OnMouseMove@CMiniFrameWnd@@QAEXIVCPoint@@@Z',
    4788:'?OnMouseMove@COleControl@@IAEXIVCPoint@@@Z',
    4789:'?OnMouseMove@CSplitterWnd@@IAEXIVCPoint@@@Z',
    4790:'?OnMove@CDaoRecordView@@UAEHI@Z',
    4791:'?OnMove@COleControl@@IAEXHH@Z',
    4792:'?OnMove@CRecordView@@UAEHI@Z',
    4793:'?OnNcActivate@CFrameWnd@@IAEHH@Z',
    4794:'?OnNcActivate@CMDIChildWnd@@IAEHH@Z',
    4795:'?OnNcActivate@CMiniFrameWnd@@QAEHH@Z',
    4796:'?OnNcCalcSize@CDockBar@@IAEXHPAUtagNCCALCSIZE_PARAMS@@@Z',
    4797:'?OnNcCalcSize@CMiniFrameWnd@@QAEXHPAUtagNCCALCSIZE_PARAMS@@@Z',
    4798:'?OnNcCalcSize@COleControl@@IAEXHPAUtagNCCALCSIZE_PARAMS@@@Z',
    4799:'?OnNcCalcSize@CStatusBar@@IAEXHPAUtagNCCALCSIZE_PARAMS@@@Z',
    4800:'?OnNcCalcSize@CToolBar@@IAEXHPAUtagNCCALCSIZE_PARAMS@@@Z',
    4801:'?OnNcCreate@CMDIChildWnd@@IAEHPAUtagCREATESTRUCTA@@@Z',
    4802:'?OnNcCreate@CMiniFrameWnd@@QAEHPAUtagCREATESTRUCTA@@@Z',
    4803:'?OnNcCreate@COleControl@@IAEHPAUtagCREATESTRUCTA@@@Z',
    4804:'?OnNcCreate@CPropertySheet@@IAEHPAUtagCREATESTRUCTA@@@Z',
    4805:'?OnNcCreate@CSplitterWnd@@IAEHPAUtagCREATESTRUCTA@@@Z',
    4806:'?OnNcCreate@CToolBar@@IAEHPAUtagCREATESTRUCTA@@@Z',
    4807:'?OnNcDestroy@CListCtrl@@IAEXXZ',
    4808:'?OnNcDestroy@CListView@@QAEXXZ',
    4809:'?OnNcDestroy@CWnd@@IAEXXZ',
    4810:'?OnNcHitTest@CMiniFrameWnd@@QAEIVCPoint@@@Z',
    4811:'?OnNcHitTest@COleControl@@IAEIVCPoint@@@Z',
    4812:'?OnNcHitTest@CStatusBar@@IAEIVCPoint@@@Z',
    4813:'?OnNcHitTest@CToolBar@@IAEIVCPoint@@@Z',
    4814:'?OnNcLButtonDblClk@CMiniDockFrameWnd@@QAEXIVCPoint@@@Z',
    4815:'?OnNcLButtonDown@CMiniDockFrameWnd@@QAEXIVCPoint@@@Z',
    4816:'?OnNcLButtonDown@CMiniFrameWnd@@QAEXIVCPoint@@@Z',
    4817:'?OnNcLButtonDown@COleControl@@IAEXIVCPoint@@@Z',
    4818:'?OnNcPaint@CDockBar@@IAEXXZ',
    4819:'?OnNcPaint@CMiniFrameWnd@@QAEXXZ',
    4820:'?OnNcPaint@COleControl@@IAEXXZ',
    4821:'?OnNcPaint@CStatusBar@@IAEXXZ',
    4822:'?OnNcPaint@CToolBar@@IAEXXZ',
    4823:'?OnNewDocument@CDocument@@UAEHXZ',
    4824:'?OnNewDocument@COleDocument@@UAEHXZ',
    4825:'?OnNewDocument@COleLinkingDoc@@UAEHXZ',
    4826:'?OnNewEmbedding@COleServerDoc@@MAEXPAUIStorage@@@Z',
    4827:'?OnNext@CEnumArray@@MAEHPAX@Z',
    4828:'?OnNext@CEnumConnPoints@@MAEHPAX@Z',
    4829:'?OnNext@CEnumFormatEtc@@MAEHPAX@Z',
    4830:'?OnNext@CEnumOleVerb@@MAEHPAX@Z',
    4831:'?OnNext@CEnumUnknown@@MAEHPAX@Z',
    4832:'?OnNextPage@CPreviewView@@IAEXXZ',
    4833:'?OnNextPaneCmd@CView@@IAEHI@Z',
    4834:'?OnNotify@CFileDialog@@MAEHIJPAJ@Z',
    4835:'?OnNotify@CPropertyPage@@MAEHIJPAJ@Z',
    4836:'?OnNotify@CSplitterWnd@@MAEHIJPAJ@Z',
    4837:'?OnNotify@CWnd@@MAEHIJPAJ@Z',
    4838:'?OnNotRespondingDialog@COleMessageFilter@@UAEHPAUHTASK__@@@Z',
    4839:'?OnNTCtlColor@CWnd@@IAEJIJ@Z',
    4840:'?OnNumPageChange@CPreviewView@@IAEXXZ',
    4841:'?OnObjectsChanged@CColorPropPage@@MAEXXZ',
    4842:'?OnObjectsChanged@CFontPropPage@@MAEXXZ',
    4843:'?OnObjectsChanged@COlePropertyPage@@UAEXXZ',
    4844:'?OnObjectsChanged@CPicturePropPage@@MAEXXZ',
    4845:'?OnOcmCtlColorBtn@COleControl@@IAEJIJ@Z',
    4846:'?OnOcmCtlColorDlg@COleControl@@IAEJIJ@Z',
    4847:'?OnOcmCtlColorEdit@COleControl@@IAEJIJ@Z',
    4848:'?OnOcmCtlColorListBox@COleControl@@IAEJIJ@Z',
    4849:'?OnOcmCtlColorMsgBox@COleControl@@IAEJIJ@Z',
    4850:'?OnOcmCtlColorScrollBar@COleControl@@IAEJIJ@Z',
    4851:'?OnOcmCtlColorStatic@COleControl@@IAEJIJ@Z',
    4852:'?OnOK@CCommonDialog@@MAEXXZ',
    4853:'?OnOK@CDialog@@MAEXXZ',
    4854:'?OnOK@CPropertyPage@@UAEXXZ',
    4855:'?OnOpen@CDocObjectServerItem@@MAEXXZ',
    4856:'?OnOpen@COleControl@@MAEJHPAUtagMSG@@@Z',
    4857:'?OnOpen@COleServerItem@@MAEXXZ',
    4858:'?OnOpenDocument@CDocument@@UAEHPBD@Z',
    4859:'?OnOpenDocument@COleDocument@@UAEHPBD@Z',
    4860:'?OnOpenDocument@COleLinkingDoc@@UAEHPBD@Z',
    4861:'?OnOpenEmbedding@COleServerDoc@@MAEXPAUIStorage@@@Z',
    4862:'?OnOpenRecentFile@CWinApp@@IAEHI@Z',
    4863:'?OnOutOfBandData@CAsyncSocket@@MAEXH@Z',
    4864:'?OnPaint@CControlBar@@QAEXXZ',
    4865:'?OnPaint@CCtrlView@@IAEXXZ',
    4866:'?OnPaint@CDockBar@@IAEXXZ',
    4867:'?OnPaint@CFontPropPage@@MAEXXZ',
    4868:'?OnPaint@COleControl@@IAEXPAVCDC@@@Z',
    4869:'?OnPaint@COleResizeBar@@IAEXXZ',
    4870:'?OnPaint@CPicturePropPage@@IAEXXZ',
    4871:'?OnPaint@CSplitterWnd@@IAEXXZ',
    4872:'?OnPaint@CStatusBar@@IAEXXZ',
    4873:'?OnPaint@CToolBar@@IAEXXZ',
    4874:'?OnPaint@CView@@IAEXXZ',
    4875:'?OnPaletteChanged@CFrameWnd@@IAEXPAVCWnd@@@Z',
    4876:'?OnPaletteChanged@COleFrameHook@@UAEXPAVCWnd@@@Z',
    4877:'?OnParaAlign@CRichEditView@@QAEXG@Z',
    4878:'?OnParaCenter@CRichEditView@@IAEXXZ',
    4879:'?OnParaLeft@CRichEditView@@IAEXXZ',
    4880:'?OnParaRight@CRichEditView@@IAEXXZ',
    4881:'?OnParentNotify@CWnd@@IAEXIJ@Z',
    4882:'?OnPasteNativeObject@CRichEditView@@MAEHPAUIStorage@@@Z',
    4883:'?OnPopMessageString@CFrameWnd@@IAEJIJ@Z',
    4884:'?OnPosRectChange@XOleIPSite@COleClientItem@@UAGJPBUtagRECT@@@Z',
    4885:'?OnPosRectChange@XOleIPSite@COleControlSite@@UAGJPBUtagRECT@@@Z',
    4886:'?OnPrepareDC@CEditView@@UAEXPAVCDC@@PAUCPrintInfo@@@Z',
    4887:'?OnPrepareDC@CPreviewView@@UAEXPAVCDC@@PAUCPrintInfo@@@Z',
    4888:'?OnPrepareDC@CRichEditView@@UAEXPAVCDC@@PAUCPrintInfo@@@Z',
    4889:'?OnPrepareDC@CScrollView@@UAEXPAVCDC@@PAUCPrintInfo@@@Z',
    4890:'?OnPrepareDC@CView@@UAEXPAVCDC@@PAUCPrintInfo@@@Z',
    4891:'?OnPreparePrinting@CEditView@@MAEHPAUCPrintInfo@@@Z',
    4892:'?OnPreparePrinting@CView@@MAEHPAUCPrintInfo@@@Z',
    4893:'?OnPreTranslateMessage@COleFrameHook@@UAEHPAUtagMSG@@@Z',
    4894:'?OnPreviewClose@CPreviewView@@IAEXXZ',
    4895:'?OnPreviewPrint@CPreviewView@@IAEXXZ',
    4896:'?OnPrevPage@CPreviewView@@IAEXXZ',
    4897:'?OnPrint@CEditView@@MAEXPAVCDC@@PAUCPrintInfo@@@Z',
    4898:'?OnPrint@CRichEditView@@MAEXPAVCDC@@PAUCPrintInfo@@@Z',
    4899:'?OnPrint@CView@@MAEXPAVCDC@@PAUCPrintInfo@@@Z',
    4900:'?OnPrinterChanged@CRichEditView@@MAEXABVCDC@@@Z',
    4901:'?OnPrintSetup@CPrintDialog@@IAEXXZ',
    4902:'?OnProgress@CAsyncMonikerFile@@MAEXKKKPBD@Z',
    4903:'?OnProperties@COleControl@@UAEHPAUtagMSG@@PAUHWND__@@PBUtagRECT@@@Z',
    4904:'?OnQuery3dControls@CWnd@@IAEJIJ@Z',
    4905:'?OnQueryCancel@CPropertyPage@@UAEHXZ',
    4906:'?OnQueryCenterWnd@CMiniFrameWnd@@QAEJIJ@Z',
    4907:'?OnQueryEndSession@CFrameWnd@@IAEHXZ',
    4908:'?OnQueryHitPoint@COleControl@@UAEHKPBUtagRECT@@UtagPOINT@@JPAK@Z',
    4909:'?OnQueryHitRect@COleControl@@UAEHKPBUtagRECT@@0JPAK@Z',
    4910:'?OnQueryNewPalette@CFrameWnd@@IAEHXZ',
    4911:'?OnQueryNewPalette@COleFrameHook@@UAEHXZ',
    4912:'?OnQueryUpdateItems@COleServerItem@@UAEHXZ',
    4913:'?OnRButtonDblClk@COleControl@@IAEXIVCPoint@@@Z',
    4914:'?OnRButtonDown@COleControl@@IAEXIVCPoint@@@Z',
    4915:'?OnRButtonUp@COleControl@@IAEXIVCPoint@@@Z',
    4916:'?OnReactivateAndUndo@COleServerDoc@@MAEHXZ',
    4917:'?OnRecalcLayout@COleFrameHook@@UAEXXZ',
    4918:'?OnRecalcParent@COleIPFrameWnd@@IAEJIJ@Z',
    4919:'?OnReceive@CAsyncSocket@@MAEXH@Z',
    4920:'?OnReflectorDestroyed@COleControl@@MAEXXZ',
    4921:'?OnRemoveMenus@COleClientItem@@UAEXPAVCMenu@@@Z',
    4922:'?OnRename@XAdviseSink@COleClientItem@@UAGXPAUIMoniker@@@Z',
    4923:'?OnRenderData@CControlDataSource@COleControl@@MAEHPAUtagFORMATETC@@PAUtagSTGMEDIUM@@@Z',
    4924:'?OnRenderData@CItemDataSource@COleServerItem@@MAEHPAUtagFORMATETC@@PAUtagSTGMEDIUM@@@Z',
    4925:'?OnRenderData@COleControl@@UAEHPAUtagFORMATETC@@PAUtagSTGMEDIUM@@@Z',
    4926:'?OnRenderData@COleDataSource@@UAEHPAUtagFORMATETC@@PAUtagSTGMEDIUM@@@Z',
    4927:'?OnRenderData@COleServerItem@@UAEHPAUtagFORMATETC@@PAUtagSTGMEDIUM@@@Z',
    4928:'?OnRenderFileData@CControlDataSource@COleControl@@MAEHPAUtagFORMATETC@@PAVCFile@@@Z',
    4929:'?OnRenderFileData@CItemDataSource@COleServerItem@@MAEHPAUtagFORMATETC@@PAVCFile@@@Z',
    4930:'?OnRenderFileData@COleControl@@UAEHPAUtagFORMATETC@@PAVCFile@@@Z',
    4931:'?OnRenderFileData@COleDataSource@@UAEHPAUtagFORMATETC@@PAVCFile@@@Z',
    4932:'?OnRenderFileData@COleServerItem@@UAEHPAUtagFORMATETC@@PAVCFile@@@Z',
    4933:'?OnRenderGlobalData@CControlDataSource@COleControl@@MAEHPAUtagFORMATETC@@PAPAX@Z',
    4934:'?OnRenderGlobalData@CItemDataSource@COleServerItem@@MAEHPAUtagFORMATETC@@PAPAX@Z',
    4935:'?OnRenderGlobalData@COleControl@@UAEHPAUtagFORMATETC@@PAPAX@Z',
    4936:'?OnRenderGlobalData@COleDataSource@@UAEHPAUtagFORMATETC@@PAPAX@Z',
    4937:'?OnRenderGlobalData@COleServerItem@@UAEHPAUtagFORMATETC@@PAPAX@Z',
    4938:'?OnReplaceAll@CEditView@@MAEXPBD0H@Z',
    4939:'?OnReplaceAll@CRichEditView@@MAEXPBD0HH@Z',
    4940:'?OnReplaceSel@CEditView@@MAEXPBDHH0@Z',
    4941:'?OnReplaceSel@CRichEditView@@MAEXPBDHHH0@Z',
    4942:'?OnRequestEdit@XFontNotification@COleControl@@UAGJJ@Z',
    4943:'?OnRequestEdit@XPropertyNotifySink@COleControlSite@@UAGJJ@Z',
    4944:'?OnRequestEdit@XPropNotifySink@COlePropertyPage@@UAGJJ@Z',
    4945:'?OnRequestPositionChange@COleDocIPFrameWnd@@MAEXPBUtagRECT@@@Z',
    4946:'?OnRequestPositionChange@COleIPFrameWnd@@MAEXPBUtagRECT@@@Z',
    4947:'?OnReset@CEnumArray@@MAEXXZ',
    4948:'?OnReset@CPropertyPage@@UAEXXZ',
    4949:'?OnResetState@COleControl@@UAEXXZ',
    4950:'?OnResizeBorder@COleServerDoc@@UAEXPBUtagRECT@@PAUIOleInPlaceUIWindow@@H@Z',
    4951:'?OnResizeChild@COleIPFrameWnd@@IAEJIJ@Z',
    4952:'?OnSave@XAdviseSink@COleClientItem@@UAGXXZ',
    4953:'?OnSaveDocument@CDocument@@UAEHPBD@Z',
    4954:'?OnSaveDocument@COleDocument@@UAEHPBD@Z',
    4955:'?OnSaveDocument@COleLinkingDoc@@UAEHPBD@Z',
    4956:'?OnSaveDocument@COleServerDoc@@UAEHPBD@Z',
    4957:'?OnSaveEmbedding@COleServerDoc@@MAEXPAUIStorage@@@Z',
    4958:'?OnSaveEmbedding@COleServerItem@@MAEXPAUIStorage@@@Z',
    4959:'?OnSaveViewState@CDocObjectServer@@MAEXAAVCArchive@@@Z',
    4960:'?OnScroll@CScrollView@@UAEHIIH@Z',
    4961:'?OnScroll@CView@@UAEHIIH@Z',
    4962:'?OnScrollBy@COleClientItem@@MAEHVCSize@@@Z',
    4963:'?OnScrollBy@CScrollView@@UAEHVCSize@@H@Z',
    4964:'?OnScrollBy@CView@@UAEHVCSize@@H@Z',
    4965:'?OnSelChange@CRichEditView@@IAEXPAUtagNMHDR@@PAJ@Z',
    4966:'?OnSelchangeColorprop@CColorPropPage@@AAEXXZ',
    4967:'?OnSelchangeFontnames@CFontPropPage@@IAEXXZ',
    4968:'?OnSelchangeFontprop@CFontPropPage@@IAEXXZ',
    4969:'?OnSelchangeFontsizes@CFontPropPage@@IAEXXZ',
    4970:'?OnSelchangeFontstyles@CFontPropPage@@IAEXXZ',
    4971:'?OnSelchangePictProp@CPicturePropPage@@IAEXXZ',
    4972:'?OnSelchangePropname@CStockPropPage@@IAEXAAVCComboBox@@@Z',
    4973:'?OnSelchangeSystemcolors@CColorPropPage@@AAEXXZ',
    4974:'?OnSelect@CColorPropPage@@AAEXXZ',
    4975:'?OnSend@CAsyncSocket@@MAEXH@Z',
    4976:'?OnSetActive@CPropertyPage@@UAEHXZ',
    4977:'?OnSetBitmapSize@CToolBar@@IAEJIJ@Z',
    4978:'?OnSetButtonSize@CToolBar@@IAEJIJ@Z',
    4979:'?OnSetClientSite@COleControl@@UAEXXZ',
    4980:'?OnSetColorScheme@COleServerItem@@UAEHPBUtagLOGPALETTE@@@Z',
    4981:'?OnSetCursor@CFrameWnd@@IAEHPAVCWnd@@II@Z',
    4982:'?OnSetCursor@COleControl@@IAEHPAVCWnd@@II@Z',
    4983:'?OnSetCursor@COleResizeBar@@IAEHPAVCWnd@@II@Z',
    4984:'?OnSetCursor@CPreviewView@@IAEHPAVCWnd@@II@Z',
    4985:'?OnSetCursor@CSplitterWnd@@IAEHPAVCWnd@@II@Z',
    4986:'?OnSetData@CControlDataSource@COleControl@@MAEHPAUtagFORMATETC@@PAUtagSTGMEDIUM@@H@Z',
    4987:'?OnSetData@CItemDataSource@COleServerItem@@MAEHPAUtagFORMATETC@@PAUtagSTGMEDIUM@@H@Z',
    4988:'?OnSetData@COleControl@@UAEHPAUtagFORMATETC@@PAUtagSTGMEDIUM@@H@Z',
    4989:'?OnSetData@COleDataSource@@UAEHPAUtagFORMATETC@@PAUtagSTGMEDIUM@@H@Z',
    4990:'?OnSetData@COleServerItem@@UAEHPAUtagFORMATETC@@PAUtagSTGMEDIUM@@H@Z',
    4991:'?OnSetDefID@CPropertySheet@@IAEJIJ@Z',
    4992:'?OnSetExtent@COleControl@@UAEHPAUtagSIZE@@@Z',
    4993:'?OnSetExtent@COleServerItem@@UAEHW4tagDVASPECT@@ABVCSize@@@Z',
    4994:'?OnSetFocus@CFormView@@IAEXPAVCWnd@@@Z',
    4995:'?OnSetFocus@CFrameWnd@@IAEXPAVCWnd@@@Z',
    4996:'?OnSetFocus@COleControl@@IAEXPAVCWnd@@@Z',
    4997:'?OnSetFont@CCheckListBox@@IAEJIJ@Z',
    4998:'?OnSetFont@CDialog@@UAEXPAVCFont@@@Z',
    4999:'?OnSetFont@CEditView@@IAEJIJ@Z',
    5000:'?OnSetHostNames@COleServerDoc@@MAEXPBD0@Z',
    5001:'?OnSetItemRects@CDocObjectServer@@IAEXPAUtagRECT@@0@Z',
    5002:'?OnSetItemRects@COleServerDoc@@MAEXPBUtagRECT@@0@Z',
    5003:'?OnSetMenu@COleClientItem@@UAEXPAVCMenu@@PAXPAUHWND__@@@Z',
    5004:'?OnSetMessageString@CFrameWnd@@IAEJIJ@Z',
    5005:'?OnSetMessageString@COleControl@@IAEJIJ@Z',
    5006:'?OnSetMessageString@COleIPFrameWnd@@UAEJIJ@Z',
    5007:'?OnSetMinHeight@CStatusBar@@IAEJIJ@Z',
    5008:'?OnSetObjectRects@COleControl@@UAEHPBUtagRECT@@0@Z',
    5009:'?OnSetOptions@CDatabase@@UAEXPAX@Z',
    5010:'?OnSetOptions@CRecordset@@UAEXPAX@Z',
    5011:'?OnSetPageSite@COlePropertyPage@@UAEXXZ',
    5012:'?OnSetPreviewMode@CFrameWnd@@UAEXHPAUCPrintPreviewState@@@Z',
    5013:'?OnSetText@CMiniFrameWnd@@QAEJIJ@Z',
    5014:'?OnSetText@COleControl@@IAEJIJ@Z',
    5015:'?OnSetText@CStatusBar@@IAEJIJ@Z',
    5016:'?OnShareViolation@CFileDialog@@MAEIPBD@Z',
    5017:'?OnShow@CDocObjectServerItem@@MAEXXZ',
    5018:'?OnShow@COleServerItem@@MAEXXZ',
    5019:'?OnShowControlBars@COleClientItem@@UAEHPAVCFrameWnd@@H@Z',
    5020:'?OnShowControlBars@COleServerDoc@@MAEXPAVCFrameWnd@@H@Z',
    5021:'?OnShowDocument@COleServerDoc@@MAEXH@Z',
    5022:'?OnShowItem@COleClientItem@@MAEXXZ',
    5023:'?OnShowToolBars@COleControl@@UAEXXZ',
    5024:'?OnShowViews@COleDocument@@UAEXH@Z',
    5025:'?OnShowViews@COleLinkingDoc@@MAEXH@Z',
    5026:'?OnShowWindow@COleControl@@IAEXHI@Z',
    5027:'?OnShowWindow@CWnd@@IAEXHI@Z',
    5028:'?OnShowWindow@XOleClientSite@COleClientItem@@UAGJH@Z',
    5029:'?OnShowWindow@XOleClientSite@COleControlSite@@UAGJH@Z',
    5030:'?OnSize@CFrameWnd@@IAEXIHH@Z',
    5031:'?OnSize@CMDIChildWnd@@IAEXIHH@Z',
    5032:'?OnSize@CMDIFrameWnd@@IAEXIHH@Z',
    5033:'?OnSize@COleControl@@IAEXIHH@Z',
    5034:'?OnSize@COleIPFrameWnd@@IAEXIHH@Z',
    5035:'?OnSize@COleResizeBar@@IAEXIHH@Z',
    5036:'?OnSize@CPreviewView@@IAEXIHH@Z',
    5037:'?OnSize@CScrollView@@QAEXIHH@Z',
    5038:'?OnSize@CSplitterWnd@@IAEXIHH@Z',
    5039:'?OnSize@CStatusBar@@IAEXIHH@Z',
    5040:'?OnSize@CWnd@@IAEXIHH@Z',
    5041:'?OnSizeParent@CControlBar@@QAEJIJ@Z',
    5042:'?OnSizeParent@CDockBar@@IAEJIJ@Z',
    5043:'?OnSizeParent@COleResizeBar@@IAEJIJ@Z',
    5044:'?OnSkip@CEnumArray@@MAEHXZ',
    5045:'?OnSocketDead@CSocketWnd@@IAEJIJ@Z',
    5046:'?OnSocketNotify@CSocketWnd@@IAEJIJ@Z',
    5047:'?OnSplitCmd@CView@@IAEHI@Z',
    5048:'?OnStartBinding@CAsyncMonikerFile@@MAEXXZ',
    5049:'?OnStatusCallback@CInternetSession@@UAEXKKPAXK@Z',
    5050:'?OnStopBinding@CAsyncMonikerFile@@MAEXJPBD@Z',
    5051:'?OnStrikeout@CFontPropPage@@IAEXXZ',
    5052:'?OnSysColorChange@CToolBar@@IAEXXZ',
    5053:'?OnSysColorChange@CWnd@@IAEXXZ',
    5054:'?OnSysCommand@CFrameWnd@@IAEXIJ@Z',
    5055:'?OnSysCommand@CMiniFrameWnd@@QAEXIJ@Z',
    5056:'?OnSysCommand@CPropertySheet@@IAEXIJ@Z',
    5057:'?OnSysCommand@CSplitterWnd@@IAEXIJ@Z',
    5058:'?OnSysKeyDown@COleControl@@IAEXIII@Z',
    5059:'?OnSysKeyUp@COleControl@@IAEXIII@Z',
    5060:'?OnTextChanged@COleControl@@UAEXXZ',
    5061:'?OnTextNotFound@CEditView@@MAEXPBD@Z',
    5062:'?OnTextNotFound@CRichEditView@@MAEXPBD@Z',
    5063:'?OnTimer@CControlBar@@QAEXI@Z',
    5064:'?OnToolHitTest@CToolBar@@UBEHVCPoint@@PAUtagTOOLINFOA@@@Z',
    5065:'?OnToolHitTest@CWnd@@UBEHVCPoint@@PAUtagTOOLINFOA@@@Z',
    5066:'?OnToolTipText@CFrameWnd@@IAEHIPAUtagNMHDR@@PAJ@Z',
    5067:'?OnTypeChange@CFileDialog@@MAEXXZ',
    5068:'?OnUIActivate@COleControlContainer@@UAEXPAVCOleControlSite@@@Z',
    5069:'?OnUIActivate@XOleIPSite@COleClientItem@@UAGJXZ',
    5070:'?OnUIActivate@XOleIPSite@COleControlSite@@UAGJXZ',
    5071:'?OnUIDeactivate@COleControlContainer@@UAEXPAVCOleControlSite@@@Z',
    5072:'?OnUIDeactivate@XOleIPSite@COleClientItem@@UAGJH@Z',
    5073:'?OnUIDeactivate@XOleIPSite@COleControlSite@@UAGJH@Z',
    5074:'?OnUnderline@CFontPropPage@@IAEXXZ',
    5075:'?OnUpdate@COleServerItem@@UAEXPAV1@JPAVCObject@@W4tagDVASPECT@@@Z',
    5076:'?OnUpdate@CView@@MAEXPAV1@JPAVCObject@@@Z',
    5077:'?OnUpdateBullet@CRichEditView@@IAEXPAVCCmdUI@@@Z',
    5078:'?OnUpdateCharBold@CRichEditView@@IAEXPAVCCmdUI@@@Z',
    5079:'?OnUpdateCharEffect@CRichEditView@@QAEXPAVCCmdUI@@KK@Z',
    5080:'?OnUpdateCharItalic@CRichEditView@@IAEXPAVCCmdUI@@@Z',
    5081:'?OnUpdateCharUnderline@CRichEditView@@IAEXPAVCCmdUI@@@Z',
    5082:'?OnUpdateCmdUI@CDialogBar@@UAEXPAVCFrameWnd@@H@Z',
    5083:'?OnUpdateCmdUI@CDockBar@@MAEXPAVCFrameWnd@@H@Z',
    5084:'?OnUpdateCmdUI@COleResizeBar@@UAEXPAVCFrameWnd@@H@Z',
    5085:'?OnUpdateCmdUI@CStatusBar@@UAEXPAVCFrameWnd@@H@Z',
    5086:'?OnUpdateCmdUI@CToolBar@@UAEXPAVCFrameWnd@@H@Z',
    5087:'?OnUpdateContextHelp@CFrameWnd@@IAEXPAVCCmdUI@@@Z',
    5088:'?OnUpdateControlBarMenu@CFrameWnd@@QAEXPAVCCmdUI@@@Z',
    5089:'?OnUpdateControlBarMenu@COleIPFrameWnd@@IAEXPAVCCmdUI@@@Z',
    5090:'?OnUpdateDocument@COleServerDoc@@UAEHXZ',
    5091:'?OnUpdateEditChangeIcon@COleDocument@@IAEXPAVCCmdUI@@@Z',
    5092:'?OnUpdateEditLinksMenu@COleDocument@@IAEXPAVCCmdUI@@@Z',
    5093:'?OnUpdateEditPasteSpecial@CRichEditView@@IAEXPAVCCmdUI@@@Z',
    5094:'?OnUpdateEditProperties@CRichEditView@@IAEXPAVCCmdUI@@@Z',
    5095:'?OnUpdateEditUndo@CEditView@@IAEXPAVCCmdUI@@@Z',
    5096:'?OnUpdateEditUndo@CRichEditView@@IAEXPAVCCmdUI@@@Z',
    5097:'?OnUpdateFileExit@COleServerDoc@@IAEXPAVCCmdUI@@@Z',
    5098:'?OnUpdateFileSendMail@CDocument@@IAEXPAVCCmdUI@@@Z',
    5099:'?OnUpdateFileUpdate@COleServerDoc@@IAEXPAVCCmdUI@@@Z',
    5100:'?OnUpdateFrameMenu@CFrameWnd@@UAEXPAUHMENU__@@@Z',
    5101:'?OnUpdateFrameMenu@CMDIChildWnd@@UAEXHPAVCWnd@@PAUHMENU__@@@Z',
    5102:'?OnUpdateFrameMenu@CMDIFrameWnd@@UAEXPAUHMENU__@@@Z',
    5103:'?OnUpdateFrameTitle@CFrameWnd@@UAEXH@Z',
    5104:'?OnUpdateFrameTitle@CMDIChildWnd@@MAEXH@Z',
    5105:'?OnUpdateFrameTitle@CMDIFrameWnd@@UAEXH@Z',
    5106:'?OnUpdateFrameTitle@COleClientItem@@UAEHXZ',
    5107:'?OnUpdateFrameTitle@COleFrameHook@@UAEHXZ',
    5108:'?OnUpdateItems@COleServerItem@@UAEXXZ',
    5109:'?OnUpdateKeyIndicator@CFrameWnd@@IAEXPAVCCmdUI@@@Z',
    5110:'?OnUpdateMDIWindowCmd@CMDIFrameWnd@@IAEXPAVCCmdUI@@@Z',
    5111:'?OnUpdateNeedClip@CEditView@@IAEXPAVCCmdUI@@@Z',
    5112:'?OnUpdateNeedClip@CRichEditView@@IAEXPAVCCmdUI@@@Z',
    5113:'?OnUpdateNeedFind@CEditView@@IAEXPAVCCmdUI@@@Z',
    5114:'?OnUpdateNeedFind@CRichEditView@@IAEXPAVCCmdUI@@@Z',
    5115:'?OnUpdateNeedSel@CEditView@@IAEXPAVCCmdUI@@@Z',
    5116:'?OnUpdateNeedSel@CRichEditView@@IAEXPAVCCmdUI@@@Z',
    5117:'?OnUpdateNeedText@CEditView@@IAEXPAVCCmdUI@@@Z',
    5118:'?OnUpdateNeedText@CRichEditView@@IAEXPAVCCmdUI@@@Z',
    5119:'?OnUpdateNextPage@CPreviewView@@IAEXPAVCCmdUI@@@Z',
    5120:'?OnUpdateNextPaneMenu@CView@@IAEXPAVCCmdUI@@@Z',
    5121:'?OnUpdateNumPageChange@CPreviewView@@IAEXPAVCCmdUI@@@Z',
    5122:'?OnUpdateObjectVerbMenu@COleDocument@@IAEXPAVCCmdUI@@@Z',
    5123:'?OnUpdateParaAlign@CRichEditView@@QAEXPAVCCmdUI@@G@Z',
    5124:'?OnUpdateParaCenter@CRichEditView@@IAEXPAVCCmdUI@@@Z',
    5125:'?OnUpdateParaLeft@CRichEditView@@IAEXPAVCCmdUI@@@Z',
    5126:'?OnUpdateParaRight@CRichEditView@@IAEXPAVCCmdUI@@@Z',
    5127:'?OnUpdatePasteLinkMenu@COleDocument@@IAEXPAVCCmdUI@@@Z',
    5128:'?OnUpdatePasteMenu@COleDocument@@IAEXPAVCCmdUI@@@Z',
    5129:'?OnUpdatePrevPage@CPreviewView@@IAEXPAVCCmdUI@@@Z',
    5130:'?OnUpdateRecentFileMenu@CWinApp@@IAEXPAVCCmdUI@@@Z',
    5131:'?OnUpdateRecordFirst@CDaoRecordView@@IAEXPAVCCmdUI@@@Z',
    5132:'?OnUpdateRecordFirst@CRecordView@@IAEXPAVCCmdUI@@@Z',
    5133:'?OnUpdateRecordLast@CDaoRecordView@@IAEXPAVCCmdUI@@@Z',
    5134:'?OnUpdateRecordLast@CRecordView@@IAEXPAVCCmdUI@@@Z',
    5135:'?OnUpdateRecordNext@CDaoRecordView@@IAEXPAVCCmdUI@@@Z',
    5136:'?OnUpdateRecordNext@CRecordView@@IAEXPAVCCmdUI@@@Z',
    5137:'?OnUpdateRecordPrev@CDaoRecordView@@IAEXPAVCCmdUI@@@Z',
    5138:'?OnUpdateRecordPrev@CRecordView@@IAEXPAVCCmdUI@@@Z',
    5139:'?OnUpdateSplitCmd@CView@@IAEXPAVCCmdUI@@@Z',
    5140:'?OnUpdateZoomIn@CPreviewView@@IAEXPAVCCmdUI@@@Z',
    5141:'?OnUpdateZoomOut@CPreviewView@@IAEXPAVCCmdUI@@@Z',
    5142:'?OnViewChange@XAdviseSink@COleClientItem@@UAGXKJ@Z',
    5143:'?OnVKeyToItem@CWnd@@IAEHIPAVCListBox@@I@Z',
    5144:'?OnVScroll@CFrameWnd@@IAEXIIPAVCScrollBar@@@Z',
    5145:'?OnVScroll@CPreviewView@@IAEXIIPAVCScrollBar@@@Z',
    5146:'?OnVScroll@CScrollView@@QAEXIIPAVCScrollBar@@@Z',
    5147:'?OnVScroll@CSplitterWnd@@IAEXIIPAVCScrollBar@@@Z',
    5148:'?OnVScroll@CWnd@@IAEXIIPAVCScrollBar@@@Z',
    5149:'?OnWindowFromPoint@CToolTipCtrl@@IAEJIJ@Z',
    5150:'?OnWindowlessMessage@COleControl@@UAEHIIJPAJ@Z',
    5151:'?OnWindowMessage@XOleInPlaceObject@COleControl@@UAGJIIJPAJ@Z',
    5152:'?OnWindowNew@CMDIFrameWnd@@IAEXXZ',
    5153:'?OnWindowPosChanging@CControlBar@@QAEXPAUtagWINDOWPOS@@@Z',
    5154:'?OnWindowPosChanging@CDockBar@@IAEXPAUtagWINDOWPOS@@@Z',
    5155:'?OnWindowPosChanging@CMDIChildWnd@@IAEXPAUtagWINDOWPOS@@@Z',
    5156:'?OnWindowPosChanging@COleIPFrameWnd@@IAEXPAUtagWINDOWPOS@@@Z',
    5157:'?OnWindowPosChanging@CStatusBar@@IAEXPAUtagWINDOWPOS@@@Z',
    5158:'?OnWindowPosChanging@CToolBar@@IAEXPAUtagWINDOWPOS@@@Z',
    5159:'?OnWinIniChange@CWnd@@IAEXPBD@Z',
    5160:'?OnWizardBack@CPropertyPage@@UAEJXZ',
    5161:'?OnWizardFinish@CPropertyPage@@UAEHXZ',
    5162:'?OnWizardNext@CPropertyPage@@UAEJXZ',
    5163:'?OnWndMsg@CWnd@@MAEHIIJPAJ@Z',
    5164:'?OnZoomIn@CPreviewView@@IAEXXZ',
    5165:'?OnZoomOut@CPreviewView@@IAEXXZ',
    5166:'?Open@CAsyncMonikerFile@@UAEHPAUIMoniker@@PAUIBindHost@@PAVCFileException@@@Z',
    5167:'?Open@CAsyncMonikerFile@@UAEHPAUIMoniker@@PAUIServiceProvider@@PAVCFileException@@@Z',
    5168:'?Open@CAsyncMonikerFile@@UAEHPAUIMoniker@@PAUIUnknown@@PAVCFileException@@@Z',
    5169:'?Open@CAsyncMonikerFile@@UAEHPAUIMoniker@@PAVCFileException@@@Z',
    5170:'?Open@CAsyncMonikerFile@@UAEHPBDPAUIBindHost@@PAVCFileException@@@Z',
    5171:'?Open@CAsyncMonikerFile@@UAEHPBDPAUIServiceProvider@@PAVCFileException@@@Z',
    5172:'?Open@CAsyncMonikerFile@@UAEHPBDPAUIUnknown@@PAVCFileException@@@Z',
    5173:'?Open@CAsyncMonikerFile@@UAEHPBDPAVCFileException@@@Z',
    5174:'?Open@CDaoDatabase@@UAEXPBDHH0@Z',
    5175:'?Open@CDaoQueryDef@@UAEXPBD@Z',
    5176:'?Open@CDaoRecordset@@UAEXHPBDH@Z',
    5177:'?Open@CDaoRecordset@@UAEXPAVCDaoQueryDef@@HH@Z',
    5178:'?Open@CDaoRecordset@@UAEXPAVCDaoTableDef@@HH@Z',
    5179:'?Open@CDaoTableDef@@UAEXPBD@Z',
    5180:'?Open@CDaoWorkspace@@UAEXPBD@Z',
    5181:'?Open@CDatabase@@UAEHPBDHH0H@Z',
    5182:'?Open@CDataPathProperty@@UAEHPAVCFileException@@@Z',
    5183:'?Open@CDataPathProperty@@UAEHPAVCOleControl@@PAVCFileException@@@Z',
    5184:'?Open@CDataPathProperty@@UAEHPBDPAVCFileException@@@Z',
    5185:'?Open@CDataPathProperty@@UAEHPBDPAVCOleControl@@PAVCFileException@@@Z',
    5186:'?Open@CFile@@UAEHPBDIPAVCFileException@@@Z',
    5187:'?Open@CMirrorFile@@UAEHPBDIPAVCFileException@@@Z',
    5188:'?Open@CMonikerFile@@MAEHPAUIMoniker@@PAUIBindHost@@PAUIBindStatusCallback@@PAUIBindCtx@@PAVCFileException@@@Z',
    5189:'?Open@CMonikerFile@@MAEHPBDPAUIBindHost@@PAUIBindStatusCallback@@PAUIBindCtx@@PAVCFileException@@@Z',
    5190:'?Open@CMonikerFile@@UAEHPAUIMoniker@@PAVCFileException@@@Z',
    5191:'?Open@CMonikerFile@@UAEHPBDPAVCFileException@@@Z',
    5192:'?Open@CRecordset@@UAEHIPBDK@Z',
    5193:'?Open@CSocketFile@@UAEHPBDIPAVCFileException@@@Z',
    5194:'?Open@CStdioFile@@UAEHPBDIPAVCFileException@@@Z',
    5195:'?Open@XOleDocumentView@CDocObjectServer@@UAGJXZ',
    5196:'?OpenDocumentFile@CDocManager@@UAEPAVCDocument@@PBD@Z',
    5197:'?OpenDocumentFile@CMultiDocTemplate@@UAEPAVCDocument@@PBDH@Z',
    5198:'?OpenDocumentFile@CSingleDocTemplate@@UAEPAVCDocument@@PBDH@Z',
    5199:'?OpenDocumentFile@CWinApp@@UAEPAVCDocument@@PBD@Z',
    5200:'?OpenEx@CDatabase@@UAEHPBDK@Z',
    5201:'?OpenFile@CFtpConnection@@QAEPAVCInternetFile@@PBDKKK@Z',
    5202:'?OpenFile@CGopherConnection@@QAEPAVCGopherFile@@AAVCGopherLocator@@KPBDK@Z',
    5203:'?OpenLinkSource@COleUILinkInfo@@UAGJK@Z',
    5204:'?OpenRequest@CHttpConnection@@QAEPAVCHttpFile@@HPBD0KPAPBD0K@Z',
    5205:'?OpenRequest@CHttpConnection@@QAEPAVCHttpFile@@PBD00KPAPBD0K@Z',
    5206:'?OpenStream@COleStreamFile@@QAEHPAUIStorage@@PBDKPAVCFileException@@@Z',
    5207:'?OpenURL@CInternetSession@@QAEPAVCStdioFile@@PBDKK0K@Z',
    5208:'?OsErrorToException@CFileException@@SGHJ@Z',
    5209:'?OutputString@CDumpContext@@IAEXPBD@Z',
    5210:'?PaginateTo@CEditView@@IAEHPAVCDC@@PAUCPrintInfo@@@Z',
    5211:'?PaginateTo@CRichEditView@@IAEHPAVCDC@@PAUCPrintInfo@@@Z',
    5212:'?PaintHookProc@CPageSetupDialog@@KGIPAUHWND__@@IIJ@Z',
    5213:'?ParentToClient@COleControl@@UBEIPBUtagRECT@@PAUtagPOINT@@H@Z',
    5214:'?ParseCommandLine@CWinApp@@QAEXAAVCCommandLineInfo@@@Z',
    5215:'?ParseCurrency@COleCurrency@@QAEHPBDKK@Z',
    5216:'?ParseDateTime@COleDateTime@@QAEHPBDKK@Z',
    5217:'?ParseDisplayName@XOleContainer@COleControlContainer@@UAGJPAUIBindCtx@@PAGPAKPAPAUIMoniker@@@Z',
    5218:'?ParseDisplayName@XOleItemContainer@COleLinkingDoc@@UAGJPAUIBindCtx@@PAGPAKPAPAUIMoniker@@@Z',
    5219:'?ParseLast@CCommandLineInfo@@IAEXH@Z',
    5220:'?ParseParam@CCommandLineInfo@@UAEXPBDHH@Z',
    5221:'?ParseParamFlag@CCommandLineInfo@@IAEXPBD@Z',
    5222:'?ParseParamNotFlag@CCommandLineInfo@@IAEXPBD@Z',
    5223:'?PasteSpecial@CRichEditCtrl@@QAEXIKPAUHMETAFILE__@@@Z',
    5224:'?PlayMetaFile@CDC@@QAEHPAUHMETAFILE__@@@Z',
    5225:'?PolyBezierTo@CDC@@QAEHPBUtagPOINT@@H@Z',
    5226:'?PolyDraw@CDC@@QAEHPBUtagPOINT@@PBEH@Z',
    5227:'?PolylineTo@CDC@@QAEHPBUtagPOINT@@H@Z',
    5228:'?PositionPage@CPreviewView@@MAEXI@Z',
    5229:'?PostBindToStream@CAsyncMonikerFile@@MAEHPAVCFileException@@@Z',
    5230:'?PostBindToStream@CMonikerFile@@MAEHPAVCFileException@@@Z',
    5231:'?PostCreateDialog@COccManager@@UAEXPAU_AFX_OCC_DIALOG_INFO@@@Z',
    5232:'?PostModal@CDialog@@IAEXXZ',
    5233:'?PostModalDialog@COleControl@@QAEXPAUHWND__@@@Z',
    5234:'?PostNcDestroy@CControlBar@@UAEXXZ',
    5235:'?PostNcDestroy@CControlFrameWnd@@MAEXXZ',
    5236:'?PostNcDestroy@CFindReplaceDialog@@MAEXXZ',
    5237:'?PostNcDestroy@CFrameWnd@@MAEXXZ',
    5238:'?PostNcDestroy@COleCntrFrameWnd@@MAEXXZ',
    5239:'?PostNcDestroy@CReflectorWnd@@MAEXXZ',
    5240:'?PostNcDestroy@CView@@MAEXXZ',
    5241:'?PostNcDestroy@CWnd@@MAEXXZ',
    5242:'?PreBindFields@CRecordset@@UAEXXZ',
    5243:'?PreCloseFrame@CDocument@@UAEXPAVCFrameWnd@@@Z',
    5244:'?PreCloseFrame@COleDocument@@UAEXPAVCFrameWnd@@@Z',
    5245:'?PreCloseFrame@CRichEditDoc@@UAEXPAVCFrameWnd@@@Z',
    5246:'?PreCompareItem@CCheckListBox@@IAEHPAUtagCOMPAREITEMSTRUCT@@@Z',
    5247:'?PreCreateDialog@COccManager@@UAEPBUDLGTEMPLATE@@PAU_AFX_OCC_DIALOG_INFO@@PBU2@@Z',
    5248:'?PreCreateWindow@CControlBar@@UAEHAAUtagCREATESTRUCTA@@@Z',
    5249:'?PreCreateWindow@CControlFrameWnd@@UAEHAAUtagCREATESTRUCTA@@@Z',
    5250:'?PreCreateWindow@CCtrlView@@MAEHAAUtagCREATESTRUCTA@@@Z',
    5251:'?PreCreateWindow@CEditView@@MAEHAAUtagCREATESTRUCTA@@@Z',
    5252:'?PreCreateWindow@CFrameWnd@@MAEHAAUtagCREATESTRUCTA@@@Z',
    5253:'?PreCreateWindow@CListView@@UAEHAAUtagCREATESTRUCTA@@@Z',
    5254:'?PreCreateWindow@CMDIChildWnd@@UAEHAAUtagCREATESTRUCTA@@@Z',
    5255:'?PreCreateWindow@CMDIFrameWnd@@UAEHAAUtagCREATESTRUCTA@@@Z',
    5256:'?PreCreateWindow@CMiniFrameWnd@@MAEHAAUtagCREATESTRUCTA@@@Z',
    5257:'?PreCreateWindow@CRichEditView@@MAEHAAUtagCREATESTRUCTA@@@Z',
    5258:'?PreCreateWindow@CStatusBar@@UAEHAAUtagCREATESTRUCTA@@@Z',
    5259:'?PreCreateWindow@CTreeView@@UAEHAAUtagCREATESTRUCTA@@@Z',
    5260:'?PreCreateWindow@CView@@MAEHAAUtagCREATESTRUCTA@@@Z',
    5261:'?PreCreateWindow@CWnd@@UAEHAAUtagCREATESTRUCTA@@@Z',
    5262:'?PreDeleteItem@CCheckListBox@@IAEXPAUtagDELETEITEMSTRUCT@@@Z',
    5263:'?PreDrawItem@CCheckListBox@@IAEXPAUtagDRAWITEMSTRUCT@@@Z',
    5264:'?PreDrawPage@CPageSetupDialog@@UAEIGGPAUtagPSDA@@@Z',
    5265:'?PreInitDialog@CDialog@@MAEXXZ',
    5266:'?PreInitDialog@COleChangeSourceDialog@@UAEXXZ',
    5267:'?PreMeasureItem@CCheckListBox@@IAEXPAUtagMEASUREITEMSTRUCT@@@Z',
    5268:'?PreModal@CDialog@@IAEPAUHWND__@@XZ',
    5269:'?PreModalDialog@COleControl@@QAEXPAUHWND__@@@Z',
    5270:'?PrepareAndExecute@CRecordset@@IAEXXZ',
    5271:'?PrepareCtrl@CDataExchange@@QAEPAUHWND__@@H@Z',
    5272:'?PrepareEditCtrl@CDataExchange@@QAEPAUHWND__@@H@Z',
    5273:'?PrepareOleCtrl@CDataExchange@@QAEPAVCWnd@@H@Z',
    5274:'?PrepareUpdateHstmt@CRecordset@@IAEXXZ',
    5275:'?PreProcessPageTemplate@CPropertyPage@@IAEXAAU_PROPSHEETPAGEA@@H@Z',
    5276:'?PreSubclassWindow@CDragListBox@@UAEXXZ',
    5277:'?PreSubclassWindow@CWnd@@UAEXXZ',
    5278:'?PreTranslateInput@CWnd@@QAEHPAUtagMSG@@@Z',
    5279:'?PreTranslateMessage@CControlBar@@UAEHPAUtagMSG@@@Z',
    5280:'?PreTranslateMessage@CDialog@@UAEHPAUtagMSG@@@Z',
    5281:'?PreTranslateMessage@CFormView@@MAEHPAUtagMSG@@@Z',
    5282:'?PreTranslateMessage@CFrameWnd@@UAEHPAUtagMSG@@@Z',
    5283:'?PreTranslateMessage@CMDIChildWnd@@UAEHPAUtagMSG@@@Z',
    5284:'?PreTranslateMessage@CMDIFrameWnd@@UAEHPAUtagMSG@@@Z',
    5285:'?PreTranslateMessage@COleIPFrameWnd@@UAEHPAUtagMSG@@@Z',
    5286:'?PreTranslateMessage@COlePropertyPage@@MAEHPAUtagMSG@@@Z',
    5287:'?PreTranslateMessage@CPropertyPage@@MAEHPAUtagMSG@@@Z',
    5288:'?PreTranslateMessage@CPropertySheet@@UAEHPAUtagMSG@@@Z',
    5289:'?PreTranslateMessage@CWinThread@@UAEHPAUtagMSG@@@Z',
    5290:'?PreTranslateMessage@CWnd@@UAEHPAUtagMSG@@@Z',
    5291:'?Print@XPrint@CDocObjectServer@@UAGJKPAPAUtagDVTARGETDEVICE@@PAPAUtagPAGESET@@PAUtagSTGMEDIUM@@PAUIContinueCallback@@JPAJ4@Z',
    5292:'?PrinterDPtoScreenDP@CPreviewDC@@QBEXPAUtagPOINT@@@Z',
    5293:'?PrintInsideRect@CEditView@@QAEIPAVCDC@@AAUtagRECT@@II@Z',
    5294:'?PrintInsideRect@CRichEditView@@QAEJPAVCDC@@AAUtagRECT@@JJH@Z',
    5295:'?PrintPage@CRichEditView@@QAEJPAVCDC@@JJ@Z',
    5296:'?Process@COleDispatchException@@SGXPAUtagEXCEPINFO@@PBVCException@@@Z',
    5297:'?Process@COleException@@SGJPBVCException@@@Z',
    5298:'?ProcessAuxQueue@CSocket@@SGHXZ',
    5299:'?ProcessHelpMsg@CFrameWnd@@IAEHAAUtagMSG@@PAK@Z',
    5300:'?ProcessMessageFilter@CWinThread@@UAEHHPAUtagMSG@@@Z',
    5301:'?ProcessShellCommand@CWinApp@@QAEHAAVCCommandLineInfo@@@Z',
    5302:'?ProcessWndProcException@CWinApp@@UAEJPAVCException@@PBUtagMSG@@@Z',
    5303:'?ProcessWndProcException@CWinThread@@UAEJPAVCException@@PBUtagMSG@@@Z',
    5304:'?PtrOfIndex@COleSafeArray@@QAEXPAJPAPAX@Z',
    5305:'?PtVisible@CDC@@UBEHHH@Z',
    5306:'?PtVisible@CMetaFileDC@@UBEHHH@Z',
    5307:'?PumpMessage@CWinThread@@UAEHXZ',
    5308:'?PumpMessages@CSocket@@MAEHI@Z',
    5309:'?PushStackArgs@CCmdTarget@@IAEJPAEPBEPAXGPAUtagDISPPARAMS@@PAIPAUtagVARIANT@@@Z',
    5310:'?PutElement@COleSafeArray@@QAEXPAJPAX@Z',
    5311:'?PutFile@CFtpConnection@@QAEHPBD0KK@Z',
    5312:'?PX_Blob@@YAHPAVCPropExchange@@PBDAAPAXPAX@Z',
    5313:'?PX_Bool@@YAHPAVCPropExchange@@PBDAAH@Z',
    5314:'?PX_Bool@@YAHPAVCPropExchange@@PBDAAHH@Z',
    5315:'?PX_Color@@YAHPAVCPropExchange@@PBDAAK@Z',
    5316:'?PX_Color@@YAHPAVCPropExchange@@PBDAAKK@Z',
    5317:'?PX_Currency@@YAHPAVCPropExchange@@PBDAATtagCY@@@Z',
    5318:'?PX_Currency@@YAHPAVCPropExchange@@PBDAATtagCY@@T2@@Z',
    5319:'?PX_DataPath@@YAHPAVCPropExchange@@PBDAAVCDataPathProperty@@1@Z',
    5320:'?PX_DataPath@@YAHPAVCPropExchange@@PBDAAVCDataPathProperty@@ABVCString@@@Z',
    5321:'?PX_Double@@YAHPAVCPropExchange@@PBDAAN@Z',
    5322:'?PX_Double@@YAHPAVCPropExchange@@PBDAANN@Z',
    5323:'?PX_Float@@YAHPAVCPropExchange@@PBDAAM@Z',
    5324:'?PX_Float@@YAHPAVCPropExchange@@PBDAAMM@Z',
    5325:'?PX_Font@@YAHPAVCPropExchange@@PBDAAVCFontHolder@@PBUtagFONTDESC@@PAUIFontDisp@@@Z',
    5326:'?PX_IUnknown@@YAHPAVCPropExchange@@PBDAAPAUIUnknown@@ABU_GUID@@PAU2@@Z',
    5327:'?PX_Long@@YAHPAVCPropExchange@@PBDAAJ@Z',
    5328:'?PX_Long@@YAHPAVCPropExchange@@PBDAAJJ@Z',
    5329:'?PX_Picture@@YAHPAVCPropExchange@@PBDAAVCPictureHolder@@@Z',
    5330:'?PX_Picture@@YAHPAVCPropExchange@@PBDAAVCPictureHolder@@2@Z',
    5331:'?PX_Short@@YAHPAVCPropExchange@@PBDAAF@Z',
    5332:'?PX_Short@@YAHPAVCPropExchange@@PBDAAFF@Z',
    5333:'?PX_String@@YAHPAVCPropExchange@@PBDAAVCString@@@Z',
    5334:'?PX_String@@YAHPAVCPropExchange@@PBDAAVCString@@1@Z',
    5335:'?PX_String@@YAHPAVCPropExchange@@PBDAAVCString@@ABV2@@Z',
    5336:'?PX_ULong@@YAHPAVCPropExchange@@PBDAAK@Z',
    5337:'?PX_ULong@@YAHPAVCPropExchange@@PBDAAKK@Z',
    5338:'?PX_UShort@@YAHPAVCPropExchange@@PBDAAG@Z',
    5339:'?PX_UShort@@YAHPAVCPropExchange@@PBDAAGG@Z',
    5340:'?PX_VBXFontConvert@@YAHPAVCPropExchange@@AAVCFontHolder@@@Z',
    5341:'?QueryAcceptData@CRichEditView@@MAEJPAUIDataObject@@PAGKHPAX@Z',
    5342:'?QueryAcceptData@XRichEditOleCallback@CRichEditView@@UAGJPAUIDataObject@@PAGKHPAX@Z',
    5343:'?QueryAggregates@CCmdTarget@@QAEPAUIUnknown@@PBX@Z',
    5344:'?QueryContinueDrag@COleDropSource@@UAEJHK@Z',
    5345:'?QueryContinueDrag@XDropSource@COleDropSource@@UAGJHK@Z',
    5346:'?QueryDefHandler@COleControl@@IAEPAXABU_GUID@@@Z',
    5347:'?QueryGetData@XDataObject@COleControl@@UAGJPAUtagFORMATETC@@@Z',
    5348:'?QueryGetData@XDataObject@COleDataSource@@UAGJPAUtagFORMATETC@@@Z',
    5349:'?QueryGetData@XDataObject@COleServerDoc@@UAGJPAUtagFORMATETC@@@Z',
    5350:'?QueryGetData@XDataObject@COleServerItem@@UAGJPAUtagFORMATETC@@@Z',
    5351:'?QueryHitPoint@XViewObject@COleControl@@UAGJKPBUtagRECT@@UtagPOINT@@JPAK@Z',
    5352:'?QueryHitRect@XViewObject@COleControl@@UAGJKPBUtagRECT@@0JPAK@Z',
    5353:'?QueryInfo@CHttpFile@@QBEHKAAVCString@@PAK@Z',
    5354:'?QueryInfo@CHttpFile@@QBEHKPAU_SYSTEMTIME@@PAK@Z',
    5355:'?QueryInfo@CHttpFile@@QBEHKPAXPAK1@Z',
    5356:'?QueryInfoStatusCode@CHttpFile@@QBEHAAK@Z',
    5357:'?QueryInsertObject@XRichEditOleCallback@CRichEditView@@UAGJPAU_GUID@@PAUIStorage@@J@Z',
    5358:'?QueryInterface@?$_CIP@UIBindHost@@$1?IID_IBindHost@@3U_GUID@@B@@QAEJPAUIUnknown@@@Z',
    5359:'?QueryInterface@CArchiveStream@@UAGJABU_GUID@@PAPAX@Z',
    5360:'?QueryInterface@CBlobProperty@@UAGJABU_GUID@@PAPAX@Z',
    5361:'?QueryInterface@CInnerUnknown@@UAGJABU_GUID@@PAPAX@Z',
    5362:'?QueryInterface@COleConnPtContainer@@UAGJABU_GUID@@PAPAX@Z',
    5363:'?QueryInterface@COleDispatchImpl@@UAGJABU_GUID@@PAPAX@Z',
    5364:'?QueryInterface@COleUILinkInfo@@UAGJABU_GUID@@PAPAX@Z',
    5365:'?QueryInterface@XAdviseSink@COleClientItem@@UAGJABU_GUID@@PAPAX@Z',
    5366:'?QueryInterface@XAmbientProps@COleControlSite@@UAGJABU_GUID@@PAPAX@Z',
    5367:'?QueryInterface@XBoundObjectSite@COleControlSite@@UAGJABU_GUID@@PAPAX@Z',
    5368:'?QueryInterface@XClassFactory@COleObjectFactory@@UAGJABU_GUID@@PAPAX@Z',
    5369:'?QueryInterface@XConnPt@CConnectionPoint@@UAGJABU_GUID@@PAPAX@Z',
    5370:'?QueryInterface@XDataObject@COleControl@@UAGJABU_GUID@@PAPAX@Z',
    5371:'?QueryInterface@XDataObject@COleDataSource@@UAGJABU_GUID@@PAPAX@Z',
    5372:'?QueryInterface@XDataObject@COleServerDoc@@UAGJABU_GUID@@PAPAX@Z',
    5373:'?QueryInterface@XDataObject@COleServerItem@@UAGJABU_GUID@@PAPAX@Z',
    5374:'?QueryInterface@XDropSource@COleDropSource@@UAGJABU_GUID@@PAPAX@Z',
    5375:'?QueryInterface@XDropTarget@COleDropTarget@@UAGJABU_GUID@@PAPAX@Z',
    5376:'?QueryInterface@XEnumVOID@CEnumArray@@UAGJABU_GUID@@PAPAX@Z',
    5377:'?QueryInterface@XEventSink@COleControlSite@@UAGJABU_GUID@@PAPAX@Z',
    5378:'?QueryInterface@XFontNotification@COleControl@@UAGJABU_GUID@@PAPAX@Z',
    5379:'?QueryInterface@XMessageFilter@COleMessageFilter@@UAGJABU_GUID@@PAPAX@Z',
    5380:'?QueryInterface@XNotifyDBEvents@COleControlSite@@UAGJABU_GUID@@PAPAX@Z',
    5381:'?QueryInterface@XOleCache@COleControl@@UAGJABU_GUID@@PAPAX@Z',
    5382:'?QueryInterface@XOleClientSite@COleClientItem@@UAGJABU_GUID@@PAPAX@Z',
    5383:'?QueryInterface@XOleClientSite@COleControlSite@@UAGJABU_GUID@@PAPAX@Z',
    5384:'?QueryInterface@XOleCommandTarget@CDocObjectServer@@UAGJABU_GUID@@PAPAX@Z',
    5385:'?QueryInterface@XOleContainer@COleControlContainer@@UAGJABU_GUID@@PAPAX@Z',
    5386:'?QueryInterface@XOleControl@COleControl@@UAGJABU_GUID@@PAPAX@Z',
    5387:'?QueryInterface@XOleControlSite@COleControlSite@@UAGJABU_GUID@@PAPAX@Z',
    5388:'?QueryInterface@XOleDocument@CDocObjectServer@@UAGJABU_GUID@@PAPAX@Z',
    5389:'?QueryInterface@XOleDocumentView@CDocObjectServer@@UAGJABU_GUID@@PAPAX@Z',
    5390:'?QueryInterface@XOleInPlaceActiveObject@COleControl@@UAGJABU_GUID@@PAPAX@Z',
    5391:'?QueryInterface@XOleInPlaceActiveObject@COleServerDoc@@UAGJABU_GUID@@PAPAX@Z',
    5392:'?QueryInterface@XOleInPlaceFrame@COleFrameHook@@UAGJABU_GUID@@PAPAX@Z',
    5393:'?QueryInterface@XOleInPlaceObject@COleControl@@UAGJABU_GUID@@PAPAX@Z',
    5394:'?QueryInterface@XOleInPlaceObject@COleServerDoc@@UAGJABU_GUID@@PAPAX@Z',
    5395:'?QueryInterface@XOleIPFrame@COleControlContainer@@UAGJABU_GUID@@PAPAX@Z',
    5396:'?QueryInterface@XOleIPSite@COleClientItem@@UAGJABU_GUID@@PAPAX@Z',
    5397:'?QueryInterface@XOleIPSite@COleControlSite@@UAGJABU_GUID@@PAPAX@Z',
    5398:'?QueryInterface@XOleItemContainer@COleLinkingDoc@@UAGJABU_GUID@@PAPAX@Z',
    5399:'?QueryInterface@XOleObject@CDocObjectServer@@UAGJABU_GUID@@PAPAX@Z',
    5400:'?QueryInterface@XOleObject@COleControl@@UAGJABU_GUID@@PAPAX@Z',
    5401:'?QueryInterface@XOleObject@COleServerDoc@@UAGJABU_GUID@@PAPAX@Z',
    5402:'?QueryInterface@XOleObject@COleServerItem@@UAGJABU_GUID@@PAPAX@Z',
    5403:'?QueryInterface@XOleUIObjInfo@COlePropertiesDialog@@UAGJABU_GUID@@PAPAX@Z',
    5404:'?QueryInterface@XPerPropertyBrowsing@COleControl@@UAGJABU_GUID@@PAPAX@Z',
    5405:'?QueryInterface@XPersistFile@COleLinkingDoc@@UAGJABU_GUID@@PAPAX@Z',
    5406:'?QueryInterface@XPersistMemory@COleControl@@UAGJABU_GUID@@PAPAX@Z',
    5407:'?QueryInterface@XPersistPropertyBag@COleControl@@UAGJABU_GUID@@PAPAX@Z',
    5408:'?QueryInterface@XPersistStorage@COleControl@@UAGJABU_GUID@@PAPAX@Z',
    5409:'?QueryInterface@XPersistStorage@COleServerDoc@@UAGJABU_GUID@@PAPAX@Z',
    5410:'?QueryInterface@XPersistStreamInit@COleControl@@UAGJABU_GUID@@PAPAX@Z',
    5411:'?QueryInterface@XPointerInactive@COleControl@@UAGJABU_GUID@@PAPAX@Z',
    5412:'?QueryInterface@XPrint@CDocObjectServer@@UAGJABU_GUID@@PAPAX@Z',
    5413:'?QueryInterface@XPropertyNotifySink@COleControlSite@@UAGJABU_GUID@@PAPAX@Z',
    5414:'?QueryInterface@XPropertyPage@COlePropertyPage@@UAGJABU_GUID@@PAPAX@Z',
    5415:'?QueryInterface@XPropNotifySink@COlePropertyPage@@UAGJABU_GUID@@PAPAX@Z',
    5416:'?QueryInterface@XProvideClassInfo@COleControl@@UAGJABU_GUID@@PAPAX@Z',
    5417:'?QueryInterface@XQuickActivate@COleControl@@UAGJABU_GUID@@PAPAX@Z',
    5418:'?QueryInterface@XRichEditOleCallback@CRichEditView@@UAGJABU_GUID@@PAPAX@Z',
    5419:'?QueryInterface@XSpecifyPropertyPages@COleControl@@UAGJABU_GUID@@PAPAX@Z',
    5420:'?QueryInterface@XViewObject@COleControl@@UAGJABU_GUID@@PAPAX@Z',
    5421:'?QueryOption@CInternetConnection@@QBEHKAAK@Z',
    5422:'?QueryOption@CInternetConnection@@QBEHKAAVCString@@@Z',
    5423:'?QueryOption@CInternetConnection@@QBEHKPAXPAK@Z',
    5424:'?QueryOption@CInternetFile@@QBEHKAAK@Z',
    5425:'?QueryOption@CInternetFile@@QBEHKAAVCString@@@Z',
    5426:'?QueryOption@CInternetFile@@QBEHKPAXPAK@Z',
    5427:'?QueryOption@CInternetSession@@QBEHKAAK@Z',
    5428:'?QueryOption@CInternetSession@@QBEHKAAVCString@@@Z',
    5429:'?QueryOption@CInternetSession@@QBEHKPAXPAK@Z',
    5430:'?QuerySiblings@CPropertyPage@@QAEJIJ@Z',
    5431:'?QuerySinkInterface@CConnectionPoint@@UAEPAUIUnknown@@PAU2@@Z',
    5432:'?QuerySinkInterface@XEventConnPt@COleControl@@UAEPAUIUnknown@@PAU3@@Z',
    5433:'?QueryStatus@XOleCommandTarget@CDocObjectServer@@UAGJPBU_GUID@@KQAU_tagOLECMD@@PAU_tagOLECMDTEXT@@@Z',
    5434:'?QueryTextMetrics@CFontHolder@@QAEXPAUtagTEXTMETRICA@@@Z',
    5435:'?QuickActivate@COleControlSite@@UAEHXZ',
    5436:'?QuickActivate@XQuickActivate@COleControl@@UAGJPAUtagQACONTAINER@@PAUtagQACONTROL@@@Z',
    5437:'?ReactivateAndUndo@COleClientItem@@QAEHXZ',
    5438:'?ReactivateAndUndo@XOleInPlaceObject@COleControl@@UAGJXZ',
    5439:'?ReactivateAndUndo@XOleInPlaceObject@COleServerDoc@@UAGJXZ',
    5440:'?Read@CArchive@@QAEIPAXI@Z',
    5441:'?Read@CArchiveStream@@UAGJPAXKPAK@Z',
    5442:'?Read@CFile@@UAEIPAXI@Z',
    5443:'?Read@CImageList@@QAEHPAVCArchive@@@Z',
    5444:'?Read@CInternetFile@@UAEIPAXI@Z',
    5445:'?Read@CMemFile@@UAEIPAXI@Z',
    5446:'?Read@COleStreamFile@@UAEIPAXI@Z',
    5447:'?Read@CSocketFile@@UAEIPAXI@Z',
    5448:'?Read@CStdioFile@@UAEIPAXI@Z',
    5449:'?ReadClass@CArchive@@QAEPAUCRuntimeClass@@PBU2@PAIPAK@Z',
    5450:'?ReadCount@CArchive@@QAEKXZ',
    5451:'?ReadFromArchive@CEditView@@QAEXAAVCArchive@@I@Z',
    5452:'?ReadFromStream@CProperty@@QAEHPAUIStream@@@Z',
    5453:'?ReadFromStream@CPropertySection@@QAEHPAUIStream@@T_LARGE_INTEGER@@@Z',
    5454:'?ReadFromStream@CPropertySet@@QAEHPAUIStream@@@Z',
    5455:'?ReadItem@COleClientItem@@UAEXAAVCArchive@@@Z',
    5456:'?ReadItemCompound@COleClientItem@@QAEXAAVCArchive@@@Z',
    5457:'?ReadItemFlat@COleClientItem@@QAEXAAVCArchive@@@Z',
    5458:'?ReadList@CRecentFileList@@UAEXXZ',
    5459:'?ReadNameDictFromStream@CPropertySection@@QAEHPAUIStream@@@Z',
    5460:'?ReadObject@CArchive@@QAEPAVCObject@@PBUCRuntimeClass@@@Z',
    5461:'?ReadString@CArchive@@QAEHAAVCString@@@Z',
    5462:'?ReadString@CArchive@@QAEPADPADI@Z',
    5463:'?ReadString@CInternetFile@@UAEHAAVCString@@@Z',
    5464:'?ReadString@CInternetFile@@UAEPADPADI@Z',
    5465:'?ReadString@CStdioFile@@UAEHAAVCString@@@Z',
    5466:'?ReadString@CStdioFile@@UAEPADPADI@Z',
    5467:'?Realloc@CMemFile@@MAEPAEPAEK@Z',
    5468:'?Realloc@CSharedFile@@MAEPAEPAEK@Z',
    5469:'?ReallocLongBinary@CFieldExchange@@QAEPAEAAVCLongBinary@@JJ@Z',
    5470:'?RebindParams@CRecordset@@QAEXPAX@Z',
    5471:'?RecalcDelayShow@CControlBar@@UAEKPAUAFX_SIZEPARENTPARAMS@@@Z',
    5472:'?RecalcLayout@CFrameWnd@@UAEXH@Z',
    5473:'?RecalcLayout@CMiniDockFrameWnd@@UAEXH@Z',
    5474:'?RecalcLayout@COleCntrFrameWnd@@UAEXH@Z',
    5475:'?RecalcLayout@COleDocIPFrameWnd@@MAEXH@Z',
    5476:'?RecalcLayout@COleIPFrameWnd@@UAEXH@Z',
    5477:'?RecalcLayout@CSplitterWnd@@UAEXXZ',
    5478:'?Receive@CAsyncSocket@@UAEHPAXHH@Z',
    5479:'?Receive@CSocket@@UAEHPAXHH@Z',
    5480:'?ReceiveFrom@CAsyncSocket@@QAEHPAXHAAVCString@@AAIH@Z',
    5481:'?ReceiveFromHelper@CAsyncSocket@@MAEHPAXHPAUsockaddr@@PAHH@Z',
    5482:'?ReceiveFromHelper@CSocket@@MAEHPAXHPAUsockaddr@@PAHH@Z',
    5483:'?RecreateControlWindow@COleControl@@QAEXXZ',
    5484:'?rectDefault@CFrameWnd@@2VCRect@@B',
    5485:'?RectVisible@CDC@@UBEHPBUtagRECT@@@Z',
    5486:'?RectVisible@CMetaFileDC@@UBEHPBUtagRECT@@@Z',
    5487:'?Redim@COleSafeArray@@QAEXPAUtagSAFEARRAYBOUND@@@Z',
    5488:'?ReDockControlBar@CDockBar@@QAEXPAVCControlBar@@PBUtagRECT@@@Z',
    5489:'?ReDockControlBar@CFrameWnd@@QAEXPAVCControlBar@@PAVCDockBar@@PBUtagRECT@@@Z',
    5490:'?ReflectChildNotify@CWnd@@IAEHIIJPAJ@Z',
    5491:'?ReflectLastMsg@CWnd@@KGHPAUHWND__@@PAJ@Z',
    5492:'?Refresh@COleControl@@QAEXXZ',
    5493:'?RefreshLink@CDaoTableDef@@QAEXXZ',
    5494:'?RefreshRowset@CRecordset@@QAEXGG@Z',
    5495:'?Register@COleDropTarget@@QAEHPAVCWnd@@@Z',
    5496:'?Register@COleLinkingDoc@@QAEHPAVCOleObjectFactory@@PBD@Z',
    5497:'?Register@COleMessageFilter@@QAEHXZ',
    5498:'?Register@COleObjectFactory@@UAEHXZ',
    5499:'?Register@COleTemplateServer@@UAEHXZ',
    5500:'?RegisterAll@COleObjectFactory@@SGHXZ',
    5501:'?RegisterIfServerAttached@COleLinkingDoc@@MAEHPBDH@Z',
    5502:'?RegisterShellFileTypes@CDocManager@@UAEXH@Z',
    5503:'?RegisterShellFileTypes@CWinApp@@IAEXH@Z',
    5504:'?RegisterShellFileTypesCompat@CWinApp@@IAEXXZ',
    5505:'?Release@CArchiveStream@@UAGKXZ',
    5506:'?Release@CBlobProperty@@UAGKXZ',
    5507:'?Release@CInnerUnknown@@UAGKXZ',
    5508:'?Release@COleClientItem@@UAEXW4tagOLECLOSE@@@Z',
    5509:'?Release@COleConnPtContainer@@UAGKXZ',
    5510:'?Release@COleDataObject@@QAEXXZ',
    5511:'?Release@COleDispatchImpl@@UAGKXZ',
    5512:'?Release@COleUILinkInfo@@UAGKXZ',
    5513:'?Release@CString@@IAEXXZ',
    5514:'?Release@CString@@KGXPAUCStringData@@@Z',
    5515:'?Release@XAdviseSink@COleClientItem@@UAGKXZ',
    5516:'?Release@XAmbientProps@COleControlSite@@UAGKXZ',
    5517:'?Release@XBoundObjectSite@COleControlSite@@UAGKXZ',
    5518:'?Release@XClassFactory@COleObjectFactory@@UAGKXZ',
    5519:'?Release@XConnPt@CConnectionPoint@@UAGKXZ',
    5520:'?Release@XDataObject@COleControl@@UAGKXZ',
    5521:'?Release@XDataObject@COleDataSource@@UAGKXZ',
    5522:'?Release@XDataObject@COleServerDoc@@UAGKXZ',
    5523:'?Release@XDataObject@COleServerItem@@UAGKXZ',
    5524:'?Release@XDropSource@COleDropSource@@UAGKXZ',
    5525:'?Release@XDropTarget@COleDropTarget@@UAGKXZ',
    5526:'?Release@XEnumVOID@CEnumArray@@UAGKXZ',
    5527:'?Release@XEventSink@COleControlSite@@UAGKXZ',
    5528:'?Release@XFontNotification@COleControl@@UAGKXZ',
    5529:'?Release@XMessageFilter@COleMessageFilter@@UAGKXZ',
    5530:'?Release@XNotifyDBEvents@COleControlSite@@UAGKXZ',
    5531:'?Release@XOleCache@COleControl@@UAGKXZ',
    5532:'?Release@XOleClientSite@COleClientItem@@UAGKXZ',
    5533:'?Release@XOleClientSite@COleControlSite@@UAGKXZ',
    5534:'?Release@XOleCommandTarget@CDocObjectServer@@UAGKXZ',
    5535:'?Release@XOleContainer@COleControlContainer@@UAGKXZ',
    5536:'?Release@XOleControl@COleControl@@UAGKXZ',
    5537:'?Release@XOleControlSite@COleControlSite@@UAGKXZ',
    5538:'?Release@XOleDocument@CDocObjectServer@@UAGKXZ',
    5539:'?Release@XOleDocumentView@CDocObjectServer@@UAGKXZ',
    5540:'?Release@XOleInPlaceActiveObject@COleControl@@UAGKXZ',
    5541:'?Release@XOleInPlaceActiveObject@COleServerDoc@@UAGKXZ',
    5542:'?Release@XOleInPlaceFrame@COleFrameHook@@UAGKXZ',
    5543:'?Release@XOleInPlaceObject@COleControl@@UAGKXZ',
    5544:'?Release@XOleInPlaceObject@COleServerDoc@@UAGKXZ',
    5545:'?Release@XOleIPFrame@COleControlContainer@@UAGKXZ',
    5546:'?Release@XOleIPSite@COleClientItem@@UAGKXZ',
    5547:'?Release@XOleIPSite@COleControlSite@@UAGKXZ',
    5548:'?Release@XOleItemContainer@COleLinkingDoc@@UAGKXZ',
    5549:'?Release@XOleObject@CDocObjectServer@@UAGKXZ',
    5550:'?Release@XOleObject@COleControl@@UAGKXZ',
    5551:'?Release@XOleObject@COleServerDoc@@UAGKXZ',
    5552:'?Release@XOleObject@COleServerItem@@UAGKXZ',
    5553:'?Release@XOleUIObjInfo@COlePropertiesDialog@@UAGKXZ',
    5554:'?Release@XPerPropertyBrowsing@COleControl@@UAGKXZ',
    5555:'?Release@XPersistFile@COleLinkingDoc@@UAGKXZ',
    5556:'?Release@XPersistMemory@COleControl@@UAGKXZ',
    5557:'?Release@XPersistPropertyBag@COleControl@@UAGKXZ',
    5558:'?Release@XPersistStorage@COleControl@@UAGKXZ',
    5559:'?Release@XPersistStorage@COleServerDoc@@UAGKXZ',
    5560:'?Release@XPersistStreamInit@COleControl@@UAGKXZ',
    5561:'?Release@XPointerInactive@COleControl@@UAGKXZ',
    5562:'?Release@XPrint@CDocObjectServer@@UAGKXZ',
    5563:'?Release@XPropertyNotifySink@COleControlSite@@UAGKXZ',
    5564:'?Release@XPropertyPage@COlePropertyPage@@UAGKXZ',
    5565:'?Release@XPropNotifySink@COlePropertyPage@@UAGKXZ',
    5566:'?Release@XProvideClassInfo@COleControl@@UAGKXZ',
    5567:'?Release@XQuickActivate@COleControl@@UAGKXZ',
    5568:'?Release@XRichEditOleCallback@CRichEditView@@UAGKXZ',
    5569:'?Release@XSpecifyPropertyPages@COleControl@@UAGKXZ',
    5570:'?Release@XViewObject@COleControl@@UAGKXZ',
    5571:'?ReleaseAttribDC@CDC@@UAEXXZ',
    5572:'?ReleaseBuffer@CString@@QAEXH@Z',
    5573:'?ReleaseCapture@COleControl@@QAEHXZ',
    5574:'?ReleaseDC@COleControl@@QAEHPAVCDC@@@Z',
    5575:'?ReleaseDispatch@COleDispatchDriver@@QAEXXZ',
    5576:'?ReleaseDocSite@CDocObjectServer@@QAEXXZ',
    5577:'?ReleaseFile@CDocument@@UAEXPAVCFile@@H@Z',
    5578:'?ReleaseFont@CFontHolder@@QAEXXZ',
    5579:'?ReleaseOutputDC@CDC@@UAEXXZ',
    5580:'?ReleaseOutputDC@CMetaFileDC@@MAEXXZ',
    5581:'?ReleaseOutputDC@CPreviewDC@@UAEXXZ',
    5582:'?Reload@COleClientItem@@QAEHXZ',
    5583:'?Remove@CFile@@SGXPBD@Z',
    5584:'?Remove@CFtpConnection@@QAEHPBD@Z',
    5585:'?Remove@CPropertySection@@QAEXK@Z',
    5586:'?Remove@CPropertySet@@QAEXU_GUID@@@Z',
    5587:'?Remove@CPropertySet@@QAEXU_GUID@@K@Z',
    5588:'?Remove@CRecentFileList@@UAEXH@Z',
    5589:'?Remove@CSimpleList@@QAEHPAX@Z',
    5590:'?RemoveAll@CMapPtrToPtr@@QAEXXZ',
    5591:'?RemoveAll@CMapPtrToWord@@QAEXXZ',
    5592:'?RemoveAll@CMapStringToOb@@QAEXXZ',
    5593:'?RemoveAll@CMapStringToPtr@@QAEXXZ',
    5594:'?RemoveAll@CMapStringToString@@QAEXXZ',
    5595:'?RemoveAll@CMapWordToOb@@QAEXXZ',
    5596:'?RemoveAll@CMapWordToPtr@@QAEXXZ',
    5597:'?RemoveAll@CObList@@QAEXXZ',
    5598:'?RemoveAll@CPropertySection@@QAEXXZ',
    5599:'?RemoveAll@CPropertySet@@QAEXXZ',
    5600:'?RemoveAll@CPtrList@@QAEXXZ',
    5601:'?RemoveAll@CStringList@@QAEXXZ',
    5602:'?RemoveAt@CByteArray@@QAEXHH@Z',
    5603:'?RemoveAt@CDWordArray@@QAEXHH@Z',
    5604:'?RemoveAt@CObArray@@QAEXHH@Z',
    5605:'?RemoveAt@CObList@@QAEXPAU__POSITION@@@Z',
    5606:'?RemoveAt@CPtrArray@@QAEXHH@Z',
    5607:'?RemoveAt@CPtrList@@QAEXPAU__POSITION@@@Z',
    5608:'?RemoveAt@CStringArray@@QAEXHH@Z',
    5609:'?RemoveAt@CStringList@@QAEXPAU__POSITION@@@Z',
    5610:'?RemoveAt@CUIntArray@@QAEXHH@Z',
    5611:'?RemoveAt@CWordArray@@QAEXHH@Z',
    5612:'?RemoveControlBar@CDockBar@@QAEHPAVCControlBar@@HH@Z',
    5613:'?RemoveControlBar@CFrameWnd@@QAEXPAVCControlBar@@@Z',
    5614:'?RemoveDirectoryA@CFtpConnection@@QAEHPBD@Z',
    5615:'?RemoveDocument@CDocTemplate@@UAEXPAVCDocument@@@Z',
    5616:'?RemoveDocument@CMultiDocTemplate@@UAEXPAVCDocument@@@Z',
    5617:'?RemoveDocument@CSingleDocTemplate@@UAEXPAVCDocument@@@Z',
    5618:'?RemoveFrameLevelUI@COleControl@@MAEXXZ',
    5619:'?RemoveFrameWnd@CFrameWnd@@IAEXXZ',
    5620:'?RemoveHead@CObList@@QAEPAVCObject@@XZ',
    5621:'?RemoveHead@CPtrList@@QAEPAXXZ',
    5622:'?RemoveHead@CStringList@@QAE?AVCString@@XZ',
    5623:'?RemoveImageList@CListCtrl@@IAEXH@Z',
    5624:'?RemoveImageList@CListView@@IAEXH@Z',
    5625:'?RemoveImageList@CTreeCtrl@@IAEXH@Z',
    5626:'?RemoveImageList@CTreeView@@IAEXH@Z',
    5627:'?RemoveItem@COleDocument@@UAEXPAVCDocItem@@@Z',
    5628:'?RemoveKey@CMapPtrToPtr@@QAEHPAX@Z',
    5629:'?RemoveKey@CMapPtrToWord@@QAEHPAX@Z',
    5630:'?RemoveKey@CMapStringToOb@@QAEHPBD@Z',
    5631:'?RemoveKey@CMapStringToPtr@@QAEHPBD@Z',
    5632:'?RemoveKey@CMapStringToString@@QAEHPBD@Z',
    5633:'?RemoveKey@CMapWordToOb@@QAEHG@Z',
    5634:'?RemoveKey@CMapWordToPtr@@QAEHG@Z',
    5635:'?RemoveMenus@XOleInPlaceFrame@COleFrameHook@@UAGJPAUHMENU__@@@Z',
    5636:'?RemoveMenus@XOleIPFrame@COleControlContainer@@UAGJPAUHMENU__@@@Z',
    5637:'?RemovePage@CPropertySheet@@QAEXH@Z',
    5638:'?RemovePage@CPropertySheet@@QAEXPAVCPropertyPage@@@Z',
    5639:'?RemovePlaceHolder@CDockBar@@QAEXPAVCControlBar@@@Z',
    5640:'?RemoveSource@CDataBoundProperty@@QAEXXZ',
    5641:'?RemoveTail@CObList@@QAEPAVCObject@@XZ',
    5642:'?RemoveTail@CPtrList@@QAEPAXXZ',
    5643:'?RemoveTail@CStringList@@QAE?AVCString@@XZ',
    5644:'?RemoveView@CDocument@@QAEXPAVCView@@@Z',
    5645:'?Rename@CFile@@SGXPBD0@Z',
    5646:'?Rename@CFtpConnection@@QAEHPBD0@Z',
    5647:'?Render@CPictureHolder@@QAEXPAVCDC@@ABVCRect@@1@Z',
    5648:'?RepairDatabase@CDaoWorkspace@@SGXPBD@Z',
    5649:'?ReparentControlWindow@COleControl@@UAEXPAUHWND__@@0@Z',
    5650:'?ReplaceBrackets@CDatabase@@QAEXPAD@Z',
    5651:'?ReportError@CException@@UAEHII@Z',
    5652:'?ReportError@COleClientItem@@UBEHJ@Z',
    5653:'?ReportSaveLoadException@CDocument@@UAEXPBDPAVCException@@HI@Z',
    5654:'?ReportSaveLoadException@COleLinkingDoc@@UAEXPBDPAVCException@@HI@Z',
    5655:'?RepositionBars@CWnd@@QAEXIIIIPAUtagRECT@@PBU2@H@Z',
    5656:'?RepositionFrame@COleIPFrameWnd@@UAEXPBUtagRECT@@0@Z',
    5657:'?Requery@CDaoRecordset@@UAEXXZ',
    5658:'?Requery@CRecordset@@UAEHXZ',
    5659:'?RequestBorderSpace@XOleInPlaceFrame@COleFrameHook@@UAGJPBUtagRECT@@@Z',
    5660:'?RequestBorderSpace@XOleIPFrame@COleControlContainer@@UAGJPBUtagRECT@@@Z',
    5661:'?RequestLicKey@XClassFactory@COleObjectFactory@@UAGJKPAPAG@Z',
    5662:'?RequestNewObjectLayout@XOleClientSite@COleClientItem@@UAGJXZ',
    5663:'?RequestNewObjectLayout@XOleClientSite@COleControlSite@@UAGJXZ',
    5664:'?RequestPositionChange@COleServerDoc@@QAEXPBUtagRECT@@@Z',
    5665:'?Reset@XEnumVOID@CEnumArray@@UAGJXZ',
    5666:'?ResetCursor@CRecordset@@QAEXXZ',
    5667:'?ResetData@CCachedDataPathProperty@@UAEXXZ',
    5668:'?ResetData@CDataPathProperty@@UAEXXZ',
    5669:'?ResetStockProps@COleControl@@QAEXXZ',
    5670:'?ResetTimer@CControlBar@@QAEXII@Z',
    5671:'?ResetVersion@COleControl@@QAEXK@Z',
    5672:'?ResizeBorder@XOleInPlaceActiveObject@COleControl@@UAGJPBUtagRECT@@PAUIOleInPlaceUIWindow@@H@Z',
    5673:'?ResizeBorder@XOleInPlaceActiveObject@COleServerDoc@@UAGJPBUtagRECT@@PAUIOleInPlaceUIWindow@@H@Z',
    5674:'?ResizeFrameWindow@COleControl@@MAEXHH@Z',
    5675:'?ResizeOneDim@COleSafeArray@@QAEXK@Z',
    5676:'?ResizeOpenControl@COleControl@@IAEXHH@Z',
    5677:'?ResizeParentToFit@CScrollView@@QAEXH@Z',
    5678:'?RestoreDC@CDC@@UAEHH@Z',
    5679:'?RestoreDC@CPreviewDC@@UAEHH@Z',
    5680:'?RestoreState@CToolBarCtrl@@QAEXPAUHKEY__@@PBD1@Z',
    5681:'?RestoreWaitCursor@CCmdTarget@@QAEXXZ',
    5682:'?RetryRejectedCall@XMessageFilter@COleMessageFilter@@UAGKPAUHTASK__@@KK@Z',
    5683:'?ReverseFind@CString@@QBEHD@Z',
    5684:'?Revert@CArchiveStream@@UAGJXZ',
    5685:'?Revoke@COleDropTarget@@UAEXXZ',
    5686:'?Revoke@COleLinkingDoc@@QAEXXZ',
    5687:'?Revoke@COleMessageFilter@@QAEXXZ',
    5688:'?Revoke@COleObjectFactory@@QAEXXZ',
    5689:'?RevokeAll@COleObjectFactory@@SGXXZ',
    5690:'?RFX_Binary@@YGXPAVCFieldExchange@@PBDAAVCByteArray@@H@Z',
    5691:'?RFX_Binary_Bulk@@YGXPAVCFieldExchange@@PBDPAPAEPAPAJH@Z',
    5692:'?RFX_Bool@@YGXPAVCFieldExchange@@PBDAAH@Z',
    5693:'?RFX_Bool_Bulk@@YGXPAVCFieldExchange@@PBDPAPAHPAPAJ@Z',
    5694:'?RFX_Byte@@YGXPAVCFieldExchange@@PBDAAE@Z',
    5695:'?RFX_Byte_Bulk@@YGXPAVCFieldExchange@@PBDPAPAEPAPAJ@Z',
    5696:'?RFX_Date@@YGXPAVCFieldExchange@@PBDAAUtagTIMESTAMP_STRUCT@@@Z',
    5697:'?RFX_Date@@YGXPAVCFieldExchange@@PBDAAVCTime@@@Z',
    5698:'?RFX_Date_Bulk@@YGXPAVCFieldExchange@@PBDPAPAUtagTIMESTAMP_STRUCT@@PAPAJ@Z',
    5699:'?RFX_Double@@YGXPAVCFieldExchange@@PBDAAN@Z',
    5700:'?RFX_Double_Bulk@@YGXPAVCFieldExchange@@PBDPAPANPAPAJ@Z',
    5701:'?RFX_Int@@YGXPAVCFieldExchange@@PBDAAH@Z',
    5702:'?RFX_Int_Bulk@@YGXPAVCFieldExchange@@PBDPAPAHPAPAJ@Z',
    5703:'?RFX_Long@@YGXPAVCFieldExchange@@PBDAAJ@Z',
    5704:'?RFX_Long_Bulk@@YGXPAVCFieldExchange@@PBDPAPAJ2@Z',
    5705:'?RFX_LongBinary@@YGXPAVCFieldExchange@@PBDAAVCLongBinary@@@Z',
    5706:'?RFX_Single@@YGXPAVCFieldExchange@@PBDAAM@Z',
    5707:'?RFX_Single_Bulk@@YGXPAVCFieldExchange@@PBDPAPAMPAPAJ@Z',
    5708:'?RFX_Text@@YGXPAVCFieldExchange@@PBDAAVCString@@HHF@Z',
    5709:'?RFX_Text_Bulk@@YGXPAVCFieldExchange@@PBDPAPADPAPAJH@Z',
    5710:'?Right@CString@@QBE?AV1@H@Z',
    5711:'?Rollback@CDaoWorkspace@@QAEXXZ',
    5712:'?Rollback@CDatabase@@QAEHXZ',
    5713:'?Run@COleClientItem@@QAEXXZ',
    5714:'?Run@CWinApp@@UAEHXZ',
    5715:'?Run@CWinThread@@UAEHXZ',
    5716:'?RunAutomated@CWinApp@@QAEHXZ',
    5717:'?RunEmbedded@CWinApp@@QAEHXZ',
    5718:'?RunModalLoop@CWnd@@QAEHK@Z',
    5719:'?SafeSetProperty@COleControlSite@@UAAHJGZZ',
    5720:'?SameAsSelected@CEditView@@IAEHPBDH@Z',
    5721:'?SameAsSelected@CRichEditView@@IAEHPBDHH@Z',
    5722:'?Save@CBlobProperty@@UAGJPAUIStream@@H@Z',
    5723:'?Save@XPersistFile@COleLinkingDoc@@UAGJPBGH@Z',
    5724:'?Save@XPersistMemory@COleControl@@UAGJPAXHK@Z',
    5725:'?Save@XPersistPropertyBag@COleControl@@UAGJPAUIPropertyBag@@HH@Z',
    5726:'?Save@XPersistStorage@COleControl@@UAGJPAUIStorage@@H@Z',
    5727:'?Save@XPersistStorage@COleServerDoc@@UAGJPAUIStorage@@H@Z',
    5728:'?Save@XPersistStreamInit@COleControl@@UAGJPAUIStream@@H@Z',
    5729:'?SaveAllModified@CDocManager@@UAEHXZ',
    5730:'?SaveAllModified@CDocTemplate@@UAEHXZ',
    5731:'?SaveAllModified@CWinApp@@UAEHXZ',
    5732:'?SaveBarState@CFrameWnd@@QBEXPBD@Z',
    5733:'?SaveCompleted@XPersistFile@COleLinkingDoc@@UAGJPBG@Z',
    5734:'?SaveCompleted@XPersistStorage@COleControl@@UAGJPAUIStorage@@@Z',
    5735:'?SaveCompleted@XPersistStorage@COleServerDoc@@UAGJPAUIStorage@@@Z',
    5736:'?SaveDC@CDC@@UAEHXZ',
    5737:'?SaveDC@CPreviewDC@@UAEHXZ',
    5738:'?SaveEmbedding@COleServerDoc@@QAEXXZ',
    5739:'?SaveFocusControl@CFormView@@IAEHXZ',
    5740:'?SaveModified@CDocument@@UAEHXZ',
    5741:'?SaveModified@COleDocument@@UAEHXZ',
    5742:'?SaveModified@COleServerDoc@@MAEHXZ',
    5743:'?SaveModifiedPrompt@COleServerDoc@@IAEHXZ',
    5744:'?SaveObject@XOleClientSite@COleClientItem@@UAGJXZ',
    5745:'?SaveObject@XOleClientSite@COleControlSite@@UAGJXZ',
    5746:'?SaveState@CControlBarInfo@@QAEHPBDH@Z',
    5747:'?SaveState@CDockState@@QAEXPBD@Z',
    5748:'?SaveState@COleControl@@IAEJPAUIStream@@@Z',
    5749:'?SaveState@CToolBarCtrl@@QAEXPAUHKEY__@@PBD1@Z',
    5750:'?SaveStdProfileSettings@CWinApp@@IAEXXZ',
    5751:'?SaveToStorage@COleDocument@@MAEXPAVCObject@@@Z',
    5752:'?SaveToStorage@COleLinkingDoc@@MAEXPAVCObject@@@Z',
    5753:'?SaveViewState@XOleDocumentView@CDocObjectServer@@UAGJPAUIStream@@@Z',
    5754:'?ScalePoint@CDockState@@QAEXAAVCPoint@@@Z',
    5755:'?ScaleRectPos@CDockState@@QAEXAAVCRect@@@Z',
    5756:'?ScaleViewportExt@CDC@@UAE?AVCSize@@HHHH@Z',
    5757:'?ScaleViewportExt@CMetaFileDC@@UAE?AVCSize@@HHHH@Z',
    5758:'?ScaleViewportExt@CPreviewDC@@UAE?AVCSize@@HHHH@Z',
    5759:'?ScaleWindowExt@CDC@@UAE?AVCSize@@HHHH@Z',
    5760:'?ScaleWindowExt@CPreviewDC@@UAE?AVCSize@@HHHH@Z',
    5761:'?Scroll@XOleIPSite@COleClientItem@@UAGJUtagSIZE@@@Z',
    5762:'?Scroll@XOleIPSite@COleControlSite@@UAGJUtagSIZE@@@Z',
    5763:'?ScrollChildren@COleControlContainer@@UAEXHH@Z',
    5764:'?ScrollContainerBy@COleServerDoc@@QAEHVCSize@@@Z',
    5765:'?ScrollToDevicePosition@CScrollView@@IAEXUtagPOINT@@@Z',
    5766:'?ScrollToPosition@CScrollView@@QAEXUtagPOINT@@@Z',
    5767:'?ScrollWindow@COleControl@@QAEXHHPBUtagRECT@@0@Z',
    5768:'?ScrollWindow@CWnd@@QAEXHHPBUtagRECT@@0@Z',
    5769:'?ScrollWindowEx@CWnd@@QAEHHHPBUtagRECT@@0PAVCRgn@@PAU2@I@Z',
    5770:'?Seek@CArchiveStream@@UAGJT_LARGE_INTEGER@@KPAT_ULARGE_INTEGER@@@Z',
    5771:'?Seek@CDaoRecordset@@QAEHPBDPAVCOleVariant@@11@Z',
    5772:'?Seek@CDaoRecordset@@QAEHPBDPAVCOleVariant@@G@Z',
    5773:'?Seek@CFile@@UAEJJI@Z',
    5774:'?Seek@CInternetFile@@UAEJJI@Z',
    5775:'?Seek@CMemFile@@UAEJJI@Z',
    5776:'?Seek@COleStreamFile@@UAEJJI@Z',
    5777:'?Seek@CSocketFile@@UAEJJI@Z',
    5778:'?Seek@CStdioFile@@UAEJJI@Z',
    5779:'?Select@CFontHolder@@QAEPAVCFont@@PAVCDC@@JJ@Z',
    5780:'?SelectClipPath@CDC@@QAEHH@Z',
    5781:'?SelectClipRgn@CDC@@QAEHPAVCRgn@@@Z',
    5782:'?SelectClipRgn@CDC@@QAEHPAVCRgn@@H@Z',
    5783:'?SelectFontFromList@CFontPropPage@@IAEXVCString@@PAUtagMERGEOBJECT@@@Z',
    5784:'?SelectFontObject@COleControl@@QAEPAVCFont@@PAVCDC@@AAVCFontHolder@@@Z',
    5785:'?SelectGdiObject@CDC@@KGPAVCGdiObject@@PAUHDC__@@PAX@Z',
    5786:'?SelectObject@CDC@@QAEHPAVCRgn@@@Z',
    5787:'?SelectObject@CDC@@QAEPAVCBrush@@PAV2@@Z',
    5788:'?SelectObject@CDC@@QAEPAVCPen@@PAV2@@Z',
    5789:'?SelectObject@CDC@@UAEPAVCFont@@PAV2@@Z',
    5790:'?SelectObject@CPreviewDC@@UAEPAVCFont@@PAV2@@Z',
    5791:'?SelectPalette@CDC@@QAEPAVCPalette@@PAV2@H@Z',
    5792:'?SelectPrinter@CWinApp@@QAEXPAX0H@Z',
    5793:'?SelectStockFont@COleControl@@QAEPAVCFont@@PAVCDC@@@Z',
    5794:'?SelectStockObject@CDC@@UAEPAVCGdiObject@@H@Z',
    5795:'?SelectStockObject@CPreviewDC@@UAEPAVCGdiObject@@H@Z',
    5796:'?Send@CAsyncSocket@@UAEHPBXHH@Z',
    5797:'?Send@CSocket@@UAEHPBXHH@Z',
    5798:'?SendAdvise@COleControl@@IAEXI@Z',
    5799:'?SendChildNotifyLastMsg@CWnd@@QAEHPAJ@Z',
    5800:'?SendChunk@CSocket@@QAEHPBXHH@Z',
    5801:'?SendDlgItemMessageA@COleControlContainer@@UAEJHIIJ@Z',
    5802:'?SendDlgItemMessageA@CWnd@@QAEJHIIJ@Z',
    5803:'?SendInitialUpdate@CDocument@@QAEXXZ',
    5804:'?SendLongBinaryData@CRecordset@@IAEXPAX@Z',
    5805:'?SendMessageToDescendants@CWnd@@SGXPAUHWND__@@IIJHH@Z',
    5806:'?SendMnemonic@COleControlSite@@QAEXPAUtagMSG@@@Z',
    5807:'?SendRequest@CHttpFile@@QAEHAAVCString@@PAXK@Z',
    5808:'?SendRequest@CHttpFile@@QAEHPBDKPAXK@Z',
    5809:'?SendTo@CAsyncSocket@@QAEHPBXHIPBDH@Z',
    5810:'?SendToHelper@CAsyncSocket@@MAEHPBXHPBUsockaddr@@HH@Z',
    5811:'?SendToHelper@CSocket@@MAEHPBXHPBUsockaddr@@HH@Z',
    5812:'?Serialize@CByteArray@@UAEXAAVCArchive@@@Z',
    5813:'?Serialize@CControlBarInfo@@QAEXAAVCArchive@@PAVCDockState@@@Z',
    5814:'?Serialize@CDocItem@@UAEXAAVCArchive@@@Z',
    5815:'?Serialize@CDockState@@UAEXAAVCArchive@@@Z',
    5816:'?Serialize@CDWordArray@@UAEXAAVCArchive@@@Z',
    5817:'?Serialize@CEditView@@UAEXAAVCArchive@@@Z',
    5818:'?Serialize@CMapStringToOb@@UAEXAAVCArchive@@@Z',
    5819:'?Serialize@CMapStringToString@@UAEXAAVCArchive@@@Z',
    5820:'?Serialize@CMapWordToOb@@UAEXAAVCArchive@@@Z',
    5821:'?Serialize@CObArray@@UAEXAAVCArchive@@@Z',
    5822:'?Serialize@CObject@@UAEXAAVCArchive@@@Z',
    5823:'?Serialize@CObList@@UAEXAAVCArchive@@@Z',
    5824:'?Serialize@COleClientItem@@UAEXAAVCArchive@@@Z',
    5825:'?Serialize@COleControl@@MAEXAAVCArchive@@@Z',
    5826:'?Serialize@COleDocument@@UAEXAAVCArchive@@@Z',
    5827:'?Serialize@CRichEditDoc@@UAEXAAVCArchive@@@Z',
    5828:'?Serialize@CRichEditView@@UAEXAAVCArchive@@@Z',
    5829:'?Serialize@CStringArray@@UAEXAAVCArchive@@@Z',
    5830:'?Serialize@CStringList@@UAEXAAVCArchive@@@Z',
    5831:'?Serialize@CWordArray@@UAEXAAVCArchive@@@Z',
    5832:'?SerializeClass@CArchive@@QAEXPBUCRuntimeClass@@@Z',
    5833:'?SerializeElements@@YGXAAVCArchive@@PAVCOleVariant@@H@Z',
    5834:'?SerializeElements@@YGXAAVCArchive@@PAVCString@@H@Z',
    5835:'?SerializeExtent@COleControl@@QAEXAAVCArchive@@@Z',
    5836:'?SerializeRaw@CEditView@@QAEXAAVCArchive@@@Z',
    5837:'?SerializeStockProps@COleControl@@QAEXAAVCArchive@@@Z',
    5838:'?SerializeVersion@COleControl@@QAEKAAVCArchive@@KH@Z',
    5839:'?Set@CProperty@@QAEHKQAXK@Z',
    5840:'?Set@CProperty@@QAEHQAX@Z',
    5841:'?Set@CProperty@@QAEHQAXK@Z',
    5842:'?Set@CPropertySection@@QAEHKPAX@Z',
    5843:'?Set@CPropertySection@@QAEHKPAXK@Z',
    5844:'?Set@CPropertySet@@QAEHU_GUID@@KPAX@Z',
    5845:'?Set@CPropertySet@@QAEHU_GUID@@KPAXK@Z',
    5846:'?SetAbsolutePosition@CDaoRecordset@@QAEXJ@Z',
    5847:'?SetActiveObject@XOleInPlaceFrame@COleFrameHook@@UAGJPAUIOleInPlaceActiveObject@@PBG@Z',
    5848:'?SetActiveObject@XOleIPFrame@COleControlContainer@@UAGJPAUIOleInPlaceActiveObject@@PBG@Z',
    5849:'?SetActivePage@CPropertySheet@@QAEHH@Z',
    5850:'?SetActivePage@CPropertySheet@@QAEHPAVCPropertyPage@@@Z',
    5851:'?SetActivePane@CSplitterWnd@@UAEXHHPAVCWnd@@@Z',
    5852:'?SetActiveView@CFrameWnd@@QAEXPAVCView@@H@Z',
    5853:'?SetAdvise@XViewObject@COleControl@@UAGJKKPAUIAdviseSink@@@Z',
    5854:'?SetAppearance@COleControl@@QAEXF@Z',
    5855:'?SetArcDirection@CDC@@QAEHH@Z',
    5856:'?SetAt@CString@@QAEXHD@Z',
    5857:'?SetAtGrow@CByteArray@@QAEXHE@Z',
    5858:'?SetAtGrow@CDWordArray@@QAEXHK@Z',
    5859:'?SetAtGrow@CObArray@@QAEXHPAVCObject@@@Z',
    5860:'?SetAtGrow@CPtrArray@@QAEXHPAX@Z',
    5861:'?SetAtGrow@CStringArray@@QAEXHPBD@Z',
    5862:'?SetAtGrow@CUIntArray@@QAEXHI@Z',
    5863:'?SetAtGrow@CWordArray@@QAEXHG@Z',
    5864:'?SetAttribDC@CDC@@UAEXPAUHDC__@@@Z',
    5865:'?SetAttribDC@CMetaFileDC@@UAEXPAUHDC__@@@Z',
    5866:'?SetAttribDC@CPreviewDC@@UAEXPAUHDC__@@@Z',
    5867:'?SetAttributes@CDaoTableDef@@QAEXJ@Z',
    5868:'?SetBackColor@COleControl@@QAEXK@Z',
    5869:'?SetBarInfo@CControlBar@@QAEXPAVCControlBarInfo@@PAVCFrameWnd@@@Z',
    5870:'?SetBarInfo@CDockBar@@QAEXPAVCControlBarInfo@@PAVCFrameWnd@@@Z',
    5871:'?SetBarStyle@CControlBar@@QAEXK@Z',
    5872:'?SetBitmap@CToolBar@@QAEHPAUHBITMAP__@@@Z',
    5873:'?SetBkColor@CDC@@UAEKK@Z',
    5874:'?SetBkColor@CPreviewDC@@UAEKK@Z',
    5875:'?SetBkMode@CDC@@QAEHH@Z',
    5876:'?SetBookmark@CDaoRecordset@@QAEXVCOleVariant@@@Z',
    5877:'?SetBookmark@CRecordset@@QAEXABVCDBVariant@@@Z',
    5878:'?SetBorderSpace@XOleInPlaceFrame@COleFrameHook@@UAGJPBUtagRECT@@@Z',
    5879:'?SetBorderSpace@XOleIPFrame@COleControlContainer@@UAGJPBUtagRECT@@@Z',
    5880:'?SetBorderStyle@COleControl@@QAEXF@Z',
    5881:'?SetButton@CColorPropPage@@AAEXPAVCColorButton@@@Z',
    5882:'?SetButtonInfo@CToolBar@@QAEXHIIH@Z',
    5883:'?SetButtons@CToolBar@@QAEHPBIH@Z',
    5884:'?SetButtonStyle@CToolBar@@QAEXHI@Z',
    5885:'?SetButtonText@CToolBar@@QAEHHPBD@Z',
    5886:'?SetCacheSize@CDaoRecordset@@QAEXJ@Z',
    5887:'?SetCacheStart@CDaoRecordset@@QAEXVCOleVariant@@@Z',
    5888:'?SetCapture@COleControl@@QAEPAVCWnd@@XZ',
    5889:'?SetCharFormat@CRichEditView@@QAEXU_charformat@@@Z',
    5890:'?SetCheck@CCheckListBox@@QAEXHH@Z',
    5891:'?SetCheck@CCmdUI@@UAEXH@Z',
    5892:'?SetCheck@COleCmdUI@@UAEXH@Z',
    5893:'?SetCheck@CStatusCmdUI@@UAEXH@Z',
    5894:'?SetCheck@CTestCmdUI@@UAEXH@Z',
    5895:'?SetCheck@CToolCmdUI@@UAEXH@Z',
    5896:'?SetCheckStyle@CCheckListBox@@QAEXI@Z',
    5897:'?SetClassID@CPropertySet@@QAEXU_GUID@@@Z',
    5898:'?SetClientSite@CDataBoundProperty@@QAEXPAVCOleControlSite@@@Z',
    5899:'?SetClientSite@XOleObject@CDocObjectServer@@UAGJPAUIOleClientSite@@@Z',
    5900:'?SetClientSite@XOleObject@COleControl@@UAGJPAUIOleClientSite@@@Z',
    5901:'?SetClientSite@XOleObject@COleServerDoc@@UAGJPAUIOleClientSite@@@Z',
    5902:'?SetClientSite@XOleObject@COleServerItem@@UAGJPAUIOleClientSite@@@Z',
    5903:'?SetClipboard@COleDataSource@@QAEXXZ',
    5904:'?SetColorAdjustment@CDC@@QAEHPBUtagCOLORADJUSTMENT@@@Z',
    5905:'?SetColorProp@CColorPropPage@@IAEHPAVCDataExchange@@KPBD@Z',
    5906:'?SetColorScheme@XOleObject@CDocObjectServer@@UAGJPAUtagLOGPALETTE@@@Z',
    5907:'?SetColorScheme@XOleObject@COleControl@@UAGJPAUtagLOGPALETTE@@@Z',
    5908:'?SetColorScheme@XOleObject@COleServerDoc@@UAGJPAUtagLOGPALETTE@@@Z',
    5909:'?SetColorScheme@XOleObject@COleServerItem@@UAGJPAUtagLOGPALETTE@@@Z',
    5910:'?SetColumnInfo@CSplitterWnd@@QAEXHHH@Z',
    5911:'?SetConcurrencyAndCursorType@CRecordset@@IAEXPAXK@Z',
    5912:'?SetConnect@CDaoQueryDef@@QAEXPBD@Z',
    5913:'?SetConnect@CDaoTableDef@@QAEXPBD@Z',
    5914:'?SetContainerInfo@CDocTemplate@@QAEXI@Z',
    5915:'?SetContentExtent@XQuickActivate@COleControl@@UAGJPAUtagSIZE@@@Z',
    5916:'?SetControl@CReflectorWnd@@QAEXPAVCOleControl@@@Z',
    5917:'?SetControlSize@COleControl@@QAEHHH@Z',
    5918:'?SetControlStatus@COlePropertyPage@@QAEHIH@Z',
    5919:'?SetControlText@CFileDialog@@QAEXHPBD@Z',
    5920:'?SetCurrency@COleCurrency@@QAEXJJ@Z',
    5921:'?SetCurrentColor@CColorDialog@@QAEXK@Z',
    5922:'?SetCurrentDirectoryA@CFtpConnection@@QAEHPBD@Z',
    5923:'?SetCurrentHandles@CWinApp@@QAEXXZ',
    5924:'?SetCurrentIndex@CDaoRecordset@@QAEXPBD@Z',
    5925:'?SetCurrentPage@CPreviewView@@IAEXIH@Z',
    5926:'?SetCursor@CRectTracker@@QBEHPAVCWnd@@I@Z',
    5927:'?SetCursorAttributes@CDaoRecordset@@QAEXXZ',
    5928:'?SetData@XDataObject@COleControl@@UAGJPAUtagFORMATETC@@PAUtagSTGMEDIUM@@H@Z',
    5929:'?SetData@XDataObject@COleDataSource@@UAGJPAUtagFORMATETC@@PAUtagSTGMEDIUM@@H@Z',
    5930:'?SetData@XDataObject@COleServerDoc@@UAGJPAUtagFORMATETC@@PAUtagSTGMEDIUM@@H@Z',
    5931:'?SetData@XDataObject@COleServerItem@@UAGJPAUtagFORMATETC@@PAUtagSTGMEDIUM@@H@Z',
    5932:'?SetData@XOleCache@COleControl@@UAGJPAUtagFORMATETC@@PAUtagSTGMEDIUM@@H@Z',
    5933:'?SetDateTime@COleDateTime@@QAEHHHHHHH@Z',
    5934:'?SetDateTimeSpan@COleDateTimeSpan@@QAEXJHHH@Z',
    5935:'?SetDefaultButton@COccManager@@SAXPAVCWnd@@H@Z',
    5936:'?SetDefaultButton@COleControlSite@@QAEXH@Z',
    5937:'?SetDefaultCharFormat@CRichEditCtrl@@QAEHAAU_charformat@@@Z',
    5938:'?SetDefaultPassword@CDaoWorkspace@@SGXPBD@Z',
    5939:'?SetDefaultTitle@CMultiDocTemplate@@UAEXPAVCDocument@@@Z',
    5940:'?SetDefaultTitle@CSingleDocTemplate@@UAEXPAVCDocument@@@Z',
    5941:'?SetDefaultUser@CDaoWorkspace@@SGXPBD@Z',
    5942:'?SetDefExt@CFileDialog@@QAEXPBD@Z',
    5943:'?SetDialogBkColor@CWinApp@@IAEXKK@Z',
    5944:'?SetDialogResource@COlePropertyPage@@QAEXPAX@Z',
    5945:'?SetDirtyFields@CDaoRecordset@@MAEXXZ',
    5946:'?SetDirtyFieldStatus@CDaoRecordset@@QAEXI@Z',
    5947:'?SetDirtyFieldStatus@CRecordset@@QAEXK@Z',
    5948:'?SetDlgCtrlID@COleControlSite@@UAEHH@Z',
    5949:'?SetDlgCtrlID@CWnd@@QAEHH@Z',
    5950:'?SetDlgItemInt@COleControlContainer@@UAEXHIH@Z',
    5951:'?SetDlgItemInt@CWnd@@QAEXHIH@Z',
    5952:'?SetDlgItemTextA@COleControlContainer@@UAEXHPBD@Z',
    5953:'?SetDlgItemTextA@CWnd@@QAEXHPBD@Z',
    5954:'?SetDockState@CFrameWnd@@QAEXABVCDockState@@@Z',
    5955:'?SetDocSite@CDocObjectServer@@QAEXPAUIOleDocumentSite@@@Z',
    5956:'?SetDrawAspect@COleClientItem@@UAEXW4tagDVASPECT@@@Z',
    5957:'?SetDrawAspect@CRichEditCntrItem@@UAEXW4tagDVASPECT@@@Z',
    5958:'?SetDSCSite@CDataBoundProperty@@QAEXPAVCOleControlSite@@@Z',
    5959:'?SetEnabled@COleControl@@QAEXH@Z',
    5960:'?SetExtent@COleClientItem@@QAEXABVCSize@@W4tagDVASPECT@@@Z',
    5961:'?SetExtent@COleControlSite@@IAEHXZ',
    5962:'?SetExtent@XOleObject@CDocObjectServer@@UAGJKPAUtagSIZE@@@Z',
    5963:'?SetExtent@XOleObject@COleControl@@UAGJKPAUtagSIZE@@@Z',
    5964:'?SetExtent@XOleObject@COleServerDoc@@UAGJKPAUtagSIZE@@@Z',
    5965:'?SetExtent@XOleObject@COleServerItem@@UAGJKPAUtagSIZE@@@Z',
    5966:'?SetFaceColor@CColorButton@@QAEXK@Z',
    5967:'?SetFieldDirty@CDaoRecordset@@QAEXPAXH@Z',
    5968:'?SetFieldDirty@CRecordset@@QAEXPAXH@Z',
    5969:'?SetFieldNull@CDaoRecordset@@QAEXPAXH@Z',
    5970:'?SetFieldNull@CRecordset@@QAEXPAXH@Z',
    5971:'?SetFieldStatus@CRecordset@@QAEXKE@Z',
    5972:'?SetFieldValue@CDaoRecordset@@QAEXHPBD@Z',
    5973:'?SetFieldValue@CDaoRecordset@@QAEXPBD0@Z',
    5974:'?SetFieldValue@CDaoRecordset@@UAEXHABVCOleVariant@@@Z',
    5975:'?SetFieldValue@CDaoRecordset@@UAEXPBDABVCOleVariant@@@Z',
    5976:'?SetFieldValueNull@CDaoRecordset@@QAEXH@Z',
    5977:'?SetFieldValueNull@CDaoRecordset@@QAEXPBD@Z',
    5978:'?SetFilePath@CFile@@UAEXPBD@Z',
    5979:'?SetFocus@COleControl@@QAEPAVCWnd@@XZ',
    5980:'?SetFocus@COleControlSite@@UAEPAVCWnd@@XZ',
    5981:'?SetFocus@CWnd@@QAEPAV1@XZ',
    5982:'?SetFont@CDialogTemplate@@QAEHPBDG@Z',
    5983:'?SetFont@CFontHolder@@QAEXPAUIFont@@@Z',
    5984:'?SetFont@COleControl@@QAEXPAUIFontDisp@@@Z',
    5985:'?SetFontProps@CFontPropPage@@IAEHPAVCDataExchange@@UtagFONTOBJECT@@PBD@Z',
    5986:'?SetForeColor@COleControl@@QAEXK@Z',
    5987:'?SetFormatID@CPropertySection@@QAEXU_GUID@@@Z',
    5988:'?SetFormatVersion@CPropertySet@@QAEXG@Z',
    5989:'?SetHandle@CSharedFile@@QAEXPAXH@Z',
    5990:'?SetHeight@CToolBar@@QAEXH@Z',
    5991:'?SetHelpCapture@CFrameWnd@@IAEPAUHWND__@@UtagPOINT@@PAH@Z',
    5992:'?SetHelpInfo@COlePropertyPage@@QAEXPBD0K@Z',
    5993:'?SetHostNames@COleClientItem@@QAEXPBD0@Z',
    5994:'?SetHostNames@XOleObject@CDocObjectServer@@UAGJPBG0@Z',
    5995:'?SetHostNames@XOleObject@COleControl@@UAGJPBG0@Z',
    5996:'?SetHostNames@XOleObject@COleServerDoc@@UAGJPBG0@Z',
    5997:'?SetHostNames@XOleObject@COleServerItem@@UAGJPBG0@Z',
    5998:'?SetIconicMetafile@COleClientItem@@QAEHPAX@Z',
    5999:'?SetID@CProperty@@QAEXK@Z',
    6000:'?SetIndicators@CStatusBar@@QAEHPBIH@Z',
    6001:'?SetIniPath@CDaoWorkspace@@SGXPBD@Z',
    6002:'?SetInitialDataFormats@COleControl@@MAEXXZ',
    6003:'?SetInitialPageNum@XPrint@CDocObjectServer@@UAGJJ@Z',
    6004:'?SetInitialSize@COleControl@@QAEXHH@Z',
    6005:'?SetInPlaceSite@XOleDocumentView@CDocObjectServer@@UAGJPAUIOleInPlaceSite@@@Z',
    6006:'?SetIsolateODBCTrans@CDaoWorkspace@@QAEXH@Z',
    6007:'?SetItem@CListCtrl@@QAEHHHIPBDHIIJ@Z',
    6008:'?SetItem@CTreeCtrl@@QAEHPAU_TREEITEM@@IPBDHHIIJ@Z',
    6009:'?SetItemRects@COleClientItem@@QAEHPBUtagRECT@@0@Z',
    6010:'?SetLength@CFile@@UAEXK@Z',
    6011:'?SetLength@CInternetFile@@UAEXK@Z',
    6012:'?SetLength@CMemFile@@UAEXK@Z',
    6013:'?SetLength@COleStreamFile@@UAEXK@Z',
    6014:'?SetLength@CSocketFile@@UAEXK@Z',
    6015:'?SetLinkSource@COleUILinkInfo@@UAGJKPADKPAKH@Z',
    6016:'?SetLinkUpdateOptions@COleClientItem@@QAEXW4tagOLEUPDATE@@@Z',
    6017:'?SetLinkUpdateOptions@COleUILinkInfo@@UAGJKK@Z',
    6018:'?SetLockingMode@CDaoRecordset@@QAEXH@Z',
    6019:'?SetLockingMode@CRecordset@@QAEXI@Z',
    6020:'?SetLoginTimeout@CDaoWorkspace@@SGXF@Z',
    6021:'?SetMapMode@CDC@@UAEHH@Z',
    6022:'?SetMapMode@CPreviewDC@@UAEHH@Z',
    6023:'?SetMapperFlags@CDC@@QAEKK@Z',
    6024:'?SetMenu@XOleInPlaceFrame@COleFrameHook@@UAGJPAUHMENU__@@PAXPAUHWND__@@@Z',
    6025:'?SetMenu@XOleIPFrame@COleControlContainer@@UAGJPAUHMENU__@@PAXPAUHWND__@@@Z',
    6026:'?SetMessageText@CFrameWnd@@QAEXI@Z',
    6027:'?SetMessageText@CFrameWnd@@QAEXPBD@Z',
    6028:'?SetModified@CPropertyPage@@QAEXH@Z',
    6029:'?SetModifiedFlag@CDocument@@UAEXH@Z',
    6030:'?SetModifiedFlag@COleControl@@QAEXH@Z',
    6031:'?SetModifiedFlag@COlePropertyPage@@QAEXH@Z',
    6032:'?SetModifiedFlag@CRichEditDoc@@UAEXH@Z',
    6033:'?SetMoniker@XOleObject@CDocObjectServer@@UAGJKPAUIMoniker@@@Z',
    6034:'?SetMoniker@XOleObject@COleControl@@UAGJKPAUIMoniker@@@Z',
    6035:'?SetMoniker@XOleObject@COleServerDoc@@UAGJKPAUIMoniker@@@Z',
    6036:'?SetMoniker@XOleObject@COleServerItem@@UAGJKPAUIMoniker@@@Z',
    6037:'?SetName@CDaoQueryDef@@QAEXPBD@Z',
    6038:'?SetName@CDaoTableDef@@QAEXPBD@Z',
    6039:'?SetName@CPropertySection@@QAEHKPBD@Z',
    6040:'?SetNotPermitted@COleControl@@QAEXXZ',
    6041:'?SetNotSupported@CCmdTarget@@QAEXXZ',
    6042:'?SetNotSupported@COleControl@@QAEXXZ',
    6043:'?SetNullableFieldStatus@CDaoRecordset@@QAEXI@Z',
    6044:'?SetNullableKnownFieldStatus@CDaoRecordset@@QAEXI@Z',
    6045:'?SetNullFieldStatus@CDaoRecordset@@QAEXI@Z',
    6046:'?SetNullFieldStatus@CRecordset@@QAEXK@Z',
    6047:'?SetNullParamStatus@CRecordset@@QAEXK@Z',
    6048:'?SetNullValue@CDaoFieldExchange@@SGXPAXK@Z',
    6049:'?SetObjectRects@XOleInPlaceObject@COleControl@@UAGJPBUtagRECT@@0@Z',
    6050:'?SetObjectRects@XOleInPlaceObject@COleServerDoc@@UAGJPBUtagRECT@@0@Z',
    6051:'?SetObjects@XPropertyPage@COlePropertyPage@@UAGJKPAPAUIUnknown@@@Z',
    6052:'?SetOccDialogInfo@CDialog@@MAEHPAU_AFX_OCC_DIALOG_INFO@@@Z',
    6053:'?SetOccDialogInfo@CDialogBar@@MAEHPAU_AFX_OCC_DIALOG_INFO@@@Z',
    6054:'?SetOccDialogInfo@CFormView@@MAEHPAU_AFX_OCC_DIALOG_INFO@@@Z',
    6055:'?SetOccDialogInfo@CWnd@@MAEHPAU_AFX_OCC_DIALOG_INFO@@@Z',
    6056:'?SetODBCTimeout@CDaoQueryDef@@QAEXF@Z',
    6057:'?SetOption@CInternetConnection@@QAEHKPAXKK@Z',
    6058:'?SetOption@CInternetFile@@QAEHKPAXKK@Z',
    6059:'?SetOption@CInternetSession@@QAEHKPAXKK@Z',
    6060:'?SetOSVersion@CPropertySet@@QAEXK@Z',
    6061:'?SetOutputDC@CDC@@UAEXPAUHDC__@@@Z',
    6062:'?SetOutputDC@CMetaFileDC@@MAEXPAUHDC__@@@Z',
    6063:'?SetOutputDC@CPreviewDC@@UAEXPAUHDC__@@@Z',
    6064:'?SetOwner@CToolBar@@QAEXPAVCWnd@@@Z',
    6065:'?SetPageName@COlePropertyPage@@QAEXPBD@Z',
    6066:'?SetPageSite@XPropertyPage@COlePropertyPage@@UAGJPAUIPropertyPageSite@@@Z',
    6067:'?SetPaneInfo@CStatusBar@@QAEXHIIH@Z',
    6068:'?SetPaneStyle@CStatusBar@@QAEXHI@Z',
    6069:'?SetPaneText@CStatusBar@@QAEHHPBDH@Z',
    6070:'?SetParaFormat@CRichEditCtrl@@QAEHAAU_paraformat@@@Z',
    6071:'?SetParaFormat@CRichEditView@@QAEHAAU_paraformat@@@Z',
    6072:'?SetParamValue@CDaoQueryDef@@UAEXHABVCOleVariant@@@Z',
    6073:'?SetParamValue@CDaoQueryDef@@UAEXPBDABVCOleVariant@@@Z',
    6074:'?SetParamValue@CDaoRecordset@@UAEXHABVCOleVariant@@@Z',
    6075:'?SetParamValue@CDaoRecordset@@UAEXPBDABVCOleVariant@@@Z',
    6076:'?SetParamValueNull@CDaoQueryDef@@QAEXH@Z',
    6077:'?SetParamValueNull@CDaoQueryDef@@QAEXPBD@Z',
    6078:'?SetParamValueNull@CDaoRecordset@@QAEXH@Z',
    6079:'?SetParamValueNull@CDaoRecordset@@QAEXPBD@Z',
    6080:'?SetPathName@CDocument@@UAEXPBDH@Z',
    6081:'?SetPathName@COleDocument@@UAEXPBDH@Z',
    6082:'?SetPathName@CRichEditDoc@@UAEXPBDH@Z',
    6083:'?SetPercentPosition@CDaoRecordset@@QAEXM@Z',
    6084:'?SetPictureDispatch@CPictureHolder@@QAEXPAUIPictureDisp@@@Z',
    6085:'?SetPictureProp@CPicturePropPage@@IAEHPAVCDataExchange@@PAUIPictureDisp@@PBD@Z',
    6086:'?SetPolyFillMode@CDC@@QAEHH@Z',
    6087:'?SetPrintDevice@COleClientItem@@QAEHPBUtagDVTARGETDEVICE@@@Z',
    6088:'?SetPrintDevice@COleClientItem@@QAEHPBUtagPDA@@@Z',
    6089:'?SetPrinterFont@CEditView@@QAEXPAVCFont@@@Z',
    6090:'?SetPrintView@CPreviewView@@QAEHPAVCView@@@Z',
    6091:'?SetPropCheck@COlePropertyPage@@QAEHPBDH@Z',
    6092:'?SetProperty@COleControlSite@@UAAXJGZZ',
    6093:'?SetProperty@COleDispatchDriver@@QAAXJGZZ',
    6094:'?SetProperty@CWnd@@QAAXJGZZ',
    6095:'?SetPropertyV@COleControlSite@@UAEXJGPAD@Z',
    6096:'?SetPropIndex@COlePropertyPage@@QAEHPBDH@Z',
    6097:'?SetPropRadio@COlePropertyPage@@QAEHPBDH@Z',
    6098:'?SetPropsetData@COleControl@@IAEHPAUtagFORMATETC@@PAUtagSTGMEDIUM@@ABU_GUID@@@Z',
    6099:'?SetPropText@COlePropertyPage@@QAEHPBDAAE@Z',
    6100:'?SetPropText@COlePropertyPage@@QAEHPBDAAF@Z',
    6101:'?SetPropText@COlePropertyPage@@QAEHPBDAAH@Z',
    6102:'?SetPropText@COlePropertyPage@@QAEHPBDAAI@Z',
    6103:'?SetPropText@COlePropertyPage@@QAEHPBDAAJ@Z',
    6104:'?SetPropText@COlePropertyPage@@QAEHPBDAAK@Z',
    6105:'?SetPropText@COlePropertyPage@@QAEHPBDAAM@Z',
    6106:'?SetPropText@COlePropertyPage@@QAEHPBDAAN@Z',
    6107:'?SetPropText@COlePropertyPage@@QAEHPBDAAVCString@@@Z',
    6108:'?SetQueryTimeout@CDaoDatabase@@QAEXF@Z',
    6109:'?SetRadio@CCmdUI@@UAEXH@Z',
    6110:'?SetRadio@CTestCmdUI@@UAEXH@Z',
    6111:'?SetRange@CSliderCtrl@@QAEXHHH@Z',
    6112:'?SetReadBufferSize@CInternetFile@@QAEHI@Z',
    6113:'?SetRect@XOleDocumentView@CDocObjectServer@@UAGJPAUtagRECT@@@Z',
    6114:'?SetRectComplex@XOleDocumentView@CDocObjectServer@@UAGJPAUtagRECT@@000@Z',
    6115:'?SetRectInContainer@COleControl@@QAEHPBUtagRECT@@@Z',
    6116:'?SetRegistryKey@CWinApp@@IAEXI@Z',
    6117:'?SetRegistryKey@CWinApp@@IAEXPBD@Z',
    6118:'?SetReturnsRecords@CDaoQueryDef@@QAEXH@Z',
    6119:'?SetROP2@CDC@@QAEHH@Z',
    6120:'?SetRowInfo@CSplitterWnd@@QAEXHHH@Z',
    6121:'?SetRowsetCurrencyStatus@CRecordset@@UAEXFGJK@Z',
    6122:'?SetRowsetCursorPosition@CRecordset@@QAEXGG@Z',
    6123:'?SetRowsetSize@CRecordset@@UAEXK@Z',
    6124:'?SetScaledSize@CPreviewView@@IAEXI@Z',
    6125:'?SetScaleRatio@CPreviewDC@@QAEXHH@Z',
    6126:'?SetScaleToFitSize@CScrollView@@QAEXUtagSIZE@@@Z',
    6127:'?SetScreenSize@CDockState@@QAEXAAVCSize@@@Z',
    6128:'?SetScrollInfo@CWnd@@QAEHHPAUtagSCROLLINFO@@H@Z',
    6129:'?SetScrollPos@CWnd@@QAEHHHH@Z',
    6130:'?SetScrollRange@CWnd@@QAEXHHHH@Z',
    6131:'?SetScrollSizes@CScrollView@@QAEXHUtagSIZE@@ABU2@1@Z',
    6132:'?SetScrollStyle@CSplitterWnd@@QAEXK@Z',
    6133:'?SetSectionName@CPropertySection@@QAEHPBD@Z',
    6134:'?SetSel@CRichEditCtrl@@QAEXJJ@Z',
    6135:'?SetSelection@CSliderCtrl@@QAEXHH@Z',
    6136:'?SetSelectionCharFormat@CRichEditCtrl@@QAEHAAU_charformat@@@Z',
    6137:'?SetServerInfo@CDocTemplate@@QAEXIIPAUCRuntimeClass@@0@Z',
    6138:'?SetSize@CArchiveStream@@UAGJT_ULARGE_INTEGER@@@Z',
    6139:'?SetSize@CByteArray@@QAEXHH@Z',
    6140:'?SetSize@CDWordArray@@QAEXHH@Z',
    6141:'?SetSize@CObArray@@QAEXHH@Z',
    6142:'?SetSize@CPtrArray@@QAEXHH@Z',
    6143:'?SetSize@CStringArray@@QAEXHH@Z',
    6144:'?SetSize@CUIntArray@@QAEXHH@Z',
    6145:'?SetSize@CWordArray@@QAEXHH@Z',
    6146:'?SetSizes@CToolBar@@QAEXUtagSIZE@@0@Z',
    6147:'?SetSourceTableName@CDaoTableDef@@QAEXPBD@Z',
    6148:'?SetSplitCursor@CSplitterWnd@@MAEXH@Z',
    6149:'?SetSQL@CDaoQueryDef@@QAEXPBD@Z',
    6150:'?SetStandardProp@CCmdTarget@@IAEJPBUAFX_DISPMAP_ENTRY@@PAUtagDISPPARAMS@@PAI@Z',
    6151:'?SetState@CColorButton@@QAEXH@Z',
    6152:'?SetState@CRecordset@@IAEXHPBDK@Z',
    6153:'?SetStatus@CFile@@SGXPBDABUCFileStatus@@@Z',
    6154:'?SetStatusText@CControlBar@@UAEHH@Z',
    6155:'?SetStatusText@XOleInPlaceFrame@COleFrameHook@@UAGJPBG@Z',
    6156:'?SetStatusText@XOleIPFrame@COleControlContainer@@UAGJPBG@Z',
    6157:'?SetStretchBltMode@CDC@@QAEHH@Z',
    6158:'?SetString@COleVariant@@QAEXPBDG@Z',
    6159:'?SetSysString@CString@@QBEPAGPAPAG@Z',
    6160:'?SetSystemFont@CDialogTemplate@@QAEHG@Z',
    6161:'?SetTabStops@CEditView@@QAEXH@Z',
    6162:'?SetTemplate@CDialogTemplate@@IAEHPBUDLGTEMPLATE@@I@Z',
    6163:'?SetTemplate@CFileDialog@@QAEXPBD0@Z',
    6164:'?SetText@CCmdUI@@UAEXPBD@Z',
    6165:'?SetText@COleCmdUI@@UAEXPBD@Z',
    6166:'?SetText@COleControl@@QAEXPBD@Z',
    6167:'?SetText@CStatusCmdUI@@UAEXPBD@Z',
    6168:'?SetText@CTestCmdUI@@UAEXPBD@Z',
    6169:'?SetText@CToolCmdUI@@UAEXPBD@Z',
    6170:'?SetTextAlign@CDC@@QAEII@Z',
    6171:'?SetTextCharacterExtra@CDC@@QAEHH@Z',
    6172:'?SetTextColor@CDC@@UAEKK@Z',
    6173:'?SetTextColor@CPreviewDC@@UAEKK@Z',
    6174:'?SetTextJustification@CDC@@QAEHHH@Z',
    6175:'?SetTitle@CDocument@@UAEXPBD@Z',
    6176:'?SetTitle@CPropertySheet@@QAEXPBDI@Z',
    6177:'?SetTitle@CRichEditDoc@@UAEXPBD@Z',
    6178:'?SetToolRect@CToolTipCtrl@@QAEXPAVCWnd@@IPBUtagRECT@@@Z',
    6179:'?SetTopLeftOffset@CPreviewDC@@QAEXVCSize@@@Z',
    6180:'?SetType@CProperty@@QAEXK@Z',
    6181:'?SetUpdateMethod@CRecordset@@IAEXXZ',
    6182:'?SetValidationRule@CDaoTableDef@@QAEXPBD@Z',
    6183:'?SetValidationText@CDaoTableDef@@QAEXPBD@Z',
    6184:'?SetValue@CThreadSlotData@@QAEXHPAX@Z',
    6185:'?SetViewInfo@XOleUIObjInfo@COlePropertiesDialog@@UAGJKPAXKHH@Z',
    6186:'?SetViewportExt@CDC@@UAE?AVCSize@@HH@Z',
    6187:'?SetViewportExt@CMetaFileDC@@UAE?AVCSize@@HH@Z',
    6188:'?SetViewportExt@CPreviewDC@@UAE?AVCSize@@HH@Z',
    6189:'?SetViewportOrg@CDC@@UAE?AVCPoint@@HH@Z',
    6190:'?SetViewportOrg@CMetaFileDC@@UAE?AVCPoint@@HH@Z',
    6191:'?SetViewportOrg@CPreviewDC@@UAE?AVCPoint@@HH@Z',
    6192:'?SetWindowExt@CDC@@UAE?AVCSize@@HH@Z',
    6193:'?SetWindowExt@CPreviewDC@@UAE?AVCSize@@HH@Z',
    6194:'?SetWindowOrg@CDC@@QAE?AVCPoint@@HH@Z',
    6195:'?SetWindowPlacement@CWnd@@QAEHPBUtagWINDOWPLACEMENT@@@Z',
    6196:'?SetWindowPos@COleControlSite@@UAEHPBVCWnd@@HHHHI@Z',
    6197:'?SetWindowPos@CWnd@@QAEHPBV1@HHHHI@Z',
    6198:'?SetWindowTextA@COleControlSite@@UAEXPBD@Z',
    6199:'?SetWindowTextA@CWnd@@QAEXPBD@Z',
    6200:'?SetWordCharFormat@CRichEditCtrl@@QAEHAAU_charformat@@@Z',
    6201:'?SetWriteBufferSize@CInternetFile@@QAEHI@Z',
    6202:'?SetZoomState@CPreviewView@@IAEXIIVCPoint@@@Z',
    6203:'?Show@XOleDocumentView@CDocObjectServer@@UAGJH@Z',
    6204:'?Show@XPropertyPage@COlePropertyPage@@UAGJI@Z',
    6205:'?ShowAll@CDockBar@@QAEXH@Z',
    6206:'?ShowContainerUI@CRichEditCntrItem@@QAEJH@Z',
    6207:'?ShowContainerUI@CRichEditView@@QAEJH@Z',
    6208:'?ShowContainerUI@XRichEditOleCallback@CRichEditView@@UAGJH@Z',
    6209:'?ShowControlBar@CFrameWnd@@QAEXPAVCControlBar@@HH@Z',
    6210:'?ShowObject@XOleClientSite@COleClientItem@@UAGJXZ',
    6211:'?ShowObject@XOleClientSite@COleControlSite@@UAGJXZ',
    6212:'?ShowOwnedWindows@CFrameWnd@@QAEXH@Z',
    6213:'?ShowPropertyFrame@XOleControlSite@COleControlSite@@UAGJXZ',
    6214:'?ShowWindow@COleControlSite@@UAEHH@Z',
    6215:'?ShowWindow@CWnd@@QAEHH@Z',
    6216:'?sizeDefault@CScrollView@@2UtagSIZE@@B',
    6217:'?SizeToContent@CBitmapButton@@QAEXXZ',
    6218:'?SizeToolBar@CToolBar@@IAEXPAU_TBBUTTON@@HHH@Z',
    6219:'?Skip@XEnumVOID@CEnumArray@@UAGJK@Z',
    6220:'?SkipDeletedRecords@CRecordset@@QAEXGJPAKPAF@Z',
    6221:'?Socket@CAsyncSocket@@QAEHHJHH@Z',
    6222:'?SpanExcluding@CString@@QBE?AV1@PBD@Z',
    6223:'?SpanIncluding@CString@@QBE?AV1@PBD@Z',
    6224:'?SplitColumn@CSplitterWnd@@UAEHH@Z',
    6225:'?SplitDialogTemplate@COccManager@@UAEPAUDLGTEMPLATE@@PBU2@PAPAUDLGITEMTEMPLATE@@@Z',
    6226:'?SplitRow@CSplitterWnd@@UAEHH@Z',
    6227:'?StartDocA@CDC@@QAEHPBD@Z',
    6228:'?StartDrag@CDockContext@@UAEXVCPoint@@@Z',
    6229:'?StartResize@CDockContext@@UAEXHVCPoint@@@Z',
    6230:'?StartTracking@CSplitterWnd@@MAEXH@Z',
    6231:'?Stat@CArchiveStream@@UAGJPAUtagSTATSTG@@K@Z',
    6232:'?StopTracking@CSplitterWnd@@MAEXH@Z',
    6233:'?Store@CRuntimeClass@@QBEXAAVCArchive@@@Z',
    6234:'?StoreFields@CDaoRecordset@@IAEXXZ',
    6235:'?StoreFields@CRecordset@@QAEXXZ',
    6236:'?Stream@CRichEditView@@QAEXAAVCArchive@@H@Z',
    6237:'?Stretch@CDockContext@@QAEXVCPoint@@@Z',
    6238:'?StripBrackets@CDaoRecordset@@SAXPBDPAD@Z',
    6239:'?SubclassCtl3d@CWnd@@QAEHH@Z',
    6240:'?SubclassDlg3d@CWnd@@QAEHK@Z',
    6241:'?SubclassDlgItem@CWnd@@QAEHIPAV1@@Z',
    6242:'?SubclassWindow@CWnd@@QAEHPAUHWND__@@@Z',
    6243:'?SyncAfter@XNotifyDBEvents@COleControlSite@@UAGJKKQAUtagDBNOTIFYREASON@@@Z',
    6244:'?SyncBefore@XNotifyDBEvents@COleControlSite@@UAGJKKQAUtagDBNOTIFYREASON@@@Z',
    6245:'?SyncToRichEditObject@CRichEditCntrItem@@QAEXAAU_reobject@@@Z',
    6246:'?TabbedTextOutA@CDC@@UAE?AVCSize@@HHPBDHHPAHH@Z',
    6247:'?TabbedTextOutA@CMetaFileDC@@UAE?AVCSize@@HHPBDHHPAHH@Z',
    6248:'?TabbedTextOutA@CPreviewDC@@UAE?AVCSize@@HHPBDHHPAHH@Z',
    6249:'?TextNotFound@CRichEditView@@QAEXPBD@Z',
    6250:'?TextOutA@CDC@@UAEHHHPBDH@Z',
    6251:'?TextOutA@CMetaFileDC@@UAEHHHPBDH@Z',
    6252:'?TextOutA@CPreviewDC@@UAEHHHPBDH@Z',
    6253:'?ThrowDaoException@CDaoDatabase@@MAEXH@Z',
    6254:'?ThrowDaoException@CDaoQueryDef@@MAEXH@Z',
    6255:'?ThrowDaoException@CDaoRecordset@@MAEXH@Z',
    6256:'?ThrowDaoException@CDaoTableDef@@MAEXH@Z',
    6257:'?ThrowDaoException@CDaoWorkspace@@MAEXH@Z',
    6258:'?ThrowDBException@CDatabase@@MAEXF@Z',
    6259:'?ThrowDBException@CRecordset@@UAEXFPAX@Z',
    6260:'?ThrowErrno@CFileException@@SGXHPBD@Z',
    6261:'?ThrowError@COleControl@@QAEXJII@Z',
    6262:'?ThrowError@COleControl@@QAEXJPBDI@Z',
    6263:'?ThrowGetRowsDaoException@@YAXJ@Z',
    6264:'?ThrowOsError@CFileException@@SGXJPBD@Z',
    6265:'?ToggleDocking@CDockContext@@UAEXXZ',
    6266:'?Track@CDockContext@@QAEHXZ',
    6267:'?Track@CRectTracker@@QAEHPAVCWnd@@VCPoint@@H0@Z',
    6268:'?TrackColumnSize@CSplitterWnd@@MAEXHH@Z',
    6269:'?TrackHandle@CRectTracker@@IAEHHPAVCWnd@@VCPoint@@0@Z',
    6270:'?TrackPopupMenu@CMenu@@QAEHIHHPAVCWnd@@PBUtagRECT@@@Z',
    6271:'?TrackRowSize@CSplitterWnd@@MAEXHH@Z',
    6272:'?TrackRubberBand@CRectTracker@@QAEHPAVCWnd@@VCPoint@@H@Z',
    6273:'?TransformCoords@COleControl@@QAEXPAU_POINTL@@PAUtagPOINTF@@K@Z',
    6274:'?TransformCoords@XOleControlSite@COleControlSite@@UAGJPAU_POINTL@@PAUtagPOINTF@@K@Z',
    6275:'?TranslateAcceleratorA@XOleControlSite@COleControlSite@@UAGJPAUtagMSG@@K@Z',
    6276:'?TranslateAcceleratorA@XOleInPlaceActiveObject@COleControl@@UAGJPAUtagMSG@@@Z',
    6277:'?TranslateAcceleratorA@XOleInPlaceActiveObject@COleServerDoc@@UAGJPAUtagMSG@@@Z',
    6278:'?TranslateAcceleratorA@XOleInPlaceFrame@COleFrameHook@@UAGJPAUtagMSG@@G@Z',
    6279:'?TranslateAcceleratorA@XOleIPFrame@COleControlContainer@@UAGJPAUtagMSG@@G@Z',
    6280:'?TranslateAcceleratorA@XPropertyPage@COlePropertyPage@@UAGJPAUtagMSG@@@Z',
    6281:'?TranslateColor@COleControl@@QAEKKPAUHPALETTE__@@@Z',
    6282:'?TrimLeft@CString@@QAEXXZ',
    6283:'?TrimRight@CString@@QAEXXZ',
    6284:'?UIActivate@XOleDocumentView@CDocObjectServer@@UAGJH@Z',
    6285:'?UIActivateControl@COccManager@@KAXPAVCWnd@@@Z',
    6286:'?UIDeactivate@XOleInPlaceObject@COleControl@@UAGJXZ',
    6287:'?UIDeactivate@XOleInPlaceObject@COleServerDoc@@UAGJXZ',
    6288:'?UIDeactivateIfNecessary@COccManager@@KAXPAVCWnd@@0@Z',
    6289:'?UnaccessData@COleSafeArray@@QAEXXZ',
    6290:'?Unadvise@XConnPt@CConnectionPoint@@UAGJK@Z',
    6291:'?Unadvise@XOleObject@CDocObjectServer@@UAGJK@Z',
    6292:'?Unadvise@XOleObject@COleControl@@UAGJK@Z',
    6293:'?Unadvise@XOleObject@COleServerDoc@@UAGJK@Z',
    6294:'?Unadvise@XOleObject@COleServerItem@@UAGJK@Z',
    6295:'?UnbindFieldsForUpdate@CRecordset@@QAEXXZ',
    6296:'?Uncache@XOleCache@COleControl@@UAGJK@Z',
    6297:'?Unfreeze@XViewObject@COleControl@@UAGJK@Z',
    6298:'?Unlock@CCriticalSection@@UAEHXZ',
    6299:'?Unlock@CEvent@@UAEHXZ',
    6300:'?Unlock@CMultiLock@@QAEHJPAJ@Z',
    6301:'?Unlock@CMultiLock@@QAEHXZ',
    6302:'?Unlock@CMutex@@UAEHXZ',
    6303:'?Unlock@COleSafeArray@@QAEXXZ',
    6304:'?Unlock@CSemaphore@@UAEHJPAJ@Z',
    6305:'?Unlock@CSemaphore@@UAEHXZ',
    6306:'?Unlock@CSingleLock@@QAEHJPAJ@Z',
    6307:'?Unlock@CSingleLock@@QAEHXZ',
    6308:'?Unlock@CSyncObject@@UAEHJPAJ@Z',
    6309:'?Unlock@CTypeLibCache@@QAEXXZ',
    6310:'?UnlockBuffer@CEditView@@QBEXXZ',
    6311:'?UnlockBuffer@CString@@QAEXXZ',
    6312:'?UnlockRange@CFile@@UAEXKK@Z',
    6313:'?UnlockRange@CInternetFile@@UAEXKK@Z',
    6314:'?UnlockRange@CMemFile@@UAEXKK@Z',
    6315:'?UnlockRange@COleStreamFile@@UAEXKK@Z',
    6316:'?UnlockRange@CSocketFile@@UAEXKK@Z',
    6317:'?UnlockRange@CStdioFile@@UAEXKK@Z',
    6318:'?UnlockRegion@CArchiveStream@@UAGJT_ULARGE_INTEGER@@0K@Z',
    6319:'?Unregister@CWinApp@@QAEHXZ',
    6320:'?UnsubclassWindow@CWnd@@QAEPAUHWND__@@XZ',
    6321:'?Update@CDaoRecordset@@UAEXXZ',
    6322:'?Update@CRecordset@@UAEHXZ',
    6323:'?Update@XOleObject@CDocObjectServer@@UAGJXZ',
    6324:'?Update@XOleObject@COleControl@@UAGJXZ',
    6325:'?Update@XOleObject@COleServerDoc@@UAGJXZ',
    6326:'?Update@XOleObject@COleServerItem@@UAGJXZ',
    6327:'?UpdateAllItems@COleServerDoc@@QAEXPAVCOleServerItem@@JPAVCObject@@W4tagDVASPECT@@@Z',
    6328:'?UpdateAllPanes@CStatusBar@@IAEXHH@Z',
    6329:'?UpdateAllViews@CDocument@@QAEXPAVCView@@JPAVCObject@@@Z',
    6330:'?UpdateBars@CScrollView@@IAEXXZ',
    6331:'?UpdateClientEdge@CMDIChildWnd@@IAEHPAUtagRECT@@@Z',
    6332:'?UpdateControls@CDataSourceControl@@UAEJXZ',
    6333:'?UpdateCursor@CDataSourceControl@@UAEJXZ',
    6334:'?UpdateData@CWnd@@QAEHH@Z',
    6335:'?UpdateDialogControls@CWnd@@QAEXPAVCCmdTarget@@H@Z',
    6336:'?UpdateFrameCounts@CDocument@@UAEXXZ',
    6337:'?UpdateFrameTitleForDocument@CFrameWnd@@QAEXPBD@Z',
    6338:'?UpdateInsertDelete@CRecordset@@IAEHXZ',
    6339:'?UpdateItemType@COleClientItem@@QAEXXZ',
    6340:'?UpdateLink@COleClientItem@@QAEHXZ',
    6341:'?UpdateLink@COleUILinkInfo@@UAGJKHH@Z',
    6342:'?UpdateLogFont@CSizeComboBox@@QAEXPAUtagLOGFONTA@@H@Z',
    6343:'?UpdateMenu@CRecentFileList@@UAEXPAVCCmdUI@@@Z',
    6344:'?UpdateModifiedFlag@COleDocument@@UAEXXZ',
    6345:'?UpdateModifiedFlag@CRichEditDoc@@UAEXXZ',
    6346:'?UpdateObjectCache@CRichEditDoc@@IAEXXZ',
    6347:'?UpdatePrinterSelection@CWinApp@@IAEXH@Z',
    6348:'?UpdateRegistry@CColorPropPageFactory@CColorPropPage@@UAEHH@Z',
    6349:'?UpdateRegistry@CFontPropPageFactory@CFontPropPage@@UAEHH@Z',
    6350:'?UpdateRegistry@COleObjectFactory@@MAEHH@Z',
    6351:'?UpdateRegistry@COleObjectFactory@@QAEXPBD@Z',
    6352:'?UpdateRegistry@COleTemplateServer@@QAEXW4OLE_APPTYPE@@PAPBD1@Z',
    6353:'?UpdateRegistry@CPicturePropPageFactory@CPicturePropPage@@UAEHH@Z',
    6354:'?UpdateRegistryAll@COleObjectFactory@@SGHH@Z',
    6355:'?UpdateSampleFont@CFontPropPage@@IAEXXZ',
    6356:'?UpdateState@CDockContext@@QAEXPAHH@Z',
    6357:'?UpdateTipText@CToolTipCtrl@@QAEXIPAVCWnd@@I@Z',
    6358:'?UpdateTipText@CToolTipCtrl@@QAEXPBDPAVCWnd@@I@Z',
    6359:'?UpdateUsingHostObj@COleServerDoc@@IAEXIPAVCCmdUI@@@Z',
    6360:'?UpdateVisibleLock@COleLinkingDoc@@IAEXHH@Z',
    6361:'?VerifyConnect@CDatabase@@IAEXXZ',
    6362:'?VerifyCursorSupport@CRecordset@@IAEKXZ',
    6363:'?VerifyDriverBehavior@CRecordset@@IAEXXZ',
    6364:'?VerifyLicenseKey@COleObjectFactory@@MAEHPAG@Z',
    6365:'?VerifyUserLicense@COleObjectFactory@@MAEHXZ',
    6366:'?VKeyToItem@CListBox@@UAEHII@Z',
    6367:'?WalkPreTranslateTree@CWnd@@SGHPAUHWND__@@PAUtagMSG@@@Z',
    6368:'?WillAmbientsBeValidDuringLoad@COleControl@@QAEHXZ',
    6369:'?WindowProc@CControlBar@@UAEJIIJ@Z',
    6370:'?WindowProc@COleControl@@UAEJIIJ@Z',
    6371:'?WindowProc@COlePropertyPage@@MAEJIIJ@Z',
    6372:'?WindowProc@CParkingWnd@@UAEJIIJ@Z',
    6373:'?WindowProc@CReflectorWnd@@MAEJIIJ@Z',
    6374:'?WindowProc@CWnd@@MAEJIIJ@Z',
    6375:'?WinHelpA@CWinApp@@UAEXKI@Z',
    6376:'?WinHelpA@CWnd@@UAEXKI@Z',
    6377:'?wndBottom@CWnd@@2V1@B',
    6378:'?wndNoTopMost@CWnd@@2V1@B',
    6379:'?wndTop@CWnd@@2V1@B',
    6380:'?wndTopMost@CWnd@@2V1@B',
    6381:'?WrapChanged@CRichEditView@@MAEXXZ',
    6382:'?WrapToolBar@CToolBar@@IAEHPAU_TBBUTTON@@HH@Z',
    6383:'?Write@CArchive@@QAEXPBXI@Z',
    6384:'?Write@CArchiveStream@@UAGJPBXKPAK@Z',
    6385:'?Write@CFile@@UAEXPBXI@Z',
    6386:'?Write@CGopherFile@@UAEXPBXI@Z',
    6387:'?Write@CImageList@@QAEHPAVCArchive@@@Z',
    6388:'?Write@CInternetFile@@UAEXPBXI@Z',
    6389:'?Write@CMemFile@@UAEXPBXI@Z',
    6390:'?Write@COleStreamFile@@UAEXPBXI@Z',
    6391:'?Write@CSocketFile@@UAEXPBXI@Z',
    6392:'?Write@CStdioFile@@UAEXPBXI@Z',
    6393:'?WriteClass@CArchive@@QAEXPBUCRuntimeClass@@@Z',
    6394:'?WriteCount@CArchive@@QAEXK@Z',
    6395:'?WriteItem@COleClientItem@@UAEXAAVCArchive@@@Z',
    6396:'?WriteItemCompound@COleClientItem@@QAEXAAVCArchive@@@Z',
    6397:'?WriteItemFlat@COleClientItem@@QAEXAAVCArchive@@@Z',
    6398:'?WriteList@CRecentFileList@@UAEXXZ',
    6399:'?WriteNameDictToStream@CPropertySection@@QAEHPAUIStream@@@Z',
    6400:'?WriteObject@CArchive@@QAEXPBVCObject@@@Z',
    6401:'?WriteProfileBinary@CWinApp@@QAEHPBD0PAEI@Z',
    6402:'?WriteProfileInt@CWinApp@@QAEHPBD0H@Z',
    6403:'?WriteProfileStringA@CWinApp@@QAEHPBD00@Z',
    6404:'?WriteString@CArchive@@QAEXPBD@Z',
    6405:'?WriteString@CGopherFile@@UAEXPBD@Z',
    6406:'?WriteString@CInternetFile@@UAEXPBD@Z',
    6407:'?WriteString@CStdioFile@@UAEXPBD@Z',
    6408:'?WriteToArchive@CEditView@@QAEXAAVCArchive@@@Z',
    6409:'?WriteToStream@CProperty@@QAEHPAUIStream@@@Z',
    6410:'?WriteToStream@CPropertySection@@QAEHPAUIStream@@@Z',
    6411:'?WriteToStream@CPropertySet@@QAEHPAUIStream@@@Z',
    6412:'afxChNil',
    6413:'??0CHttpConnection@@QAE@PAVCInternetSession@@PBDKG11K@Z',
    6414:'??1?$_CIP@UIBindStatusCallback@@$1?IID_IBindStatusCallback@@3U_GUID@@B@@QAE@XZ',
    6415:'??1?$_CIP@UIUnknown@@$1?IID_IUnknown@@3U_GUID@@B@@QAE@XZ=_AfxBinaryCompatibleStubFunction',
    6416:'??1?$CIP@UIBindStatusCallback@@$1?IID_IBindStatusCallback@@3U_GUID@@B@@QAE@XZ',
    6417:'??1?$CIP@UIUnknown@@$1?IID_IUnknown@@3U_GUID@@B@@QAE@XZ',
    6418:'?AfxParseURLEx@@YGHPBDAAKAAVCString@@2AAG22K@Z',
    6419:'?AfxThunkINetCanonicalizeUrl@@YGHPBDPADPAKK@Z',
    6420:'?AfxThunkINetCrackUrl@@YGHPBDKKPAUURL_COMPONENTSA@@@Z',
    6421:'?classCInternetException@CInternetException@@2UCRuntimeClass@@B',
    6422:'?CreateBindStatusCallback@CAsyncMonikerFile@@MAEPAUIUnknown@@PAU2@@Z',
    6423:'?DispatchThreadMessageEx@CWinThread@@IAEHPAUtagMSG@@@Z',
    6424:'?EndCallbacks@CAsyncMonikerFile@@IAEXXZ',
    6425:'?Flush@CMonikerFile@@UAEXXZ',
    6426:'?GetHttpConnection@CInternetSession@@QAEPAVCHttpConnection@@PBDKG00@Z',
    6427:'?Read@CAsyncMonikerFile@@UAEIPAXI@Z',
    6428:'??0CEnumConnections@@QAE@PBXI@Z',
    6429:'??1CEnumConnections@@UAE@XZ',
    6430:'??6CHtmlStream@@QAEAAV0@ABVCByteArray@@@Z',
    6431:'??6CHtmlStream@@QAEAAV0@ABVCLongBinary@@@Z',
    6432:'??6CHttpServerContext@@QAEAAV0@ABVCByteArray@@@Z',
    6433:'??6CHttpServerContext@@QAEAAV0@ABVCLongBinary@@@Z',
    6434:'?_AfxDeleteRegKey@@YGHPBD@Z',
    6435:'?AddConnection@CEnumConnections@@QAEXPAUtagCONNECTDATA@@@Z',
    6436:'?AfxGetDefaultValue@@YAXPAU_DAOField@@AAVCString@@@Z',
    6437:'?AfxGetResourceHandle@@YGPAUHINSTANCE__@@XZ=?AfxGetResourceHandleCompat@@YGPAUHINSTANCE__@@XZ',
    6438:'?AfxInitRichEdit@@YGHXZ',
    6439:'?AfxOleUnregisterHelper@@YGHPBQBD0HPAUHKEY__@@@Z',
    6440:'?AfxOleUnregisterServerClass@@YGHABU_GUID@@PBD11W4OLE_APPTYPE@@PAPBD3@Z',
    6441:'?AfxSetDefaultValue@@YAXPAU_DAOField@@AAVCString@@@Z',
    6442:'?CreateEx@CWnd@@QAEHKPBD0KABUtagRECT@@PAV1@IPAX@Z',
    6443:'?DoMouseWheel@CScrollView@@QAEHIFVCPoint@@@Z',
    6444:'?FreezeAllEvents@COleControlContainer@@QAEXH@Z',
    6445:'?FreezeEvents@COleControlSite@@QAEXH@Z',
    6446:'?GetInterfaceMap@CEnumConnections@@MBEPBUAFX_INTERFACEMAP@@XZ',
    6447:'?HashKey@@YGIPBG@Z',
    6448:'?interfaceMap@COleControlContainer@@1UAFX_INTERFACEMAP@@B',
    6449:'?interfaceMap@COleControlSite@@1UAFX_INTERFACEMAP@@B',
    6450:'?OnClone@CEnumConnections@@MAEPAVCEnumArray@@XZ',
    6451:'?OnCmdMsg@CDocTemplate@@UAEHIHPAXPAUAFX_CMDHANDLERINFO@@@Z',
    6452:'?OnCmdMsg@COleTemplateServer@@MAEHIHPAXPAUAFX_CMDHANDLERINFO@@@Z',
    6453:'?OnDestroy@CWnd@@IAEXXZ',
    6454:'?OnMouseWheel@CScrollView@@QAEHIFVCPoint@@@Z',
    6455:'?OnMouseWheel@CSplitterWnd@@IAEHIFVCPoint@@@Z',
    6456:'?OnNext@CEnumConnections@@MAEHPAX@Z',
    6457:'?OnRegisteredMouseWheel@CFrameWnd@@IAEJIJ@Z',
    6458:'?OnSettingChange@CWnd@@IAEXIPBD@Z',
    6459:'?SetParamNull@CRecordset@@QAEXHH@Z',
    6460:'?Unregister@COleObjectFactory@@QAEHXZ',
    6461:'?Unregister@COleTemplateServer@@QAEHXZ',
    6462:'?UnregisterAll@COleObjectFactory@@SGHXZ',
    6463:'?UnregisterShellFileTypes@CDocManager@@QAEXXZ',
    6464:'?UnregisterShellFileTypes@CWinApp@@IAEXXZ',
    6465:'??0AFX_CLASSINIT_COMPAT@@QAE@PAUCRuntimeClass@@@Z',
    6466:'??0AFX_EVENT@@QAE@H@Z',
    6467:'??0AFX_MAINTAIN_STATE2@@QAE@PAVAFX_MODULE_STATE@@@Z',
    6468:'??0CCachedDataPathProperty@@QAE@PAVCOleControl@@@Z',
    6469:'??0CColorPropPageFactory@CColorPropPage@@QAE@ABU_GUID@@PAUCRuntimeClass@@HPBD@Z',
    6470:'??0CDaoErrorInfo@@QAE@XZ',
    6471:'??0CDaoFieldInfo@@QAE@XZ',
    6472:'??0CDataPathProperty@@QAE@PAVCOleControl@@@Z',
    6473:'??0CDynLinkLibrary@@QAE@PAUHINSTANCE__@@0@Z',
    6474:'??0CFileStatus@@QAE@XZ',
    6475:'??0CFixedAlloc@@QAE@II@Z',
    6476:'??0CFontPropPageFactory@CFontPropPage@@QAE@ABU_GUID@@PAUCRuntimeClass@@HPBD@Z',
    6477:'??0CGopherLocator@@QAE@ABV0@@Z',
    6478:'??0CHtmlView@@IAE@XZ',
    6479:'??0CMemoryException@@QAE@HI@Z',
    6480:'??0CNotSupportedException@@QAE@HI@Z',
    6481:'??0COleDocObjectItem@@QAE@PAVCOleDocument@@@Z',
    6482:'??0COleVariant@@QAE@PBU_ITEMIDLIST@@@Z',
    6483:'??0CParkingWnd@@QAE@XZ',
    6484:'??0CPicturePropPageFactory@CPicturePropPage@@QAE@ABU_GUID@@PAUCRuntimeClass@@HPBD@Z',
    6485:'??0CPropertyPageEx@@QAE@IIII@Z',
    6486:'??0CPropertyPageEx@@QAE@PBDIII@Z',
    6487:'??0CPropertyPageEx@@QAE@XZ',
    6488:'??0CPropertySheetEx@@QAE@IPAVCWnd@@IPAUHBITMAP__@@PAUHPALETTE__@@1@Z',
    6489:'??0CPropertySheetEx@@QAE@PBDPAVCWnd@@IPAUHBITMAP__@@PAUHPALETTE__@@2@Z',
    6490:'??0CPropertySheetEx@@QAE@XZ',
    6491:'??0CReBar@@QAE@XZ',
    6492:'??0CResourceException@@QAE@HI@Z',
    6493:'??0CString@@QAE@PBGH@Z',
    6494:'??0CUserException@@QAE@HI@Z',
    6495:'??1?$CEmbeddedButActsLikePtr@VCMapPtrToPtr@@@@QAE@XZ',
    6496:'??1?$CEmbeddedButActsLikePtr@VCPtrList@@@@QAE@XZ',
    6497:'??1AFX_MAINTAIN_STATE2@@QAE@XZ',
    6498:'??1CArchiveException@@UAE@XZ',
    6499:'??1CColorDialog@@UAE@XZ',
    6500:'??1CColorPropPage@@UAE@XZ',
    6501:'??1CColorPropPageFactory@CColorPropPage@@UAE@XZ',
    6502:'??1CComboBoxEx@@UAE@XZ',
    6503:'??1CControlBarInfo@@QAE@XZ',
    6504:'??1CControlDataSource@COleControl@@UAE@XZ',
    6505:'??1CControlFrameWnd@@UAE@XZ',
    6506:'??1CControlRectTracker@@UAE@XZ',
    6507:'??1CDaoErrorInfo@@QAE@XZ',
    6508:'??1CDateTimeCtrl@@UAE@XZ',
    6509:'??1CFindReplaceDialog@@UAE@XZ',
    6510:'??1CFixedAlloc@@QAE@XZ',
    6511:'??1CFontPropPage@@UAE@XZ',
    6512:'??1CFontPropPageFactory@CFontPropPage@@UAE@XZ',
    6513:'??1CHandleMap@@QAE@XZ',
    6514:'??1CHtmlView@@UAE@XZ',
    6515:'??1CIPAddressCtrl@@UAE@XZ',
    6516:'??1CMDIChildWnd@@UAE@XZ',
    6517:'??1CMDIFrameWnd@@UAE@XZ',
    6518:'??1CMemoryException@@UAE@XZ',
    6519:'??1CMiniDockFrameWnd@@UAE@XZ',
    6520:'??1CMonthCalCtrl@@UAE@XZ',
    6521:'??1CNotSupportedException@@UAE@XZ',
    6522:'??1COccManager@@UAE@XZ',
    6523:'??1COleDocObjectItem@@UAE@XZ',
    6524:'??1COleException@@UAE@XZ',
    6525:'??1COleSafeArray@@QAE@XZ',
    6526:'??1CParkingWnd@@UAE@XZ',
    6527:'??1CPicturePropPageFactory@CPicturePropPage@@UAE@XZ',
    6528:'??1CPropertyPageEx@@UAE@XZ',
    6529:'??1CPropertySheetEx@@UAE@XZ',
    6530:'??1CPushRoutingFrame@@QAE@XZ',
    6531:'??1CPushRoutingView@@QAE@XZ',
    6532:'??1CReBar@@UAE@XZ',
    6533:'??1CReflectorWnd@@UAE@XZ',
    6534:'??1CResourceException@@UAE@XZ',
    6535:'??1CRichEditDoc@@UAE@XZ',
    6536:'??1CSocketWnd@@UAE@XZ',
    6537:'??1CTempDC@@UAE@XZ',
    6538:'??1CTempGdiObject@@UAE@XZ',
    6539:'??1CTempImageList@@UAE@XZ',
    6540:'??1CTempMenu@@UAE@XZ',
    6541:'??1CTempWnd@@UAE@XZ',
    6542:'??1CThreadData@@UAE@XZ',
    6543:'??1CTypeLibCacheMap@@UAE@XZ',
    6544:'??1CUserException@@UAE@XZ',
    6545:'??1CWindowlessDC@@UAE@XZ',
    6546:'??3CTempWnd@@SAXPAX@Z',
    6547:'??5CArchive@@QAEAAV0@AAE@Z',
    6548:'??5CArchive@@QAEAAV0@AAG@Z',
    6549:'??5CArchive@@QAEAAV0@AAJ@Z',
    6550:'??5CArchive@@QAEAAV0@AAK@Z',
    6551:'??5CArchive@@QAEAAV0@AAM@Z',
    6552:'??5CArchive@@QAEAAV0@AAN@Z',
    6553:'??6CArchive@@QAEAAV0@E@Z',
    6554:'??6CArchive@@QAEAAV0@G@Z',
    6555:'??6CArchive@@QAEAAV0@J@Z',
    6556:'??6CArchive@@QAEAAV0@K@Z',
    6557:'??6CArchive@@QAEAAV0@M@Z',
    6558:'??6CArchive@@QAEAAV0@N@Z',
    6559:'??8@YAHABU_GUID@@0@Z',
    6560:'??MCTime@@QBEHV0@@Z',
    6561:'??OCTime@@QBEHV0@@Z',
    6562:'?ActivateAndShow@COleDocObjectItem@@IAEXXZ',
    6563:'?ActivateMe@XOleDocumentSite@COleDocObjectItem@@UAGJPAUIOleDocumentView@@@Z',
    6564:'?AddBar@CReBar@@QAEHPAVCWnd@@KKPBDK@Z',
    6565:'?AddBar@CReBar@@QAEHPAVCWnd@@PBDPAVCBitmap@@K@Z',
    6566:'?AddClassIDToList@COleInsertDialog@@IAEXAAPAU_GUID@@AAH1PAU2@@Z',
    6567:'?AddHead@CStringList@@QAEPAU__POSITION@@ABVCString@@@Z',
    6568:'?AddPage@CPropertySheetEx@@QAEXPAVCPropertyPageEx@@@Z',
    6569:'?AddTail@CStringList@@QAEPAU__POSITION@@ABVCString@@@Z',
    6570:'?AfxCheckError@@YGXJ@Z',
    6571:'?AfxClassInit@@YGXPAUCRuntimeClass@@@Z',
    6572:'?AfxDelRegTreeHelper@@YGJPAUHKEY__@@ABVCString@@@Z',
    6573:'?AfxGetExceptionContext@@YAPAUAFX_EXCEPTION_CONTEXT@@XZ',
    6574:'?AfxGetHENV@@YGPAXXZ',
    6575:'?AfxGetMainWnd@@YGPAVCWnd@@XZ',
    6576:'?AfxGetResourceHandleCompat@@YGPAUHINSTANCE__@@XZ',
    6577:'?afxMapHDC@@YGPAVCHandleMap@@H@Z',
    6578:'?afxMapHGDIOBJ@@YGPAVCHandleMap@@H@Z',
    6579:'?afxMapHIMAGELIST@@YGPAVCHandleMap@@H@Z',
    6580:'?afxMapHMENU@@YGPAVCHandleMap@@H@Z',
    6581:'?afxMapHWND@@YGPAVCHandleMap@@H@Z',
    6582:'?AfxOleRegisterServerClass@@YGHABU_GUID@@PBD11W4OLE_APPTYPE@@PAPBD3H11@Z',
    6583:'?AfxTrackerTerm@@YAXXZ',
    6584:'?AfxWingdixTerm@@YAXXZ',
    6585:'?Alloc@CFixedAlloc@@QAEPAXXZ',
    6586:'?AtlA2WHelper@@YGPAGPAGPBDH@Z',
    6587:'?AtlDevModeA2W@@YAPAU_devicemodeW@@PAU1@PAU_devicemodeA@@@Z',
    6588:'?AtlDevModeW2A@@YGPAU_devicemodeA@@PAU1@PAU_devicemodeW@@@Z',
    6589:'?AtlW2AHelper@@YGPADPADPBGH@Z',
    6590:'?AttachControlSite@COleControlContainer@@QAEXPAVCWnd@@@Z',
    6591:'?BeforeNavigate2@CHtmlView@@MAEXPAUIDispatch@@PAUtagVARIANT@@1111PAH@Z',
    6592:'?BuildPropPageArray@CPropertySheetEx@@UAEXXZ',
    6593:'?CalcDynamicLayout@CReBar@@UAE?AVCSize@@HK@Z',
    6594:'?CalcFixedLayout@CReBar@@UAE?AVCSize@@HH@Z',
    6595:'?CheckHstmt@CDatabase@@QBGHFPAX@Z',
    6596:'?classCComboBoxEx@CComboBoxEx@@2UCRuntimeClass@@B',
    6597:'?classCHtmlView@CHtmlView@@2UCRuntimeClass@@B',
    6598:'?classCIPAddressCtrl@CIPAddressCtrl@@2UCRuntimeClass@@B',
    6599:'?classCOleDBRecordView@COleDBRecordView@@2UCRuntimeClass@@B',
    6600:'?classCOleDocObjectItem@COleDocObjectItem@@2UCRuntimeClass@@B',
    6601:'?classCPropertyPageEx@CPropertyPageEx@@2UCRuntimeClass@@B',
    6602:'?classCPropertySheetEx@CPropertySheetEx@@2UCRuntimeClass@@B',
    6603:'?classCReBar@CReBar@@2UCRuntimeClass@@B',
    6604:'?classCReBarCtrl@CReBarCtrl@@2UCRuntimeClass@@B',
    6605:'?ClientToScreen@CWnd@@QBEXPAUtagRECT@@@Z',
    6606:'?CommonConstruct@CPropertyPageEx@@IAEXPBDIII@Z',
    6607:'?CommonConstruct@CPropertySheetEx@@QAEXPAVCWnd@@IPAUHBITMAP__@@PAUHPALETTE__@@1@Z',
    6608:'?Construct@CPropertyPageEx@@QAEXIIII@Z',
    6609:'?Construct@CPropertyPageEx@@QAEXPBDIII@Z',
    6610:'?Construct@CPropertySheetEx@@QAEXIPAVCWnd@@IPAUHBITMAP__@@PAUHPALETTE__@@1@Z',
    6611:'?Construct@CPropertySheetEx@@QAEXPBDPAVCWnd@@IPAUHBITMAP__@@PAUHPALETTE__@@2@Z',
    6612:'?Create@CComboBoxEx@@QAEHKABUtagRECT@@PAVCWnd@@I@Z',
    6613:'?Create@CDateTimeCtrl@@QAEHKABUtagRECT@@PAVCWnd@@I@Z',
    6614:'?Create@CHtmlView@@UAEHPBD0KABUtagRECT@@PAVCWnd@@IPAUCCreateContext@@@Z',
    6615:'?Create@CImageList@@QAEHPAV1@@Z',
    6616:'?Create@CIPAddressCtrl@@QAEHKABUtagRECT@@PAVCWnd@@I@Z',
    6617:'?Create@CMonthCalCtrl@@QAEHKABUtagPOINT@@PAVCWnd@@I@Z',
    6618:'?Create@CMonthCalCtrl@@QAEHKABUtagRECT@@PAVCWnd@@I@Z',
    6619:'?Create@CReBar@@QAEHPAVCWnd@@KKI@Z',
    6620:'?Create@CReBarCtrl@@QAEHKABUtagRECT@@PAVCWnd@@I@Z',
    6621:'?CreateControl@COleControlContainer@@QAEHPAVCWnd@@ABU_GUID@@PBDKPBUtagPOINT@@PBUtagSIZE@@IPAVCFile@@HPAGPAPAVCOleControlSite@@@Z',
    6622:'?CreateControl@COleControlSite@@QAEJPAVCWnd@@ABU_GUID@@PBDKPBUtagPOINT@@PBUtagSIZE@@IPAVCFile@@HPAG@Z',
    6623:'?CreateControl@CWnd@@QAEHABU_GUID@@PBDKPBUtagPOINT@@PBUtagSIZE@@PAV1@IPAVCFile@@HPAG@Z',
    6624:'?CreateEx@CStatusBar@@QAEHPAVCWnd@@KKI@Z',
    6625:'?CreateEx@CToolBar@@QAEHPAVCWnd@@KKVCRect@@I@Z',
    6626:'?CreateNewChild@CMDIFrameWnd@@QAEPAVCMDIChildWnd@@PAUCRuntimeClass@@IPAUHMENU__@@PAUHACCEL__@@@Z',
    6627:'?CreateObject@CHtmlView@@SGPAVCObject@@XZ',
    6628:'?CreateObject@CTempDC@@SGPAVCObject@@XZ',
    6629:'?CreateObject@CTempGdiObject@@SGPAVCObject@@XZ',
    6630:'?CreateObject@CTempImageList@@SGPAVCObject@@XZ',
    6631:'?CreateObject@CTempMenu@@SGPAVCObject@@XZ',
    6632:'?CreateObject@CTempWnd@@SGPAVCObject@@XZ',
    6633:'?CreateTracker@COleControl@@IAEXHHPBUtagRECT@@@Z',
    6634:'?DDV_MinMaxDateTime@@YGXPAVCDataExchange@@AAVCOleDateTime@@PBV2@2@Z',
    6635:'?DDV_MinMaxDateTime@@YGXPAVCDataExchange@@AAVCTime@@PBV2@2@Z',
    6636:'?DDV_MinMaxMonth@@YGXPAVCDataExchange@@AAVCOleDateTime@@PBV2@2@Z',
    6637:'?DDV_MinMaxMonth@@YGXPAVCDataExchange@@AAVCTime@@PBV2@2@Z',
    6638:'?DDV_MinMaxSlider@@YGXPAVCDataExchange@@KKK@Z',
    6639:'?DDX_DateTimeCtrl@@YGXPAVCDataExchange@@HAAVCOleDateTime@@@Z',
    6640:'?DDX_DateTimeCtrl@@YGXPAVCDataExchange@@HAAVCTime@@@Z',
    6641:'?DDX_FieldSlider@@YGXPAVCDataExchange@@HAAHPAVCDaoRecordset@@@Z',
    6642:'?DDX_FieldText@@YGXPAVCDataExchange@@HPADHPAVCDaoRecordset@@@Z',
    6643:'?DDX_FieldText@@YGXPAVCDataExchange@@HPADHPAVCRecordset@@@Z',
    6644:'?DDX_MonthCalCtrl@@YGXPAVCDataExchange@@HAAVCOleDateTime@@@Z',
    6645:'?DDX_MonthCalCtrl@@YGXPAVCDataExchange@@HAAVCTime@@@Z',
    6646:'?DDX_Slider@@YGXPAVCDataExchange@@HAAH@Z',
    6647:'?DDX_Text@@YGXPAVCDataExchange@@HPADH@Z',
    6648:'?Delete@CString@@QAEHHH@Z',
    6649:'?DeleteItem@CComboBoxEx@@QAEHH@Z',
    6650:'?DocumentComplete@CHtmlView@@MAEXPAUIDispatch@@PAUtagVARIANT@@@Z',
    6651:'?DoModal@COleInsertDialog@@QAEHK@Z',
    6652:'?DrawGripper@CControlBar@@QAEXPAVCDC@@ABVCRect@@@Z',
    6653:'?DrawIndirect@CImageList@@QAEHPAU_IMAGELISTDRAWPARAMS@@@Z',
    6654:'?DrawIndirect@CImageList@@QAEHPAVCDC@@HUtagPOINT@@UtagSIZE@@1IKKK@Z',
    6655:'?EnableTrackingToolTips@CWnd@@QAEHH@Z',
    6656:'?EndDialog@CPropertyPage@@QAEXH@Z',
    6657:'?EndRequest@CHttpFile@@QAEHKPAU_INTERNET_BUFFERSA@@K@Z',
    6658:'?eventsinkMap@CHtmlView@@1UAFX_EVENTSINKMAP@@B',
    6659:'?Exec@XOleCommandTarget@COleFrameHook@@UAGJPBU_GUID@@KKPAUtagVARIANT@@1@Z',
    6660:'?ExecCommand@COleDocObjectItem@@QAEJKKPBU_GUID@@@Z',
    6661:'?ExecWB@CHtmlView@@QAEXW4OLECMDID@@W4OLECMDEXECOPT@@PAUtagVARIANT@@2@Z',
    6662:'?Find@CString@@QBEHDH@Z',
    6663:'?Find@CString@@QBEHPBDH@Z',
    6664:'?FontAttrSize@@YAHH@Z',
    6665:'?Free@CFixedAlloc@@QAEXPAX@Z',
    6666:'?FreeAll@CFixedAlloc@@QAEXXZ',
    6667:'?FreeData@CString@@KIXPAUCStringData@@@Z',
    6668:'?GetActiveView@COleDocObjectItem@@QBEPAUIOleDocumentView@@XZ',
    6669:'?GetAddress@CIPAddressCtrl@@QAEHAAE000@Z',
    6670:'?GetAddressBar@CHtmlView@@QBEHXZ',
    6671:'?GetApplication@CHtmlView@@QBEPAUIDispatch@@XZ',
    6672:'?GetAsSystemTime@COleDateTime@@QBEHAAU_SYSTEMTIME@@@Z',
    6673:'?GetAsSystemTime@CTime@@QBEHAAU_SYSTEMTIME@@@Z',
    6674:'?GetBusy@CHtmlView@@QBEHXZ',
    6675:'?GetCheck@CListCtrl@@QBEHH@Z',
    6676:'?GetCheck@CTreeCtrl@@QBEHPAU_TREEITEM@@@Z',
    6677:'?GetColorScheme@CReBarCtrl@@QAEHPAUtagCOLORSCHEME@@@Z',
    6678:'?GetColumnOrderArray@CListCtrl@@QAEHPAHH@Z',
    6679:'?GetContainer@CHtmlView@@QBEPAUIDispatch@@XZ',
    6680:'?GetCookie@CInternetSession@@SAHPBD0AAVCString@@@Z',
    6681:'?GetCookie@CInternetSession@@SAHPBD0PADK@Z',
    6682:'?GetCookieLength@CInternetSession@@SAKPBD0@Z',
    6683:'?GetCurSel@CListBox@@QBEHXZ',
    6684:'?GetCurSel@CMonthCalCtrl@@QBEHAAVCOleDateTime@@@Z',
    6685:'?GetCurSel@CMonthCalCtrl@@QBEHAAVCTime@@@Z',
    6686:'?GetCursor@CDataBoundProperty@@QAEPAUIUnknown@@XZ',
    6687:'?GetCursor@CDataSourceControl@@UAEPAUIUnknown@@XZ',
    6688:'?GetCursor@COleControlSite@@IAEJJPAPAUIUnknown@@PAPAX@Z',
    6689:'?GetDocumentCount@CDocManager@@IAEHXZ',
    6690:'?GetDropTarget@CToolBarCtrl@@QBEJPAPAUIDropTarget@@@Z',
    6691:'?GetEventSinkMap@CHtmlView@@MBEPBUAFX_EVENTSINKMAP@@XZ',
    6692:'?GetFirstDayOfWeek@CMonthCalCtrl@@QBEHPAH@Z',
    6693:'?GetFont@CWnd@@QBEPAVCFont@@XZ',
    6694:'?GetFullName@CHtmlView@@QBE?AVCString@@XZ',
    6695:'?GetFullScreen@CHtmlView@@QBEHXZ',
    6696:'?GetHeaderCtrl@CListCtrl@@QAEPAVCHeaderCtrl@@XZ',
    6697:'?GetHeight@CHtmlView@@QBEJXZ',
    6698:'?GetHelpMenu@COleDocObjectItem@@MAEPAVCMenu@@AAI@Z',
    6699:'?GetHtmlDocument@CHtmlView@@QBEPAUIDispatch@@XZ',
    6700:'?GetIconFromRegistry@COleClientItem@@QBEPAUHICON__@@XZ',
    6701:'?GetIconFromRegistry@COleClientItem@@SAPAUHICON__@@AAU_GUID@@@Z',
    6702:'?GetImageList@CReBarCtrl@@QBEPAVCImageList@@XZ',
    6703:'?GetInterfaceHook@COleFrameHook@@UAEPAUIUnknown@@PBX@Z',
    6704:'?GetInterfaceMap@COleDocObjectItem@@MBEPBUAFX_INTERFACEMAP@@XZ',
    6705:'?GetItem@CComboBoxEx@@QAEHPAUtagCOMBOBOXEXITEMA@@@Z',
    6706:'?GetItemState@CTabCtrl@@QBEKHK@Z',
    6707:'?GetLayout@CDC@@QBEKXZ',
    6708:'?GetLeft@CHtmlView@@QBEJXZ',
    6709:'?GetLocationName@CHtmlView@@QBE?AVCString@@XZ',
    6710:'?GetLocationURL@CHtmlView@@QBE?AVCString@@XZ',
    6711:'?GetMenuBar@CHtmlView@@QBEHXZ',
    6712:'?GetMessageMap@CHtmlView@@MBEPBUAFX_MSGMAP@@XZ',
    6713:'?GetMessageMap@COleDBRecordView@@MBEPBUAFX_MSGMAP@@XZ',
    6714:'?GetMessageMap@CReBar@@MBEPBUAFX_MSGMAP@@XZ',
    6715:'?GetMoniker@CMonikerFile@@QBEPAUIMoniker@@XZ',
    6716:'?GetMonthRange@CMonthCalCtrl@@QBEHAAVCOleDateTime@@0K@Z',
    6717:'?GetMonthRange@CMonthCalCtrl@@QBEHAAVCTime@@0K@Z',
    6718:'?GetMonthRange@CMonthCalCtrl@@QBEHPAU_SYSTEMTIME@@0K@Z',
    6719:'?GetOffline@CHtmlView@@QBEHXZ',
    6720:'?GetOrderArray@CHeaderCtrl@@QAEHPAHH@Z',
    6721:'?GetOwner@CWnd@@QBEPAV1@XZ',
    6722:'?GetPageCount@COleDocObjectItem@@QAEHPAJ0@Z',
    6723:'?GetPageRect@CRichEditView@@QBE?AVCRect@@XZ',
    6724:'?GetParentBrowser@CHtmlView@@QBEPAUIDispatch@@XZ',
    6725:'?GetPrintRect@CRichEditView@@QBE?AVCRect@@XZ',
    6726:'?GetProperty@CHtmlView@@QAE?AVCOleVariant@@PBD@Z',
    6727:'?GetProperty@CHtmlView@@QAEHPBDAAVCString@@@Z',
    6728:'?GetPropSheetHeader@CPropertySheet@@QAEPAU_AFX_OLDPROPSHEETHEADER@@XZ',
    6729:'?GetRange@CDateTimeCtrl@@QBEKPAVCOleDateTime@@0@Z',
    6730:'?GetRange@CDateTimeCtrl@@QBEKPAVCTime@@0@Z',
    6731:'?GetRange@CMonthCalCtrl@@QBEKPAU_SYSTEMTIME@@0@Z',
    6732:'?GetRange@CMonthCalCtrl@@QBEKPAVCOleDateTime@@0@Z',
    6733:'?GetRange@CMonthCalCtrl@@QBEKPAVCTime@@0@Z',
    6734:'?GetRange@CProgressCtrl@@QAEXAAH0@Z',
    6735:'?GetReadyState@CHtmlView@@QBE?AW4tagREADYSTATE@@XZ',
    6736:'?GetRegisterAsBrowser@CHtmlView@@QBEHXZ',
    6737:'?GetRegisterAsDropTarget@CHtmlView@@QBEHXZ',
    6738:'?GetRoutingFrame_@CCmdTarget@@KGPAVCFrameWnd@@XZ',
    6739:'?GetRoutingView_@CCmdTarget@@KGPAVCView@@XZ',
    6740:'?GetRuntimeClass@CComboBoxEx@@UBEPAUCRuntimeClass@@XZ',
    6741:'?GetRuntimeClass@CDateTimeCtrl@@UBEPAUCRuntimeClass@@XZ',
    6742:'?GetRuntimeClass@CHtmlView@@UBEPAUCRuntimeClass@@XZ',
    6743:'?GetRuntimeClass@CIPAddressCtrl@@UBEPAUCRuntimeClass@@XZ',
    6744:'?GetRuntimeClass@CMonthCalCtrl@@UBEPAUCRuntimeClass@@XZ',
    6745:'?GetRuntimeClass@COleDBRecordView@@UBEPAUCRuntimeClass@@XZ',
    6746:'?GetRuntimeClass@COleDocObjectItem@@UBEPAUCRuntimeClass@@XZ',
    6747:'?GetRuntimeClass@CPropertyPageEx@@UBEPAUCRuntimeClass@@XZ',
    6748:'?GetRuntimeClass@CPropertySheetEx@@UBEPAUCRuntimeClass@@XZ',
    6749:'?GetRuntimeClass@CReBar@@UBEPAUCRuntimeClass@@XZ',
    6750:'?GetRuntimeClass@CReBarCtrl@@UBEPAUCRuntimeClass@@XZ',
    6751:'?GetRuntimeClass@CTempDC@@UBEPAUCRuntimeClass@@XZ',
    6752:'?GetRuntimeClass@CTempGdiObject@@UBEPAUCRuntimeClass@@XZ',
    6753:'?GetRuntimeClass@CTempImageList@@UBEPAUCRuntimeClass@@XZ',
    6754:'?GetRuntimeClass@CTempMenu@@UBEPAUCRuntimeClass@@XZ',
    6755:'?GetRuntimeClass@CTempWnd@@UBEPAUCRuntimeClass@@XZ',
    6756:'?GetSafeOwner_@CWnd@@SGPAUHWND__@@PAU2@PAPAU2@@Z',
    6757:'?GetSelRange@CMonthCalCtrl@@QBEHAAVCOleDateTime@@0@Z',
    6758:'?GetSelRange@CMonthCalCtrl@@QBEHAAVCTime@@0@Z',
    6759:'?GetSelRange@CMonthCalCtrl@@QBEHPAU_SYSTEMTIME@@0@Z',
    6760:'?GetSilent@CHtmlView@@QBEHXZ',
    6761:'?GetStatusBar@CHtmlView@@QBEHXZ',
    6762:'?GetSubItemRect@CListCtrl@@QAEHHHHAAVCRect@@@Z',
    6763:'?GetTabControl@CPropertySheet@@QBEPAVCTabCtrl@@XZ',
    6764:'?GetTempName@CMirrorFile@@SA?AVCString@@PBDH@Z',
    6765:'?GetTheaterMode@CHtmlView@@QBEHXZ',
    6766:'?GetTime@CDateTimeCtrl@@QBEHAAVCOleDateTime@@@Z',
    6767:'?GetTime@CDateTimeCtrl@@QBEKAAVCTime@@@Z',
    6768:'?GetTipText@CStatusBarCtrl@@QBE?AVCString@@H@Z',
    6769:'?GetTitle@CFrameWnd@@QBE?AVCString@@XZ',
    6770:'?GetToday@CMonthCalCtrl@@QBEHAAVCOleDateTime@@@Z',
    6771:'?GetToday@CMonthCalCtrl@@QBEHAAVCTime@@@Z',
    6772:'?GetToolBar@CHtmlView@@QBEHXZ',
    6773:'?GetTop@CHtmlView@@QBEJXZ',
    6774:'?GetTopLevelContainer@CHtmlView@@QBEHXZ',
    6775:'?GetType@CHtmlView@@QBE?AVCString@@XZ',
    6776:'?GetVisible@CHtmlView@@QBEHXZ',
    6777:'?HasFont@@YAHPBUDLGTEMPLATE@@@Z',
    6778:'?Insert@CString@@QAEHHD@Z',
    6779:'?Insert@CString@@QAEHHPBD@Z',
    6780:'?InsertAfter@CStringList@@QAEPAU__POSITION@@PAU2@ABVCString@@@Z',
    6781:'?InsertAt@CStringArray@@QAEXHABVCString@@H@Z',
    6782:'?InsertBefore@CStringList@@QAEPAU__POSITION@@PAU2@ABVCString@@@Z',
    6783:'?InsertEmpty@CStringArray@@IAEXHH@Z',
    6784:'?InsertItem@CComboBoxEx@@QAEHPBUtagCOMBOBOXEXITEMA@@@Z',
    6785:'?InsertItem@CTabCtrl@@QAEHIHPBDHJ@Z',
    6786:'?InsertItem@CTabCtrl@@QAEHIHPBDHJKK@Z',
    6787:'?interfaceMap@COleDocObjectItem@@1UAFX_INTERFACEMAP@@B',
    6788:'?IsButtonEnabled@CPropertyPage@@IAEHH@Z',
    6789:'?IsButtonUp@@YAHPAUtagMSG@@@Z',
    6790:'?IsDialogEx@@YAHPBUDLGTEMPLATE@@@Z',
    6791:'?IsDirSep@@YAHD@Z',
    6792:'?IsEnterKey@@YAHPAUtagMSG@@@Z',
    6793:'?IsHelpKey@@YGHPAUtagMSG@@@Z',
    6794:'?Layout@CToolBar@@IAEXXZ',
    6795:'?LoadBitmapA@CBitmap@@QAEHI@Z',
    6796:'?LoadFromResource@CHtmlView@@QAEHI@Z',
    6797:'?LoadFromResource@CHtmlView@@QAEHPBD@Z',
    6798:'?MDIActivate@CMDIChildWnd@@QAEXXZ',
    6799:'?MDIDestroy@CMDIChildWnd@@QAEXXZ',
    6800:'?messageMap@CHtmlView@@1UAFX_MSGMAP@@B',
    6801:'?messageMap@COleDBRecordView@@1UAFX_MSGMAP@@B',
    6802:'?messageMap@CReBar@@1UAFX_MSGMAP@@B',
    6803:'?Navigate@CHtmlView@@QAEXPBDK00PAXK@Z',
    6804:'?Navigate2@CHtmlView@@QAEXPAU_ITEMIDLIST@@KPBD@Z',
    6805:'?Navigate2@CHtmlView@@QAEXPBDK00PAXK@Z',
    6806:'?Navigate2@CHtmlView@@QAEXPBDKAAVCByteArray@@00@Z',
    6807:'?NavigateComplete2@CHtmlView@@MAEXPAUIDispatch@@PAUtagVARIANT@@@Z',
    6808:'?OnBeforeNavigate2@CHtmlView@@UAEXPBDK0AAVCByteArray@@0PAH@Z',
    6809:'?OnButtonDblClk@COleControl@@IAEXGIVCPoint@@@Z',
    6810:'?OnButtonDown@COleControl@@IAEXGIVCPoint@@@Z',
    6811:'?OnButtonUp@COleControl@@IAEXGIVCPoint@@@Z',
    6812:'?OnCommandStateChange@CHtmlView@@UAEXJH@Z',
    6813:'?OnDestroy@CHtmlView@@IAEXXZ',
    6814:'?OnDocumentComplete@CHtmlView@@UAEXPBD@Z',
    6815:'?OnDownloadBegin@CHtmlView@@UAEXXZ',
    6816:'?OnDownloadComplete@CHtmlView@@UAEXXZ',
    6817:'?OnDraw@CHtmlView@@UAEXPAVCDC@@@Z',
    6818:'?OnEnable@CToolTipCtrl@@IAEXH@Z',
    6819:'?OnEraseBkgnd@COleControl@@IAEHPAVCDC@@@Z',
    6820:'?OnEraseBkgnd@CReBar@@IAEHPAVCDC@@@Z',
    6821:'?OnFieldChange@XRowsetNotify@COleControlSite@@UAGJPAUIRowset@@KKQAKKKH@Z',
    6822:'?OnFilePrint@CHtmlView@@QAEXXZ',
    6823:'?OnFullScreen@CHtmlView@@UAEXH@Z',
    6824:'?OnGetItemPosition@COleDocObjectItem@@MAEXAAVCRect@@@Z',
    6825:'?OnHeightChange@CReBar@@IAEXPAUtagNMHDR@@PAJ@Z',
    6826:'?OnHelpPromptAddr@CFrameWnd@@IAEJIJ@Z',
    6827:'?OnInitialUpdate@COleDBRecordView@@UAEXXZ',
    6828:'?OnInitMenu@CFrameWnd@@IAEXPAVCMenu@@@Z',
    6829:'?OnInitMenu@COleFrameHook@@UAEXPAVCMenu@@@Z',
    6830:'?OnInitMenuPopup@COleFrameHook@@UAEHPAVCMenu@@HH@Z',
    6831:'?OnInsertMenus@COleDocObjectItem@@UAEXPAVCMenu@@PAUtagOleMenuGroupWidths@@@Z',
    6832:'?OnMenuBar@CHtmlView@@UAEXH@Z',
    6833:'?OnMenuSelect@COleFrameHook@@UAEHIIPAUHMENU__@@@Z',
    6834:'?OnMove@COleDBRecordView@@UAEHI@Z',
    6835:'?OnNavigateComplete2@CHtmlView@@UAEXPBD@Z',
    6836:'?OnNcCalcSize@CReBar@@IAEXHPAUtagNCCALCSIZE_PARAMS@@@Z',
    6837:'?OnNcCreate@CReBar@@IAEHPAUtagCREATESTRUCTA@@@Z',
    6838:'?OnNcPaint@CReBar@@IAEXXZ',
    6839:'?OnNewWindow2@CHtmlView@@UAEXPAPAUIDispatch@@PAH@Z',
    6840:'?OnPaint@CHtmlView@@IAEXXZ',
    6841:'?OnPaint@CReBar@@IAEXXZ',
    6842:'?OnPreparePrinting@COleDocObjectItem@@SAHPAVCView@@PAUCPrintInfo@@H@Z',
    6843:'?OnPreserveZeroBorderHelper@CToolBar@@IAEJIJ@Z',
    6844:'?OnPrint@COleDocObjectItem@@SAXPAVCView@@PAUCPrintInfo@@H@Z',
    6845:'?OnProgressChange@CHtmlView@@UAEXJJ@Z',
    6846:'?OnPropertyChange@CHtmlView@@UAEXPBD@Z',
    6847:'?OnQuit@CHtmlView@@UAEXXZ',
    6848:'?OnRecalcParent@CReBar@@IAEXXZ',
    6849:'?OnRemoveMenus@COleDocObjectItem@@UAEXPAVCMenu@@@Z',
    6850:'?OnRowChange@XRowsetNotify@COleControlSite@@UAGJPAUIRowset@@KQBKKKH@Z',
    6851:'?OnRowsetChange@XRowsetNotify@COleControlSite@@UAGJPAUIRowset@@KKH@Z',
    6852:'?OnSetSizeHelper@CToolBar@@IAEJAAVCSize@@J@Z',
    6853:'?OnShowBand@CReBar@@IAEJIJ@Z',
    6854:'?OnSize@CHtmlView@@IAEXIHH@Z',
    6855:'?OnStatusBar@CHtmlView@@UAEXH@Z',
    6856:'?OnStatusTextChange@CHtmlView@@UAEXPBD@Z',
    6857:'?OnTheaterMode@CHtmlView@@UAEXH@Z',
    6858:'?OnTitleChange@CHtmlView@@UAEXPBD@Z',
    6859:'?OnToolBar@CHtmlView@@UAEXH@Z',
    6860:'?OnToolHitTest@CReBar@@UBEHVCPoint@@PAUtagTOOLINFOA@@@Z',
    6861:'?OnToolTipText@CMDIChildWnd@@IAEHIPAUtagNMHDR@@PAJ@Z',
    6862:'?OnUpdateCmdUI@CReBar@@UAEXPAVCFrameWnd@@H@Z',
    6863:'?OnUpdateRecordFirst@COleDBRecordView@@IAEXPAVCCmdUI@@@Z',
    6864:'?OnUpdateRecordLast@COleDBRecordView@@IAEXPAVCCmdUI@@@Z',
    6865:'?OnUpdateRecordNext@COleDBRecordView@@IAEXPAVCCmdUI@@@Z',
    6866:'?OnUpdateRecordPrev@COleDBRecordView@@IAEXPAVCCmdUI@@@Z',
    6867:'?OnVisible@CHtmlView@@UAEXH@Z',
    6868:'?PutProperty@CHtmlView@@QAEXPBDABUtagVARIANT@@@Z',
    6869:'?QueryInfo@CHttpFile@@QBEHKAAKPAK@Z',
    6870:'?QueryStatus@XOleCommandTarget@COleFrameHook@@UAGJPBU_GUID@@KQAU_tagOLECMD@@PAU_tagOLECMDTEXT@@@Z',
    6871:'?QueryStatusWB@CHtmlView@@QBE?AW4OLECMDF@@W4OLECMDID@@@Z',
    6872:'?Release@COleDocObjectItem@@UAEXW4tagOLECLOSE@@@Z',
    6873:'?ReleaseCaches@COleControl@@IAEXXZ',
    6874:'?Remove@CString@@QAEHD@Z',
    6875:'?RemoveAll@CTypeLibCacheMap@@UAEXPAX@Z',
    6876:'?Replace@CString@@QAEHDD@Z',
    6877:'?Replace@CString@@QAEHPBD0@Z',
    6878:'?RFX_Date@@YGXPAVCFieldExchange@@PBDAAVCOleDateTime@@@Z',
    6879:'?RFX_Text@@YGXPAVCFieldExchange@@PBDPADHHF@Z',
    6880:'?ScreenToClient@CWnd@@QBEXPAUtagRECT@@@Z',
    6881:'?SendRequestEx@CHttpFile@@QAEHKKK@Z',
    6882:'?SendRequestEx@CHttpFile@@QAEHPAU_INTERNET_BUFFERSA@@0KK@Z',
    6883:'?SetAtGrow@CStringArray@@QAEXHABVCString@@@Z',
    6884:'?SetBinding@CAsyncMonikerFile@@IAEXPAUIBinding@@@Z',
    6885:'?SetBkImage@CListCtrl@@QAEHPADHHH@Z',
    6886:'?SetBkImage@CListCtrl@@QAEHPAUHBITMAP__@@HHH@Z',
    6887:'?SetBorders@CControlBar@@QAEXHHHH@Z',
    6888:'?SetCheck@CListCtrl@@QAEHHH@Z',
    6889:'?SetCheck@CTreeCtrl@@QAEHPAU_TREEITEM@@H@Z',
    6890:'?SetColorScheme@CReBarCtrl@@QAEXPBUtagCOLORSCHEME@@@Z',
    6891:'?SetColumnOrderArray@CListCtrl@@QAEHHPAH@Z',
    6892:'?SetCookie@CInternetSession@@SAHPBD00@Z',
    6893:'?SetCurSel@CMonthCalCtrl@@QAEHABVCOleDateTime@@@Z',
    6894:'?SetCurSel@CMonthCalCtrl@@QAEHABVCTime@@@Z',
    6895:'?SetDayState@CMonthCalCtrl@@QAEHHPAK@Z',
    6896:'?SetFirstDayOfWeek@CMonthCalCtrl@@QAEHHPAH@Z',
    6897:'?SetFontNotifySink@CFontHolder@@QAEXPAUIPropertyNotifySink@@@Z',
    6898:'?SetFormatEtc@CAsyncMonikerFile@@IAEXPAUtagFORMATETC@@@Z',
    6899:'?SetHandles@CMDIChildWnd@@QAEXPAUHMENU__@@PAUHACCEL__@@@Z',
    6900:'?SetIconSpacing@CListCtrl@@QAE?AVCSize@@HH@Z',
    6901:'?SetIconSpacing@CListCtrl@@QAE?AVCSize@@V2@@Z',
    6902:'?SetImageList@CReBarCtrl@@QAEHPAVCImageList@@@Z',
    6903:'?SetItem@CComboBoxEx@@QAEHPBUtagCOMBOBOXEXITEMA@@@Z',
    6904:'?SetItemCountEx@CListCtrl@@QAEHHK@Z',
    6905:'?SetItemState@CListCtrl@@QAEHHII@Z',
    6906:'?SetItemState@CTabCtrl@@QAEHHKK@Z',
    6907:'?SetItemText@CListCtrl@@QAEHHHPBD@Z',
    6908:'?SetLayout@CDC@@QAEKK@Z',
    6909:'?SetOrderArray@CHeaderCtrl@@QAEHHPAH@Z',
    6910:'?SetRange@CDateTimeCtrl@@QAEHPBVCOleDateTime@@0@Z',
    6911:'?SetRange@CDateTimeCtrl@@QAEHPBVCTime@@0@Z',
    6912:'?SetRange@CMonthCalCtrl@@QAEHPBVCOleDateTime@@0@Z',
    6913:'?SetRange@CMonthCalCtrl@@QAEHPBVCTime@@0@Z',
    6914:'?SetRange@CMonthCalCtrl@@QAEHQAU_SYSTEMTIME@@0@Z',
    6915:'?SetSelectionCheck@CCheckListBox@@IAEXH@Z',
    6916:'?SetSelRange@CMonthCalCtrl@@QAEHABVCOleDateTime@@0@Z',
    6917:'?SetSelRange@CMonthCalCtrl@@QAEHABVCTime@@0@Z',
    6918:'?SetSelRange@CMonthCalCtrl@@QAEHQAU_SYSTEMTIME@@0@Z',
    6919:'?SetTime@CDateTimeCtrl@@QAEHABVCOleDateTime@@@Z',
    6920:'?SetTime@CDateTimeCtrl@@QAEHPAU_SYSTEMTIME@@@Z',
    6921:'?SetTime@CDateTimeCtrl@@QAEHPBVCTime@@@Z',
    6922:'?SetToday@CMonthCalCtrl@@QAEXABVCOleDateTime@@@Z',
    6923:'?SetToday@CMonthCalCtrl@@QAEXPBVCTime@@@Z',
    6924:'?SizeMinReq@CMonthCalCtrl@@QAEHH@Z',
    6925:'?SupportsIPrint@COleDocObjectItem@@IAEHXZ',
    6926:'?ToVariant@CDataSourceControl@@QAE?AVCOleVariant@@H@Z',
    6927:'?TrimLeft@CString@@QAEXD@Z',
    6928:'?TrimLeft@CString@@QAEXPBD@Z',
    6929:'?TrimRight@CString@@QAEXD@Z',
    6930:'?TrimRight@CString@@QAEXPBD@Z',
    6931:'?WindowProc@CReBar@@MAEJIIJ@Z',
}
